/*
 * Returns a cursor with the data for the 'Products with Changes
 * to Copay Tier Data' report.
 *
 * start_date - Report will include products with changes to copay
 * tier data beginning on or after start_date. If start_date is null,
 * all dates will be included.
 *
 * include_active_products - If value is 1, active products are included
 * in the report. Otherwise, active products are not included in the
 * report.
 *
 * include_inactive_products - If value is 1, inactive products are included
 * in the report. Otherwise, inactive products are not included in the
 * report.
 *
 * subfield_delimiter - For products with more than one associated drug class,
 * this value will be used to delimit the drug class codes in the output, as in:
 * T300|XA200|XA201
 *
 * out_cursor - The cursor returned to the client for processing.
 */
CREATE OR REPLACE PROCEDURE PPSNEPL.report_copay_tier_change (
  start_date IN DATE
  , include_active_products IN NUMBER
  , include_inactive_products IN NUMBER
  , subfield_delimiter IN VARCHAR2
  , out_cursor OUT SYS_REFCURSOR) 
AUTHID CURRENT_USER IS
  ignore_start_date NUMBER;
BEGIN
  -- Convert start_date into a true/false value, so that it can be used in the WHERE clause.
  IF start_date IS NULL 
  THEN
    ignore_start_date := 1;
  ELSE
    ignore_start_date := 0;
  END IF;

  -- Return the cursor to the client for processing.
  OPEN out_cursor FOR 
  -- Return one row per product.
  SELECT DISTINCT
      PRODUCTS.EPL_ID
      , PRODUCTS.VA_PRODUCT_NAME
      , PRODUCTS.CMOP_ID
      , PRODUCTS.NATIONAL_FORMULARY_NAME
      , PRODUCTS.INACTIVATION_DATE AS STATUS_DATE
      , PRODUCTS.ITEM_STATUS
      , CURRENT_TIER.TIER_LEVEL AS TIER
      , CURRENT_TIER.EFFECTIVE_DATE
      , CURRENT_TIER.END_DATE
      , FUTURE_TIER.TIER_LEVEL AS FUTURE_TIER_LEVEL
      , FUTURE_TIER.EFFECTIVE_DATE AS FUTURE_EFFECTIVE_DATE
      
      -- Concatenate associated drug class codes using the subfield_delimiter.
      , (
        SELECT LISTAGG(CODE, subfield_delimiter) WITHIN GROUP (ORDER BY CODE)
        FROM EPL_VA_DRUG_CLASSES CLASSES
        INNER JOIN EPL_PROD_DRUG_CLASS_ASSOCS ASSOCS
          ON CLASSES.EPL_ID = ASSOCS.DRUG_CLASS_ID_FK
        WHERE
          ASSOCS.EPL_ID_PRODUCT_FK = PRODUCTS.EPL_ID
      ) AS DRUG_CLASS
    FROM EPL_ITEM_AUDIT_HISTORY HISTORY
    INNER JOIN EPL_ITEM_AUDIT_HISTORY_DETAILS DETAILS
      ON HISTORY.EPL_ID = DETAILS.IAH_ID_FK
    INNER JOIN EPL_PRODUCTS PRODUCTS
      ON HISTORY.AUDIT_ITEM_ID = PRODUCTS.EPL_ID
      
    -- Retain all product records, plus matching current copay tier records if they exist.
    LEFT JOIN (
      SELECT
        ASSOCS.EPL_ID_PRODUCT_FK
        , TIER_LEVEL
        , EFFECTIVE_DATE
        , END_DATE
      FROM
        EPL_PROD_COPAY_TIER_ASSOCS ASSOCS
      INNER JOIN EPL_COPAY_TIER TIER
        ON ASSOCS.EPL_ID_COPAY_TIER_FK = TIER.EPL_ID
      WHERE
          ASSOCS.EFFECTIVE_DATE <= CURRENT_DATE
          AND (
            ASSOCS.END_DATE IS NULL
            OR ASSOCS.END_DATE >= CURRENT_DATE
          )
    ) CURRENT_TIER
      ON CURRENT_TIER.EPL_ID_PRODUCT_FK = PRODUCTS.EPL_ID
    
    -- Retain all product records, plus matching future copay tier records if they exist.
    LEFT JOIN (
      SELECT
        ASSOCS.EPL_ID_PRODUCT_FK
        , TIER_LEVEL
        , EFFECTIVE_DATE
      FROM
        EPL_PROD_COPAY_TIER_ASSOCS ASSOCS
      INNER JOIN EPL_COPAY_TIER TIER
        ON ASSOCS.EPL_ID_COPAY_TIER_FK = TIER.EPL_ID
      WHERE
          ASSOCS.EFFECTIVE_DATE > CURRENT_DATE
    ) FUTURE_TIER
      ON FUTURE_TIER.EPL_ID_PRODUCT_FK = PRODUCTS.EPL_ID
    WHERE
      HISTORY.EVENT_CATEGORY = 'NATIONAL_MODIFICATION'
      AND DETAILS.COL_NM = 'copay.tier.assocs'
      AND (
        -- If ignore_start_date = 1, this OR will return true, thereby including all dates.
        ignore_start_date = 1 OR DETAILS.CREATED_DTM BETWEEN start_date AND CURRENT_DATE
      )
      AND (
        -- If include_inactive_products = 1, this AND will check for 'INACTIVE' products.
        -- Otherwise, this AND will return FALSE, thereby excluding 'INACTIVE' products.
        (include_inactive_products = 1 AND PRODUCTS.ITEM_STATUS = 'INACTIVE')
        OR
        -- If include_active_products = 1, this AND will check for 'ACTIVE' products.
        -- Otherwise, this AND will return FALSE, thereby excluding 'ACTIVE' products. 
        (include_active_products = 1 AND PRODUCTS.ITEM_STATUS = 'ACTIVE')
      )
    ORDER BY
      PRODUCTS.EPL_ID; 
END;
/
GRANT EXECUTE ON PPSNEPL.report_copay_tier_change TO PPSNEPL_READ_ONLY_ROLE;
GRANT EXECUTE ON PPSNEPL.report_copay_tier_change TO PPSN_ROLE;   