SET SERVEROUTPUT ON
SET LINESIZE 2000
SET PAGESIZE 60 
SPOOL PPSNEPL_RESET_PRODUCT_NDF_IEN_WHERE_NULL.log

SELECT 'Update to EPL_PRODUCTS_NDF_PRODUCT_IEN Started: ' || CURRENT_TIMESTAMP FROM DUAL;
 
SET ECHO ON

REM ######################################################################################
REM #  Description: Update the NDF_PRODUCTS_IEN column where it is NULL with an IEN 
REM #               based on a new sequence created from the max value of the same column 
REM #               and incrementing as neeeded
REM ######################################################################################
REM # Change History
REM #
REM # DATE       User Name         DESCRIPTION
REM # ---------- ----------------- -------------------------------------------
REM # 11/01/2017 David Lumb        Created script
REM ######################################################################################

DECLARE

   cc_schema_name   CONSTANT VARCHAR2 (35) := 'PPSNEPL';

   vc_sql_statement          VARCHAR2 (4000);
   vc_errm                   VARCHAR2 (64);
   vn_code                   NUMBER;
   vn_continue               NUMBER;
   
   vn_row_count              NUMBER;

   -----------------------------------------------------------------------------
   -- NAME:         check_for_NDF_IEN_SEQ
   -- TYPE:         Function
   -- DESCRIPTION:  Check to see if PPSN 3 sequence exists for drop/reset
   --               - if so, drop and reset it as appropriate
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
   FUNCTION check_for_NDF_IEN_SEQ
      RETURN NUMBER
   IS
   BEGIN
      SELECT COUNT (*)
        INTO vn_row_count
        FROM dba_sequences
       WHERE sequence_owner = cc_schema_name
          AND sequence_name IN ('EPL_PRODUCTS_NDF_IEN_SEQ');

      IF vn_row_count > 0
      THEN
         RETURN (1);
      END IF;

      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         DBMS_OUTPUT.put_line (
            'check_for_NDF_IEN_SEQ: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
         RETURN (1);
   END check_for_NDF_IEN_SEQ;

   -----------------------------------------------------------------------------
   -- NAME:         create_PRODUCTS_NDF_IEN_SEQ
   -- TYPE:         Function
   -- DESCRIPTION:  Create the sequence needed for mig of package sizes 
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
       
    FUNCTION create_PRODUCTS_NDF_IEN_SEQ
       RETURN NUMBER
    IS
       vc_ndf_products_ien	 	NUMBER;

    BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   START:  create_PRODUCTS_NDF_IEN_SEQ'); 
          
      SELECT NVL(MAX(NDF_PRODUCT_IEN), 0) + 1 INTO vc_ndf_products_ien FROM PPSNEPL.EPL_PRODUCTS ;
     
      vc_sql_statement := 'CREATE SEQUENCE PPSNEPL.EPL_PRODUCTS_NDF_IEN_SEQ'
         || ' MINVALUE 1 '
         || ' MAXVALUE 9999999999999999999999999999 '
         || ' INCREMENT BY 1 '
         || ' START WITH '
         || vc_ndf_products_ien
         || ' NOCACHE '
         || ' NOCYCLE '
         || ' NOORDER ' ;
           
      EXECUTE IMMEDIATE vc_sql_statement;
      DBMS_OUTPUT.put_line('Executed CREATE SEQUENCE PPSNEPL.EPL_PRODUCTS_NDF_IEN_SEQ...') ;  
      
      vc_sql_statement := 'GRANT SELECT, ALTER ON PPSNEPL.EPL_PRODUCTS_NDF_IEN_SEQ TO PPSNEPL_UPDATE_ROLE' ;
      EXECUTE IMMEDIATE vc_sql_statement;
      DBMS_OUTPUT.put_line('Executed GRANT SELECT, ALTER ON PPSNEPL.EPL_PRODUCTS_NDF_IEN_SEQ TO PPSNEPL_UPDATE_ROLE') ;  
      
      vc_sql_statement := 'CREATE OR REPLACE PUBLIC SYNONYM EPL_PRODUCTS_NDF_IEN_SEQ FOR PPSNEPL.EPL_PRODUCTS_NDF_IEN_SEQ' ;
      EXECUTE IMMEDIATE vc_sql_statement;
      DBMS_OUTPUT.put_line('CREATE OR REPLACE PUBLIC SYNONYM EPL_PRODUCTS_NDF_IEN_SEQ FOR PPSNEPL.EPL_PRODUCTS_NDF_IEN_SEQ') ;  
                   
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  create_PRODUCTS_NDF_IEN_SEQ');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'create_PRODUCTS_NDF_IEN_SEQ: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'create_PRODUCTS_NDF_IEN_SEQ: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'create_PRODUCTS_NDF_IEN_SEQ: ' || vc_sql_statement);
         RETURN (1);
   END create_PRODUCTS_NDF_IEN_SEQ; 


   -----------------------------------------------------------------------------
   -- NAME:         update_EPL_PRODUCTS_NDF_IEN
   -- TYPE:         Function
   -- DESCRIPTION:  Update the NDF_PRODUCTS_IEN value where IS NULL   
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
       
    FUNCTION update_EPL_PRODUCTS_NDF_IEN
       RETURN NUMBER
    IS

    BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   START:  update_EPL_PRODUCTS_NDF_IEN'); 
       
      vc_sql_statement := 'UPDATE PPSNEPL.EPL_PRODUCTS SET NDF_PRODUCT_IEN = '
         || 'PPSNEPL.EPL_PRODUCTS_NDF_IEN_SEQ.NEXTVAL '
         || 'WHERE NDF_PRODUCT_IEN IS NULL';

      EXECUTE IMMEDIATE vc_sql_statement;
 
      vc_sql_statement := 'COMMIT' ;

      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  update_EPL_PRODUCTS_NDF_IEN');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'update_EPL_PRODUCTS_NDF_IEN: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'update_EPL_PRODUCTS_NDF_IEN: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'update_EPL_PRODUCTS_NDF_IEN: ' || vc_sql_statement);
         RETURN (1);
   END update_EPL_PRODUCTS_NDF_IEN; 

   -----------------------------------------------------------------------------
   -- NAME:         update_EPL_IEN_GEN
   -- TYPE:         Function
   -- DESCRIPTION:  Insert/update EPL_EIN_GENERATOR based on maxval + 1 of associated table
   --               This is only used for the package sizes IEN row insert now for insert,
   --               and all others for simple reset. Note well, package sizes still not
   --               populated at this time
   -- INPUTS:       None
   -- OUTPUTS:      Number: (0 - good: >0 - error)
   -----------------------------------------------------------------------------
       
    FUNCTION update_EPL_IEN_GEN
       RETURN NUMBER
    IS

       vc_ien_product 		NUMBER;

    BEGIN
      sys.DBMS_OUTPUT.put_line (CHR (10));
      sys.DBMS_OUTPUT.put_line ('INFORMATION -   START:  update_EPL_IEN_GEN'); 
      
      SELECT NVL(MAX(NDF_PRODUCT_IEN),1) 	INTO vc_ien_product 		FROM PPSNEPL.EPL_PRODUCTS ;
       
      vc_sql_statement := 'UPDATE PPSNEPL.EPL_IEN_GENERATOR SET LAST_USED_IEN = '
         ||vc_ien_product
         || ' WHERE CONCEPT = ''EPL_PRODUCTS'' ';
         
      EXECUTE IMMEDIATE vc_sql_statement;

      sys.DBMS_OUTPUT.put_line ('INFORMATION -   END:  update_EPL_IEN_GEN');
      RETURN (0);
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'update_EPL_IEN_GEN: Error code ' || vn_code || ': ' || vc_errm);
         sys.DBMS_OUTPUT.put_line (
            'update_EPL_IEN_GEN: last value for vc_sql_statement ');
         sys.DBMS_OUTPUT.put_line (
            'update_EPL_IEN_GEN: ' || vc_sql_statement);
         RETURN (1);
   END update_EPL_IEN_GEN; 
   
   
-----------------------------------------------------------------------------
-- MAIN SECTION ***
-----------------------------------------------------------------------------
BEGIN
   sys.DBMS_OUTPUT.put_line (CHR (10));
   sys.DBMS_OUTPUT.put_line (
      'INFORMATION -   Checking to see if PPSN NDF_PRODUCTS_IEN_SEQ object already exists.');

   vn_continue := check_for_NDF_IEN_SEQ;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   TEST OF FAILED -> SEQUENCE EXISTS. Stopping NDF_PRODUCTS_IEN UPDATE on EPL_PRODUCTS.');
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPSN 3 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;


   vn_continue := create_PRODUCTS_NDF_IEN_SEQ;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPSNEPL 3 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;


   vn_continue := update_EPL_PRODUCTS_NDF_IEN;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPSNEPL 3 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;
   
   
   vn_continue := update_EPL_IEN_GEN;

   IF vn_continue > 0
   THEN
      sys.DBMS_OUTPUT.put_line (
         'ERROR   -   EXECUTE PPSNEPL 3 ROLLBACK PROCESS');
      sys.DBMS_OUTPUT.put_line ('ERROR   -   EXITING SCRIPT...');
      RETURN;
   END IF;
         
      -- Purge recycle bin
      
      vc_sql_statement := 'purge RECYCLEBIN';
      COMMIT;
   
      EXECUTE IMMEDIATE vc_sql_statement;
   
      -- Recompile Schema and Gather Statistics
      
      DBMS_UTILITY.compile_schema (schema => 'PPSNEPL');
      DBMS_STATS.gather_schema_stats ('PPSNEPL');
      
   EXCEPTION
      WHEN OTHERS
      THEN
         vn_code := SQLCODE;
         vc_errm := SUBSTR (SQLERRM, 1, 64);
         sys.DBMS_OUTPUT.put_line (
            'MAIN: Error code ' || vn_code || ': ' || vc_errm);
   END;
   /
      
   SELECT 'Update to EPL_PRODUCTS_NDF_PRODUCT_IEN Stopped: ' || CURRENT_TIMESTAMP FROM DUAL;
   
   SPOOL OFF
EXIT