package gov.va.med.pharmacy.peps.common.utility;

import org.apache.commons.lang3.StringUtils;

import gov.va.med.pharmacy.peps.common.vo.DispenseUnitVo;
import gov.va.med.pharmacy.peps.common.vo.DosageFormVo;
import gov.va.med.pharmacy.peps.common.vo.DrugClassVo;
import gov.va.med.pharmacy.peps.common.vo.DrugUnitVo;
import gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorVo;
import gov.va.med.pharmacy.peps.common.vo.GenericNameVo;
import gov.va.med.pharmacy.peps.common.vo.OrderableItemVo;
import gov.va.med.pharmacy.peps.common.vo.PossibleDosagesAutoCreate;
import gov.va.med.pharmacy.peps.common.vo.ProductPackage;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;

/**
 * Data Conversion Utility for converting Product data elements to Strings for sending to VistA.
 */
public final class NdfProductDataConversionUtility {

  /**
   * Private constructor to prevent instantiation, since this is a static utility class.
   */
  private NdfProductDataConversionUtility() {
  }

  /** A single space, which is sent to VistA to denote blank. */
  private static final String BLANK = " ";

  /**
   * Gets the old value string from the specified Diff. Returns {@link BLANK} if the value is null or blank.
   *
   * @param diff the Diff from which to get the old value
   * @return the old value string
   */
  public static String getOldValueString(Difference diff) {
    String oldValue = StringUtils.isNotBlank((String) diff.getOldValue()) ? (String) diff.getOldValue() : BLANK;
    return oldValue;
  }

  /**
   * Gets the new value string from the specified Diff. Returns {@link BLANK} if the value is null or blank.
   *
   * @param diff the Diff from which to get the new value
   * @return the old value string
   */
  public static String getNewValueString(Difference diff) {
    String newValue = StringUtils.isNotBlank((String) diff.getNewValue()) ? (String) diff.getNewValue() : BLANK;
    return newValue;

  }

  /**
   * Returns the specified String or {@link BLANK} if it is null or blank.
   *
   * @param value the value
   * @return the old value string
   */
  public static String getStringValue(String value) {
    return StringUtils.isNotBlank(value) ? value : BLANK;
  }

  /**
   * Converts the ProductPackage to a String value for sending to VistA. Returns {@link BLANK} if the value is null or blank.
   *
   * @param productPackage the product package
   * @return the string
   */
  public static String convertProductPackageValue(ProductPackage productPackage) {

    if (productPackage != null) {
      if (productPackage.equals(ProductPackage.INPATIENT)) {
        return "I";
      } else if (productPackage.equals(ProductPackage.OUTPATIENT)) {
        return "O";
      } else {
        return "IO";
      }
    } else {
      return BLANK;
    }
  }

  /**
   * Converts the PossibleDosagesAutoCreate to a String value for sending to VistA. Returns {@link BLANK} if the value is null.
   *
   * @param autoDosage the auto dosage
   * @return the string
   */
  public static String convertDosageAutoCreate(PossibleDosagesAutoCreate autoDosage) {

    if (autoDosage != null) {
      if (autoDosage.equals(PossibleDosagesAutoCreate.NO_POSSIBLE_DOSAGES)) {
        return "N";
      } else if (autoDosage.equals(PossibleDosagesAutoCreate.ONE_X_POSSIBLE_DOSAGES)) {
        return "O";
      } else {
        return "B";
      }
    } else {
      return BLANK;
    }

  }

  /**
   * Converts the CSFedSchedule String value to a String value for sending to VistA.
   *
   * @param csFedScheduleValue String containing the cs fed schedule value
   * @return the string
   */
  public static String convertFedSchedule(String csFedScheduleValue) {

    if (csFedScheduleValue.startsWith("0", 0)) {
      return "0";
    } else if (csFedScheduleValue.startsWith("1", 0)) {
      return "1";
    } else if (csFedScheduleValue.startsWith("2n", 0)) {
      return "2n";
    } else if (csFedScheduleValue.startsWith("2", 0)) {
      return "2";
    } else if (csFedScheduleValue.startsWith("3n", 0)) {
      return "3n";
    } else if (csFedScheduleValue.startsWith("3", 0)) {
      return "3";
    } else if (csFedScheduleValue.startsWith("4", 0)) {
      return "4";
    } else {
      return "5";
    }
  }

  /**
   * Converts the Drug Class old value in the specified Diff to a String value for sending to VistA. Returns {@link BLANK} if
   * the value is null or blank.
   *
   * @param diff the diff containing the DrugClass old value
   * @return the string
   */
  public static String convertOldDrugClass(Difference diff) {

    DrugClassVo oldDrugClass = (DrugClassVo) diff.getOldValue();
    return oldDrugClass.getCode() != null
        ? StringUtils.isNotBlank(oldDrugClass.getDrugClassIen()) ? oldDrugClass.getDrugClassIen() : BLANK : BLANK;

  }

  /**
   * Converts the Drug Class new value in the specified Diff to a String value for sending to VistA. Returns {@link BLANK} if
   * the value is null or blank.
   *
   * @param diff the diff containing the DrugClass old value
   * @param vaProductVo the ProductVo
   * @return the string
   */
  public static String convertNewDrugClass(Difference diff, ProductVo vaProductVo) {

    DrugClassVo drugClass = (DrugClassVo) diff.getNewValue();
    return drugClass.getCode() != null ? StringUtils.isNotBlank(vaProductVo.getPrimaryDrugClass().getDrugClassIen())
        ? vaProductVo.getPrimaryDrugClass().getDrugClassIen() : BLANK : BLANK;

  }

  /**
   * Converts the Drug Class new value in the specified ProductVo to a String value for sending to VistA. Returns {@link BLANK}
   * if the value is null or blank.
   *
   * @param vaProductVo the ProductVo
   * @return the string
   */
  public static String convertNewDrugClass(ProductVo vaProductVo) {

    return vaProductVo.getPrimaryDrugClass().getCode() != null
        ? StringUtils.isNotBlank(vaProductVo.getPrimaryDrugClass().getDrugClassIen())
            ? vaProductVo.getPrimaryDrugClass().getDrugClassIen() : BLANK
        : BLANK;

  }

  /**
   * Converts the DispenseUnitVo to a String value for sending to VistA. Returns {@link BLANK} if the value is null or blank.
   *
   * @param dispenseUnit the dispense unit
   * @return the string
   */
  public static String convertDispenseUnit(DispenseUnitVo dispenseUnit) {

    return dispenseUnit != null
        ? StringUtils.isNotBlank(dispenseUnit.getDispenseUnitIen()) ? dispenseUnit.getDispenseUnitIen() : BLANK : BLANK;

  }

  /**
   * Converts the DrugUnitVo to a String value for sending to VistA. Returns {@link BLANK} if the value is null or blank.
   *
   * @param drugUnit the drug unit
   * @return the string
   */
  public static String convertDrugUnit(DrugUnitVo drugUnit) {

    return drugUnit != null ? StringUtils.isNotBlank(drugUnit.getDrugUnitIen()) ? drugUnit.getDrugUnitIen() : BLANK : BLANK;

  }

  /**
   * Converts the FormularyDesignatorVo to a String value for sending to VistA. Returns {@link BLANK} if the value is null or
   * blank.
   *
   * @param formularyDesignator the formulary designator
   * @return the string
   */
  public static String convertFormularyDesignator(FormularyDesignatorVo formularyDesignator) {

    return formularyDesignator != null
        ? StringUtils.isNotBlank(formularyDesignator.getName()) ? formularyDesignator.getName() : BLANK : BLANK;

  }

  /**
   * Converts the GenericNameVo to a String value for sending to VistA. Returns {@link BLANK} if the value is null or blank.
   *
   * @param genericName the generic name
   * @return the string
   */
  public static String convertGenericName(GenericNameVo genericName) {

    return genericName != null ? StringUtils.isNotBlank(genericName.getGenericIen()) ? genericName.getGenericIen() : BLANK
        : BLANK;

  }

  /**
   * Converts the OrderableItemVo to a String value for sending to VistA. Returns {@link BLANK} if the value is null or blank.
   *
   * @param orderableItem the orderable item
   * @return the string
   */
  public static String convertOrderableItem(OrderableItemVo orderableItem) {

    return orderableItem.getDosageForm() != null ? StringUtils.isNotBlank(orderableItem.getDosageForm().getDosageFormIen())
        ? orderableItem.getDosageForm().getDosageFormIen() : BLANK : BLANK;
  }

  /**
   * Converts the DosageFormVo to a String value for sending to VistA. Returns {@link BLANK} if the value is null or blank.
   *
   * @param dosageForm the dosage form
   * @return the string
   */
  public static String convertDosageForm(DosageFormVo dosageForm) {

    return dosageForm != null ? StringUtils.isNotBlank(dosageForm.getDosageFormIen()) ? dosageForm.getDosageFormIen() : BLANK
        : BLANK;
  }

  /**
   * Converts the specified Boolean to a "Y/N" String value for sending to VistA. If the specified value is true, "Y"; otherwise
   * "N".
   *
   * @param value the value
   * @return String "Y" if the specified value is true; otherwise "N".
   */
  public static String convertYesNoValue(Boolean value) {

    return value ? "Y" : "N";

  }

  /**
   * Converts the specified Object to a "1/0" String value for sending to VistA. If the specified value is true, "1"; otherwise
   * "0".
   *
   * @param value the value
   * @return String "1" if the specified value is true; otherwise "0".
   */
  public static String convertBooleanDigit(Object value) {

    return value != null ? (Boolean) value ? "1" : "0" : BLANK;

  }

  /**
   * Converts the specified boolean to a "1/0" String value for sending to VistA. If the specified value is true, "1"; otherwise
   * "0".
   *
   * @param value the value
   * @return String "1" if the specified value is true; otherwise "0".
   */
  public static String convertBooleanDigit(boolean value) {

    return value ? "1" : "0";

  }

  /**
   * Converts the specified Diff's old value to a "1/BLANK" String value for sending to VistA. If the specified Diff's old value
   * is true, "1"; otherwise " " (BLANK).
   *
   * @param diff the diff
   * @return String "1" if the specified value is true; otherwise " ".
   */
  public static String convertOldBooleanBlank(Difference diff) {

    return diff.getOldValue() != null ? (Boolean) diff.getOldValue() ? "1" : BLANK : null;

  }

  /**
   * Converts the specified Diff's new value to a "1/BLANK" String value for sending to VistA. If the specified Diff's new value
   * is true, "1"; otherwise " " (BLANK).
   *
   * @param diff the diff
   * @return String "1" if the specified value is true; otherwise " ".
   */
  public static String convertNewBooleanBlank(Difference diff) {

    return diff.getNewValue() != null ? (Boolean) diff.getNewValue() ? "1" : BLANK : null;

  }

  /**
   * Converts the specified boolean to a "1/BLANK" String value for sending to VistA. If the specified boolean value is true,
   * "1"; otherwise " " (BLANK).
   *
   * @param value the value
   * @return String "1" if the specified value is true; otherwise " ".
   */
  public static String convertNewBooleanBlank(boolean value) {

    return value ? "1" : BLANK;

  }

}
