package gov.va.med.pharmacy.peps.common.vo;

import java.util.ArrayList;
import java.util.List;

/**
 * Represents the data in a single row on the Formulary Designator import spreadsheet, along with
 * any errors occuring during the validation and processing of that data.
 */
public class FormularyDesignatorImportRow {
    private Long eplId;
    private String orderableItemName;
    private String formularyDesignatorName;
    private String orderableItemText;
    private List<String> errors;
    
    /**
     * The constructor.
     * 
     * @param eplId uniquely identifying the associated orderable item
     * @param orderableItemName textual name for the orderable item
     * @param formularyDesignatorName name of the formulary designator to be 
     * applied to the orderable item uniquely identified by eplId, as well as 
     * all associated products
     * @param orderableItemText text to be applied to the orderable item 
     * uniquely identified by eplId, as well as all associated products.
     */
    public FormularyDesignatorImportRow(Long eplId, String orderableItemName, String formularyDesignatorName,
            String orderableItemText) {
        this.eplId = eplId;
        this.orderableItemName = orderableItemName;
        this.formularyDesignatorName = formularyDesignatorName != null ? formularyDesignatorName : "";
        this.orderableItemText = orderableItemText != null ? orderableItemText : "";
        errors = new ArrayList<String>();
    }

    public Long getEplId() {
        return eplId;
    }
    
    public String getOrderableItemName() {
        return orderableItemName;
    }
    
    public String getFormularyDesignatorName() {
        return formularyDesignatorName;
    }
    
    public String getOrderableItemText() {
        return orderableItemText;
    }

    public boolean isValid() {
        return errors.size() == 0;
    }

    /**
     * Adds a message describing a validation or processing error
     * which occurred while processing the data for this row.
     * 
     * @param errorMessage to include in the error file for this
     * row
     */
    public void addError(String errorMessage) {
        errors.add(errorMessage);
    }

    public String getErrorMessage() {
        return String.join(" ", errors);
    }

    /**
     * Returns "Y" or "N" to indicate whether an error
     * occurred while processing this import row.
     * 
     * @return "Y" if an error occurred; otherwise, return
     * "N".
     */
    public String getErrorFlag() {
        if(isValid()) {
            return "N";
        }
        else {
            return "Y";
        }
    }
}
