package gov.va.med.pharmacy.peps.common.vo;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Classes implementing this interface are part of the NDF file
 * synchronization process. 
 */
public interface NdfSyncable {
    /**
     * Converts NdfSyncables (i.e., PPS-N value objects which are sync'd with VistA)
     * into a list of SyncProcessorElements suitable for converting to outgoing
     * differences.
     * 
     * @param syncables to transform
     * @param vistaIen The product's IEN - note, this should not have the sub-field ien on it (i.e. no comma!) 
     * @param userValue The user processing the change
     * @return list of SyncProcessor elements
     */
    public static List<SyncProcessorElement> syncProcessorElementsForNdfSyncables(
        Collection<? extends NdfSyncable> syncables, 
        String vistaIen,
        String userValue) {
        
        List<SyncProcessorElement> elements = new ArrayList<>();
        int pseudoSubfieldIen = 1;
        
        for(NdfSyncable syncable : syncables) {
            elements.addAll(
                syncable.asSyncProcessorElements(
                    vistaIen, 
                    pseudoSubfieldIen, 
                    userValue));            
            pseudoSubfieldIen++;
        }
        
        return elements;
    }
    
    /**
     * Returns a list of SyncProcessorElements representing the state
     * of this object using the format required for the information 
     * to be stored as NDF outgoing differences.
     * 
     * @param vistaIen identifies the top-level entity in VistA, such as
     * the particular product, dosage form, etc.
     * @param pseudoSubfieldIen a placeholder identifier for subfields
     * associated with the top-level entity, to be used if and only if
     * the subfield does not have such an identifier (e.g., copay tiers do
     * not have their own ien's, whereas active ingredients do).
     * @param userValue the name of the user creating the outgoing
     * differences
     * @return list of SyncProcessor elements, or an empty list if no
     * elements are created; does not return null.
     */
    List<SyncProcessorElement> asSyncProcessorElements(
        String vistaIen, 
        int pseudoSubfieldIen, 
        String userValue);
}
