package gov.va.med.pharmacy.peps.common.vo.validator;


import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.FormularyDesignatorVo;


/**
 * Validates that FormularyDesignatorVo conforms to business validation rules. 
 */
public class FormularyDesignatorValidator extends AbstractValidator<FormularyDesignatorVo> {
    /**
     * Constant for min length of the description.
     */
    private static final int MIN_DESCRIPTION_LENGTH = 1;
    
    /**
     * Constant for max length of the description.
     */
    private static final int MAX_DESCRIPTION_LENGTH = 255;
    
    /**
     * Constant for min length of the name.
     */
    private static final int MIN_NAME_LENGTH = 1;
    
    /**
     * Constant for max length of the name.
     */
    private static final int MAX_NAME_LENGTH = 30;
    
    /**
     * Check validation rules for the target FormularyDesignatorVo and
     * accumulate errors in the Errors object.
     * 
     * Description must not be null and must have length 1-255 inclusive.
     * 
     * Name must not be null and must have length 1-30 inclusive.
     * Item status must not be null.
     * 
     * @param target is FormularyDesignatorVo being validated
     * 
     * @param errors list accumulating errors for this validation
     */
    public final void validate(FormularyDesignatorVo target, Errors errors) {
         rejectIfNull(target, errors, FieldKey.FORMULARY_DESIGNATOR);         
         
         if(target != null) {
             rejectIfNull(target.getDescription(), errors, FieldKey.DESCRIPTION);
             rejectIfNull(target.getName(), errors, FieldKey.FORMULARY_DESIGNATOR);
             rejectIfNull(target.getItemStatus(), errors, FieldKey.ITEM_STATUS);
             
             if(target.getDescription() != null) {
               rejectIfLengthOutsideRangeInclusive(
                   target.getDescription(), 
                   MIN_DESCRIPTION_LENGTH, 
                   MAX_DESCRIPTION_LENGTH, 
                   errors, 
                   FieldKey.DESCRIPTION);
             }
             
             if(target.getName() != null) {
               rejectIfLengthOutsideRangeInclusive(
                     target.getName(), 
                     MIN_NAME_LENGTH, 
                     MAX_NAME_LENGTH, 
                     errors, 
                     FieldKey.FORMULARY_DESIGNATOR);
             }
         }
    }
}
