package gov.va.med.pharmacy.peps.common.object.test;

import static gov.va.med.pharmacy.peps.common.utility.IOUtility.safeClose;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.nio.charset.Charset;
import java.util.Arrays;

import org.apache.commons.io.IOUtils;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.object.VoListEnum;
import gov.va.med.pharmacy.peps.common.utility.SerializationUtility;
import gov.va.med.pharmacy.peps.common.utility.security.SafeObjectInputStream;
import gov.va.med.pharmacy.peps.common.utility.test.generator.ProductGenerator;
import gov.va.med.pharmacy.peps.common.vo.ManagedEntityVo;
import gov.va.med.pharmacy.peps.common.vo.NationalProductVo;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;

/**
 * JUnit test class to ensure SafeObjectInputStream functions as expected.
 */
public class SafeObjectInputStreamTest {

    /** Generate a ProductVo for test. */
    private static final ProductVo PRODUCT = new ProductGenerator().getFirst();

    /** This test triggers expected exceptions. Boolean to indicate printing those exceptions to the console. */
    private static Boolean printExpectedErrors = false;

    /** Character set constant. */
    private static final Charset CHARSET = Charset.forName("ISO-8859-1");

    /**
     * Test a successful deserialization passing in VoListEnum as the white list of classes. SafeObjectInputStream uses
     * StdJavaListEnum as part of the white list by default.
     */
    @Test
    public void testSuccessfulProductDeserialization()
        throws FileNotFoundException, IOException, SecurityException, ClassNotFoundException {
        // serialize PRODUCT to a file.
        String filename = SerializationUtility.serialize(PRODUCT);
        File file = new File(filename);

        // deserialize the file using the VoListEnum as part of the list of safe classes
        ProductVo productVo = null;
        SafeObjectInputStream inputStream = new SafeObjectInputStream(new FileInputStream(filename), VoListEnum.getVoList());
        productVo = (ProductVo) inputStream.readObject();
        assertNotNull("PRODUCT deserialization returned null", productVo);

        safeClose(inputStream);
        file.delete();
    }

    /**
     * Test successful deserialization of a String. Pass in an empty list of classes for the safe list so that only the
     * StdJavaListEnum is used as the safe list.
     */
    @Test
    public void testSuccessfulStringDeserialization() throws IOException, SecurityException, ClassNotFoundException {
        // serialize a String to a String
        String testStr = "Test string to deserialize. !@#$%^&*()_+=";
        String serializedStr = SerializationUtility.serializeToString(testStr);

        // deserialize the string passing in an empty list of safe classes
        Class<?>[] safeClassList = {};
        String deserializedStr = null;

        SafeObjectInputStream inputStream = new SafeObjectInputStream(IOUtils.toInputStream(serializedStr, CHARSET),
            Arrays.asList(safeClassList));
        deserializedStr = (String) inputStream.readObject();
        assertNotNull("String is null", deserializedStr);

        safeClose(inputStream);
    }

    /**
     * Test attempt to deserialize an unsafe class using an incomplete list of safe classes.
     */
    @Test
    public void testSecurityException() throws FileNotFoundException, IOException, ClassNotFoundException {
        // serialize PRODUCT to a file.
        String filename = SerializationUtility.serialize(PRODUCT);
        File file = new File(filename);
        Class<?>[] safeClassList = { NationalProductVo.class, ProductVo.class, ManagedEntityVo.class };

        // deserialize the file passing in an incomplete list of safe classes
        SafeObjectInputStream inputStream = null;
        ProductVo productVo = null;

        try {
            inputStream = new SafeObjectInputStream(new FileInputStream(filename), Arrays.asList(safeClassList));
            productVo = (ProductVo) inputStream.readObject();
        } catch (SecurityException e) {
            if (printExpectedErrors) {
                System.out.println("Received expected SecurityException for JUnit test");
                System.out.println(e.toString());
            }
        } finally {
            assertNull("PRODUCT deserialization was expected to fail with a SecurityException", productVo);
            safeClose(inputStream);
            file.delete();
        }
    }

    /**
     * Attempt to deserialize a deleted file to trigger FileNotFoundException.
     */
    @Test
    public void testFileNotFoundException() throws IOException, SecurityException, ClassNotFoundException {
        // serialize PRODUCT to a file.
        String filename = SerializationUtility.serialize(PRODUCT);
        File file = new File(filename);

        // delete the serialized file and attempt to deserialize the file
        SafeObjectInputStream inputStream = null;
        ProductVo productVo = null;

        try {
            assertTrue("File should exist", file.exists());
            file.delete();
            inputStream = new SafeObjectInputStream(new FileInputStream(filename), VoListEnum.getVoList());
            productVo = (ProductVo) inputStream.readObject();
        } catch (FileNotFoundException e) {
            if (printExpectedErrors) {
                System.out.println("Received expected FileNotFoundException for JUnit test");
                System.out.println(e.toString());
            }
        } finally {
            assertNull("PRODUCT deserialization was expected to fail with a FileNotFoundException", productVo);
            safeClose(inputStream);
        }
    }

    /**
     * Attempt to deserialize an improper string to trigger an IOException.
     */
    @Test
    public void testIOException() throws SecurityException, ClassNotFoundException {
        // setup attempt to deserialize a string
        String testStr = "Test string to deserialize. !@#$%^&*()_+=";
        Class<?>[] safeClassList = {};
        SafeObjectInputStream inputStream = null;
        String deserializedStr = null;

        try {
            inputStream = new SafeObjectInputStream(IOUtils.toInputStream(testStr, CHARSET), Arrays.asList(safeClassList));
            deserializedStr = (String) inputStream.readObject();
        } catch (IOException e) {
            if (printExpectedErrors) {
                System.out.println("Received expected IOException for JUnit test");
                System.out.println(e.toString());
            }
        } finally {
            assertNull("String deserialization was expected to fail with an IOException", deserializedStr);
            safeClose(inputStream);
        }
    }

}
