package gov.va.med.pharmacy.peps.common.object.test;

import static org.junit.Assert.assertFalse;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.net.URL;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.object.VoListEnum;
import gov.va.med.pharmacy.peps.common.utility.security.StdJavaListEnum;

/**
 * The VoListEnumTest is used to walk through all of the ValueObject, Difference, and AbstractValidator assignable classes
 * within the PS_PPS_common project to determine which classes need to be added to the VoListEnum or StdJavaListEnum as a safe
 * class. The VoListEnum is used in conjunction with the StdJavaListEnum to create a white list of safe classes used by
 * SafeObjectInputStream for safe deserialization of Objects.
 * 
 * As ValueObject classes are added to the code, the classes associated with the the new ValueObject class need to be added to
 * the list of safe classes. If this JUnit test passes, then no new classes need to be added to the enumerations. If the test
 * fails, a list of missing classes will be provided on the console. Pharmacy classes on the missing classes list should be
 * added to the VoListEnum. Other Java classes should be added to the StdJavaListEnum.
 *
 * This test can also be used to gather data about classes within the PS_PPS_common project that are assignable from
 * ValueObject, Difference, and AbstractValidator. Within verifySafeClassEnumsAreComplete, set listClassInfo = true to print all
 * the class names along with their declared fields (including type and name) to the console.
 * 
 */
public class VoListEnumTest {

    private static final int DOT_JAVA_EXTENSION_LENGTH = 5;
    private static final String SEPARATOR = File.separator;

    /**
     * List of classes within the PS_PPS_common project that are assignable from ValueObject, Difference, 
     * and AbstractValidator.
     */
    private ArrayList<Class<?>> listOfCommonClasses = new ArrayList<Class<?>>();

    /**
     *List of classes that have already been processed through the test to determine if they are on the list of safe classes.
     */
    private ArrayList<String> verifiedClasses = new ArrayList<String>();

    /**
     *List of classes that need to be added to the VoListEnum or StdJavaListEnum as a safe class. 
     */
    private ArrayList<String> classesMissingFromSafeList = new ArrayList<String>();

    /**
     * Walks through all of the ValueObject, Difference, and AbstractValidator assignable classes within the PS_PPS_common
     * project to determine which classes need to be added to the VoListEnum or StdJavaListEnum as a safe class. This test can
     * also be used to gather data about the classes as they are processed. Set listClassInfo = true to print all the class
     * names along with their declared fields (including type and name) to the console.
     */
    @Test
    public void verifySafeClassEnumsAreComplete() {
        // listClassInfo controls printing class data to the console 
        Boolean listClassInfo = false;

        // get list of class files within PS_PPS_common that are assignable from ValueObject, Difference, or AbstractValidator
        try {
            Enumeration<URL> roots = ClassLoader.getSystemResources("") ;

            while (roots.hasMoreElements()) {
                String PathRoot = (String) roots.nextElement().getPath() ;
                if (PathRoot.toLowerCase().contains("ps_pps_common" )) {
                    PathRoot = PathRoot.split("target")[0] + "src/main/java/gov/va/med/pharmacy/peps/common";
                    PathRoot = PathRoot.startsWith("/") ? PathRoot.substring(1) : PathRoot;
                    classesAssignableFrom(PathRoot, Class.forName("gov.va.med.pharmacy.peps.common.vo.ValueObject")) ;
                    classesAssignableFrom(PathRoot, Class.forName("gov.va.med.pharmacy.peps.common.vo.diff.Difference")) ;
                    classesAssignableFrom(PathRoot,
                        Class.forName("gov.va.med.pharmacy.peps.common.vo.validator.AbstractValidator"));
                }
            }

        } catch (IOException e) {
            System.out.println("IOException encountered while building the list of class files in PS_PPS_common project.");
            assertFalse("IOException encountered while building the list of class files in PS_PPS_common project.", true);
        } catch (ClassNotFoundException e) {
            System.out.println("ClassNotFoundException encountered while "
                + "building the list of class files in PS_PPS_common project.");
            assertFalse("ClassNotFoundException encountered while building "
                + "the list of class files in PS_PPS_common project.", true);
        }

        // check the list of classes from the PS_PPS_common project against the list of safe classes 
        ArrayList <Class<?>> unprocessedClasses =checkClassesAgainstSafeEnums(listOfCommonClasses, listClassInfo);

        // loop through until all classes and their declared fields are checked against the list of safe classes 
        while (unprocessedClasses.size() > 0) {
            unprocessedClasses=checkClassesAgainstSafeEnums(unprocessedClasses, listClassInfo);
        }

        // print out the list of classes missing from the safe list    
        if (classesMissingFromSafeList.size() > 0) {
            System.out.println("\r\nThe following classes need to be added to "
                + "VoListEnum or StdJavaListEnum to be included as a safe class:");
            for (int i = 0; i < classesMissingFromSafeList.size(); i++) {
                System.out.println(classesMissingFromSafeList.get(i).toString());
            }
        }

        assertFalse("Items are missing from the safe class enums. See the Console.", (classesMissingFromSafeList.size() > 0));
    }

    /**
     * This method iterates through a list of classes checking if the class and all its declared fields are on the list of safe
     * classes for desrialization. Classes not on the list of safe classes are added to the classesMissingFromSafeList. As
     * classes are verified, they are added to the verifiedClasses list. If a declared field for a class is not on the list of
     * verified classes and is also not on the list to be processed, it will be added to the unprocessedFieldTypes list and
     * returned to the calling method. Unprocessed field types are those classes whose declared fields have not yet been
     * verified against the list of safe classes.
     * 
     * @param classFiles - list of classes to process
     * @param listClassInfo - boolean indicator to print class name and class declared field data to the console
     * @return ArrayList - list of classes whose declared fields have not been checked against the safe list
     */
    private ArrayList<Class<?>> checkClassesAgainstSafeEnums(ArrayList<Class<?>> classFiles, Boolean listClassInfo) {
        ArrayList<Class<?>> unprocessedFieldTypes = new ArrayList<Class<?>>();

        // walk through the array of classes and verify they are represented on the safe classes list 
        for (Class<?> cls : classFiles) {
            if (listClassInfo) {
                System.out.println("\r\nProcessing Class: " + cls.toString());
                System.out.println("Class fields include: ");
            }

            // check if the class is on the list of safe classes
            verifiedClasses.add(cls.getName());
            if ( !classesMissingFromSafeList.contains(cls.getName()) && !checkIfSafeClass(cls)) {
                classesMissingFromSafeList.add(cls.getName());
            }

            // loop through declared fields for the class and verify the field type is on the list as a safe class
            Field[] fields = cls.getDeclaredFields();

            for (Field fld : fields) {
                if (processField(classFiles, listClassInfo, fld)) {
                    unprocessedFieldTypes.add(fld.getType());
                }
            }
        }

        return unprocessedFieldTypes;
    }

    /**
     * Process a field from a class. If the class is not on the safe list, add the field type to classesMissingFromSafeList. If
     * The field is not on the list of classes to be processed, set an indication to add it to the list of unprocessed field
     * types.
     * 
     * @param classFiles - list of classes to be processed
     * @param listClassInfo - indicator to print field data to the console
     * @param fld - field to process
     * @return - Boolean indicator to add the field type to the list of unprocessed types
     */
    private Boolean processField(ArrayList<Class<?>> classFiles, Boolean listClassInfo, Field fld) {
        Boolean addFieldToUnprocessedList = false;
        String fieldName = fld.getName();
        String fieldType = fld.getType().getName();

        if (listClassInfo) {
            System.out.println(fieldType + " - " + fieldName);
        }

        // serialization ignores static fields, so we can skip static fields
        // if the field type is not static and has not yet been processed, check if it is a safe class
        if (!verifiedClasses.contains(fieldType) && !Modifier.isStatic(fld.getModifiers())) {
            verifiedClasses.add(fieldType);
            Class<?> clazz = fld.getType();

            // if the type is a pharmacy class, check if it is on the list of classes to process
            if (fieldType.startsWith("gov.va.med.pharmacy.peps.") && !classFiles.contains(clazz)) {
                // add to the list of classes whose declared fields have not been verified against the safe list
                addFieldToUnprocessedList = true;
            }

            // check if the Type is on the list of safe classes
            if (!classesMissingFromSafeList.contains(fieldType) && !checkIfSafeClass(clazz)) {
                classesMissingFromSafeList.add(fieldType);
            }
        }

        return addFieldToUnprocessedList;
    }

    /**
     * Check if the provided class is on the list of safe classes for deserialization. The list of safe classes includes the
     * VoListEnum and the StdJavaListEnum. The logic used to verify if the class is on the safe list is the same as the logic
     * used by the SafeObjectInputStream.resolveClass method. Any changes to that method should be reviewed to determine if
     * changes need to be made to this test.
     * 
     * @param clazz - class to be checked against the list of safe classes
     * @return Boolean indicating if the class is on the safe list
     */
    private Boolean checkIfSafeClass(Class<?> clazz) {
        return clazz.isPrimitive() 
            || clazz.isArray() 
            || Enum.class.isAssignableFrom(clazz) 
            || List.class.isAssignableFrom(clazz)
            || StdJavaListEnum.getStdJavaList().contains(clazz) 
            || VoListEnum.getVoList().contains(clazz);
    }

    /**
     * Locates classes within the provided root path that are assignable by the provided class. Adds these classes to the
     * listOfCommonClasses.
     * 
     * @param rootPath - path in which to locate class files
     * @param clazz -classes located must be assignable from this provided class
     */
    public void classesAssignableFrom(String rootPath, Class<?> clazz) {
        File root = new File(rootPath);
        File[] list = root.listFiles();
        ArrayList<Class<?>> localListOfCommonClasses = new ArrayList<Class<?>>();
        String fileWithPath = null;
        String fullyQualClass = null;

        if (list != null) {
            for (File f : list) {
                if (f.isDirectory()) {
                    classesAssignableFrom(f.getAbsolutePath(), clazz);
                } else {
                    fileWithPath = f.getAbsoluteFile().toString();
                    fullyQualClass = fileWithPath.substring(fileWithPath.lastIndexOf("gov" + SEPARATOR + "va"),
                            fileWithPath.length() - DOT_JAVA_EXTENSION_LENGTH);
                    fullyQualClass = fullyQualClass.replace(SEPARATOR, ".");

                    fileWithPath = fileWithPath.substring(fileWithPath.lastIndexOf(SEPARATOR) + 1, fileWithPath.length());
                    fileWithPath = fileWithPath.replace(".java", "");

                    try {
                        if (!fullyQualClass.equals("gov.va.med.pharmacy.peps.common.servlet.AbstractPepsServlet")) {
                            if (clazz.isAssignableFrom(Class.forName(fullyQualClass))) {
                                localListOfCommonClasses.add(Class.forName(fullyQualClass));
                                listOfCommonClasses.add(Class.forName(fullyQualClass));
                            }
                        }
                    } catch (ClassNotFoundException e) {
                        System.out.println("ClassNotFoundException encountered while "
                            + "building the list of class files in PS_PPS_common project.");
                        assertFalse("ClassNotFoundException encountered while building "
                            + "the list of class files in PS_PPS_common project.", true);
                    }
                }
            }
        }
    }

}
