package gov.va.med.pharmacy.peps.common.utility;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThat;

import java.sql.Date;
import java.time.LocalDate;
import java.util.Calendar;
import java.util.GregorianCalendar;

import org.junit.Test;

/**
 * Unit tests for {@link MumpsConverter}.
 */
public class MumpsConverterTest {

    private static final Long JANUARY_1_1985 = 473428800000L;

    /**
     * Milliseconds since epoch representing 12/31/2017@1000 GMT.
     */
    private static final Long DECEMBER_31_2017 = 1514714400000L;

    private static final String JANUARY_1_1985_CONVERTED = "2850101";
    private static final String TIME_RESULT = "38058";
    private static final String DATE_TIME_RESULT = JANUARY_1_1985_CONVERTED + "." + TIME_RESULT;
    
    @Test
    public void testDateConversion() {
        Date dateToConvert = new Date(JANUARY_1_1985);
        String convertedDate = MumpsConverter.convertDate(dateToConvert);
        assertEquals(JANUARY_1_1985_CONVERTED, convertedDate);
    }
    
    @Test
    public void testConvertDateDec31() {
        final Date dateToConvert = new Date(DECEMBER_31_2017);
        assertEquals("3171231", MumpsConverter.convertDate(dateToConvert));
    }
    
    @Test
    public void testLocalDateConversion() {
      final String convertedDate = MumpsConverter.convertDate(LocalDate.of(1985, 7, 4));
      assertThat(convertedDate, equalTo("2850704"));
    }
    
    @Test
    public void testTimeConversion() {
        GregorianCalendar timeToConvert = new GregorianCalendar();
        timeToConvert.setTimeInMillis(JANUARY_1_1985);
        timeToConvert.set(Calendar.HOUR, 22);
        timeToConvert.set(Calendar.MINUTE, 34);
        timeToConvert.set(Calendar.SECOND, 18);
        timeToConvert.set(Calendar.MILLISECOND, 0);
        String convertedTime = MumpsConverter.convertTime(timeToConvert.getTime());
        assertEquals(TIME_RESULT, convertedTime);
    }
    
    @Test
    public void testDateTimeConversion() {
        GregorianCalendar timeToConvert = new GregorianCalendar();
        timeToConvert.setTimeInMillis(JANUARY_1_1985);
        timeToConvert.set(Calendar.HOUR, 22);
        timeToConvert.set(Calendar.MINUTE, 34);
        timeToConvert.set(Calendar.SECOND, 18);
        timeToConvert.set(Calendar.MILLISECOND, 0);
        String convertedTime = MumpsConverter.convertDateTime(timeToConvert.getTime());
        assertEquals(DATE_TIME_RESULT, convertedTime);
    }
    
    /**
     * Tests that {@link MumpsConverter#mumpsToLocalDate(String)} can parse a proper String.
     */
    @Test
    public void testMumpsToLocalDateSuccess() {
      final LocalDate actualDate = MumpsConverter.mumpsToLocalDate("3160809");
      assertEquals(LocalDate.of(2016, 8, 9), actualDate);
    }
    
    /**
     * Tests that {@link MumpsConverter#mumpsToLocalDate(String)} properly fails for a String that doesn't match the
     * correct format.
     */
    @Test(expected = IllegalArgumentException.class)
    public void testMumpsToLocalDateBadFormat() {
      MumpsConverter.mumpsToLocalDate("12345678"); // Not 7 characters.
    }

    /**
     * Tests that {@link MumpsConverter#mumpsToLocalDate(String)} properly fails for an invalid date.
     */
    @Test(expected = IllegalArgumentException.class)
    public void testMumpsToLocalDateInvalidDate() {
      MumpsConverter.mumpsToLocalDate("3160001"); // We don't support month 0.
    }
}
