package gov.va.med.pharmacy.peps.common.vo;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.test.generator.ProductGenerator;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.common.vo.diff.test.DifferenceGenerator;

/**
 * Automated tests for {@link RematchVo} .
 */
public class RematchVoTest {

  private final ProductGenerator productGenerator = new ProductGenerator();
  
  @Test
  public void testAutomaticRematch() {
    /* Setup */
    List<ProductVo> productList = productGenerator.getList();
    ProductVo fromProduct = productList.get(0);
    ProductVo toAutomaticProduct = productList.get(1);
    RematchVo rematch = RematchVo.automaticRematch(
        fromProduct, toAutomaticProduct, RematchStatus.APPROVED);
    /* Behavior & Verification */
    assertEquals(true, rematch.getIsAutomaticRematch());
    assertEquals(toAutomaticProduct, rematch.getToAutomaticProduct());
    assertEquals(new HashSet<>(Arrays.asList(toAutomaticProduct)), rematch.getMatchProducts());
    assertEquals("Automatic rematch to " + toAutomaticProduct.toShortString(), rematch.getToProductsString());

  }
  
  @Test
  public void testSuggestedRematch() {
    /* Setup */
    List<ProductVo> productList = productGenerator.getList();
    Set<ProductVo> suggestedProductSet = new HashSet<>(Arrays.asList(productList.get(1), productList.get(2)));
    RematchVo rematch = RematchVo.suggestedRematch(
        productList.get(0), suggestedProductSet, RematchStatus.APPROVED);
    /* Behavior & Verification*/
    assertEquals(false, rematch.getIsAutomaticRematch());
    assertEquals(suggestedProductSet, rematch.getToSuggestedProducts());
    assertEquals(suggestedProductSet, rematch.getMatchProducts());
    // Just test the beginning of the string because we don't know the iteration order.
    final String toProductsString = rematch.getToProductsString();
    assertThat(toProductsString, startsWith("Suggested rematch to "));
  }
  
  @Test
  public void testIsObsoletePerChanges() {
    /* Setup */
    List<ProductVo> productList = productGenerator.getList();
    ProductVo fromProduct = productList.get(0);
    ProductVo toAutomaticProduct = productList.get(1);
    RematchVo futureRematch = RematchVo.automaticRematch(
        fromProduct, toAutomaticProduct, RematchStatus.FUTURE);
    RematchVo currentRematch = RematchVo.automaticRematch(
        fromProduct, toAutomaticProduct, RematchStatus.APPROVED);

    final List<Difference> setProposedDateDiffs = Arrays.asList(
        DifferenceGenerator.makeDifferenceForDataField(FieldKey.PROPOSED_INACTIVATION_DATE, null, new Date()));
    final List<Difference> clearProposedDateDiffs = Arrays.asList(
        DifferenceGenerator.makeDifferenceForDataField(FieldKey.PROPOSED_INACTIVATION_DATE, new Date(), null));

    /* Behavior & Verification */
    assertFalse("setting Proposed Inactivation Date does not obsolete rematch",
        futureRematch.isObsoletePerChanges(setProposedDateDiffs));
    assertFalse("clearing Proposed Inactivation Date on current rematch does not obsolete it",
        currentRematch.isObsoletePerChanges(clearProposedDateDiffs));
    assertTrue("clearing Proposed Inactivation Date on future rematch does obsolete it",
        futureRematch.isObsoletePerChanges(clearProposedDateDiffs));
  }
  
    /**
     * Tests that the rematch stays approved even if there are no triggering changes.
     */
    @Test
    public void testUpdateStatusPerChanges_StaysApproved() {
        /* Setup */
        List<ProductVo> productList = productGenerator.getList();
        ProductVo fromProduct = productList.get(0);
        ProductVo toAutomaticProduct = productList.get(1);
        RematchVo rematch = RematchVo.automaticRematch(fromProduct, toAutomaticProduct, RematchStatus.APPROVED);

        /* Behavior & Verification */
        rematch.updateStatusPerChanges(Collections.emptySet(), true);

        assertThat("should not change status", rematch.getStatus(), equalTo(RematchStatus.APPROVED));
    }
  
    @Test
    public void testUpdateStatusPerChanges_BecomesObsolete() {
        /* Setup */
        List<ProductVo> productList = productGenerator.getList();
        ProductVo fromProduct = productList.get(0);
        ProductVo toAutomaticProduct = productList.get(1);
        RematchVo rematch = RematchVo.automaticRematch(fromProduct, toAutomaticProduct, RematchStatus.PENDING);

        /* Behavior & Verification */
        rematch.updateStatusPerChanges(Collections.emptySet(), true);

        assertThat("should now be obsolete", rematch.getStatus(), equalTo(RematchStatus.OBSOLETE));
    }
    
    @Test
    public void testUpdateStatusPerChanges_BecomesFuture() {
        /* Setup */
        List<ProductVo> productList = productGenerator.getList();
        ProductVo fromProduct = productList.get(0);
        ProductVo toAutomaticProduct = productList.get(1);
        RematchVo rematch = RematchVo.automaticRematch(fromProduct, toAutomaticProduct, RematchStatus.PENDING);
        final List<Difference> setProposedDateDiffs = Arrays.asList(
            DifferenceGenerator.makeDifferenceForDataField(FieldKey.PROPOSED_INACTIVATION_DATE, null, new Date()));

        /* Behavior & Verification */
        rematch.updateStatusPerChanges(setProposedDateDiffs, true);
        
        assertThat("should now be future", rematch.getStatus(), equalTo(RematchStatus.FUTURE));
    }

}
