package gov.va.med.pharmacy.peps.common.vo.test;

import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.EnumSet;
import java.util.List;

import org.apache.commons.collections4.CollectionUtils;
import org.hamcrest.CoreMatchers;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.IntendedUseVo;
import gov.va.med.pharmacy.peps.common.vo.ItemStatus;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.MatchReason;
import gov.va.med.pharmacy.peps.common.vo.NdcVo;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.SynonymVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.common.vo.diff.test.DifferenceGenerator;

/**
 * Automated tests for {@link ProductVo} .
 */
public class ProductVoTest {
  
  private ProductVo productVo = new ProductVo();
  
  /**
   * Tests {@link ProductVo#getEntityType()}.
   */
  public void testGetEntityType() {
    assertThat(productVo.getEntityType(), CoreMatchers.equalTo(EntityType.PRODUCT));
  }
  
  /**
   * Tests {@link ManagedItemVo#determineRematchTriggers(Collection)} for the empty case.
   */
  @Test
  public void testDetermineRematchTriggersEmptyCase() {
    
    /* Behavior & Verification */
    assertThat(productVo.determineRematchTriggers(Collections.emptyList()), equalTo(Collections.emptySet()));
    assertThat(
        "canRematch() should still return true",
        productVo.canRematch(), equalTo(true));
  }
  
  /**
   * Tests {@link ManagedItemVo#determineRematchTriggers(Collection)} for the product item status true case.
   */
  @Test
  public void testDetermineRematchTriggersItemStatusTrueCase() {
    
    /* Setup */
    Difference differenceItemStatusTrueCase = 
        new Difference(FieldKey.ITEM_STATUS, ItemStatus.ACTIVE, ItemStatus.INACTIVE);
    final List<Difference> differences = Arrays.asList(differenceItemStatusTrueCase);
    
    /* Behavior & Verification */
    assertThat(productVo.determineRematchTriggers(differences), equalTo(EnumSet.of(MatchReason.INACTIVATION)));
    
  }
  
  /**
   * Tests {@link ManagedItemVo#determineRematchTriggers(Collection)} returns for the product item status false case.
   */
  @Test
  public void testDetermineRematchTriggersItemStatusFalseCase() {
    
    /* Setup */
    Difference differenceItemStatusFalseCase = 
        new Difference(FieldKey.ITEM_STATUS, ItemStatus.INACTIVE, ItemStatus.ACTIVE);
    final List<Difference> differences = Arrays.asList(differenceItemStatusFalseCase);
    
    /* Behavior & Verification */
    assertThat(productVo.determineRematchTriggers(differences), equalTo(Collections.emptySet()));
    
  }
  
  @Test
  public void testDetermineRematchTriggersSetProposedInactivation() {
    final Difference difference = DifferenceGenerator.makeDifferenceForDataField(
        FieldKey.PROPOSED_INACTIVATION_DATE, null, new Date());
    final List<Difference> differences = Arrays.asList(difference);
    
    /* Behavior & Verification */
    assertThat(productVo.determineRematchTriggers(differences), equalTo(EnumSet.of(MatchReason.FUTURE_INACTIVATION)));
  }
  
  @Test
  public void testDetermineRematchTriggersChangeProposedInactivation() {
    final Difference difference = DifferenceGenerator.makeDifferenceForDataField(
        FieldKey.PROPOSED_INACTIVATION_DATE,
        new Date(1488384660000L),   // March 1, 2017
        new Date());                // Now
    final List<Difference> differences = Arrays.asList(difference);

    /* Behavior & Verification */
    assertThat(productVo.determineRematchTriggers(differences), equalTo(Collections.emptySet()));
  }

  @Test
  public void testDetermineRematchTriggersClearProposedInactivation() {
    final Difference difference = DifferenceGenerator.makeDifferenceForDataField(
        FieldKey.PROPOSED_INACTIVATION_DATE, new Date(), null);
    final List<Difference> differences = Arrays.asList(difference);
    
    /* Behavior & Verification */
    assertThat(productVo.determineRematchTriggers(differences), equalTo(Collections.emptySet()));
  }

  @Test
  public void testDetermineRematchTriggersAll() {
    final Collection<Difference> diffs = Arrays.asList(
        new Difference(FieldKey.ITEM_STATUS, ItemStatus.ACTIVE, ItemStatus.INACTIVE),
        DifferenceGenerator.makeDifferenceForDataField(FieldKey.PROPOSED_INACTIVATION_DATE, null, new Date()),
        new Difference(FieldKey.VA_PRINT_NAME, "Old Name", "New Name"));
    
    /* Behavior & Verification */
    assertThat(
        productVo.determineRematchTriggers(diffs),
        equalTo(EnumSet.of(MatchReason.INACTIVATION, MatchReason.FUTURE_INACTIVATION, MatchReason.VA_PRINT_NAME)));
  }
  
  /**
   * Tests that {@link ManagedItemVo#determineRematchTriggers(Collection)} returns an empty Optional if only differences
   * that do not trigger a rematch are present.
   */
  @Test
  public void testDetermineRematchTriggersDifferenceField() {
    
    /* Setup */
    Difference differenceFalseField =
        new Difference(FieldKey.FDA_MED_GUIDE, ItemStatus.ACTIVE, ItemStatus.INACTIVE);
    final List<Difference> differences = Arrays.asList(differenceFalseField);
    
    /* Behavior & Verification */
    assertThat(productVo.determineRematchTriggers(differences), equalTo(Collections.emptySet()));
  }
  
  /**
   * Test to clear unique synonym.
   */
  @Test
    public void testClearPatientSynonymsUnique() {
     
     ProductVo parent = new ProductVo();
     parent.setVaProductName("ASPIRIN");
     parent.setId("100");
     
     //Ndcs remaining on this product after some of the ndcs moved to another product
     ArrayList<NdcVo> ndcs = new ArrayList<NdcVo>();
     
     NdcVo ndc = new NdcVo();
     ndc.setId("1000");
     ndc.setNdc("1000-777");
     ndc.setTradeName("ASPIRIN NT");
     ndcs.add(ndc);
     
     parent.setNdcs(ndcs );
     
     Collection<SynonymVo> synonyms = new ArrayList<SynonymVo>();
     IntendedUseVo iu = new IntendedUseVo();
     iu.setIntendedUse("0-TRADE NAME");
     
     SynonymVo synonym = new SynonymVo();
     synonym.setSynonymName("ASPIRINx");
     synonym.setNdcCode("1000-222");
     synonym.setSynonymIntendedUse(iu);
     synonyms.add(synonym);
     
     synonym = new SynonymVo();
     synonym.setSynonymName("ASPIRIN NT");
     synonym.setNdcCode("1000-777");
     synonyms.add(synonym);
     synonym.setSynonymIntendedUse(iu);
     parent.setSynonyms(synonyms);
     
     //Ndc to be moved to another product
     NdcVo ndcToMove = new NdcVo();
     ndcToMove.setId("1000");
     ndcToMove.setNdc("1000-222");
     ndcToMove.setTradeName("ASPIRINx");
     ndcToMove.setProduct(parent);
     
     // Method call
     parent.updateParentAssociations(Arrays.asList(ndcToMove));
      
     //Assertions
     assertEquals(1, parent.getSynonyms().size());
     assertEquals("ASPIRIN NT", parent.getSynonyms().iterator().next().getSynonymName());
     assertEquals("1000-777", parent.getSynonyms().iterator().next().getNdcCode());
   }

  /**
   * Test to Clear Patient Synonyms Not Unique Trade Name Reassign.
   */
  @Test
  public void testClearPatientSynonymsNotUniqueTradeNameReassign(){

    //Product
    ProductVo parent = new ProductVo();
    parent.setVaProductName("ASPIRIN");
    parent.setId("100");
    
    //Ndcs remaining on this product after some of the ndcs moved to another product
    ArrayList<NdcVo> ndcs = new ArrayList<NdcVo>();
    
    NdcVo ndc = new NdcVo();
    ndc.setId("1000");
    ndc.setNdc("1000-777");
    ndc.setTradeName("ASPIRIN Rx");
    ndcs.add(ndc);
    
    ndc = new NdcVo();
    ndc.setId("9999");
    ndc.setNdc("9999-777");
    ndc.setTradeName("ASPIRIN Rx");
    ndcs.add(ndc);
    parent.setNdcs(ndcs );
    
    Collection<SynonymVo> synonyms = new ArrayList<SynonymVo>();
    IntendedUseVo iu = new IntendedUseVo();
    iu.setIntendedUse("0-TRADE NAME");

    SynonymVo synonym = new SynonymVo();
    synonym.setSynonymName("ASPIRIN Rx");
    synonym.setNdcCode("1000-777");
    synonym.setSynonymIntendedUse(iu);
    synonyms.add(synonym);
    
    synonym = new SynonymVo();
    synonym.setSynonymName("ASPIRIN Xx");
    synonym.setNdcCode("8888-123");
    synonym.setSynonymIntendedUse(iu);
    synonyms.add(synonym);
    parent.setSynonyms(synonyms);
        
    //Mock ndc to be moved to another product
    NdcVo ndcToMove = new NdcVo();
    ndcToMove.setId("1000");
    ndcToMove.setNdc("1000-777");
    ndcToMove.setTradeName("ASPIRIN Rx");
    ndcToMove.setProduct(parent);
    // Method call
    parent.updateParentAssociations(Arrays.asList(ndcToMove));
    
    assertEquals(2, parent.getSynonyms().size());
    //first synonym was reassigned
    assertEquals("ASPIRIN Rx", ((SynonymVo)CollectionUtils.get(parent.getSynonyms(),0)).getSynonymName());
    assertEquals("9999-777", ((SynonymVo)CollectionUtils.get(parent.getSynonyms(),0)).getNdcCode());
    //second synonym is left alone
    assertEquals("ASPIRIN Xx", ((SynonymVo)CollectionUtils.get(parent.getSynonyms(),1)).getSynonymName());
    assertEquals("8888-123", ((SynonymVo)CollectionUtils.get(parent.getSynonyms(),1)).getNdcCode());
  }
  
    
  /**
   * Test Clear Patient Synonyms All Ndc Moved.
   */
  @Test
  public void testClearPatientSynonymsAllNdcMoved() {

    //Product
    ProductVo parent = new ProductVo();
    parent.setVaProductName("ASPIRIN");
    parent.setId("100");
    
    //Ndcs remaining on this product after some of the ndcs moved to another product
    ArrayList<NdcVo> ndcs = new ArrayList<NdcVo>();
    
    NdcVo ndc = new NdcVo();
    ndc.setId("1000");
    ndc.setNdc("1000-777");
    ndc.setTradeName("ASPIRIN Rx");
    ndcs.add(ndc);      
    parent.setNdcs(ndcs );
    
    Collection<SynonymVo> synonyms = new ArrayList<SynonymVo>();
    IntendedUseVo iu = new IntendedUseVo();
    iu.setIntendedUse("0-TRADE NAME");

    SynonymVo synonym = new SynonymVo();
    synonym.setSynonymName("ASPIRIN Rx");
    synonym.setNdcCode("1000-777");
    synonym.setSynonymIntendedUse(iu);
    synonyms.add(synonym);      
    parent.setSynonyms(synonyms);
       
    //Ndc to be moved to another product
    NdcVo ndcToMove = new NdcVo();
    ndcToMove.setId("1000");
    ndcToMove.setNdc("1000-777");
    ndcToMove.setTradeName("ASPIRIN Rx");
    ndcToMove.setProduct(parent);
   
    // Method call
    parent.updateParentAssociations(Arrays.asList(ndcToMove));
    assertEquals(0, parent.getSynonyms().size());
   }
  
  /**
   * Test Do Not Patient Synonyms when Intended Use is not Trade Name.
   */
  @Test
  public void testLeaveMatchingSynonymWithOtherIntendedUse() {
    //Product
    ProductVo parent = new ProductVo();
    parent.setVaProductName("ASPIRIN");
    parent.setId("100");
    
    //Ndcs remaining on this product after some of the ndcs moved to another product
    ArrayList<NdcVo> ndcs = new ArrayList<NdcVo>();
    
    NdcVo ndc = new NdcVo();
    ndc.setId("1000");
    ndc.setNdc("1000-727");
    ndc.setTradeName("ASPIRIN Rx");
    ndcs.add(ndc);      
    parent.setNdcs(ndcs );
    
    Collection<SynonymVo> synonyms = new ArrayList<SynonymVo>();
    IntendedUseVo iu = new IntendedUseVo();
    iu.setIntendedUse("BLA-BLA");

    SynonymVo synonym = new SynonymVo();
    synonym.setSynonymName("ASPIRIN Rx");
    synonym.setNdcCode("1000-777");
    synonym.setSynonymIntendedUse(iu);
    synonyms.add(synonym);      
    parent.setSynonyms(synonyms);
       
    //Ndc to be moved to another product
    NdcVo ndcToMove = new NdcVo();
    ndcToMove.setId("1000");
    ndcToMove.setNdc("1000-727");
    ndcToMove.setTradeName("ASPIRIN Rx");
    ndcToMove.setProduct(parent);
   
    // Method call
    parent.updateParentAssociations(Arrays.asList(ndcToMove));
    assertEquals(1, parent.getSynonyms().size());
  }
  
  /**
   * Test Do Not remove Patient Synonyms when Intended Use is not Trade Name.
   */
  @Test
  public void testLeaveMatchingSynonymWhenIntendedUseIsNotSet() {
    //Product
    ProductVo parent = new ProductVo();
    parent.setVaProductName("ASPIRIN");
    parent.setId("100");
    
    //Ndcs remaining on this product after some of the ndcs moved to another product
    ArrayList<NdcVo> ndcs = new ArrayList<NdcVo>();
    
    NdcVo ndc = new NdcVo();
    ndc.setId("1000");
    ndc.setNdc("1000-727");
    ndc.setTradeName("ASPIRIN Rx");
    ndcs.add(ndc);      
    parent.setNdcs(ndcs );
    
    Collection<SynonymVo> synonyms = new ArrayList<SynonymVo>();
    

    SynonymVo synonym = new SynonymVo();
    synonym.setSynonymName("ASPIRIN Rx");
    synonym.setNdcCode("1000-777");
    synonyms.add(synonym);      
    parent.setSynonyms(synonyms);
       
    //Ndc to be moved to another product
    NdcVo ndcToMove = new NdcVo();
    ndcToMove.setId("1000");
    ndcToMove.setNdc("1000-727");
    ndcToMove.setTradeName("ASPIRIN Rx");
    ndcToMove.setProduct(parent);
   
    // Method call
    parent.updateParentAssociations(Arrays.asList(ndcToMove));

    assertEquals(1, parent.getSynonyms().size());
  }
  
  /**
   * Test Clear Patient Synonyms Last Ndc With Matching Trade Name Moved.
   */
  @Test
  public void testClearPatientSynonymsLastNdcWithMatchingTradeNameMoved() {
    //Product
    ProductVo parent = new ProductVo();
    parent.setVaProductName("ASPIRIN");
    parent.setId("100");
    
    //Ndcs remaining on this product after some of the ndcs moved to another product
    ArrayList<NdcVo> ndcs = new ArrayList<NdcVo>();
    
    NdcVo ndc = new NdcVo();
    ndc.setId("1000");
    ndc.setNdc("1000-727");
    ndc.setTradeName("ASPIRIN Rx");
    ndcs.add(ndc);      
    parent.setNdcs(ndcs );
    
    Collection<SynonymVo> synonyms = new ArrayList<SynonymVo>();
    IntendedUseVo iu = new IntendedUseVo();
    iu.setIntendedUse("0-TRADE NAME");

    SynonymVo synonym = new SynonymVo();
    synonym.setSynonymName("ASPIRIN Rx");
    synonym.setNdcCode("1000-777");
    synonym.setSynonymIntendedUse(iu);
    synonyms.add(synonym); 
    parent.setSynonyms(synonyms);
       
    //Ndc to be moved to another product
    NdcVo ndcToMove = new NdcVo();
    ndcToMove.setId("1000");
    ndcToMove.setNdc("1000-727");
    ndcToMove.setTradeName("ASPIRIN Rx");
    ndcToMove.setProduct(parent);
   
    // Method call
    parent.updateParentAssociations(Arrays.asList(ndcToMove));
    assertEquals(0, parent.getSynonyms().size());
  }

}
