package gov.va.med.pharmacy.peps.domain.common.capability.impl;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;

import org.hibernate.Criteria;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.vo.ComplianceSiteVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.capability.ComplianceSiteDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.dao.EplComplianceSiteDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplComplianceSiteDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.ComplianceSiteConverter;

/**
 * Perform CRUD operations on Compliance Site.
 */
public class ComplianceSiteDomainCapabilityImpl implements ComplianceSiteDomainCapability {

    /**
     * The data access object for Compliance Site.
     */
    private EplComplianceSiteDao eplComplianceSiteDao;

    /**
     * The converter for Compliance Site.
     */
    private ComplianceSiteConverter complianceSiteConverter;

    /**
     * Constructor.
     *
     * @param eplComplianceSiteDao the EplComplianceSiteDao
     * @param complianceSiteConverter the ComplianceSiteConverter
     */
    @Autowired
    public ComplianceSiteDomainCapabilityImpl(final EplComplianceSiteDao eplComplianceSiteDao,
        final ComplianceSiteConverter complianceSiteConverter) {
        super();
        this.eplComplianceSiteDao = eplComplianceSiteDao;
        this.complianceSiteConverter = complianceSiteConverter;
    }

    @Override
    public ComplianceSiteVo create(ComplianceSiteVo complianceSiteVo, UserVo user) {

        EplComplianceSiteDo eplComplianceSiteDo = complianceSiteConverter.convert(complianceSiteVo);
        EplComplianceSiteDo returnComplianceSiteDo = eplComplianceSiteDao.insert(eplComplianceSiteDo, user);

        return complianceSiteConverter.convert(returnComplianceSiteDo);
    }

    @Override
    public ComplianceSiteVo update(ComplianceSiteVo complianceSiteVo, UserVo user) {
        EplComplianceSiteDo eplComplianceSiteDo = complianceSiteConverter.convert(complianceSiteVo);
        eplComplianceSiteDao.update(eplComplianceSiteDo, user);

        return complianceSiteVo;
    }

    @Override
    public void delete(ComplianceSiteVo complianceSiteVo, UserVo user) {
        EplComplianceSiteDo eplComplianceSiteDo = complianceSiteConverter.convert(complianceSiteVo);
        eplComplianceSiteDao.delete(eplComplianceSiteDo);

        return;
    }    
    
    @Override
    public List<ComplianceSiteVo> retrieveByIncludeExcludeIndicator(String includeExcludeIndicator) {
        Criteria criteria = eplComplianceSiteDao.getCriteria();
        criteria.add(Restrictions.eq(EplComplianceSiteDo.INCLUDE_EXCLUDE_INDICATOR, includeExcludeIndicator));
        List<EplComplianceSiteDo> values = criteria.list();
        List<ComplianceSiteVo> returnedValues = complianceSiteConverter.convert(values);
        returnedValues.stream().sorted(Comparator.comparing(ComplianceSiteVo::getStationName));

        return returnedValues;
    }
    

    @Override
    public List<ComplianceSiteVo> saveList(final List<ComplianceSiteVo> sitesToSave, String includeExcludeIndicator, 
        UserVo user){

        List<ComplianceSiteVo> existingSites = retrieveByIncludeExcludeIndicator(includeExcludeIndicator);
        
        List<String> existingSiteStations = existingSites.stream()
            .map(existingSite -> existingSite.getVistaSiteVo().getStationNumber())
            .collect(Collectors.toList());

        List<String> sitesToSaveStations = sitesToSave.stream()
            .map(site -> site.getVistaSiteVo().getStationNumber())
            .collect(Collectors.toList());
        
        List<String> stationsToUpdate = new ArrayList(sitesToSaveStations);
        stationsToUpdate.retainAll(existingSiteStations);
        
        List<String> stationsToInsert = new ArrayList(sitesToSaveStations);
        stationsToInsert.removeAll(existingSiteStations);
        
        List<String> stationsToDelete = new ArrayList(existingSiteStations);
        stationsToDelete.removeAll(sitesToSaveStations);

        List<ComplianceSiteVo> results = new ArrayList<ComplianceSiteVo>();
        
        for (ComplianceSiteVo siteToSave : sitesToSave) {
            String stationNumber = siteToSave.getVistaSiteVo().getStationNumber();
            if (stationsToUpdate.contains(stationNumber)){
                results.add(update(siteToSave, user));
            }
            else if (stationsToInsert.contains(stationNumber)){
                results.add(create(siteToSave, user));
            }
        }
        
        for (ComplianceSiteVo existingSite : existingSites) {                
            if (stationsToDelete.contains(existingSite.getVistaSiteVo().getStationNumber())){
                delete(existingSite, user);
            }
        }          
        
        return results;         
    }
    
}
