package gov.va.med.pharmacy.peps.domain.common.capability.impl;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.vo.VistaSiteVo;
import gov.va.med.pharmacy.peps.domain.common.capability.SiteListDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.model.FileBasedSiteDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.VistaSiteConverter;

/**
 * Implementation of SiteListDomainCapability that is based on a configuration file containing line with the with the format:
 * DEACTIVATIONDATE^NAME^STATIONNUMBER^FACILITYTYPECODE. The file is expected to have a header line.
 */
public class FileBasedSiteListDomainCapabilityImpl implements SiteListDomainCapability {

    private static final Logger LOG = LogManager.getLogger(FileBasedSiteListDomainCapabilityImpl.class);
        
    private static final int FACILITY_TYPE_CODE_POS = 3;
    private static final int STATION_NUMBER_POS = 2;
    private static final int NAME_POS = 1;
    private static final int DEACTIVATION_DATE_POS = 0;
    private static final List<String> FILTERED_FACILITY_TYPE_CODES = Arrays.asList("VAMC", "RO-OC", "OC", "M&ROC", "CMOP");
    private VistaSiteConverter vistaSiteConverter;

    /**
     * Constructor with autowired parameters.
     *
     * @param pVistaSiteConverter the vista site converter
     */
    @Autowired
    public FileBasedSiteListDomainCapabilityImpl(VistaSiteConverter pVistaSiteConverter) {
        super();
        this.vistaSiteConverter = pVistaSiteConverter;
    }

    @Override
    public Collection<VistaSiteVo> retrieveAll() {
        try {
            return vistaSiteConverter.convert(readListFromFile()).stream().filter(site -> isNumeric(site.getStationNumber()))
                .sorted(Comparator.comparing(VistaSiteVo::getName)).collect(Collectors.toList());
        } catch (Exception ex) {
            LOG.error("Exception retrieving sites fron text file.", ex);
            return new ArrayList<>();
        }
    }

    @Override
    public Collection<VistaSiteVo> retrieveFiltered() throws Exception{        
        Predicate<FileBasedSiteDo> activePred = site -> site != null && Strings.isEmpty(site.getDeactivationDate());
        Predicate<FileBasedSiteDo> numericStationNumberPred = site -> site != null && isNumeric(site.getStationNumber());
        Predicate<FileBasedSiteDo> facilityTypePred = site -> site != null
            && FILTERED_FACILITY_TYPE_CODES.contains(site.getFacilityTypeCode());
        return vistaSiteConverter
            .convert(readListFromFile().stream().filter(activePred.and(facilityTypePred).and(numericStationNumberPred))
                .sorted(Comparator.comparing(FileBasedSiteDo::getName)).collect(Collectors.toList()));
    }

    private boolean isNumeric(String str) {
        if (Strings.isEmpty(str)) {
            return false;
        }
        for (int i = 0; i < str.length(); i++) {
            if (!Character.isDigit(str.charAt(i))) {
                return false;
            }
        }
        return true;
    }

    private Collection<FileBasedSiteDo> readListFromFile() throws Exception{
        URL url = this.getClass().getClassLoader().getResource("siteList.txt");
        File file = new File(url.getFile());
        Path path = file.toPath();

        Collection<FileBasedSiteDo> list = new ArrayList<>();
        Stream<String> stream = Files.lines(path);
        list = stream.map(mapLineToFileBasedSiteDo).collect(Collectors.toList());
        return list;
    }

    private Function<String, FileBasedSiteDo> mapLineToFileBasedSiteDo = (line) -> {
        String[] tokens = line.split("\\^");
        if (tokens[STATION_NUMBER_POS] == null || tokens[STATION_NUMBER_POS].trim().length() == 0) {
            // if station number is missing, skip the record
            return null;
        }
        FileBasedSiteDo fileBasedSiteDo = new FileBasedSiteDo();
        if (tokens[DEACTIVATION_DATE_POS] != null && tokens[DEACTIVATION_DATE_POS].trim().length() > 0) {
            fileBasedSiteDo.setDeactivationDate(tokens[DEACTIVATION_DATE_POS]);
        }
        if (tokens[NAME_POS] != null && tokens[NAME_POS].trim().length() > 0) {
            fileBasedSiteDo.setName(tokens[NAME_POS]);
        }
        if (tokens[STATION_NUMBER_POS] != null && tokens[STATION_NUMBER_POS].trim().length() > 0) {
            fileBasedSiteDo.setStationNumber(tokens[STATION_NUMBER_POS]);
        }
        if (tokens[FACILITY_TYPE_CODE_POS] != null && tokens[FACILITY_TYPE_CODE_POS].trim().length() > 0) {
            fileBasedSiteDo.setFacilityTypeCode(tokens[FACILITY_TYPE_CODE_POS]);
        }
        return fileBasedSiteDo;
    };

    @Override
    public VistaSiteVo retrieveByStationNumber(String stationNumber) {
        if (stationNumber == null || stationNumber.isEmpty()) {
            return null;
        }

        Collection<VistaSiteVo> sites = retrieveAll();
        List<VistaSiteVo> matchingSites = sites.stream().filter(site -> Objects.equals(stationNumber, site.getStationNumber()))
            .collect(Collectors.toList());
        // Expect zero or one match. If the provider has more, then provide the first one.
        if (matchingSites.size() > 0) {
            return matchingSites.get(0);
        }
        return null;
    }
}
