/**
 * Source file created in 2007 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.domain.common.capability.impl;


import gov.va.med.pharmacy.peps.common.exception.DuplicateItemException;
import gov.va.med.pharmacy.peps.common.exception.ItemNotFoundException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.utility.PPSConstants;
import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.GenericNameVo;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.VuidItemType;
import gov.va.med.pharmacy.peps.common.vo.VuidStatusHistoryVo;
import gov.va.med.pharmacy.peps.domain.common.capability.GenericNameDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.dao.EplProductDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplVaGenericNameDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplVuidStatusHistoryDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplDrugUnitDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplVaGenNameDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplVuidStatusHistoryDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.Converter;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.GenericNameConverter;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.VuidStatusHistoryConverter;
import gov.va.med.pharmacy.peps.domain.common.utility.querybuilder.HqlBuilder;

import java.util.Collection;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.hibernate.Criteria;
import org.hibernate.criterion.Restrictions;


/**
 * Perform CRUD operations on generic name.
 */
public class GenericNameDomainCapabilityImpl extends
        ManagedDataDomainCapabilityImpl<GenericNameVo, EplVaGenNameDo>
        implements GenericNameDomainCapability {
    private EplVaGenericNameDao eplVaGenericNameDao;
    private GenericNameConverter genericNameConverter;
    private EplVuidStatusHistoryDao eplVuidStatusHistoryDao;
    private EplProductDao eplProductDao;
    private VuidStatusHistoryConverter vuidStatusHistoryConverter;

    /**
     * Insert the given {@link ManagedItemVo} with a duplicate check.
     * 
     * @param managedItem {@link ManagedItemVo}
     * @param user {@link UserVo} performing the action
     * @return {@link ManagedItemVo} inserted with new ID
     * @throws DuplicateItemException if item already exists by uniqueness fields
     */
    @Override
    public GenericNameVo create(GenericNameVo managedItem, UserVo user) throws DuplicateItemException {      
        return super.create(managedItem, user);
    }
    
    /**
     * Create Hibernate {@link Criteria} that will find an item by its
     * uniqueness fields.
     * 
     * @param item
     *            {@link ManagedItemVo} for which to create uniqueness
     *            {@link Criteria}
     * @return {@link Criteria}
     */
    @Override
    protected Criteria createUniquenessCriteria(GenericNameVo item) {
        Criteria criteria = getDataAccessObject().getCriteria();
        criteria.add(Restrictions.ilike(EplVaGenNameDo.NAME, item.getValue()));

        return criteria;
    }
    
    
    /**
     * Delete the specified Generic Name.
     * @param genericNameVo genericNameVo
     * @throws ValidationException ValidationException
     */
    @Override
    public void deleteItem(GenericNameVo genericNameVo) throws ValidationException  {
        List<EplVaGenNameDo> genericNameDo = getDataAccessObject().retrieve(EplDrugUnitDo.EPL_ID, 
            Long.valueOf(genericNameVo.getId()));
       
        if (genericNameDo.size() == 1) {
            try {
                checkForActiveDependencies(genericNameVo, null);
            } catch (Exception e) {
                throw new ValidationException(ValidationException.CANNOT_DELETE, genericNameVo.getValue(),
                    " because another item is using it.");
            }

            getDataAccessObject().delete(genericNameDo.get(0));
        }
    }
    
    
    /**
     * GenericNameVo cannot be in-activated if it has an active product.
     *  
     * @param genericNameVo genericNameVo
     * @param user {@link UserVo} performing the action
     * @throws ValidationException ValidationException
     */
    @Override
    public void checkForActiveDependencies(GenericNameVo genericNameVo, UserVo user) throws ValidationException {
        //TODO - parameterize query or change to Criteria for better DB statement caching and move to EplProductDao.
        StringBuffer query = new StringBuffer();
        query.append(HqlBuilder.create("SELECT  item FROM  ").append(EplProductDo.class)
                .append(PPSConstants.ITEM));
        String whereClause = "  WHERE VA_GEN_NAME_ID_FK = " + genericNameVo.getId() + " AND ITEM_STATUS LIKE 'ACTIVE'";
        query.append(whereClause);
        
        List<EplProductDo> products = eplProductDao.executeHqlQuery(query.toString());

        if (products == null || products.size() == 0) {
            return;
        }

        // return the validation if checkForActiveDependencies returned products.
        throw new ValidationException(ValidationException.INACTIVATE_GENERICNAME,
                                      products.size(), products.get(0).getVaProductName());
    }

    /**
     * Insert the given GenericNameVo.
     * 
     * @param genericNameVo genericNameVo
     * @param user {@link UserVo} performing the action
     * @return GenericNameVo with new ID
     */
    @Override
    public GenericNameVo createWithoutDuplicateCheck(
            GenericNameVo genericNameVo, UserVo user) {

        if (genericNameVo.getId() == null) {
            genericNameVo.setId(getSeqNumDomainCapability().generateId(
                    genericNameVo.getEntityType(), user));

            if ((genericNameVo.getVaDataFields() != null)
                    && (!(genericNameVo.getVaDataFields().isEmpty()))) {
                genericNameVo.getVaDataFields().setVaDfOwnerId(
                        getSeqNumDomainCapability().generatedOwnerId(user));
            }
        }
       
        // We generate the IEN immediately upon creating the item in the database. If the item is subsequently deleted
        // before sending it to VistA, then we will create a gap in VistA's IEN sequence. That is OK according to the
        // VistA developers.
        Long id = getSeqNumDomainCapability().generateIen("EPL_VA_GEN_NAMES", user); 
        genericNameVo.setGenericIen(id.toString());
        EplVaGenNameDo genericNameDo = genericNameConverter.convert(genericNameVo);
        eplVaGenericNameDao.insert(genericNameDo, user);

        if (genericNameVo.getEffectiveDates() != null) {
            for (VuidStatusHistoryVo vuidStatusHistoryVo : genericNameVo
                    .getEffectiveDates()) {
    
                vuidStatusHistoryVo.setId(getSeqNumDomainCapability().generateId(
                        vuidStatusHistoryVo.getEntityType(), user));
                vuidStatusHistoryVo.setItemType(VuidItemType.GENERIC);
                EplVuidStatusHistoryDo vuidStatusHistoryDo = vuidStatusHistoryConverter
                        .convert(vuidStatusHistoryVo);
                eplVuidStatusHistoryDao.insert(vuidStatusHistoryDo, user);
            }
        }

        return (genericNameVo);
    }

    /**
     * Retrieve by EPL Id.
     * @param eplId eplId
     * @return GenericNameVo
     * @throws ItemNotFoundException 
     */
    public GenericNameVo retrieve(String eplId) throws ItemNotFoundException {
        GenericNameVo genericNameVo = super.retrieve(eplId);
        
        List<EplVuidStatusHistoryDo> vuidStatusHistoryList = eplVuidStatusHistoryDao.retrieveByVuidAndItemType(
            genericNameVo.getVuid(), PPSConstants.VUID_GENERIC);     
        List<VuidStatusHistoryVo> vuidStatusHistroyVoList = vuidStatusHistoryConverter.convert(vuidStatusHistoryList);
        genericNameVo.setEffectiveDates(vuidStatusHistroyVoList);
       
        return genericNameVo;
        
    }
    
    @Override
    public GenericNameVo retrieveByName(String name) {
        
        String name2 = name;
     
        // replace ' with double tick.
        if (name2.contains("'")) {
            name2 = name2.replace("'", "''");
        }
        
        StringBuffer query = new StringBuffer();
        query.append(HqlBuilder.create("SELECT item FROM").append(EplVaGenNameDo.class)
                .append("item"));
        String whereClause = " where GENERIC_NAME LIKE '" + name2 + "'";
        query.append(whereClause);

        List<EplVaGenNameDo> returnedDos = eplVaGenericNameDao.executeHqlQuery(query.toString());

        if (returnedDos == null || returnedDos.size() < 1) {
            return null;
        }
        
        Collection<GenericNameVo> vos =  genericNameConverter.convert(returnedDos);
        
        for (GenericNameVo vo : vos) {
            if (StringUtils.equals(vo.getValue(), name2)) {
                return vo;
            }
        }
             
        return null;
    }
    
    /**
     * Update the specified Generic Name.
     * @param generic GenericNameVo to update
     * @param user user making the updates
     * @return GenericNameVo the updated GenericNameVo
     * @throws DuplicateItemException  DuplicateItemException
     */
    public GenericNameVo update(GenericNameVo generic, UserVo user) throws DuplicateItemException {
        
        EplVaGenNameDo updatedGenName = genericNameConverter.convert(generic);

        // update attributes on product table
        eplVaGenericNameDao.update(updatedGenName, user);

        // check to see if we need to add the VuidStatusHistoryRecord
        List<EplVuidStatusHistoryDo> vuidStatusHistoryList = eplVuidStatusHistoryDao.retrieveByVuidAndItemType(
            generic.getVuid(), PPSConstants.VUID_GENERIC);          
        
        // if there is a vuid and no history record then create the history record.
        if (StringUtils.isNotBlank(generic.getVuid())) {
            if (vuidStatusHistoryList.size() == 0) {
                if (generic.getEffectiveDates() != null) {
                    generic.getEffectiveDates().get(0).setId(getSeqNumDomainCapability().generateId(
                        EntityType.VUID_STATUS_HISTORY, user));
                    EplVuidStatusHistoryDo data = vuidStatusHistoryConverter.convert(generic.getEffectiveDates().get(0));
                    eplVuidStatusHistoryDao.insert(data, user);
                }
            }
        }
       
        return generic;
        
    }

    @Override
    public EplVaGenericNameDao getDataAccessObject() {
        return eplVaGenericNameDao;
    }

    /**
     * Return the {@link Converter} instance for this capability.
     * 
     * @return {@link Converter}
     */
    public GenericNameConverter getConverter() {
        return genericNameConverter;
    }

    /**
     * Set the EplVaGenericNameDao.
     * @param eplVaGenericNameDao eplVaGenericNameDao property
     */
    public void setEplVaGenericNameDao(EplVaGenericNameDao eplVaGenericNameDao) {
        this.eplVaGenericNameDao = eplVaGenericNameDao;
    }

    /**
     * Set the GenericNameConverter.
     * @param genericNameConverter genericNameConverter property
     */
    public void setGenericNameConverter(
            GenericNameConverter genericNameConverter) {
        this.genericNameConverter = genericNameConverter;
    }

    /**
     * Set the VuidStatusHistoryConverter.
     * @param vuidStatusHistoryConverter vuidStatusHistoryConverter property
     */
    public void setVuidStatusHistoryConverter(
            VuidStatusHistoryConverter vuidStatusHistoryConverter) {
        this.vuidStatusHistoryConverter = vuidStatusHistoryConverter;
    }

    /**
     * Set the EplVuidStatusHistoryDao.
     * @param eplVuidStatusHistoryDao
     *            eplVuidStatusHistoryDao property
     */
    public void setEplVuidStatusHistoryDao(
            EplVuidStatusHistoryDao eplVuidStatusHistoryDao) {
        this.eplVuidStatusHistoryDao = eplVuidStatusHistoryDao;
    }
    
    /**
     * Set the EplProductDao.
     * @param eplProductDao eplProductDao property
     */
    public void setEplProductDao(EplProductDao eplProductDao) {
        this.eplProductDao = eplProductDao;
    }


}
