package gov.va.med.pharmacy.peps.domain.common.dao.impl;

import java.sql.Timestamp;
import java.util.Date;
import java.util.List;
import java.util.Optional;

import gov.va.med.pharmacy.peps.common.vo.NdfUpdateFileVo;
import gov.va.med.pharmacy.peps.common.vo.RematchStatus;
import gov.va.med.pharmacy.peps.domain.common.dao.EplRematchDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplRematchDo;
import gov.va.med.pharmacy.peps.domain.common.utility.converter.NdfUpdateFileConverter;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.criterion.Restrictions;


/**
 * Primary implementation of {@link EplRematchDao}.
 */
public class EplRematchDaoImpl extends DataAccessObjectImpl<EplRematchDo, Long>
  implements EplRematchDao {
  
  private static final Logger LOG = LogManager.getLogger(EplRematchDaoImpl.class);

    /**
     * The query HQL to find the default set of candidate rematch products. The intent of this query is to find active
     * products with the same strength, orderable item, and drug unit.
     */
    private static final String FIND_REMATCH_PRODUCTS_HQL = 
        "select candidate "
        +   "from EplProductDo candidate "
            // Join to the unmatched product to compare the properties.
        +   "join EplProductDo unmatchedProduct "
        +   "with unmatchedProduct.id = :unmatchedProductId "
        +       "and candidate <> unmatchedProduct "
                // Treating NULL and 0 as the same for strength. This could possibly grab more products than appropriate,
                // but that is not harmful since we're just pulling _candidate_ products. The actual products used must
                // be culled from the returned list anyway.
        +       "and coalesce(candidate.strength, 0) = coalesce(unmatchedProduct.strength, 0) "
        +       "and candidate.eplOrderableItem = unmatchedProduct.eplOrderableItem "
        // Have to explicitly left join DrugUnits. If we do something like "candidate.eplDrugUnit =
        // unmatchedProduct.eplDrugUnit" it creates an inner join, which is harmful since eplDrugUnit is nullable.
        +   "left join candidate.eplDrugUnit candidateDrugUnit "
        +   "left join unmatchedProduct.eplDrugUnit unmatchedProductDrugUnit "
        +   "where (candidateDrugUnit = unmatchedProductDrugUnit or (candidateDrugUnit is null and unmatchedProductDrugUnit is null)) "
        +       "and candidate.requestStatus = 'APPROVED' "
        +       "and candidate.itemStatus = 'ACTIVE' "
        +       "and candidate.inactivationDate is null ";
      
    private NdfUpdateFileConverter ndfUpdateFileConverter;    

    @Override
    public Optional<EplRematchDo> retrieveForProduct(long productId) {
      LOG.debug("Retrieving rematch for product {}.", productId);

      final Session session = getCurrentSession();

      final Criteria criteria = session.createCriteria(EplRematchDo.class)
          // Null update file means we haven't sent the rematch yet.
          .add(Restrictions.isNull("ndfUpdateFile"))
          .createCriteria("fromProduct").add(Restrictions.eq("eplId", productId));
      // The Hibernate API returns only Object, sadly, but it should always be an EplRematchDo.
      // Note that uniqueResult will throw an Exception if the query returns more than one result.
      final EplRematchDo rematchDo = (EplRematchDo)criteria.uniqueResult();

      return Optional.ofNullable(rematchDo);
    }
    
    @Override   
    public List<EplProductDo> findRematchProducts(long productId) {
        LOG.debug("Finding rematch products for product {}.", productId);

        Session session = getCurrentSession();
        Query query = session.createQuery(FIND_REMATCH_PRODUCTS_HQL)
            .setLong("unmatchedProductId",productId);
        @SuppressWarnings("unchecked")  // Trust Hibernate to return the right type.
        List<EplProductDo> returnedRows = query.list();

        return returnedRows;
    } 

   /**
    * Update
    */
    public EplRematchDo update(EplRematchDo eplRematchSuggestion) {
        
        Session session = getCurrentSession();
        session.update(eplRematchSuggestion);
        
        return eplRematchSuggestion;
    }

    @Override
    public List<EplRematchDo> findUnsentRematches() {
        final Session session = getCurrentSession();

        final Criteria criteria = session.createCriteria(EplRematchDo.class)
            // Null update file means we haven't sent the rematch yet.
            .add(Restrictions.isNull("ndfUpdateFile"));

        @SuppressWarnings("unchecked") // Trust Hibernate that it will return the right class
        final List<EplRematchDo> list = criteria.list();

        return list;
    }

    @Override
    public List<EplRematchDo> findOutgoingRematches(Date startDTM) {
        LOG.debug("Querying for outgoing rematches.");

        final Timestamp startDate = new Timestamp(startDTM.getTime());
        final Session session = getCurrentSession();

        final Criteria criteria = session.createCriteria(EplRematchDo.class)
            // Null update file means we haven't sent the rematch yet.
            .add(Restrictions.isNull("ndfUpdateFile"))
            .add(Restrictions.le("createdDtm", startDate))
            .add(Restrictions.eq("status", RematchStatus.APPROVED));

        @SuppressWarnings("unchecked")
        final List<EplRematchDo> list = criteria.list();

        return list;
    }
    
    @Override
    public void updateRematchNdfUpdateFileFk(Date startDTM, NdfUpdateFileVo ndfUpdateFileVo){        

        List<EplRematchDo> returnedValues = findOutgoingRematches(startDTM);
                
        for (EplRematchDo eplRematchDo : returnedValues) {
            eplRematchDo.setNdfUpdateFile(ndfUpdateFileConverter.convert(ndfUpdateFileVo));
            update(eplRematchDo);
        }
   }
    
    @Override
    public void removeNdfUpdateFileFk(Long fileSeqNo) {
        
        Session session = getCurrentSession();
        Query query = session.createQuery("update EplRematchDo set ndfUpdateFile = null "
            + " where ndfUpdateFile = " + fileSeqNo);
        query.executeUpdate();
    }
    
    public NdfUpdateFileConverter getNdfUpdateFileConverter() {
        return ndfUpdateFileConverter;
    }

    
    public void setNdfUpdateFileConverter(NdfUpdateFileConverter ndfUpdateFileConverter) {
        this.ndfUpdateFileConverter = ndfUpdateFileConverter;
    }   

}
    
    
