package gov.va.med.pharmacy.peps.domain.common.model;

import java.util.Collections;
import java.util.HashSet;
import java.util.Objects;
import java.util.Set;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Transient;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import gov.va.med.pharmacy.peps.common.vo.RematchStatus;


/**
 * Data Object version of {@link gov.va.med.pharmacy.peps.common.vo.RematchVo}.
 */
@Entity
@Table(name = "EPL_REMATCH", schema = "PPSNEPL")
public class EplRematchDo extends AnnotatedDataObject implements java.io.Serializable {

    /**
     * Change this when changing the class fields!
     */
    private static final long serialVersionUID = 6L;

    private static final Logger LOG = LogManager.getLogger(EplRematchDo.class);

    /**
     * The name of the database constraint that enforces only one rematch per product per update file.
     */
    public static final String UNIQUE_CONSTRAINT_NAME = "PPSNEPL.EPL_REMATCH_U01";

    private Long eplRematchId;
    private EplProductDo fromProduct;
    private EplProductDo toAutoProduct;
    private RematchStatus status;
    private EplNdfUpdateFileDo ndfUpdateFile;

    /**
     * Backing attribute for {@link #getToSuggestedProducts()}. We guarantee that the getter never returns null so this starts
     * off as an empty set instead of null.
     */
    private Set<EplProductDo> toSuggestedProducts = new HashSet<>();

    /**
     * Constructs an instance. (Necessary for Hibernate.)
     */
    public EplRematchDo() {
    }

    /**
     * Minimal constructor useful for constructing test instances. All properties not passed in the constructor will be set to
     * null.
     * 
     * @param pFromProduct See {@link #setFromProduct(EplProductDo)}.
     * @param pToAutoProduct See {@link #setToAutoProduct(EplProductDo)}.
     * @param pStatus See {@link #setStatus(RematchStatus)}.
     */
    public EplRematchDo(final EplProductDo pFromProduct, final EplProductDo pToAutoProduct, final RematchStatus pStatus) {
        fromProduct = pFromProduct;
        toAutoProduct = pToAutoProduct;
        status = pStatus;
    }

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "rematchSeqGen")
    @SequenceGenerator(name = "rematchSeqGen", sequenceName = "EPL_REMATCH_SEQ")
    @Column(name = "EPL_REMATCH_ID", unique = true, nullable = false, precision = ID_PRECISION, scale = 0)
    public Long getEplRematchId() {
        return eplRematchId;
    }

    public void setEplRematchId(final Long eplRematchId) {
        this.eplRematchId = eplRematchId;
    }

    /**
     * Returns the product we are matching from. (See {@link gov.va.med.pharmacy.peps.common.vo.RematchVo#getOldProduct()}.)
     * 
     * @return the product object
     */
    @ManyToOne( // Using default of no cascade.
    fetch = FetchType.EAGER) // Note: this association could probably be lazy-loaded, but loading even over the VPN
                             // took <200ms so it's probably worth loading initially.
    @JoinColumn(name = "FROM_PRODUCT_ID_FK", nullable = false)
    public EplProductDo getFromProduct() {
        return fromProduct;
    }

    public void setFromProduct(final EplProductDo pFromProduct) {
        fromProduct = pFromProduct;
    }

    /**
     * Returns the product we are automatically matching to.
     * 
     * @return Possibly null if this is a suggested rematch.
     */
    @ManyToOne( // Using default of no cascade.
    fetch = FetchType.EAGER) // Note: this association could probably be lazy-loaded, but loading even over the VPN
                             // took <200ms so it's probably worth loading initially.
    @JoinColumn(name = "TO_PRODUCT_AUTO_ID_FK", nullable = true)
    public EplProductDo getToAutoProduct() {
        return toAutoProduct;
    }

    public void setToAutoProduct(final EplProductDo pToAutoProduct) {
        toAutoProduct = pToAutoProduct;
    }

    /**
     * Return the suggested rematch products.
     * 
     * @return never null but possibly empty.
     */
    @ManyToMany(fetch = FetchType.LAZY)
    @JoinTable(
        schema = "PPSNEPL", name = "EPL_REMATCH_SUGGESTION",
        joinColumns = @JoinColumn(name = "EPL_REMATCH_ID") ,
        inverseJoinColumns = @JoinColumn(name = "TO_PRODUCT_ID_FK") )
    public Set<EplProductDo> getToSuggestedProducts() {
        return toSuggestedProducts;
    }

    /**
     * Set the suggested rematch products.
     * 
     * @param value Must not be null.
     */
    public void setToSuggestedProducts(final Set<EplProductDo> value) {
        toSuggestedProducts = Objects.requireNonNull(value);
    }

    @Enumerated(EnumType.ORDINAL)
    @Column(name = "STATUS")
    public RematchStatus getStatus() {
        return status;
    }

    public void setStatus(RematchStatus status) {
        this.status = status;
    }

    @ManyToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "NDF_UPDATE_FILE_FK")
    public EplNdfUpdateFileDo getNdfUpdateFile() {
        return ndfUpdateFile;
    }

    public void setNdfUpdateFile(final EplNdfUpdateFileDo ndfUpdateFile) {
        this.ndfUpdateFile = ndfUpdateFile;
    }

    /**
     * Returns the "to" product(s), whether automatic or suggested. Note that this will always return a set of 1 product in the
     * case of automatic rematches.
     * 
     * @return never null
     * @see #getToAutoProduct()
     * @see #getToSuggestedProducts()
     */
    @Transient // Calculated field.
    public Set<EplProductDo> getToProducts() {
        if (getToAutoProduct() != null) {
            LOG.debug("Returning automatic product.");
            return Collections.singleton(getToAutoProduct());
        } else {
            LOG.debug("Returning suggested product(s).");
            return getToSuggestedProducts();
        }
    }

}
