

package gov.va.med.pharmacy.peps.domain.common.utility;


import gov.va.med.pharmacy.peps.common.exception.DomainException;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplDdiSeverityDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplDrugDrugInteractionDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplIenGeneratorDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplIngredientDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplDdiSeverityDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplDrugDrugInteractionDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplIenGeneratorDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplIngredientDo;

import java.util.Calendar;
import java.util.Date;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;


/**
 * <p>Processes DDI migration DAT files and puts them into the database.</p>
 * 
 * <p>This class is <em>not</em> thread-safe: it must only be used by one thread at a time.</p>
 */
public class DDIFileReader extends NdfFileReader {
    
    /**
     * Return value of {@link #getHighestIenProcessed()} indicating no records have been processed yet. According to 
     * Linda Ellzey, VistA IENs start at 1.
     */
    protected static final long NO_IENS_PROCESSED = 0;
    
    private static final Logger LOG = LogManager.getLogger(DDIFileReader.class);

    private static final int IEN = 0;

    private static final int INTERACTION_NAME = 1;

    private static final int INGREDIENT1 = 2;

    private static final int INGREDIENT2 = 3;

    private static final int SEVERITY = 4;

    private static final int NATIONALLY_ENTERED = 5;

    private static final int TOTAL_INDEXES = 6;

    private static final int LOCALLY_ENTERED = 7;

    private static final int INACTIVATION_DATE = 8;

    private final EplIngredientDao eplIngredientDao;
    private final EplDdiSeverityDao eplDdiSeverityDao;
    private final EplDrugDrugInteractionDao drugDrugInteractionDao;
    private final EplIenGeneratorDao eplIenGeneratorDao;
    private final UserVo user;
    private long highestIenProcessed;

    /**
     * Constructs an instance with the required dependencies.
     * @param pEplIngredientDao the {@link EplIngredientDao} to use.
     * @param pEplDdiSeverityDao the {@link EplDdiSeverityDao} to use.
     * @param pDrugDrugInteractionDao the {@link EplDrugDrugInteractionDao} to use.
     * @param pEplIenGeneratorDao the {@link EplIenGeneratorDao} to use.
     * @param pUser the user executing the migration
     */
    public DDIFileReader(
        final EplIngredientDao pEplIngredientDao,
        final EplDdiSeverityDao pEplDdiSeverityDao,
        final EplDrugDrugInteractionDao pDrugDrugInteractionDao,
        final EplIenGeneratorDao pEplIenGeneratorDao,
        final UserVo pUser) {
        highestIenProcessed = NO_IENS_PROCESSED;
        eplIngredientDao = pEplIngredientDao;
        eplDdiSeverityDao = pEplDdiSeverityDao;
        drugDrugInteractionDao = pDrugDrugInteractionDao;
        eplIenGeneratorDao = pEplIenGeneratorDao;
        user = pUser;
    }
   
    /**
     * Returns the highest IEN processed by this file reader. This IEN may not have been successfully processed: it was
     * just in the file.
     * @return see description
     */
    protected long getHighestIenProcessed() {
        return highestIenProcessed;
    }
    
    private void updateHighestIenProcessed(final long newIen) {
        if (newIen > highestIenProcessed) {
            highestIenProcessed = newIen;
        }
    }

    /**
     * Takes a single line from the DDI migration DAT file and saves it to the database.
     * Original line below:
     * 
     * ABARELIX/AMIODARONE;AMIODARONE;ABARELIX;CRITICAL;YES;170674;;;
     * 6859^ABARELIX/AMIODARONE^AMIODARONE^ABARELIX^CRITICAL^YES^168896^^
     */
    @Override
    protected void processLine(String[] tokens) throws Exception{
        
        //ABARELIX/AMIODARONE;AMIODARONE;ABARELIX;CRITICAL;YES;170674;;;
        EplDrugDrugInteractionDo ddi = new EplDrugDrugInteractionDo();
        // Allowing createdBy to be set by insert().
        ddi.setCreatedDtm(Calendar.getInstance().getTime());
        final long ien = Long.valueOf(tokens[IEN]);
        ddi.setVistaIen(ien);
        ddi.setInteractionName(tokens[INTERACTION_NAME]);
        ddi.setDrugIngredient1(findIngredient(tokens[INGREDIENT1]));
        ddi.setDrugIngredient2(findIngredient(tokens[INGREDIENT2]));
        ddi.setSeverity(findSeverity(tokens[SEVERITY]));
        ddi.setNationallyEntered(stringToBool(tokens[NATIONALLY_ENTERED]));
        ddi.setTotalIndexes(Long.valueOf(tokens[TOTAL_INDEXES]));

        // Update the IEN before validating the data. See method javadoc on getHighestIenProcessed().
        updateHighestIenProcessed(ien);
        
        if (ddi.getDrugIngredient1() == null) {
            throw new DomainException(DomainException.DATA_NOT_FOUND, "DrugIngredient1 - " + tokens[INGREDIENT1]);
        }

        if (ddi.getDrugIngredient2() == null) {
            throw new DomainException(DomainException.DATA_NOT_FOUND, "DrugIngredient2 - " + tokens[INGREDIENT2]);
        }

        if (!StringUtils.isEmpty(tokens[LOCALLY_ENTERED].trim())) {
            ddi.setLocallyEdited(stringToBool(tokens[LOCALLY_ENTERED]));
        }
        
        if (tokens.length == (INACTIVATION_DATE + 1) && !StringUtils.isEmpty(tokens[INACTIVATION_DATE].trim())) {
            Date date = stringToDate(tokens[INACTIVATION_DATE]);
            ddi.setInactivationDate(date);
        }
        getDrugDrugInteractionDao().insert(ddi, user);
    }

    private EplIngredientDo findIngredient(String name) {
        return getEplIngredientDao().findIngredientByName(name);
    }

    private EplDdiSeverityDo findSeverity(String name) {
        return getEplDdiSeverityDao().findByName(name);

    }
    
    @Override
    protected void updateIenGenerator() {
        LOG.info("Inserting IEN generator record with last-used IEN {}", getHighestIenProcessed());
        final EplIenGeneratorDo ienGeneratorRecord = new EplIenGeneratorDo(
            EplDrugDrugInteractionDo.TABLE_NAME, getHighestIenProcessed(), null, null);
        eplIenGeneratorDao.insert(ienGeneratorRecord, user);
    }

    private EplIngredientDao getEplIngredientDao() {
        return eplIngredientDao;
    }

    private EplDdiSeverityDao getEplDdiSeverityDao() {
        return eplDdiSeverityDao;
    }

    private EplDrugDrugInteractionDao getDrugDrugInteractionDao() {
        return drugDrugInteractionDao;
    }

}
