package gov.va.med.pharmacy.peps.service.common.capability;

import java.util.List;

import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.vo.diff.GroupDifference;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;

/**
 * Interface for operations related to synchronizing PPS-N changes with VistA
 * via the NDF update file.
 */
public interface NdfFileSyncCapability {
    /**
     * Replace any pending outgoing differences with reconciled records representing what should be transferred to VistA. 
     * If there had been data in PPS-N and VistA, and it was all removed in PPS-N, the new value of 'DELETED' will be
     * stored on the outgoing difference. Otherwise, the whole list is stored. Since the whole list is sent in each change set,
     * this also removes any previous change sets that have not yet been included in an update file, replacing them with this
     * latest list. This method applies to group list data fields that are synchronized with VistA, specifically only those
     * VO classes which implement the NdfSyncable interface.
     * 
     * @param difference GroupDifference containing the changes that occurred in the group list field
     * @param vistaFileNumber the file number in VistA
     * @param vistaIen The data element's IEN - note, this should not have the sub-field ien on it (i.e. no comma!)
     * @param vistaFieldNumberBase identifies the particular VistA field itself, without specifying whether this
     * is a parent (e.g. ',.01') or child (e.g., ',1'; ',2'; etc.) element
     * @param vistaFieldNumber the vistaFieldNumberBase with the parent or child indicator appended, as in '14,.01'
     * or '14,1', etc.
     * @param userValue The user processing the change
     * @throws ValidationException 
     */
    void processModifiedNdfSyncables(GroupDifference difference, String vistaFileNumber, String vistaIen,
        String vistaFieldNumberBase, String vistaFieldNumber, String userValue);

    /**
     * Accepts an EplNdfOugoingDifferences object and saves the information for sending to VistA later.
     * 
     * @param eplNdfOutgoingDifferences updates to send to VistA
     */
    void save(EplNdfOutgoingDifferences eplNdfOutgoingDifferences);

    /**
     * Removes all pending outgoing differences (that have not been associated to a file yet) for the specified file, multiple
     * field number and vistaIen. All fields with the VistA field number base would be included in the delete. Ex. For Copay
     * Tier, the caller would specify 45. Fields [45,.01], [45,1], and [45,2] would be deleted for the vistaIen specified.
     * 
     * The VistA IEN specified should be the main entity's ien. For example, a Copay Tier is associated to a product. THe Vista
     * IEN on the outgoing differences file is a comma-separated aggregation of the product's IEN and the pseudo IEN of the
     * Copay Tier (Ex. [123456,1], [123456,2],...). The caller should specify just the 123456 in this case.
     * 
     * @param vistaFileNumber VistA File Number
     * @param vistaFieldNumberBase Base of the VistA Field Number of the multiple field. Ex. Copay Tier would be 45.
     * @param vistaIen VistA IEN to which the record is associated.
     */
    void removePendingOutgoingDifferencesForMultipleField(String vistaFileNumber, String vistaFieldNumberBase, String vistaIen);

    /**
     * Find pending multiple records for field and ien. The field number specified should be the base field number of the
     * multiple field, without any comma. This will retrieve what VistA terms a "Multiple" field. For example, for product copay
     * tier, passing "45" will find all records with field number "45,.01" "45,.02".
     *
     * @param vistaFileNumber the vista file number
     * @param vistaFieldNumberBase the vista field number base of the VistA Multiple field
     * @param vistaIen the vista ien
     * @return the list
     */
    List<EplNdfOutgoingDifferences> findPendingRecordsForFileFieldBaseAndIen(String vistaFileNumber,
        String vistaFieldNumberBase, String vistaIen);
}
