

package gov.va.med.pharmacy.peps.service.common.update;


import java.util.Objects;

import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.vo.DispenseUnitVo;
import gov.va.med.pharmacy.peps.common.vo.DosageFormVo;
import gov.va.med.pharmacy.peps.common.vo.DrugClassVo;
import gov.va.med.pharmacy.peps.common.vo.DrugDrugInteractionVo;
import gov.va.med.pharmacy.peps.common.vo.DrugUnitVo;
import gov.va.med.pharmacy.peps.common.vo.GenericNameVo;
import gov.va.med.pharmacy.peps.common.vo.IngredientVo;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.ManufacturerVo;
import gov.va.med.pharmacy.peps.common.vo.NdcVo;
import gov.va.med.pharmacy.peps.common.vo.PackageSizeVo;
import gov.va.med.pharmacy.peps.common.vo.PackageTypeVo;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.domain.common.capability.DrugSeverityDomainCapability;
import gov.va.med.pharmacy.peps.service.common.capability.NdfFileSyncCapability;
import gov.va.med.pharmacy.peps.service.common.update.impl.DosageFormNdfFileSyncProcessor;
import gov.va.med.pharmacy.peps.service.common.update.impl.DrugDrugInteractionNdfFileSyncProcessor;
import gov.va.med.pharmacy.peps.service.common.update.impl.DrugIngredientsNdfFileSyncProcessor;
import gov.va.med.pharmacy.peps.service.common.update.impl.DrugUnitNdfFileSyncProcessor;
import gov.va.med.pharmacy.peps.service.common.update.impl.GenericObjectProcessor;
import gov.va.med.pharmacy.peps.service.common.update.impl.ManufacturerNdfFileSyncProcessor;
import gov.va.med.pharmacy.peps.service.common.update.impl.NdcNdfFileSyncProcessor;
import gov.va.med.pharmacy.peps.service.common.update.impl.PackageSizeNdfFileSyncProcessor;
import gov.va.med.pharmacy.peps.service.common.update.impl.PackageTypeNdfFileSyncProcessor;
import gov.va.med.pharmacy.peps.service.common.update.impl.VADispenseUnitNdfFileSyncProcessor;
import gov.va.med.pharmacy.peps.service.common.update.impl.VADrugClassNdfFileSyncProcessor;
import gov.va.med.pharmacy.peps.service.common.update.impl.VaGenericNameNdfFileSyncProcessor;
import gov.va.med.pharmacy.peps.service.common.update.impl.VaProductNdfFileSyncProcessor;


/**
 * A factory for creating NdfFileSyncProcessor objects.
 */
public class NdfFileSyncProcessorFactory {

  private final NdfFileSyncCapability ndfFileSyncCapability;
  private final DrugSeverityDomainCapability drugSeverityDomainCapability;

  /**
   * Constructs an instance with the appropriate dependencies.
   * @param pNdfFileSyncCapability the {@link NdfFileSyncCapability} to use. Must not be null.
   * @param pDrugSeverityDomainCapability the {@link DrugSeverityDomainCapability} to use. Must not be null.
   */
  @Autowired
  public NdfFileSyncProcessorFactory(
      final NdfFileSyncCapability pNdfFileSyncCapability,
      final DrugSeverityDomainCapability pDrugSeverityDomainCapability) {
    ndfFileSyncCapability = Objects.requireNonNull(pNdfFileSyncCapability);
    drugSeverityDomainCapability = Objects.requireNonNull(pDrugSeverityDomainCapability);
  }
      
    /**
     * Simply constructs the processor without injecting dependencies.
     */
    private NdfFileSyncProcessor constructProcessor(Object clazz) {
        NdfFileSyncProcessor processor = null;

        if (clazz instanceof PackageTypeVo) {
            processor = new PackageTypeNdfFileSyncProcessor(ndfFileSyncCapability);
        } else if (clazz instanceof PackageSizeVo) {
            processor = new PackageSizeNdfFileSyncProcessor(ndfFileSyncCapability);            
        }
        else if (clazz instanceof ManufacturerVo) {
            processor = new ManufacturerNdfFileSyncProcessor(ndfFileSyncCapability);
        }
        else if (clazz instanceof DispenseUnitVo) {
            processor = new VADispenseUnitNdfFileSyncProcessor(ndfFileSyncCapability);
        }
        else if (clazz instanceof DosageFormVo) {
            processor = new DosageFormNdfFileSyncProcessor(ndfFileSyncCapability);
        }
        else if (clazz instanceof GenericNameVo) {
            processor = new VaGenericNameNdfFileSyncProcessor(ndfFileSyncCapability);
        }
        else if (clazz instanceof DrugUnitVo) {
            processor = new DrugUnitNdfFileSyncProcessor(ndfFileSyncCapability);
        }
        else if (clazz instanceof NdcVo){
              processor = new NdcNdfFileSyncProcessor(ndfFileSyncCapability);
        }
        else if (clazz instanceof IngredientVo) {
            processor = new DrugIngredientsNdfFileSyncProcessor(ndfFileSyncCapability);
        }
        else if (clazz instanceof DrugClassVo) {
            processor = new VADrugClassNdfFileSyncProcessor(ndfFileSyncCapability);
        }
        else if (clazz instanceof DrugDrugInteractionVo) {            
            processor = new DrugDrugInteractionNdfFileSyncProcessor(ndfFileSyncCapability, drugSeverityDomainCapability);
        }
        
        else if (clazz instanceof ProductVo) {
            processor = new VaProductNdfFileSyncProcessor(ndfFileSyncCapability);
        }
        
        else {
            processor = new GenericObjectProcessor();
        }

        return processor;
    }
    
    /**
     * Given the incoming managed item, the appropriate NdfFileSyncProcessor implementation is determined
     * and returned.
     *
     * @param managedItem the managed item for which the caller wants a processor
     * @return an instance of an appropriate NdfFileSyncProcessor
     */
    public NdfFileSyncProcessor getProcessor(final ManagedItemVo managedItem) {
        NdfFileSyncProcessor fileProcessor = constructProcessor(managedItem);
        return fileProcessor;
    }

}
