package gov.va.med.pharmacy.peps.service.common.update.impl;

import static gov.va.med.pharmacy.peps.common.vista.VistaDosageFormFile.DISPENSE_UNIT_PER_DOSE_FIELD;
import static gov.va.med.pharmacy.peps.common.vista.VistaDosageFormFile.DISPENSE_UNIT_PER_DOSE_FIELD_BASE;
import static gov.va.med.pharmacy.peps.common.vista.VistaDosageFormFile.EXCLUDE_FROM_DOSE_CHECK_FIELD;
import static gov.va.med.pharmacy.peps.common.vista.VistaDosageFormFile.FILE_NUMBER;
import static gov.va.med.pharmacy.peps.common.vista.VistaDosageFormFile.INACTIVATION_DATE_FIELD;
import static gov.va.med.pharmacy.peps.common.vista.VistaDosageFormFile.NAME_FIELD;
import static gov.va.med.pharmacy.peps.common.vista.VistaDosageFormFile.UNITS_FIELD;
import static gov.va.med.pharmacy.peps.common.vista.VistaDosageFormFile.UNITS_FIELD_BASE;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.GregorianCalendar;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringUtils;

import gov.va.med.pharmacy.peps.common.utility.MumpsConverter;
import gov.va.med.pharmacy.peps.common.vista.VistaFile;
import gov.va.med.pharmacy.peps.common.vo.DosageFormVo;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.NdfSyncable;
import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;
import gov.va.med.pharmacy.peps.common.vo.SyncProcessorElement;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.common.vo.diff.GroupDifference;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.external.common.preencapsulation.utility.update.item.DosageFormConverter;
import gov.va.med.pharmacy.peps.external.common.vo.outbound.common.ItemAction;
import gov.va.med.pharmacy.peps.service.common.capability.NdfFileSyncCapability;
import gov.va.med.pharmacy.peps.service.common.update.NdfFileSyncProcessor;

/**
 * DosageFormNdfFileSyncProcessor generates outgoing difference records for the NDF Update File for Dosage Form changes.
 */
public class DosageFormNdfFileSyncProcessor extends AbstractSimpleNdfFileSyncProcessor implements NdfFileSyncProcessor{
    
    /** FieldKeys of the fields that, if modified, would be processed by the modification logic of this processor. */
    public static final Set<FieldKey> FIELDS = Collections.unmodifiableSet(new LinkedHashSet<FieldKey>(Arrays
        .asList(
            FieldKey.DOSAGE_FORM_NAME, 
            FieldKey.LOCAL_MED_ROUTES, 
            FieldKey.LOCAL_MEDICATION_ROUTE,
            FieldKey.VERB, 
            FieldKey.OTHER_LANGUAGE_VERB, 
            FieldKey.PREPOSITION,
            FieldKey.OTHER_LANGUAGE_PREPOSITION, 
            FieldKey.DF_NOUNS, 
            FieldKey.INACTIVATION_DATE,
            FieldKey.DF_UNITS, 
            FieldKey.DF_DISPENSE_UNITS_PER_DOSE, 
            FieldKey.EXCLUDE_FROM_DOSAGE_CHKS,
            FieldKey.CONJUNCTION, 
            FieldKey.ITEM_STATUS
            )));
    
    private static final String VISTA_YES = "1";
    private static final String VISTA_NO = "0";
    
    private String userName;
    private String ien;
   
    /**
     * Instantiates a new DosageFormNdfFileSyncProcessor.
     *
     * @param pNdfFileSyncCapability the ndf file sync capability
     * @see AbstractSimpleNdfFileSyncProcessor#AbstractSimpleNdfFileSyncProcessor(Set, String, String)
     */
    public DosageFormNdfFileSyncProcessor(NdfFileSyncCapability pNdfFileSyncCapability) {
        super(FIELDS, FILE_NUMBER, NAME_FIELD, pNdfFileSyncCapability);
    }

    @Override
    public void processNew(ManagedItemVo managedItem, UserVo user) {
        
        DosageFormVo dosageFormVo = (DosageFormVo) managedItem;
        
        if (RequestItemStatus.APPROVED.equals(dosageFormVo.getRequestItemStatus()) && dosageFormVo.getValue() != null) {
            
            // IEN
            setIen(dosageFormVo.getDosageFormIen());
            
            //User
            setUserName(user.getUsername());
            
            
            //NAME_FIELD
            insertNewElement(getIen(),dosageFormVo.getDosageFormName(), getUserName() );
        
            insertNewChildElement(
                getIen(), 
                dosageFormVo.getExcludeFromDosageChks() != null 
                    ? convertYesNo(dosageFormVo.getExcludeFromDosageChks().toString()) 
                    : VistaFile.BLANK, getUserName(),
                EXCLUDE_FROM_DOSE_CHECK_FIELD);
            
            List<SyncProcessorElement> elements = new ArrayList<>();
            
            //process Dispense Unit if present
            if (dosageFormVo.getDfDispenseUnitsPerDose().size() > 0) {
                elements.addAll(NdfSyncable.syncProcessorElementsForNdfSyncables(
                    dosageFormVo.getDfDispenseUnitsPerDose(),
                    getIen(),
                    getUserName()));
            }
            
            //process Units if present
            if (dosageFormVo.getDfUnits().size() > 0){
                elements.addAll(NdfSyncable.syncProcessorElementsForNdfSyncables(
                    dosageFormVo.getDfUnits(), 
                    getIen(),
                    getUserName()));
            }
            
            for(SyncProcessorElement element : elements) {
                String requiredNewValue = 
                    (StringUtils.isNotBlank(element.getNewValue())) 
                    ? element.getNewValue() 
                        : VistaFile.BLANK;
                
                insertNewChildElement(
                    element.getVistaIen(), 
                    requiredNewValue, 
                    element.getUserName(), 
                    element.getFieldId(),
                    EplNdfOutgoingDifferences.ADD_ACTION); 
            }
        }
    }
     
    @Override
    public void processModified(ManagedItemVo managedItem, UserVo user, Collection<Difference> differences) {
        Map<FieldKey, Difference> setDifference = Difference.toDifferenceMap(differences);
        boolean hasDifference = DosageFormConverter.hasNewOrModifiedFields(super.getFields(), setDifference, ItemAction.MODIFY);

        DosageFormVo dosageFormVo = (DosageFormVo) managedItem;

        if (hasDifference) {
            if (RequestItemStatus.APPROVED.equals(dosageFormVo.getRequestItemStatus()) && dosageFormVo.getValue() != null) {

                setIen(dosageFormVo.getDosageFormIen());

                setUserName(user.getUsername());

                // Exclude From Dosage Checks
                if (DosageFormConverter.fieldWasModified(FieldKey.EXCLUDE_FROM_DOSAGE_CHKS, differences)) {
                    insertModifiedElement(
                        getIen(),
                        dosageFormVo.getExcludeFromDosageChks() != null
                            ? convertYesNo(dosageFormVo.getExcludeFromDosageChks().toString()) 
                            : VistaFile.BLANK,
                        getUserName(), 
                        EXCLUDE_FROM_DOSE_CHECK_FIELD, 
                        null);
                }
                
                // Inactivation date
                if (DosageFormConverter.fieldWasModified(FieldKey.ITEM_STATUS, differences)) {
                    processActivationDate(dosageFormVo);
                }
                
                // Dispense Units
                if (DosageFormConverter.fieldWasModified(FieldKey.DF_DISPENSE_UNITS_PER_DOSE, differences)) {
                    getNdfFileSyncCapability().processModifiedNdfSyncables(
                        (GroupDifference) setDifference.get(FieldKey.DF_DISPENSE_UNITS_PER_DOSE), 
                        FILE_NUMBER, 
                        getIen(), 
                        DISPENSE_UNIT_PER_DOSE_FIELD_BASE, 
                        DISPENSE_UNIT_PER_DOSE_FIELD, 
                        getUserName());
                }

                // Units
                if (DosageFormConverter.fieldWasModified(FieldKey.DF_UNITS, differences)) {
                    getNdfFileSyncCapability().processModifiedNdfSyncables(
                        (GroupDifference) setDifference.get(FieldKey.DF_UNITS), 
                        FILE_NUMBER, 
                        getIen(), 
                        UNITS_FIELD_BASE, 
                        UNITS_FIELD, 
                        getUserName());
                }
            }
        }
    }

    /**
     * Process activation date.
     *
     * @param dosageFormVo the dosage form vo
     */
    private void processActivationDate(DosageFormVo dosageFormVo) {
        if(dosageFormVo.getInactivationDate() != null) {
            GregorianCalendar c = new GregorianCalendar();
            c.setTime(dosageFormVo.getInactivationDate());
            insertModifiedElement(getIen(), MumpsConverter.convertDate(c.getTime()), getUserName(), INACTIVATION_DATE_FIELD,
                null);
        } else {
            insertModifiedElement(getIen(), VistaFile.BLANK, getUserName(), INACTIVATION_DATE_FIELD, null);
        }
    }
    
    /**
     * Convert YES to 1 and NO (or all other non-YES values) to 0 for VistA.  The method is case-insensitive.
     *  
     * @param value String containing YES/NO to convert to 1/0
     * @return String converted value 1 or 0
     */
    public static String convertYesNo(String value){
        return ("YES".equalsIgnoreCase(value) ? VISTA_YES : VISTA_NO);
    }
    
    /**
     * Get the name of the PPS-N user to list as the creator/modifier on the outgoing difference.
     * @return the user name
     */
    public String getUserName() {
        return userName;
    }

    /**
     * Set the name of the PPS-N user to list as the creator/modifier on the outgoing difference.
     *
     * @param pUserName the new user name
     */
    public void setUserName(String pUserName) {
        this.userName = pUserName;
    }
    
    /**
     * Get the Dosage Form IEN.
     * @return the ien
     */
    public String getIen() {
        return ien;
    }
    
    /**
     * Set the Dosage Form IEN.
     * @param ien the ien to set
     */
    public void setIen(String ien) {
        this.ien = ien;
    }
}   
