package gov.va.med.pharmacy.peps.service.common.update.impl;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;

import gov.va.med.pharmacy.peps.common.utility.MumpsConverter;
import gov.va.med.pharmacy.peps.common.utility.PPSConstants;
import gov.va.med.pharmacy.peps.common.vista.VistaFile;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.NdcVo;
import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.external.common.preencapsulation.utility.update.item.NdcConverter;
import gov.va.med.pharmacy.peps.service.common.capability.NdfFileSyncCapability;
import gov.va.med.pharmacy.peps.service.common.update.NdfFileSyncProcessor;

/**
 * This class implements methods which populate the new NDF outgoing difference table for vista file 50.67.
 */
public class NdcNdfFileSyncProcessor extends AbstractSimpleNdfFileSyncProcessor implements NdfFileSyncProcessor{
    
    /** Fields that this processor will process. */
    public static final Set<FieldKey> FIELDS = Collections.unmodifiableSet(new LinkedHashSet<FieldKey>(Arrays
        .asList(
            FieldKey.NDC, 
            FieldKey.UPC_UPN, 
            FieldKey.MANUFACTURER, 
            FieldKey.TRADE_NAME, 
            FieldKey.PRODUCT,
            FieldKey.INACTIVATION_DATE, 
            FieldKey.PACKAGE_SIZE, 
            FieldKey.PACKAGE_TYPE, 
            FieldKey.OTC_RX_INDICATOR,
            FieldKey.ITEM_STATUS
            )));
    
    private static final String FILE_NUMBER = "50.67";    
    
    /** FIELDS */

    private static final org.apache.logging.log4j.Logger LOG = org.apache.logging.log4j.LogManager
        .getLogger(NdcNdfFileSyncProcessor.class);
    private static final String NDF_NDC_IEN = ".01";
    private static final String NDC = "1";
    private static final String UPN = "2";
    private static final String MANUFACTURER_IEN= "3";
    private static final String TRADE_NAME = "4";
    private static final String VA_PRODUCT_IEN = "5";
    private static final String INACTIVATION_DATE = "7";
    private static final String PACKAGE_SIZE = "8";
    private static final String PACKAGE_TYPE_IEN = "9";
    private static final String OTX_RX_INDICATOR = "10";
    private static final String ROUTE_OF_ADMINISTRATION = "6,.01";
    
    private String userName;
    private String ien;
  
    /**
     * Instantiates a new NdcNdfFileSyncProcessor.
     *
     * @param pNdfFileSyncCapability the ndf file sync capability
     * @see AbstractSimpleNdfFileSyncProcessor#AbstractSimpleNdfFileSyncProcessor(Set, String, String)
     */
    public NdcNdfFileSyncProcessor(NdfFileSyncCapability pNdfFileSyncCapability) {
        super(FIELDS, FILE_NUMBER, NDF_NDC_IEN, pNdfFileSyncCapability); 
    }
    
    @Override
    public void processNew(ManagedItemVo managedItem, UserVo user) {
      
      LOG.debug("Processing new NDC.");
        
        NdcVo ndcvo = (NdcVo) managedItem;
        
        if (RequestItemStatus.APPROVED.equals(ndcvo.getRequestItemStatus()) && ndcvo.getValue() != null) {
            
            // IEN
            setIen(ndcvo.getNdcIen());
           
            setUserName(user.getUsername());
            
            //This field is no longer used default ien to 1 and value of N/A on add.
            insertNewChildElement(getIen()+ ",1","N/A",getUserName(),ROUTE_OF_ADMINISTRATION);
                  
            insertNewChildElement(getIen(), ndcvo.getNdcIen(),getUserName(),NDF_NDC_IEN);
            
            //vista requires zero to precede the ncd value 
            insertNewChildElement(getIen(), "0" + ndcvo.getNdc().replaceAll("-",""), getUserName(),NDC );
            
            if(ndcvo.getUpcUpn() != null)//not provided to vista if null
                insertNewChildElement(getIen(), ndcvo.getUpcUpn(), getUserName(),UPN);
            
            insertNewChildElement(getIen(), ndcvo.getManufacturer().getManufacturerIen(),getUserName(),MANUFACTURER_IEN);
             
            insertNewChildElement(getIen(), ndcvo.getTradeName(),getUserName(),TRADE_NAME);
            
            insertNewChildElement(getIen(), ndcvo.getProduct().getNdfProductIen().toString(),getUserName(),VA_PRODUCT_IEN);
            
            insertNewChildElement(getIen(), ndcvo.getPackageSizeVo().getNdfPackageSizeIen(), getUserName(),PACKAGE_SIZE);
            
            insertNewChildElement(getIen(), ndcvo.getPackageType().getPackagetypeIen(),getUserName(),PACKAGE_TYPE_IEN);
            
            String otxField = VistaFile.BLANK;
            if (PPSConstants.OVER_THE_COUNTER.equalsIgnoreCase(ndcvo.getOtcRxIndicator().getValue())) { // Over the Counter
                otxField ="O";
            } else if (PPSConstants.PRESCRIPTION.equalsIgnoreCase(ndcvo.getOtcRxIndicator().getValue())) { // Prescription
                otxField = "R";
            }

            insertNewChildElement(getIen(), otxField, getUserName(),OTX_RX_INDICATOR);
        }
    }

    @Override
    public void processModified(ManagedItemVo managedItem, UserVo user, Collection<Difference> differences) {
      
      LOG.debug("Processing modified NDC.");

       NdcVo ndcvo = (NdcVo) managedItem;
        
        if (RequestItemStatus.APPROVED.equals(ndcvo.getRequestItemStatus()) && ndcvo.getValue() != null) {
            
            // IEN
            setIen(ndcvo.getNdcIen());
                        
            setUserName(user.getUsername());
            
            if(NdcConverter.fieldWasModified(FieldKey.UPC_UPN, differences)){
                if(ndcvo.getUpcUpn() != null){ 
                    insertModifiedElement(getIen(), ndcvo.getUpcUpn(), getUserName(),UPN, null);
                }else{
                    insertModifiedElement(getIen(), VistaFile.BLANK, getUserName(),UPN, null);
                }
            }
            
            if(ndcvo.getManufacturer() != null
                && NdcConverter.fieldWasModified(FieldKey.MANUFACTURER, differences)){
                insertModifiedElement(getIen(), ndcvo.getManufacturer().getManufacturerIen(), getUserName(), MANUFACTURER_IEN,
                    null);
            } 
            
            if(ndcvo.getTradeName() != null && NdcConverter.fieldWasModified(FieldKey.TRADE_NAME, differences)){
                insertModifiedElement(getIen(), ndcvo.getTradeName(), getUserName(),TRADE_NAME,null);
            }
            
            if(ndcvo.getProduct() != null
                && NdcConverter.fieldWasModified(FieldKey.PRODUCT, differences)){
                insertModifiedElement(getIen(),ndcvo.getProduct().getNdfProductIen().toString(), getUserName(), 
                    VA_PRODUCT_IEN,null);
            }
            
            if(NdcConverter.fieldWasModified(FieldKey.ITEM_STATUS, differences)){
                if(ndcvo.getInactivationDate() != null){
                    insertModifiedElement(getIen(),MumpsConverter.convertDate(ndcvo.getInactivationDate()), 
                        getUserName(),INACTIVATION_DATE,null);
                }else{
                    //TODO test reactivate
                    insertModifiedElement(getIen(),VistaFile.BLANK, getUserName(),INACTIVATION_DATE,null);
                }
            }
            
            if(ndcvo.getPackageSize() != null
                && NdcConverter.fieldWasModified(FieldKey.PACKAGE_SIZE, differences)){
                insertModifiedElement(getIen(), ndcvo.getPackageSizeVo().getNdfPackageSizeIen(), getUserName(), PACKAGE_SIZE,
                    null);
            }
            
            if(ndcvo.getPackageType() != null 
                && NdcConverter.fieldWasModified(FieldKey.PACKAGE_TYPE, differences)){
                insertModifiedElement(getIen(),ndcvo.getPackageType().getPackagetypeIen(),getUserName(),PACKAGE_TYPE_IEN,null);
            }
            
            if(ndcvo.getOtcRxIndicator() !=null 
                && NdcConverter.fieldWasModified(FieldKey.OTC_RX_INDICATOR, differences)){
                String otxField = VistaFile.BLANK;
                if (PPSConstants.OVER_THE_COUNTER.equalsIgnoreCase(ndcvo.getOtcRxIndicator().getValue())) { // Over the Counter
                    otxField ="O";
                } else if (PPSConstants.PRESCRIPTION.equalsIgnoreCase(ndcvo.getOtcRxIndicator().getValue())) { // Prescription
                    otxField = "R";
                }
                insertModifiedElement(getIen(),otxField, getUserName(),OTX_RX_INDICATOR,null);
            }   
        }
    }
    
    /**
     * Get the name of the PPS-N user to list as the creator/modifier on the outgoing difference.
     *
     * @return the user
     */
    public String getUserName() {
        return userName;
    }

    /**
     * Set the name of the PPS-N user to list as the creator/modifier on the outgoing difference.
     *
     * @param pUser the new user name
     */
    public void setUserName(String pUser) {
        this.userName = pUser;
    }
    
    /**
     * Gets the NDC ien.
     *
     * @return the ien
     */
    public String getIen() {
        return ien;
    }
    
    /**
     * Sets the NDC ien.
     *
     * @param ien the ien to set
     */
    public void setIen(String ien) {
        this.ien = ien;
    }
}   
