package gov.va.med.pharmacy.peps.domain.common.dao.impl.test;

import gov.va.med.pharmacy.peps.common.utility.MumpsConverter;
import gov.va.med.pharmacy.peps.common.utility.VistaFileNumber;
import gov.va.med.pharmacy.peps.common.vista.VistaProductFile;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplNdfUpdateFileDao;
import gov.va.med.pharmacy.peps.domain.common.dao.impl.EplNdfOutgoingDifferencesDaoImpl;
import gov.va.med.pharmacy.peps.domain.common.dao.impl.EplStatusDaoImpl;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfUpdateFileDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplStatusDo;
import gov.va.med.pharmacy.peps.domain.common.model.dto.ProductChangeDto;
import gov.va.med.pharmacy.peps.domain.common.utility.test.AbstractDomainIT;
import gov.va.med.pharmacy.peps.domain.common.utility.test.EplNdfOutgoingDifferencesGenerator;
import gov.va.med.pharmacy.peps.service.common.update.CopayTierChange;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Optional;

import org.apache.commons.collections4.ListUtils;
import org.apache.logging.log4j.Logger;
import org.hamcrest.core.IsNull;

import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

/**
 * Automated tests for {@link EplNdfOutgoingDifferencesDaoImpl}. These tests require a database.
 */
public class EplNdfOutgoingDifferencesDaoImplIT extends AbstractDomainIT {

  /**
   * The NDF IEN of "RIBOFLAVIN 100MG TAB". Some tests assume this product exists and is active.
   */
  private static final String RIBOFLAVIN_100MG_IEN = "7775";

  private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(
      EplNdfOutgoingDifferencesDaoImplIT.class);

    @Autowired
    private EplNdfOutgoingDifferencesDaoImpl eplNdfOutgoingDifferencesDao;
    
    @Autowired
    private EplNdfUpdateFileDao eplNdfUpdateFileDao;    
    
    @Autowired
    private EplStatusDaoImpl eplStatusDaoImpl;
    
    
  /**
   * Tests that {@link EplNdfOutgoingDifferencesDaoImpl#findByFileAndField(String, String, Date)} returns the expected data.
   */
  @Test
  @Transactional
  public void testFindByFileAndField() {

    /* Test Parameters */

    final Date testDate = new Date();
    final String vistaFileNumber = VistaFileNumber.FIFTY_6.getFileNumber();
    // This field should not exist to ensure no existing data interferes with this test.
    final String vistaFieldNumber = "49";

    /* Setup */

    // Verify assumption
    assertThat("Should return empty list at beginning of test.",
        eplNdfOutgoingDifferencesDao.findByFileAndField(vistaFileNumber, vistaFieldNumber, testDate),
        equalTo(Collections.emptyList()));

    final EplNdfOutgoingDifferences diff = new EplNdfOutgoingDifferences();
    diff.setActionType(EplNdfOutgoingDifferences.MODIFY_ACTION);
    diff.setCreatedBy("Automated Test");
    diff.setCreatedDtm(testDate);
    diff.setNewValue("NewValue");
    diff.setVistaFileNumber(vistaFileNumber);
    diff.setVistaFieldNumber(vistaFieldNumber);
    diff.setVistaIen(RIBOFLAVIN_100MG_IEN);
    diff.setNdcUpdateFileFk(null);

    final Object[] expectedColumns = new Object[] { diff.getVistaIen(), diff.getNewValue() };

    eplNdfOutgoingDifferencesDao.save(diff);

    /* Behavior */
    final List<Object[]> list = eplNdfOutgoingDifferencesDao.findByFileAndField(vistaFileNumber, vistaFieldNumber, testDate);

    /* Verification */
    // We can't simply compare the actual list to an expected list because equals() compares arrays by reference
    // equality, not value equality.
    assertThat(list.size(), equalTo(1));
    assertThat(list.get(0), equalTo(expectedColumns));  // Note: equalTo() has special support for arrays
  }

  /**
   * Tests that {@link EplNdfOutgoingDifferencesDaoImpl#findPendingRecordsForFileFieldAndIen(String, String, String)} returns the
   * expected data.
   */
  @Test
  @Transactional
  public void testFindPendingRecordsForFieldAndIen() {

    /* Test Parameters */
    final String targetFileNumber = "123.456";
    final String targetFieldNumber = "867,5309";
    final String targetIen = RIBOFLAVIN_100MG_IEN;
    final String extraFileNumber = "654.321";
    final String extraFieldNumber = "555,1212";
    final String extraIen = "7776";

    // Verify pre-conditions
    assertThat("Should return empty list at beginning of test.",
        eplNdfOutgoingDifferencesDao.findPendingRecordsForFileFieldAndIen(targetFileNumber, targetFieldNumber, targetIen),
        equalTo(Collections.emptyList()));

    EplNdfUpdateFileDo ndcUpdateFileFk = createNewUpdateFile();
    //these records should be returned by find method
    createAndSaveDiff(targetFileNumber, targetFieldNumber, targetIen, EplNdfOutgoingDifferences.MODIFY_ACTION);
    createAndSaveDiff(targetFileNumber, targetFieldNumber, targetIen, EplNdfOutgoingDifferences.ADD_ACTION);
    //extra distraction records that match some, but not all of the criteria and should not be returned by find method
    createAndSaveDiff(targetFileNumber, targetFieldNumber, extraIen, EplNdfOutgoingDifferences.MODIFY_ACTION);
    createAndSaveDiff(targetFileNumber, extraFieldNumber, targetIen, EplNdfOutgoingDifferences.MODIFY_ACTION);
    createAndSaveDiff(extraFileNumber, targetFieldNumber, targetIen, EplNdfOutgoingDifferences.MODIFY_ACTION);
    createAndSaveDiff(targetFileNumber, targetFieldNumber, targetIen, EplNdfOutgoingDifferences.ADD_ACTION, ndcUpdateFileFk);

    /* Behavior */
    final List<EplNdfOutgoingDifferences> list = eplNdfOutgoingDifferencesDao.findPendingRecordsForFileFieldAndIen(
        targetFileNumber, targetFieldNumber, targetIen);

    /* Verification */
    // We can't simply compare the actual list to an expected list because equals() compares arrays by reference
    // equality, not value equality.
    assertThat(list.size(), equalTo(2));
    for (EplNdfOutgoingDifferences diff : list) {
      assertThat(diff.getVistaFileNumber(), equalTo(targetFileNumber));
      assertThat(diff.getVistaFieldNumber(), equalTo(targetFieldNumber));
      assertThat(diff.getVistaIen(), equalTo(targetIen));
    }
  }

  /**
   * Create and Persist an EplNdfOutgoingDifferences record with the specified values, other fields will be set by the method. 
   *
   * @param fileNumber the file number
   * @param fieldNumber the field number
   * @param ien the ien
   * @param action the action
   * @param ndcUpdateFileFk the ndc update file fk
   * @return the epl ndf outgoing differences
   */
  private EplNdfOutgoingDifferences createAndSaveDiff(final String fileNumber, final String fieldNumber, final String ien, 
      final String action, EplNdfUpdateFileDo ndcUpdateFileFk) {
    EplNdfOutgoingDifferences diff = EplNdfOutgoingDifferencesGenerator.createDiff(fileNumber, fieldNumber, ien, action,
        ndcUpdateFileFk);
    return eplNdfOutgoingDifferencesDao.save(diff);
  }  
  
  /**
   * Create and Persist an EplNdfOutgoingDifferences record with the specified values, other fields will be set by the method. 
   *
   * @param fileNumber the file number
   * @param fieldNumber the field number
   * @param ien the ien
   * @param action the action
   */
  private EplNdfOutgoingDifferences createAndSaveDiff(final String fileNumber, final String fieldNumber, 
      final String ien, final String action) {
    return createAndSaveDiff(fileNumber, fieldNumber, ien, action, null);
  }    
  /**
   * Simply verifies that {@link EplNdfOutgoingDifferencesDaoImpl#findDrugClassChanges(String, String, Date)} returns a list. In
   * the future this test should be expanded to verify the returned content.
   */
  @Test
  @Transactional(readOnly = true)
  public void testFindDrugClassChanges() {
    final List<Object[]> list =
        eplNdfOutgoingDifferencesDao.findDrugClassChanges(VistaFileNumber.FIFTY_68.getFileNumber(), "15", new Date());
    assertNotNull(list);
  }

  /**
   * Simply verifies that {@link EplNdfOutgoingDifferencesDaoImpl#findInactiveGenericName(String, String, Date)} returns a list.
   * In the future this test should be expanded to verify the returned content.
   */
  @Test
  @Transactional(readOnly = true)
  public void testFindInactiveGenericName() {
    final List<String> list =
        eplNdfOutgoingDifferencesDao.findInactiveGenericName(VistaFileNumber.FIFTY_6.getFileNumber(), "1", new Date());
    assertNotNull(list);
  }

  /**
   * Simply verifies that {@link EplNdfOutgoingDifferencesDaoImpl#findInactivatedProductsForLocal(String, Date)} returns a list.
   * In the future this test should be expanded to verify the returned content.
   */
  @Test
  @Transactional(readOnly = true)
  public void testFindInactivatedProductsForLocal() {
    final List<Object[]> list =
        eplNdfOutgoingDifferencesDao.findInactivatedProductsForLocal(VistaFileNumber.FIFTY_68.getFileNumber(), new Date());
    assertNotNull(list);
  }
  
  @Test
  @Transactional(readOnly = true)
  public void testFindInactivatedProducts() {
    
    /* Setup */
    
    // Save off the initial list because the database may already have inactivated products.
    final List<ProductChangeDto> initialList = eplNdfOutgoingDifferencesDao.findInactivatedProducts(new Date());
    LOG.debug("Initial list was: {}.", initialList);
    
    // Create the outgoing diff.
    final EplNdfOutgoingDifferences inactivationDiff = EplNdfOutgoingDifferencesGenerator.createDiff(
        VistaProductFile.FILE_NUMBER, VistaProductFile.STATUS_FIELD_NUMBER, RIBOFLAVIN_100MG_IEN,
        EplNdfOutgoingDifferences.MODIFY_ACTION);
    // The difference must be from no inactivation date to an inactivation date.
    inactivationDiff.setOldValue(null);
    inactivationDiff.setNewValue("3170201");
    eplNdfOutgoingDifferencesDao.save(inactivationDiff);
    
    /* Behavior and verification */
    final List<ProductChangeDto> listAfterSavingDiff = eplNdfOutgoingDifferencesDao.findInactivatedProducts(new Date());
    LOG.debug("List after saving diff was: {}.", listAfterSavingDiff);
    assertThat(listAfterSavingDiff.size(), equalTo(initialList.size() + 1));
  }
    
    /**
     * Tests only that {@link EplNdfOutgoingDifferencesDaoImpl#retrieveDifferencesForOutputFile(Date)
     * } returns a non-null list. This test is really more for manual than automated execution, but
     * running it as part of the automated suite has some value.
     */
    @Test
    @Transactional(readOnly=true)
    public void testRetrieveDifferencesForOutputFile() {
      final List<EplNdfOutgoingDifferences> diffs =
          eplNdfOutgoingDifferencesDao.retrieveDifferencesForOutputFile(new Date());
      assertNotNull(diffs);
    }

    /**
     * Simply verifies that {@link EplNdfOutgoingDifferencesDaoImpl#findCMOPIdChanges(Date)} returns a list. In
     * the future this test should be expanded to verify the returned content.
     */
    @Test
    @Transactional(readOnly = true)
    public void testFindCMOPIdChanges() {
      final List<Object[]> list =
          eplNdfOutgoingDifferencesDao.findCMOPIdChanges(new Date());
      assertNotNull(list);
    }    
    
    /**
     * Simply verifies that {@link EplNdfOutgoingDifferencesDaoImpl#findInactivatedCMOPProducts(Date)} returns a list. In
     * the future this test should be expanded to verify the returned content.
     */
    @Test
    @Transactional(readOnly = true)
    public void testFindInactivatedCMOPProducts() {
      final List<Object[]> list =
          eplNdfOutgoingDifferencesDao.findInactivatedCMOPProducts(new Date());
      assertNotNull(list);
    }
    
    /**
     * Simply verifies that {@link EplNdfOutgoingDifferencesDaoImpl#genNameAllChange(Date)} returns a list. In
     * the future this test should be expanded to verify the returned content.
     */
    @Test
    @Transactional(readOnly = true)
    public void testGenNameAllChange() {
        final List<ProductChangeDto> list = eplNdfOutgoingDifferencesDao.genNameAllChange(new Date());
        assertNotNull(list);
    }
    
    /**
     * Simply verifies that {@link EplNdfOutgoingDifferencesDaoImpl#drugUnitsChange(Date)} returns a list. In
     * the future this test should be expanded to verify the returned content.
     */
    @Test
    @Transactional(readOnly = true)
    public void testDrugUnitsChange() {
        final List<ProductChangeDto> list = eplNdfOutgoingDifferencesDao.drugUnitsChange(new Date());
        assertNotNull(list);
    }
    
    /**
     * Simply verifies that {@link EplNdfOutgoingDifferencesDaoImpl#dispenseUnitsChange(Date)} returns a list. In
     * the future this test should be expanded to verify the returned content.
     */
    @Test
    @Transactional(readOnly = true)
    public void testDispenseUnitsChange() {
        final List<ProductChangeDto> list = eplNdfOutgoingDifferencesDao.dispenseUnitsChange(new Date());
        assertNotNull(list);
    }

    /**
     * Simply verifies that {@link EplNdfOutgoingDifferencesDaoImpl#dosageFormChange(Date)} returns a list. In
     * the future this test should be expanded to verify the returned content.
     */
    @Test
    @Transactional(readOnly = true)
    public void testDosageFormChange() {
        final List<ProductChangeDto> list = eplNdfOutgoingDifferencesDao.dosageFormChange(new Date());
        assertNotNull(list);
    }
    
    /**
     * Simply verifies that {@link EplNdfOutgoingDifferencesDaoImpl#messageGenericChange(String, Date)} returns a list.
     * In the future this test should be expanded to verify the returned content.
     */
    @Test
    @Transactional(readOnly = true)
    public void testMessageGenericChange() {
        final List<ProductChangeDto> list = eplNdfOutgoingDifferencesDao.messageGenericChange(
            VistaProductFile.STRENGTH_FIELD_NUMBER, new Date());
        assertNotNull(list);
    }
    
  /**
   * Generates the given number of sequential Copay Tier objects.
   * @param productIen all generated objects will have this productIen.
   * @param productName all generated objects will have this productName.
   * @param productCmopId all generated objects will have this cmopId.
   * @param numToGenerate the number of objects to generate.
   * @return a List containing numToGenerate objects in proper date order.
   */
  private static List<CopayTierChange> makeCopayTierChanges(
      final String productIen, final String productName, final String productCmopId, final int numToGenerate) {
    final LocalDate baseDate = LocalDate.of(2016, 3, 3);
    final ArrayList<CopayTierChange> generatedChanges = new ArrayList<>();
    for (int i = 0; i < numToGenerate; ++i) {
      final LocalDate effectiveDate = baseDate.plusDays(i * 2);
      final boolean lastIteration = (i == numToGenerate - 1);
      generatedChanges.add(new CopayTierChange(
          productIen,
          productName,
          productCmopId,
          Integer.toString(i + 1),
          Optional.of(effectiveDate),
          // Set no end date for last copay tier.
          lastIteration ? Optional.empty() : Optional.of(effectiveDate.plusDays(1))));
    }
    return generatedChanges;
  }
  
  /**
   * Creates an {@link EplNdfOutgoingDifferences} for the given VistA field and IEN with the given value. The file is
   * always {@link VistaFileNumber#FIFTY_68}.
   * @return a new object every time
   */
  private static EplNdfOutgoingDifferences makeOutgoingDifference(
      final String vistaFieldNumber, final String vistaIen, final String actionType, final String newValue) {   

    final EplNdfOutgoingDifferences diff = EplNdfOutgoingDifferencesGenerator.createDiff(
        VistaProductFile.FILE_NUMBER, vistaFieldNumber, vistaIen, actionType);
    diff.setNewValue(newValue);
    return diff;
  }
    
  /**
   * Generate {@link EplNdfOutgoingDifferences} objects for the given copay tier changes. Note that we are using
   * {@link CopayTierChange} objects for the opposite of the normal purpose: normally we make them <em>from</em>
   * {@link EplNdfOutgoingDifferences}.
   */
  private List<EplNdfOutgoingDifferences> createDiffsForCopayTiers(final List<CopayTierChange> changes) {
    final ArrayList<EplNdfOutgoingDifferences> diffs = new ArrayList<EplNdfOutgoingDifferences>();
    for (int i = 0; i < changes.size(); ++i) {
      final CopayTierChange change = changes.get(i);
      final int copayTierIndex = i + 1;
      final String combinedIen = String.format("%s,%d", change.getProductIen(), copayTierIndex);
      
      // Tier level
      diffs.add(makeOutgoingDifference(
          VistaProductFile.COPAY_TIER_FIELD_NUMBER, combinedIen,
          EplNdfOutgoingDifferences.MODIFY_ACTION, change.getCopayTier()));
      
      // Effective date
      if (change.getEffectiveDate().isPresent()) {
        final String filemanDate = MumpsConverter.convertDate(change.getEffectiveDate().get());
        diffs.add(makeOutgoingDifference(
            VistaProductFile.COPAY_TIER_START_DATE_FIELD_NUMBER, combinedIen,
            EplNdfOutgoingDifferences.MODIFY_ACTION, filemanDate));
      }
      
      // End date
      if (change.getEndDate().isPresent()) {
        final String filemanDate = MumpsConverter.convertDate(change.getEndDate().get());
        diffs.add(makeOutgoingDifference(
            VistaProductFile.COPAY_TIER_END_DATE_FIELD_NUMBER, combinedIen,
            EplNdfOutgoingDifferences.MODIFY_ACTION, filemanDate));
      }
    }

    return diffs;
  }
    
  /**
   * Just a convenience method to save all of the given differences objects in iteration order.
   */
  private void saveAllDiffs(final Collection<EplNdfOutgoingDifferences> diffs) {
    for (final EplNdfOutgoingDifferences diff : diffs) {
      eplNdfOutgoingDifferencesDao.save(diff);
    }
  }
    
  /**
   * Tests {@link EplNdfOutgoingDifferencesDaoImpl#findCopayTierChanges(Date)}.
   */
  @Test
  @Transactional
  public void testFindCopayTierChanges() {

    /* Setup */

    // Save off the initial list in case there are already any in the database.
    final List<CopayTierChange> initialTierChanges = eplNdfOutgoingDifferencesDao.findCopayTierChanges(new Date());
    LOG.debug("Initial tier changes are: {}", initialTierChanges);

    // These have to be known values from the database.
    final String product1Ien = "23479";
    final String product1Name = "OXYCODONE HCL 5MG/ACETAMINOPHEN 300MG TAB";
    final String product1CmopId = "O0328";
    // Product 2 has a higher IEN but a "lower" name.
    final String product2Ien = "23489";
    final String product2Name = "CYANOCOBALAMIN 5,000MCG TAB";
    final String product2CmopId = "C1531";
    final List<CopayTierChange> product1Tiers = makeCopayTierChanges(product1Ien, product1Name, product1CmopId, 1);
    final List<CopayTierChange> product2Tiers = makeCopayTierChanges(product2Ien, product2Name, product2CmopId, 11);
    // Product 1 comes after product 2 due to "higher" name.
    final List<CopayTierChange> expectedTierChanges = ListUtils.union(product2Tiers, product1Tiers);

    // Save our test data to the database. Intentionally save product 1 first to test that it will
    // be retrieved last.
    saveAllDiffs(createDiffsForCopayTiers(product1Tiers));
    saveAllDiffs(createDiffsForCopayTiers(product2Tiers));

    /* Behavior */

    final List<CopayTierChange> actualTierChanges = eplNdfOutgoingDifferencesDao.findCopayTierChanges(new Date());

    /* Verification */

    // Remove the initial changes for the below comparison.
    actualTierChanges.removeAll(initialTierChanges);
    assertThat(actualTierChanges, equalTo(expectedTierChanges));
  }


    /**
     * Test save.
     */
    @Test
    @Transactional
    public void testSave() {
        EplNdfUpdateFileDo ndcUpdateFileFk = createNewUpdateFile();
        
        EplNdfOutgoingDifferences differences = new EplNdfOutgoingDifferences();
        differences.setActionType("type");
        differences.setCreatedBy("me");
        differences.setCreatedDtm(Calendar.getInstance().getTime());
        differences.setNewValue("1");
        differences.setVistaFieldNumber("01");
        differences.setVistaFileNumber("3212");
        differences.setVistaIen("111L");
        differences.setNdcUpdateFileFk(ndcUpdateFileFk);
        
        differences = eplNdfOutgoingDifferencesDao.save(differences);
        System.out.println(differences.getId());
        assertNotNull(differences.getId());
        
    }

    /**
     * 
     */
    private EplNdfUpdateFileDo createNewUpdateFile() {
        EplNdfUpdateFileDo eplNdfUpdateFileDo = new EplNdfUpdateFileDo();
        eplNdfUpdateFileDo.setComments("sdfsfd");
        eplNdfUpdateFileDo.setCreatedBy("me");
        eplNdfUpdateFileDo.setCreatedDtm(Calendar.getInstance().getTime());
        eplNdfUpdateFileDo.setDirectoryPath("/path");
        eplNdfUpdateFileDo.setFileName("JUNIT_TEST_FILE");
        EplStatusDo status = eplStatusDaoImpl.retrieve(1L);
        eplNdfUpdateFileDo.setEplStatus(status);
        eplNdfUpdateFileDao.insert(eplNdfUpdateFileDo, new UserVo());
        return eplNdfUpdateFileDo;
    }

    /**
     * Test delete.
     */
    @Test
    @Transactional
    public void testDelete() {
        
        EplNdfOutgoingDifferences differences = new EplNdfOutgoingDifferences();
        differences.setActionType("type");
        differences.setCreatedBy("me");
        differences.setCreatedDtm(Calendar.getInstance().getTime());
        differences.setNewValue("1");
        differences.setVistaFieldNumber("01");
        differences.setVistaFileNumber("3212");
        differences.setVistaIen("111L");
        
        differences = eplNdfOutgoingDifferencesDao.save(differences);
        assertNotNull(differences.getId());

        Long id = differences.getId();
        
        eplNdfOutgoingDifferencesDao.delete(differences);
        EplNdfOutgoingDifferences postDeleteDiff = eplNdfOutgoingDifferencesDao.findById(id);
        assertThat(postDeleteDiff, is(IsNull.nullValue()));
        
    }    
    
  /**
   * Tests that
   * {@link EplNdfOutgoingDifferencesDaoImpl#removePendingOutgoingDifferencesForMultipleField(String, String, String)} properly
   * removes existing ADD records.
   */
  @Test
  @Transactional
  public void testRemovePendingOutgoingDifferencesForMultipleFieldExistingAdds() {
    commonRemovePendingOutgoingDifferencesForMultipleFields(EplNdfOutgoingDifferences.ADD_ACTION);

  }

  /**
   * Tests that
   * {@link EplNdfOutgoingDifferencesDaoImpl#removePendingOutgoingDifferencesForMultipleField(String, String, String)} properly
   * removes existing MODIFY records.
   */
  @Test
  @Transactional
  public void testRemovePendingOutgoingDifferencesForMultipleFieldExistingMods() {
    commonRemovePendingOutgoingDifferencesForMultipleFields(EplNdfOutgoingDifferences.MODIFY_ACTION);

  }  
  

  /**
   * Tests that the proper records are removed by removePendingOutgoingDifferencesForMultipleField.
   * @param existingRecordsActionType Action Type of the existing record types that will be removed.
   */
  private void commonRemovePendingOutgoingDifferencesForMultipleFields(String existingRecordsActionType) {
    final String VISTA_FILE_NUMBER = "867.5309";
    final String VISTA_IEN_PRODUCT1 = "9991234";
    final String VISTA_IEN_PRODUCT2 = "9994321";
    final int NUMBER_DIFF_RECORDS_ADDED = 5;
    List<EplNdfOutgoingDifferences> diffsFromDB = eplNdfOutgoingDifferencesDao.retrieveDifferencesForOutputFile(new Date());
    int countDbRecordsAtStart = diffsFromDB.size();
    List<String> fields = Arrays.asList("127,.01", "127,1", "127,2");
    List<String> diffVistaIens = new ArrayList<String>();
    int inx = 1;
    for (String field : fields) {
      EplNdfOutgoingDifferences differences = new EplNdfOutgoingDifferences();
      differences.setCreatedBy("TESTER");
      differences.setCreatedDtm(Calendar.getInstance().getTime());
      differences.setNewValue("1");
      differences.setVistaFieldNumber(field);
      differences.setVistaFileNumber(VISTA_FILE_NUMBER);
      String vistaIen = VISTA_IEN_PRODUCT1 + "," + inx++;
      diffVistaIens.add(vistaIen);
      differences.setVistaIen(vistaIen);
      differences.setActionType(existingRecordsActionType);
      differences.setNdcUpdateFileFk(null);
      differences = eplNdfOutgoingDifferencesDao.save(differences);
    }
    // add another record with same vista Ien but diff field that should not get deleted.
    EplNdfOutgoingDifferences differences = new EplNdfOutgoingDifferences();
    differences.setCreatedBy("TESTER");
    differences.setCreatedDtm(Calendar.getInstance().getTime());
    differences.setNewValue("1");
    differences.setVistaFieldNumber("1278");
    differences.setVistaFileNumber(VISTA_FILE_NUMBER);
    differences.setVistaIen(VISTA_IEN_PRODUCT1 + ",1");
    differences.setActionType(EplNdfOutgoingDifferences.MODIFY_ACTION);
    differences.setNdcUpdateFileFk(null);
    differences = eplNdfOutgoingDifferencesDao.save(differences);

    // add another record with diff vista Ien but same field that should not get deleted.
    EplNdfOutgoingDifferences differences2 = new EplNdfOutgoingDifferences();
    differences2.setCreatedBy("TESTER");
    differences2.setCreatedDtm(Calendar.getInstance().getTime());
    differences2.setNewValue("1");
    differences2.setVistaFieldNumber("127,.01");
    differences2.setVistaFileNumber(VISTA_FILE_NUMBER);
    differences2.setVistaIen(VISTA_IEN_PRODUCT2 + ",1");
    differences2.setActionType(EplNdfOutgoingDifferences.MODIFY_ACTION);
    differences2.setNdcUpdateFileFk(null);
    differences2 = eplNdfOutgoingDifferencesDao.save(differences2);

    Date queryDate = Calendar.getInstance().getTime();
    diffsFromDB = eplNdfOutgoingDifferencesDao.retrieveDifferencesForOutputFile(queryDate);

    assertThat("Records were not added to DB properly.", diffsFromDB.size(), equalTo(countDbRecordsAtStart 
        + NUMBER_DIFF_RECORDS_ADDED));

    int countVerifyAdded = 0;
    for (EplNdfOutgoingDifferences eplNdfOutgoingDifferences : diffsFromDB) {
      if (diffVistaIens.contains(eplNdfOutgoingDifferences.getVistaIen())
          && fields.contains(eplNdfOutgoingDifferences.getVistaFieldNumber())) {
        countVerifyAdded++;
      }
    }
    assertThat("Records were not added to DB properly.", countVerifyAdded, equalTo(fields.size()));

    eplNdfOutgoingDifferencesDao.removePendingOutgoingDifferencesForMultipleField(VISTA_FILE_NUMBER, "127", VISTA_IEN_PRODUCT1);

    diffsFromDB = eplNdfOutgoingDifferencesDao.retrieveDifferencesForOutputFile(queryDate);
    assertThat("Records were not deleted from DB properly.", diffsFromDB.size(), equalTo(countDbRecordsAtStart + 2));

    for (EplNdfOutgoingDifferences eplNdfOutgoingDifferences : diffsFromDB) {
      if (diffVistaIens.contains(eplNdfOutgoingDifferences.getVistaIen())
          && fields.contains(eplNdfOutgoingDifferences.getVistaFieldNumber())) {
        fail("Record that was supposed to be deleted was still found: " + eplNdfOutgoingDifferences.toString());
      }
    }
  }

}
