package gov.va.med.pharmacy.peps.domain.common.utility;

import org.junit.Before;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.test.generator.UserGenerator;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.dao.EplDdiSeverityDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplDrugDrugInteractionDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplIenGeneratorDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplIngredientDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplDdiSeverityDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplDrugDrugInteractionDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplIenGeneratorDo;
import gov.va.med.pharmacy.peps.domain.common.model.EplIngredientDo;

import static org.easymock.EasyMock.*;
import static org.junit.Assert.assertThat;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.net.URISyntaxException;

import org.easymock.EasyMockSupport;

import static org.hamcrest.CoreMatchers.equalTo;

/**
 * Automated unit tests for {@link DDIFileReader}.
 */
public class DDIFileReaderTest extends EasyMockSupport {
    
    private EplIngredientDao mockIngredientDao;
    private EplDdiSeverityDao mockDdiSeverityDao;
    private EplDrugDrugInteractionDao mockDrugDrugInteractionDao;
    private EplIenGeneratorDao mockIenGeneratorDao;
    
    /**
     * Constructs necessary dependencies as mocks for convenience.
     */
    @Before
    public void createMocks() {
        mockIngredientDao = strictMock(EplIngredientDao.class);
        mockDdiSeverityDao = strictMock(EplDdiSeverityDao.class);
        mockDrugDrugInteractionDao = strictMock(EplDrugDrugInteractionDao.class);
        mockIenGeneratorDao = strictMock(EplIenGeneratorDao.class);
    }

    /**
     * Test method for {@link gov.va.med.pharmacy.peps.domain.common.utility.NdfFileReader#stringToBool(java.lang.String)}.
     */
    @Test
    public void testStringToBool() {
        // using Hamcrest matchers for better failed assertion messaging.
        assertThat(NdfFileReader.stringToBool("Yes"), equalTo(true));
        assertThat(NdfFileReader.stringToBool("YES"), equalTo(true));
        assertThat(NdfFileReader.stringToBool("yes"), equalTo(true));
        assertThat(NdfFileReader.stringToBool("Yest"), equalTo(false));
        assertThat(NdfFileReader.stringToBool("No"), equalTo(false));
        assertThat(NdfFileReader.stringToBool("no"), equalTo(false));
        assertThat(NdfFileReader.stringToBool("NO"), equalTo(false));
    }

    @Test
    public void testProcessFile() throws URISyntaxException, IOException {

        /* Setup */

        // These constants must match the input file.
        final String anagrelideIngredientName = "ANAGRELIDE HYDROCHLORIDE";
        final String abarelixIngredientName = "ABARELIX";
        final String propafenoneIngredientName = "PROPAFENONE";
        final String criticalSeverityName = "CRITICAL";
        final String significantSeverityName = "SIGNIFICANT";
        final long highestIen = 10857;

        final UserVo testUser = new UserGenerator().getFirst();
        final DDIFileReader ddiReader = new DDIFileReader(
            mockIngredientDao, mockDdiSeverityDao, mockDrugDrugInteractionDao, mockIenGeneratorDao, testUser);
        final File testFile = new File(getClass().getResource("/test_data/EPL_DRUG_INTERACTION.DAT").toURI());

        final EplIngredientDo anagrelideIngredient = new EplIngredientDo();
        anagrelideIngredient.setName(anagrelideIngredientName);
        final EplIngredientDo abarelixIngredient = new EplIngredientDo();
        abarelixIngredient.setName(abarelixIngredientName);
        final EplIngredientDo propafenoneIngredient = new EplIngredientDo();
        propafenoneIngredient.setName(propafenoneIngredientName);
        final EplDdiSeverityDo criticalSeverity = new EplDdiSeverityDo(1, criticalSeverityName);
        final EplDdiSeverityDo significantSeverity = new EplDdiSeverityDo(2, significantSeverityName);
        final EplIenGeneratorDo expectedIenGeneratorRecord =
            new EplIenGeneratorDo(EplDrugDrugInteractionDo.TABLE_NAME, highestIen, null, null);

        // Expected method calls
        expectLookupAndInsert(testUser, anagrelideIngredient, abarelixIngredient, criticalSeverity);
        expectLookupAndInsert(testUser, propafenoneIngredient, abarelixIngredient, significantSeverity);
        expect(mockIenGeneratorDao.insert(expectedIenGeneratorRecord, testUser))
            .andReturn(null);  // Kludge: DDIFileReader ignores the return value.

        replayAll();

        /* Behavior */
        
        assertThat(ddiReader.getHighestIenProcessed(), equalTo(DDIFileReader.NO_IENS_PROCESSED));

        try (final RandomAccessFile raFile = new RandomAccessFile(testFile, "r")) {
            ddiReader.setFile(raFile);
            ddiReader.processFile();
        }
        
        /* Verification */
        assertThat(ddiReader.getHighestIenProcessed(), equalTo(highestIen));

        verifyAll();
    }

    /**
     * Sets up EasyMock expectations for looking up the given ingredients and severity then inserting the
     * {@link EplDrugDrugInteractionDo}.
     */
    private void expectLookupAndInsert(
        final UserVo testUser, final EplIngredientDo anagrelideIngredient, final EplIngredientDo abarelixIngredient,
        final EplDdiSeverityDo criticalSeverity) {

        expect(mockIngredientDao.findIngredientByName(anagrelideIngredient.getName())).andReturn(anagrelideIngredient);
        expect(mockIngredientDao.findIngredientByName(abarelixIngredient.getName())).andReturn(abarelixIngredient);
        expect(mockDdiSeverityDao.findByName(criticalSeverity.getName())).andReturn(criticalSeverity);
        // Not bothering to test the actual inserted DDI content right now: createdDate makes it unreliable.
        expect(mockDrugDrugInteractionDao.insert(anyObject(EplDrugDrugInteractionDo.class), eq(testUser)))
            .andReturn(null);  // Kludge: DDIFileReader ignores the return value.
    }
}
