package gov.va.med.pharmacy.peps.domain.common.utility.converter;

import static org.easymock.EasyMock.expect;
import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.notNullValue;
import static org.hamcrest.CoreMatchers.nullValue;
import static org.junit.Assert.assertThat;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.easymock.EasyMockSupport;
import org.junit.Before;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.vo.ComplianceSiteVo;
import gov.va.med.pharmacy.peps.common.vo.VistaSiteVo;
import gov.va.med.pharmacy.peps.domain.common.capability.SiteListDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.model.EplComplianceSiteDo;

/**
 * Junit Test class for ComplianceSiteConverter.
 */
public class ComplianceSiteConverterTest extends EasyMockSupport{
    
    private SiteListDomainCapability mockSiteListCapability;
    private ComplianceSiteConverter converter;
    private List<VistaSiteVo> siteList;

    /**
     * Setup before each test.
     *
     * @throws Exception the exception
     */
    @Before
    public void setUp() throws Exception {
        mockSiteListCapability = mock(SiteListDomainCapability.class);
        converter = new ComplianceSiteConverter(mockSiteListCapability);
        
        siteList = Arrays.asList(
            new VistaSiteVo("SITE1", "SITE1 VAMC"), 
            new VistaSiteVo("SITE2", "SITE2 VAMC"),
            new VistaSiteVo("SITE3", "SITE3 VAMC"),
            new VistaSiteVo("SITE4", "SITE4 VAMC"),
            new VistaSiteVo("SITE5", "SITE5 VAMC")
            );
    }

    /**
     * Tests common toValueObject method call.
     * @see ComplianceSiteConverter#convert(ComplianceSiteVo)
     */
    @Test
    public final void testToValueObjectEplComplianceSiteDo() {                
        EplComplianceSiteDo eplComplianceSiteDo = new EplComplianceSiteDo("SITE2", ComplianceSiteVo.EXCLUDE);
        expect(mockSiteListCapability.retrieveByStationNumber("SITE2")).andReturn(new VistaSiteVo("SITE2", "SITE2 VAMC"));
        replayAll();
        
        ComplianceSiteVo complianceSiteVo = converter.toValueObject(eplComplianceSiteDo);
        assertThat(complianceSiteVo, notNullValue());
        assertThat(complianceSiteVo.getVistaSiteVo(), notNullValue());
        assertThat(complianceSiteVo.getVistaSiteVo().getStationNumber(), equalTo("SITE2"));
        assertThat(complianceSiteVo.getVistaSiteVo().getName(), equalTo("SITE2 VAMC"));
        assertThat(complianceSiteVo.getIncludeExcludeIndicator(), equalTo(ComplianceSiteVo.EXCLUDE));        
    }

    /**
     * Tests toValueObject method call where the station number is not found when looking up VistaSiteVo.
     * @see ComplianceSiteConverter#convert(ComplianceSiteVo)
     */    
    @Test
    public final void testToValueObjectEplComplianceSiteDoVistaSiteNotFound() {                
        EplComplianceSiteDo eplComplianceSiteDo = new EplComplianceSiteDo("SITEX", ComplianceSiteVo.EXCLUDE);
        expect(mockSiteListCapability.retrieveByStationNumber("SITEX")).andReturn(null);
        replayAll();
        
        ComplianceSiteVo complianceSiteVo = converter.toValueObject(eplComplianceSiteDo);
        assertThat(complianceSiteVo, notNullValue());
        assertThat(complianceSiteVo.getVistaSiteVo(), notNullValue());
        assertThat(complianceSiteVo.getIncludeExcludeIndicator(), equalTo(ComplianceSiteVo.EXCLUDE));        
    }    
    
    /**
     * Tests toValueObject method call when EplComplianceSiteDo specified is null.
     * @see ComplianceSiteConverter#convert(ComplianceSiteVo)
     */    
    @Test
    public final void testToValueObjectEplComplianceSiteDoNull() {                
        EplComplianceSiteDo eplComplianceSiteDo = null;
        
        ComplianceSiteVo complianceSiteVo = converter.toValueObject(eplComplianceSiteDo);
        assertThat(complianceSiteVo, nullValue());
    }    

    /**
     * Tests common toValueObject method call.
     * @see ComplianceSiteConverter#convert(java.util.Collection)
     */    
    @Test
    public final void testToValueObjectListOfEplComplianceSiteDo() {
        List<EplComplianceSiteDo> eplComplianceSiteDoList = new ArrayList<>();
        for (int i = 1; i < siteList.size()+1; i++){
            EplComplianceSiteDo eplComplianceSiteDo = new EplComplianceSiteDo("SITE" + i, ComplianceSiteVo.INCLUDE);    
            eplComplianceSiteDoList.add(eplComplianceSiteDo);
        }
        
        siteList.stream().forEach(site -> 
            expect(mockSiteListCapability.retrieveByStationNumber(site.getStationNumber())).andReturn(site));
        replayAll();        
        
        List<ComplianceSiteVo> ComplianceSiteVoList = converter.toValueObject(eplComplianceSiteDoList);
        assertThat(ComplianceSiteVoList, notNullValue());
        assertThat(ComplianceSiteVoList.size(), equalTo(siteList.size()));
        
        assertThat(ComplianceSiteVoList.get(0).getVistaSiteVo().getStationNumber(), 
            equalTo(siteList.get(0).getStationNumber()));
        assertThat(ComplianceSiteVoList.get(0).getVistaSiteVo().getName(), equalTo(siteList.get(0).getName()));
        assertThat(ComplianceSiteVoList.get(0).getIncludeExcludeIndicator(), equalTo(ComplianceSiteVo.INCLUDE));
        assertThat(ComplianceSiteVoList.get(siteList.size()-1).getVistaSiteVo().getStationNumber(), 
            equalTo(siteList.get(siteList.size()-1).getStationNumber()));
        assertThat(ComplianceSiteVoList.get(siteList.size()-1).getVistaSiteVo().getName(), 
            equalTo(siteList.get(siteList.size()-1).getName())); 
    }
    
    /**
     * Test toValueObject when List specified is null.
     * @see ComplianceSiteConverter#toValueObject(List)
     */
    @Test
    public final void testToValueObjectListOfEplComplianceSiteDoNull() {
        List<EplComplianceSiteDo> eplComplianceSiteDoList = null;
        
        List<ComplianceSiteVo> ComplianceSiteVoList = converter.toValueObject(eplComplianceSiteDoList);
        assertThat(ComplianceSiteVoList.size(), equalTo(0));        
    }    
    
    /**
     * Test common toDataObject method call.
     * @see ComplianceSiteConverter#toDataObject(ComplianceSiteVo)
     */    
    @Test
    public final void testToDataObjectComplianceSiteVo() {
        VistaSiteVo vistaSiteVo = new VistaSiteVo("SITE1", "SITE1 VAMC");
        ComplianceSiteVo complianceSiteVo = new ComplianceSiteVo(vistaSiteVo, ComplianceSiteVo.INCLUDE);
        EplComplianceSiteDo eplComplianceSiteDo = converter.toDataObject(complianceSiteVo);
        assertThat(eplComplianceSiteDo, notNullValue());
        assertThat(eplComplianceSiteDo.getStationNumber(), equalTo("SITE1"));
        assertThat(eplComplianceSiteDo.getIncludeExcludeIndicator(), equalTo(ComplianceSiteVo.INCLUDE));   
    }

    /**
     * Test toDataObject method call when ComplianceSiteVo specified is null.
     * @see ComplianceSiteConverter#toDataObject(ComplianceSiteVo)
     */        
    @Test
    public final void testToDataObjectComplianceSiteVoNull() {
        ComplianceSiteVo complianceSiteVo = null;
        EplComplianceSiteDo eplComplianceSiteDo = converter.toDataObject(complianceSiteVo);
        assertThat(eplComplianceSiteDo, nullValue());
    }    
    
    /**
     * Test common toDataObject method call.
     * @see ComplianceSiteConverter#toDataObject(List)
     */       
    @Test
    public final void testToDataObjectListOfComplianceSiteVo() {       
        List<ComplianceSiteVo> complianceSiteVoList = new ArrayList<>();
        siteList.stream().forEach(site ->   
            complianceSiteVoList.add(new ComplianceSiteVo(site, ComplianceSiteVo.INCLUDE)));
        
        siteList.stream().forEach(site -> 
            expect(mockSiteListCapability.retrieveByStationNumber(site.getStationNumber())).andReturn(site));
        replayAll();        
        
        List<EplComplianceSiteDo> eplComplianceSiteDoList = converter.toDataObject(complianceSiteVoList);
        assertThat(eplComplianceSiteDoList, notNullValue());
        assertThat(eplComplianceSiteDoList.size(), equalTo(siteList.size()));
        
        assertThat(eplComplianceSiteDoList.get(0).getStationNumber(), equalTo(siteList.get(0).getStationNumber()));
        assertThat(eplComplianceSiteDoList.get(0).getIncludeExcludeIndicator(), equalTo(ComplianceSiteVo.INCLUDE));
        assertThat(eplComplianceSiteDoList.get(siteList.size()-1).getStationNumber(), 
            equalTo(siteList.get(siteList.size()-1).getStationNumber()));
    }
    
    /**
     * Test toDataObject method call when List specified is null.
     * @see ComplianceSiteConverter#toDataObject(List)
     */       
    @Test
    public final void testToDataObjectListOfComplianceSiteVoNull() {
        List<ComplianceSiteVo> complianceSiteVoList = null;
        List<EplComplianceSiteDo> eplComplianceSiteDoList = converter.toDataObject(complianceSiteVoList);
        assertThat(eplComplianceSiteDoList, notNullValue());
        assertThat(eplComplianceSiteDoList.size(), equalTo(0));
    }
    
}
