package gov.va.med.pharmacy.peps.domain.common.utility.impl;

import java.util.Date;
import java.util.List;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.pharmacy.peps.common.utility.ConfigFileUtility;
import gov.va.med.pharmacy.peps.common.utility.impl.test.EnvironmentUtilityStub;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.domain.common.capability.impl.SeqNumDomainCapabilityImpl;
import gov.va.med.pharmacy.peps.domain.common.dao.impl.EplProductDaoImpl;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.utility.RxNormCapability;
import gov.va.med.pharmacy.peps.domain.common.utility.client.rxnorm.RxNormClient;
import gov.va.med.pharmacy.peps.domain.common.utility.client.rxnorm.test.RxNormClientIT;
import gov.va.med.pharmacy.peps.domain.common.utility.test.AbstractDomainIT;

/**
 * Tests RxNormCapabilityImpl functionality. In order to run these integration tests we need to properly setup the data source
 * configuration properties in the src\test\resources\xml\spring\test\DomainContext.xml file. We also need to activate the
 * "oracle" maven profile for the PS_PPS_domain project.
 * 
 * @author VHAISDRUSTRA
 *
 */
public class RxNormCapabilityImplIT extends AbstractDomainIT implements ApplicationContextAware {

    /**
     * Product id for vuid 4001164.
     */
    public static final long PRODUCT_VUID_4001164_ID = 9991350428094730L;

    /**
     * Product id for vuid 4003547.
     */
    public static final long PRODUCT_VUID_4003547_ID = 9991350428536081L;

    /**
     * Product id for vuid 4001954.
     */
    public static final long PRODUCT_VUID_4001954_ID = 9991350428218557L;

    /**
     * Product id for vuid 4007158.
     */
    public static final long PRODUCT_VUID_4031757_ID = 9991353516543760L;

    private ApplicationContext context;
    private UserVo user;

    @Autowired
    private EplProductDaoImpl eplProductDao;

    private RxNormCapability rxNormCapability;
    private RxNormClient rxnormClient;
    private EnvironmentUtilityStub environmentUtility;
    private ConfigFileUtility configFile;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.context = applicationContext;

    }

    /**
     * Prepares the class to run the tests.
     */
    @Before
    public void setup() {
        user = new UserVo();
        user.setUsername("AutomatedTest");

        rxNormCapability = (RxNormCapability) context.getBean("rxNormCapability");
        rxnormClient = (RxNormClient) context.getBean("rxNormClient");

        configFile =  new ConfigFileUtility();
        
        environmentUtility = new EnvironmentUtilityStub();
        environmentUtility.setNational();

        SeqNumDomainCapabilityImpl seqNumCapability = (SeqNumDomainCapabilityImpl) context.getBean("seqNumDomainCapability");
        seqNumCapability.setEnvironmentUtility(environmentUtility);

        rxNormCapability.setEnvironmentUtility(environmentUtility);
    }

    /**
     * Tests the getDifferences method.
     */
    @Test
    public void testGetDifferences() {
        EplProductDo product = new EplProductDo();

        // test both null
        product.setCodingSystemCode(null);
        List<Difference> differences = rxNormCapability.getDifferences(null, product);
        Assert.assertTrue(differences.isEmpty());

        // test both equal
        product.setCodingSystemCode("1234");
        differences = rxNormCapability.getDifferences("1234", product);
        Assert.assertTrue(differences.isEmpty());

        // test different code
        product.setCodingSystemCode("1235");
        differences = rxNormCapability.getDifferences("1234", product);
        Assert.assertTrue(differences.size() == 2);

        // test different system
        product.setCodingSystemCode("1235");
        product.setCodingSystem("Not RX Norm");
        differences = rxNormCapability.getDifferences("1235", product);
        Assert.assertTrue(differences.size() == 2);

        // test new code
        differences = rxNormCapability.getDifferences("1235", product);
        Assert.assertTrue(differences.size() == 2);

        // test deleted code
        product.setCodingSystemCode("1235");
        differences = rxNormCapability.getDifferences(null, product);
        Assert.assertTrue(differences.size() == 2);

    }

    /**
     * Tests when new rx norm code is added. The EPL_PRODUCTS table needs to be setup so the coding system related columns are
     * set to null for VUID = 4001164.
     */
    @Test
    @Transactional
    public void testPopulateAndPersistNullCoding() {

        EplProductDo eplProductDo = retrieveProduct(PRODUCT_VUID_4001164_ID);
        Assert.assertNull(eplProductDo.getCodingSystemCode());

        String code = rxnormClient.getCode(eplProductDo.getVuid(), configFile);
        populateRxNormForProduct(user, PRODUCT_VUID_4001164_ID);

        eplProductDo = retrieveProduct(PRODUCT_VUID_4001164_ID);
        Assert.assertEquals(code, eplProductDo.getCodingSystemCode());
        Assert.assertEquals(EplProductDo.RX_NORM_CODING_SYSTEM, eplProductDo.getCodingSystem());
    }

    /**
     * Tests when new rx norm code is updated. The EPL_PRODUCTS table needs to be setup so coding_system = 'Test' and
     * coding_system_code = '99999' are set to null for VUID = 4003547.
     */
    @Test
    @Transactional
    public void testPopulateAndPersistUpdateCoding() {

        EplProductDo eplProductDo = retrieveProduct(PRODUCT_VUID_4003547_ID);
        Assert.assertNotNull(eplProductDo.getCodingSystemCode());
        Assert.assertEquals("99999", eplProductDo.getCodingSystemCode());
        Assert.assertEquals("Test", eplProductDo.getCodingSystem());

        String code = rxnormClient.getCode(eplProductDo.getVuid(), configFile);
        populateRxNormForProduct(user, PRODUCT_VUID_4003547_ID);

        Assert.assertEquals(code, eplProductDo.getCodingSystemCode());
        Assert.assertEquals(EplProductDo.RX_NORM_CODING_SYSTEM, eplProductDo.getCodingSystem());
    }

    /**
     * Tests when new rx norm code does not need to be updated. The EPL_PRODUCTS table needs to be setup so coding_system =
     * 'RxNorm' and coding_system_code = '198334' are set to null for VUID = 4001954.
     */
    @Test
    @Transactional
    public void testPopulateAndPersistNoUpdateCoding() {

        EplProductDo eplProductDo = retrieveProduct(PRODUCT_VUID_4001954_ID);
        String code = rxnormClient.getCode(eplProductDo.getVuid(), configFile);

        Assert.assertNotNull(eplProductDo.getCodingSystemCode());
        Assert.assertEquals(code, eplProductDo.getCodingSystemCode());
        Assert.assertEquals(EplProductDo.RX_NORM_CODING_SYSTEM, eplProductDo.getCodingSystem());

        populateRxNormForProduct(user, PRODUCT_VUID_4001954_ID);

        Assert.assertEquals(code, eplProductDo.getCodingSystemCode());
        Assert.assertEquals(EplProductDo.RX_NORM_CODING_SYSTEM, eplProductDo.getCodingSystem());

    }

    /**
     * Tests when a previously existing rx norm code has been deleted from ISAAC. The EPL_PRODUCTS table needs to be setup so
     * coding_system = 'RxNorm' and coding_system_code is not null for VUID = 4031757.
     */
    @Test
    @Transactional
    public void testPopulateAndPersistDeleteCoding() {

        EplProductDo eplProductDo = retrieveProduct(PRODUCT_VUID_4031757_ID);
        String code = rxnormClient.getCode(eplProductDo.getVuid(), configFile); // fake vuid to get a null

        Assert.assertNotNull(eplProductDo.getCodingSystemCode());
        Assert.assertNull(code);
        Assert.assertEquals(EplProductDo.RX_NORM_CODING_SYSTEM, eplProductDo.getCodingSystem());

        populateRxNormForProduct(user, PRODUCT_VUID_4031757_ID);

        eplProductDo = retrieveProduct(PRODUCT_VUID_4031757_ID);
        Assert.assertNull(eplProductDo.getCodingSystemCode());
        Assert.assertNull(eplProductDo.getCodingSystem());

    }

    /**
     * returns a product from the database on an id.
     * 
     * @param id long id
     * @return EplProductDo
     */
    public EplProductDo retrieveProduct(long id) {
        return eplProductDao.retrieve(id);
    }

    /**
     * Exercises the populateAndPersist method in the rxNormCapability class.
     * 
     * @param user UserVo
     * @param productId long id
     */
    public void populateRxNormForProduct(UserVo user, Long productId) {
        Date runDate = new Date();
        EplProductDo product = eplProductDao.retrieve(productId);
        rxNormCapability.populateAndPersist(user, product, runDate, configFile);
    }

}
