/**
 * 
 */
package gov.va.med.pharmacy.peps.service.common.capability.impl;

import static org.easymock.EasyMock.capture;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;
import static org.easymock.EasyMock.mock;
import static org.easymock.EasyMock.newCapture;
import static org.easymock.EasyMock.partialMockBuilder;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.hamcrest.CoreMatchers.equalTo;
import static org.hamcrest.CoreMatchers.is;
import static org.junit.Assert.assertThat;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.time.DateUtils;
import org.easymock.Capture;
import org.easymock.CaptureType;
import org.easymock.EasyMock;
import org.hamcrest.core.IsNull;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.test.generator.CopayTierAssocVoGenerator;
import gov.va.med.pharmacy.peps.common.vista.VistaProductFile;
import gov.va.med.pharmacy.peps.common.vo.CopayTierAssocVo;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.diff.GroupDifference;
import gov.va.med.pharmacy.peps.domain.common.dao.EplNdfOutgoingDifferencesDao;
import gov.va.med.pharmacy.peps.domain.common.dao.impl.EplNdfOutgoingDifferencesDaoImpl;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;
import gov.va.med.pharmacy.peps.domain.common.utility.test.EplNdfOutgoingDifferencesGenerator;
import gov.va.med.pharmacy.peps.service.common.update.impl.ChecksumCalculator;
import gov.va.med.pharmacy.peps.service.common.update.impl.NdfFileSyncProcessorTestHelper;
import gov.va.med.pharmacy.peps.service.common.update.impl.NdfOutgoingDifferenceConsolidator;

/**
 * JUnit Test class for NdfFileSyncCapabilityImpl.
 */
public class NdfFileSyncCapabilityImplTest {

  private static final long EXISTING_RECORD_ID = 9999L;
  
  /** Product File Number - should match the one used in VaProductNdfFileSyncProcessor. */
  private static final String PRODUCT_FILE_NUMBER = "50.68";

  /**
   * Test method for
   * {@link NdfFileSyncCapabilityImpl#removePendingOutgoingDifferencesForMultipleField(String, String, String)}
   * . Scenario is an new record with Add Action and no existing records. Expect save to be called top insert method.
   */
  @Test
  public void testUpdatePendingQueueAddNoExisting() {
    // Data
    final String VISTA_FILE_NUMBER = "321.123";
    final String VISTA_FIELD_NUMBER = "123,456";
    final String VISTA_IEN = "12345,6789";
    final String ACTION = EplNdfOutgoingDifferences.ADD_ACTION;
    final EplNdfOutgoingDifferences diff = EplNdfOutgoingDifferencesGenerator.createDiff(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER,
        VISTA_IEN, ACTION);

    // Expectations
    EplNdfOutgoingDifferencesDao dao = mock(EplNdfOutgoingDifferencesDao.class);
    NdfFileSyncCapabilityImpl impl = new NdfFileSyncCapabilityImpl(dao, null);
    Capture<EplNdfOutgoingDifferences> capturedOutgoingDiffs = newCapture(CaptureType.ALL);    
    EasyMock.expect(dao.findPendingRecordsForFileFieldAndIen(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER, VISTA_IEN))
        .andReturn(Collections.emptyList());
    EasyMock.expect(dao.save(capture(capturedOutgoingDiffs))).andReturn(diff).once();
    replay(dao);

    // Test
    impl.updatePendingQueue(diff);
    assertThat(capturedOutgoingDiffs.getValues().size(), equalTo(1));
    // object without id is the object we passed in. The mock won't populate it like the real DAO,
    // but this makes it easy to tell our existing and unsaved records apart.
    assertThat(capturedOutgoingDiffs.getValues().get(0).getId(), is(IsNull.nullValue()));
    verify(dao);
  }

  /**
   * Test method for
   * {@link NdfFileSyncCapabilityImpl#removePendingOutgoingDifferencesForMultipleField(String, String, String)}
   * . Scenario is an new record with Add Action and an existing Add record. Expect update to be called to update the existing
   * record.
   */
  @Test
  public void testUpdatePendingQueueAddWithExistingAdd() {
    // Data
    final String VISTA_FILE_NUMBER = "321.123";
    final String VISTA_FIELD_NUMBER = "123,456";
    final String VISTA_IEN = "12345,6789";
    final String ACTION = EplNdfOutgoingDifferences.ADD_ACTION;
    final EplNdfOutgoingDifferences diff = EplNdfOutgoingDifferencesGenerator.createDiff(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER,
        VISTA_IEN, ACTION);
    final EplNdfOutgoingDifferences existingDiff = createExistingDiff(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER, VISTA_IEN, 
        EplNdfOutgoingDifferences.ADD_ACTION);
    List<EplNdfOutgoingDifferences> existingDiffs = new ArrayList<EplNdfOutgoingDifferences>();
    existingDiffs.add(existingDiff);

    String expectedNewValue = diff.getNewValue();
    String expectedCreatedBy = diff.getCreatedBy();
    Date expectedCreatedDtm = diff.getCreatedDtm();

    // Expectations
    EplNdfOutgoingDifferencesDao dao = mock(EplNdfOutgoingDifferencesDao.class);
    NdfFileSyncCapabilityImpl impl = new NdfFileSyncCapabilityImpl(dao, null);
    Capture<EplNdfOutgoingDifferences> capturedOutgoingDiffs = newCapture(CaptureType.ALL);
    EasyMock.expect(dao.findPendingRecordsForFileFieldAndIen(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER, VISTA_IEN))
        .andReturn(existingDiffs);
    EasyMock.expect(dao.update(capture(capturedOutgoingDiffs))).andReturn(diff).once();
    replay(dao);

    // Test
    impl.updatePendingQueue(diff);
    assertThat(capturedOutgoingDiffs.getValues().size(), equalTo(1));
    //We have two objects - the unsaved one without an id, and an "existing" one with an id.
    //Since the mock dao save method doesn't populate the id, we can tell whether the unsaved
    //record or the existing record was saved.  We expect the existing record to be saved.
    assertThat(capturedOutgoingDiffs.getValues().get(0).getId(), equalTo(EXISTING_RECORD_ID));
    assertThat(capturedOutgoingDiffs.getValues().get(0).getNewValue(), equalTo(expectedNewValue));
    assertThat(capturedOutgoingDiffs.getValues().get(0).getCreatedBy(), equalTo(expectedCreatedBy));
    assertThat(capturedOutgoingDiffs.getValues().get(0).getCreatedDtm(), equalTo(expectedCreatedDtm));
    assertThat(capturedOutgoingDiffs.getValues().get(0).getActionType(), equalTo(EplNdfOutgoingDifferences.ADD_ACTION));
    verify(dao);
  }

  /**
   * Test method for
   * {@link NdfFileSyncCapabilityImpl#removePendingOutgoingDifferencesForMultipleField(String, String, String)}
   * . Scenario is an new record with Modify Action and an existing Add record. Expect update to be called to update the
   * existing record.
   */
  @Test
  public void testUpdatePendingQueueModifyWithExistingAdd() {
    // Data
    final String VISTA_FILE_NUMBER = "321.123";
    final String VISTA_FIELD_NUMBER = "123,456";
    final String VISTA_IEN = "12345,6789";
    final EplNdfOutgoingDifferences diff = EplNdfOutgoingDifferencesGenerator.createDiff(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER,
        VISTA_IEN, EplNdfOutgoingDifferences.MODIFY_ACTION);
    final EplNdfOutgoingDifferences existingDiff = createExistingDiff(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER, VISTA_IEN, 
        EplNdfOutgoingDifferences.ADD_ACTION);
    List<EplNdfOutgoingDifferences> existingDiffs = new ArrayList<EplNdfOutgoingDifferences>();
    existingDiffs.add(existingDiff);

    String expectedNewValue = diff.getNewValue();
    String expectedCreatedBy = diff.getCreatedBy();
    Date expectedCreatedDtm = diff.getCreatedDtm();

    // Expectations
    EplNdfOutgoingDifferencesDao dao = mock(EplNdfOutgoingDifferencesDao.class);
    NdfFileSyncCapabilityImpl impl = new NdfFileSyncCapabilityImpl(dao, null);
    Capture<EplNdfOutgoingDifferences> capturedOutgoingDiffs = newCapture(CaptureType.ALL);
    EasyMock.expect(dao.findPendingRecordsForFileFieldAndIen(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER, VISTA_IEN))
        .andReturn(existingDiffs);
    EasyMock.expect(dao.update(capture(capturedOutgoingDiffs))).andReturn(diff).once();
    replay(dao);

    // Test
    impl.updatePendingQueue(diff);
    assertThat(capturedOutgoingDiffs.getValues().size(), equalTo(1));
    //We have two objects - the unsaved one without an id, and an "existing" one with an id.
    //Since the mock dao save method doesn't populate the id, we can tell whether the unsaved
    //record or the existing record was saved.  We expect the existing record to be saved.
    assertThat(capturedOutgoingDiffs.getValues().get(0).getId(), equalTo(EXISTING_RECORD_ID));
    assertThat(capturedOutgoingDiffs.getValues().get(0).getNewValue(), equalTo(expectedNewValue));
    assertThat(capturedOutgoingDiffs.getValues().get(0).getCreatedBy(), equalTo(expectedCreatedBy));
    assertThat(capturedOutgoingDiffs.getValues().get(0).getCreatedDtm(), equalTo(expectedCreatedDtm));
    assertThat(capturedOutgoingDiffs.getValues().get(0).getActionType(), equalTo(EplNdfOutgoingDifferences.ADD_ACTION));
    verify(dao);
  }

  /**
   * @param vistFileNumber
   * @param action TODO
   * @param VISTA_FIELD_NUMBER
   * @param VISTA_IEN
   * @return
   */
  private EplNdfOutgoingDifferences createExistingDiff(final String vistFileNumber, final String vistaFileNumber,
      final String vistaIen, String action) {
    final int TEN_DAYS_AGO = -10;
    final EplNdfOutgoingDifferences existingDiff = EplNdfOutgoingDifferencesGenerator.createDiff(vistFileNumber,
        vistaFileNumber, vistaIen, action);
    existingDiff.setId(EXISTING_RECORD_ID);
    existingDiff.setNewValue("ExistingNewValue");
    existingDiff.setCreatedBy("ExistingCreatedBy");
    existingDiff.setCreatedDtm(DateUtils.addDays(new Date(), TEN_DAYS_AGO));
    return existingDiff;
  }

  /**
   * Test method for
   * {@link NdfFileSyncCapabilityImpl#removePendingOutgoingDifferencesForMultipleField(String, String, String)}
   * . Scenario is an new record with Modify Action and an existing Modify record, and it does not revert the value. Expect
   * update to be called to update the existing record.
   */
  @Test
  public void testUpdatePendingQueueModifyWithExistingModifyDiffValue() {
    // Data
    final String VISTA_FILE_NUMBER = "321.123";
    final String VISTA_FIELD_NUMBER = "123,456";
    final String VISTA_IEN = "12345,6789";
    final EplNdfOutgoingDifferences diff = EplNdfOutgoingDifferencesGenerator.createDiff(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER,
        VISTA_IEN, EplNdfOutgoingDifferences.MODIFY_ACTION);
    final EplNdfOutgoingDifferences existingDiff = createExistingDiff(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER, VISTA_IEN, 
        EplNdfOutgoingDifferences.MODIFY_ACTION);
    List<EplNdfOutgoingDifferences> existingDiffs = new ArrayList<EplNdfOutgoingDifferences>();
    existingDiffs.add(existingDiff);

    String expectedNewValue = diff.getNewValue();
    String expectedCreatedBy = diff.getCreatedBy();
    Date expectedCreatedDtm = diff.getCreatedDtm();

    // Expectations
    EplNdfOutgoingDifferencesDao dao = mock(EplNdfOutgoingDifferencesDao.class);
    NdfFileSyncCapabilityImpl impl = new NdfFileSyncCapabilityImpl(dao, null);
    Capture<EplNdfOutgoingDifferences> capturedOutgoingDiffs = newCapture(CaptureType.ALL);
    EasyMock.expect(dao.findPendingRecordsForFileFieldAndIen(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER, VISTA_IEN))
        .andReturn(existingDiffs);
    EasyMock.expect(dao.update(capture(capturedOutgoingDiffs))).andReturn(diff).once();
    replay(dao);

    // Test
    impl.updatePendingQueue(diff);
    assertThat(capturedOutgoingDiffs.getValues().size(), equalTo(1));
    //We have two objects - the unsaved one without an id, and an "existing" one with an id.
    //Since the mock dao save method doesn't populate the id, we can tell whether the unsaved
    //record or the existing record was saved.  We expect the existing record to be saved.
    assertThat(capturedOutgoingDiffs.getValues().get(0).getId(), equalTo(EXISTING_RECORD_ID));
    assertThat(capturedOutgoingDiffs.getValues().get(0).getNewValue(), equalTo(expectedNewValue));
    assertThat(capturedOutgoingDiffs.getValues().get(0).getCreatedBy(), equalTo(expectedCreatedBy));
    assertThat(capturedOutgoingDiffs.getValues().get(0).getCreatedDtm(), equalTo(expectedCreatedDtm));
    assertThat(capturedOutgoingDiffs.getValues().get(0).getActionType(), equalTo(EplNdfOutgoingDifferences.MODIFY_ACTION));
    verify(dao);
  }

  /**
   * Test method for
   * {@link NdfFileSyncCapabilityImpl#removePendingOutgoingDifferencesForMultipleField(String, String, String)}
   * . Scenario is an new record with Modify Action and an existing Modify record, reverting the value. Expect delete to be
   * called to delete the existing record.
   */
  @Test
  public void testUpdatePendingQueueModifyWithExistingModifyRevertValue() {
    // Data
    final String VISTA_FILE_NUMBER = "321.123";
    final String VISTA_FIELD_NUMBER = "123,456";
    final String VISTA_IEN = "12345,6789";
    final EplNdfOutgoingDifferences diff = EplNdfOutgoingDifferencesGenerator.createDiff(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER,
        VISTA_IEN, EplNdfOutgoingDifferences.MODIFY_ACTION);
    diff.setNewValue("");
    final EplNdfOutgoingDifferences existingDiff = createExistingDiff(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER, VISTA_IEN, 
        EplNdfOutgoingDifferences.MODIFY_ACTION);
    List<EplNdfOutgoingDifferences> existingDiffs = new ArrayList<EplNdfOutgoingDifferences>();
    existingDiffs.add(existingDiff);

    // Expectations
    EplNdfOutgoingDifferencesDao dao = mock(EplNdfOutgoingDifferencesDao.class);
    NdfFileSyncCapabilityImpl impl = new NdfFileSyncCapabilityImpl(dao, null);
    Capture<EplNdfOutgoingDifferences> capturedOutgoingDiffs = newCapture(CaptureType.ALL);
    EasyMock.expect(dao.findPendingRecordsForFileFieldAndIen(VISTA_FILE_NUMBER, VISTA_FIELD_NUMBER, VISTA_IEN))
        .andReturn(existingDiffs);
    dao.delete(capture(capturedOutgoingDiffs));
    EasyMock.expectLastCall().once();
    replay(dao);

    // Test
    impl.updatePendingQueue(diff);
    assertThat(capturedOutgoingDiffs.getValues().size(), equalTo(1));
    //We have two objects - the unsaved one without an id, and an "existing" one with an id.
    //Since the mock dao save method doesn't populate the id, we can tell whether the unsaved
    //record or the existing record was saved.  We expect the existing record to be saved.
    assertThat(capturedOutgoingDiffs.getValues().get(0).getId(), equalTo(EXISTING_RECORD_ID));
    verify(dao);
  }
  
    /**
     * Test for processModifiedCopayTiers when all copay tiers have been deleted. Expects that one outgoing difference record is
     * recorded for field 45,.01 with a value of 'DELETED'.
     * 
     * @throws Exception if an error is encountered during the test
     */
    @Test
    public void testProcessModifiedCopayTiersAllDeleted() throws Exception {
        String vistaIen = "123";
        String userValue = "TESTUSER";

        List<CopayTierAssocVo> oldValue = new CopayTierAssocVoGenerator().getList();
        List<CopayTierAssocVo> newValue = Collections.emptyList();
        GroupDifference difference = new GroupDifference(FieldKey.COPAY_TIER_ASSOC, oldValue, newValue);

        Capture<EplNdfOutgoingDifferences> capturedOutgoingDiffs = newCapture(CaptureType.ALL);
        int numDiffRecordsExpected = 1;

        NdfFileSyncCapabilityImpl ndfFileSyncCapability = partialMockBuilder(NdfFileSyncCapabilityImpl.class)
            .addMockedMethod("findPendingRecordsForFileFieldBaseAndIen")
            .addMockedMethod("removePendingOutgoingDifferencesForMultipleField").addMockedMethod("save")
            .withConstructor(new EplNdfOutgoingDifferencesDaoImpl(), new NdfOutgoingDifferenceConsolidator()).createMock();
        expect(ndfFileSyncCapability.findPendingRecordsForFileFieldBaseAndIen(VistaProductFile.FILE_NUMBER,
            VistaProductFile.COPAY_TIER_FIELD_NUMBER_BASE, vistaIen)).andReturn(new ArrayList<EplNdfOutgoingDifferences>());
        ndfFileSyncCapability.removePendingOutgoingDifferencesForMultipleField(PRODUCT_FILE_NUMBER,
            VistaProductFile.COPAY_TIER_FIELD_NUMBER_BASE, vistaIen);
        ndfFileSyncCapability.save(capture(capturedOutgoingDiffs));
        expectLastCall().times(numDiffRecordsExpected);

        replay(ndfFileSyncCapability);

        ndfFileSyncCapability.processModifiedNdfSyncables(difference, VistaProductFile.FILE_NUMBER, vistaIen,
            VistaProductFile.COPAY_TIER_FIELD_NUMBER_BASE, VistaProductFile.COPAY_TIER_FIELD_NUMBER, userValue);

        assertThat(capturedOutgoingDiffs.getValues().size(), equalTo(numDiffRecordsExpected));

        EplNdfOutgoingDifferences diff = capturedOutgoingDiffs.getValues().get(0);
        assertThat(diff.getVistaFileNumber(), equalTo(PRODUCT_FILE_NUMBER));
        assertThat(diff.getVistaIen(), equalTo(vistaIen + "," + 1));

        String expectedChecksum = ChecksumCalculator.forCopayTierAssocVos(oldValue);
        assertThat(diff.getOldValue(), equalTo(expectedChecksum));

        assertThat(diff.getNewValue(), equalTo("DELETED"));
        assertThat(diff.getActionType(), equalTo(EplNdfOutgoingDifferences.MODIFY_ACTION));
        assertThat(diff.getCreatedBy(), equalTo(userValue));
    }
  
    /**
     * Test for processModifiedCopayTiers when a copay tier has been added to an existing product. Expects that all copay tiers
     * will be sent in outgoing difference records, all with action 'M'.
     * 
     * @param copayTierAssocs
     * @throws Exception if an error is encountered during the test
     */
    @Test
    public void testProcessModifiedCopayTiersAddCopayTier() throws Exception {
        String vistaIen = "123";
        String userValue = "TESTUSER";

        List<CopayTierAssocVo> newValue = new CopayTierAssocVoGenerator().getList();
        List<CopayTierAssocVo> oldValue = newValue.subList(0, newValue.size() - 1);
        GroupDifference difference = new GroupDifference(FieldKey.COPAY_TIER_ASSOC, oldValue, newValue);

        Capture<EplNdfOutgoingDifferences> capturedOutgoingDiffs = newCapture(CaptureType.ALL);
        int numDiffRecordsExpected = NdfFileSyncProcessorTestHelper.calculateNumberOfExpectedRecordsForCopayTest(newValue);

        NdfFileSyncCapabilityImpl ndfFileSyncCapability = partialMockBuilder(NdfFileSyncCapabilityImpl.class)
            .addMockedMethod("findPendingRecordsForFileFieldBaseAndIen")
            .addMockedMethod("removePendingOutgoingDifferencesForMultipleField").addMockedMethod("save")
            .withConstructor(new EplNdfOutgoingDifferencesDaoImpl(), new NdfOutgoingDifferenceConsolidator()).createMock();
        expect(ndfFileSyncCapability.findPendingRecordsForFileFieldBaseAndIen(VistaProductFile.FILE_NUMBER,
            VistaProductFile.COPAY_TIER_FIELD_NUMBER_BASE, vistaIen)).andReturn(new ArrayList<EplNdfOutgoingDifferences>());
        ndfFileSyncCapability.removePendingOutgoingDifferencesForMultipleField(PRODUCT_FILE_NUMBER,
            VistaProductFile.COPAY_TIER_FIELD_NUMBER_BASE, vistaIen);
        ndfFileSyncCapability.save(capture(capturedOutgoingDiffs));
        expectLastCall().times(numDiffRecordsExpected);

        replay(ndfFileSyncCapability);

        ndfFileSyncCapability.processModifiedNdfSyncables(difference, VistaProductFile.FILE_NUMBER, vistaIen,
            VistaProductFile.COPAY_TIER_FIELD_NUMBER_BASE, VistaProductFile.COPAY_TIER_FIELD_NUMBER, userValue);

        assertThat(capturedOutgoingDiffs.getValues().size(), equalTo(numDiffRecordsExpected));

        String expectedChecksum = ChecksumCalculator.forCopayTierAssocVos(oldValue);

        NdfFileSyncProcessorTestHelper.assertResultsForCopayTest(newValue, vistaIen, userValue, capturedOutgoingDiffs,
            expectedChecksum);
    }
}
