package gov.va.med.pharmacy.peps.service.common.update.impl;

import static org.hamcrest.CoreMatchers.equalTo;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.fail;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.easymock.Capture;

import gov.va.med.pharmacy.peps.common.utility.MumpsConverter;
import gov.va.med.pharmacy.peps.common.vista.VistaProductFile;
import gov.va.med.pharmacy.peps.common.vo.CopayTierAssocVo;
import gov.va.med.pharmacy.peps.common.vo.SyncProcessorElement;
import gov.va.med.pharmacy.peps.domain.common.model.EplNdfOutgoingDifferences;

/**
 * Helper class with reusable operations for tests related to the NDF update
 * file processing.
 */
public abstract class NdfFileSyncProcessorTestHelper {
    /**
     * Calculates the number of expected outgoing difference records for Copay Tier.
     * A record is expected for each:
     * <ul>
     * <li>tier level</li>
     * <li>effective date</li>
     * <li>end date - if populated</li>
     * </ul>
     * 
     * @param copayTierAssocs List of copay tiers that were specified in the test
     * @return int containing the number of expected outgoing difference records.
     */
    public static int calculateNumberOfExpectedRecordsForCopayTest(List<CopayTierAssocVo> copayTierAssocs) {
        int numDiffRecordsExpected = 0;
        if (copayTierAssocs != null) {
            for (CopayTierAssocVo copayTierAssocVo : copayTierAssocs) {
                numDiffRecordsExpected += 2;
                if (copayTierAssocVo.getEndDate() != null) {
                    numDiffRecordsExpected++;
                }
            }
        }
        return numDiffRecordsExpected;
    }

    /** Product File Number - should match the one used in VaProductNdfFileSyncProcessor. */
    static final String PRODUCT_FILE_NUMBER = "50.68";
    /** Field Number for Copay Tier End Date - should match the one used in VaProductNdfFileSyncProcessor. */
    static final String FIELD_COPAY_TIER_END_DATE = "45,2";
    /** Field Number for Copay Tier Start Date - should match the one used in VaProductNdfFileSyncProcessor. */
    static final String FIELD_COPAY_TIER_START_DATE = "45,1";
    /** Field Number for Copay Tier Value - should match the one used in VaProductNdfFileSyncProcessor. */
    static final String FIELD_COPAY_TIER = "45,.01";
    /**
     * Creates a set of expected outgoing differences record fields for each copay tier.
     * If the end date was not populated, then the FIELD_END_DATE would not exist. 
     * @return Set of Strings containing the field numbers that are expected in the outgoing difference records.
     */
    public static Set<String> createSetOfExpectedFieldsForCopayTest() {
        Set<String> expectedFields = new HashSet<String>();
        expectedFields.add(FIELD_COPAY_TIER);
        expectedFields.add(FIELD_COPAY_TIER_START_DATE);
        expectedFields.add(FIELD_COPAY_TIER_END_DATE);
        return expectedFields;
    }
    /**
     * Asserts that all the expected fields had outgoing difference records.
     * @param copayTierAssocs List of copay tiers that were specified in the test
     * @param assocIndex How many records were captured for a specific copay tier
     * @param expectedFields Set of expected fields that should have outgoing differences records for each copay tier
     */
    public static void assertExpectedFieldsFoundForCopayTest(List<CopayTierAssocVo> copayTierAssocs, int assocIndex,
            Set<String> expectedFields) {
        if (!expectedFields.isEmpty()) {
            if (!(assocIndex == copayTierAssocs.size() && expectedFields.contains(FIELD_COPAY_TIER_END_DATE)
                    && expectedFields.size() == 1)) {
                // It's ok for the last record to not have end date, but any
                // other condition is a failure
                StringBuilder sb = new StringBuilder("Expected Outgoing Differences for Fields: ");
                for (String field : expectedFields) {
                    sb.append("[");
                    sb.append(field);
                    sb.append("] ");
                }
                fail(sb.toString());
            }
        }
    }
    /**
     * Asserts the results of processNewCopayTiers test methods.
     * @param copayTierAssocs List of copay tiers that were specified in the test
     * @param vistaIen Product VistA IEN that was specified in the test
     * @param userValue User that was specified in the test
     * @param elementsToSendList The SyncProcessorElements that were captured by EasyMock during the test
     */
    public static void assertResultsForCopayTest(
        List<CopayTierAssocVo> copayTierAssocs, 
        String vistaIen, 
        String userValue,
        List<SyncProcessorElement> elementsToSendList) {
        Iterator<SyncProcessorElement> elementsIter = elementsToSendList.iterator();
    
        int assocIndex = 1;
        for (CopayTierAssocVo copayTierAssocVo : copayTierAssocs) {
            Set<String> expectedFields = createSetOfExpectedFieldsForCopayTest();
    
            while (elementsIter.hasNext() && !expectedFields.isEmpty()) {
              SyncProcessorElement element = elementsIter.next();
                assertThat(element.getVistaIen(), equalTo(vistaIen + "," + assocIndex));
                assertThat(element.getOldValue(), equalTo(null));
                assertThat(element.getUserName(), equalTo(userValue));
    
                if (element.getFieldId().equals(FIELD_COPAY_TIER)) {
                    assertThat(element.getNewValue(), equalTo(copayTierAssocVo.getCopayTier().getValue()));
                    expectedFields.remove(FIELD_COPAY_TIER);
                } else if (element.getFieldId().equals(FIELD_COPAY_TIER_START_DATE)) {
                    assertThat(element.getNewValue(),
                            equalTo(MumpsConverter.convertDate(copayTierAssocVo.getEffectiveDate())));
                    expectedFields.remove(FIELD_COPAY_TIER_START_DATE);
                } else if (element.getFieldId().equals(FIELD_COPAY_TIER_END_DATE)) {
                    assertThat(element.getNewValue(), equalTo(MumpsConverter.convertDate(copayTierAssocVo.getEndDate())));
                    expectedFields.remove(FIELD_COPAY_TIER_END_DATE);
                } else {
                    fail("Unexpected Field in the List of SyncProcessorElements");
                }
            }
    
            assertExpectedFieldsFoundForCopayTest(copayTierAssocs, assocIndex, expectedFields);
            assocIndex++;
        }
    }
    /**
     * Asserts the results of insert and modify copay tier test methods.
     * @param copayTierAssocs List of copay tiers that were specified in the test
     * @param vistaIen Product VistA IEN that was specified in the test
     * @param userValue User that was specified in the test
     * @param capturedOutgoingDiffs The EplNdfOutgoingDifferences that were captured by EasyMock during the test
     * @param expectedChecksum The expected checksum
     */
    public static void assertResultsForCopayTest(List<CopayTierAssocVo> copayTierAssocs, String vistaIen, String userValue,
            Capture<EplNdfOutgoingDifferences> capturedOutgoingDiffs, String expectedChecksum) {
        Iterator<EplNdfOutgoingDifferences> diffIter = capturedOutgoingDiffs.getValues().iterator();
    
        int assocIndex = 1;
        for (CopayTierAssocVo copayTierAssocVo : copayTierAssocs) {
            Set<String> expectedFields = createSetOfExpectedFieldsForCopayTest();
    
            while (diffIter.hasNext() && !expectedFields.isEmpty()) {
                EplNdfOutgoingDifferences diff = diffIter.next();
                assertThat(diff.getVistaFileNumber(), equalTo(PRODUCT_FILE_NUMBER));
                assertThat(diff.getVistaIen(), equalTo(vistaIen + "," + assocIndex));
                
                if(diff.getVistaFieldNumber().contains(".01")) {
                    assertThat(diff.getOldValue(), equalTo(expectedChecksum));
                }
                else {
                    assertThat(diff.getOldValue(), equalTo(null));
                }
                
                assertThat(diff.getActionType(), equalTo(EplNdfOutgoingDifferences.MODIFY_ACTION));                
                assertThat(diff.getCreatedBy(), equalTo(userValue));
    
                if (diff.getVistaFieldNumber().equals(FIELD_COPAY_TIER)) {
                    assertThat(diff.getNewValue(), equalTo(copayTierAssocVo.getCopayTier().getValue()));
                    expectedFields.remove(FIELD_COPAY_TIER);
                } else if (diff.getVistaFieldNumber().equals(FIELD_COPAY_TIER_START_DATE)) {
                    assertThat(diff.getNewValue(),
                            equalTo(MumpsConverter.convertDate(copayTierAssocVo.getEffectiveDate())));
                    expectedFields.remove(FIELD_COPAY_TIER_START_DATE);
                } else if (diff.getVistaFieldNumber().equals(FIELD_COPAY_TIER_END_DATE)) {
                    assertThat(diff.getNewValue(), equalTo(MumpsConverter.convertDate(copayTierAssocVo.getEndDate())));
                    expectedFields.remove(FIELD_COPAY_TIER_END_DATE);
                } else {
                    fail("Unexpected Field in the EplNdfOutgoingDifferences");
                }
            }
    
            assertExpectedFieldsFoundForCopayTest(copayTierAssocs, assocIndex, expectedFields);
            assocIndex++;
        }
    }
    
    /**
     * Convenience method to create an EplNdfOutgoingDifferences for an "Add" record. Defaults the File number.
     *
     * @param newValue the new value
     * @param vistaFieldNumber the vista field number
     * @param vistaIen the vista ien
     * @return the epl ndf outgoing differences
     */
    static EplNdfOutgoingDifferences expectedAddDiff(String newValue, String vistaFieldNumber, String vistaIen) {
        EplNdfOutgoingDifferences diff = new EplNdfOutgoingDifferences();
        diff.setNewValue(newValue);
        diff.setVistaFileNumber(VistaProductFile.FILE_NUMBER);
        diff.setVistaFieldNumber(vistaFieldNumber);
        diff.setVistaIen(vistaIen);
        diff.setActionType(EplNdfOutgoingDifferences.ADD_ACTION);
        return diff;
    }

    /**
     * Convenience method to create an EplNdfOutgoingDifferences for an "Add" record. Defaults the File number.
     * Uses the given oldValue.
     *
     * @param newValue the new value
     * @param vistaFieldNumber the vista field number
     * @param vistaIen the vista ien
     * @param oldValue the old value
     * @return the epl ndf outgoing differences
     */
    static EplNdfOutgoingDifferences expectedAddDiff(String newValue, String vistaFieldNumber, String vistaIen,
        String oldValue) {
        EplNdfOutgoingDifferences diff = expectedAddDiff(newValue, vistaFieldNumber, vistaIen);
        diff.setOldValue(oldValue);
        return diff;
    }

}
