/**
 * Source file created in 2012 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.external.common.drugdatavendor.outbound.capability.impl;


import static gov.va.med.pharmacy.peps.common.utility.IOUtility.safeClose;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.sql.DataSource;

import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import gov.va.med.pharmacy.peps.common.utility.PPSConstants;
import gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidationType;
import gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidator;
import gov.va.med.pharmacy.peps.common.vo.FdbAutoAddVo;
import gov.va.med.pharmacy.peps.common.vo.FdbAutoUpdateVo;
import gov.va.med.pharmacy.peps.external.common.drugdatavendor.outbound.capability.DrugReferenceAutoCapability;




/**
 * Main implementation of {@link DrugReferenceAutoCapability}.
 */
public class DrugReferenceAutoCapabilityImpl implements DrugReferenceAutoCapability {

    private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(DrugReferenceAutoCapabilityImpl.class);
    
    private SimpleDateFormat dateFormat = new SimpleDateFormat(DrugReferenceAutoCapability.FDB_DATE_PATTERN, Locale.US);
    
    private DataSource fdbDataSource;
    
    /**
     * Constructs an instance with the required dependencies.
     * @param pFdbDataSource The {@link DataSource} to the FDB-DIF database.
     */
    @Autowired
    public DrugReferenceAutoCapabilityImpl(final DataSource pFdbDataSource) {
      fdbDataSource = pFdbDataSource;
    }
    
    @Override
    public List<FdbAutoAddVo> getFdbAddedItems(Date startDate) {

        Connection con = getConnection();
        List<FdbAutoAddVo> ndcList = new ArrayList<FdbAutoAddVo>();
        PreparedStatement pstmt = null;
        ResultSet rsAdd = null;

        if (con == null) {
            LOG.error("getFdbAddedItems: get Connection is null");

            return ndcList;
        }

        try {

            //  StringBuffer query = new StringBuffer();

            String query = "select PMID, GCNSEQNO, "
                    + "ADDDATE FROM FDB_PACKAGEDDRUG WHERE to_date (ADDDATE,'YYYYMMDD') between to_date("
                    + dateFormat.format(startDate)
                    + ",'YYYYMMDD') and to_date("
                    + dateFormat.format(new Date())
                    + ",'YYYYMMDD')";

            LOG.debug("Query is: " + query);
            pstmt = con.prepareStatement(query);
            rsAdd = pstmt.executeQuery();

            while (rsAdd.next()) {
                FdbAutoAddVo vo = new FdbAutoAddVo();
                vo.setNdc(rsAdd.getString(1));
                vo.setGcnSeqno(rsAdd.getLong(2));
                vo.setAddDate(rsAdd.getString(PPSConstants.I3));
                ndcList.add(vo);
            }
        } catch (Exception e) {
            LOG.error(ESAPIValidator.validateStringInput(e.getMessage(), ESAPIValidationType.LOG_FORGING));
        } finally {
            safeClose(rsAdd);
            safeClose(pstmt);
            safeClose(con);
        }

        LOG.debug("Returning " + ndcList.size() + " items.");
        
        return ndcList;
    }

    @Override
    public FdbAutoAddVo getFdbAddedItems(String ndc) {

        Connection con = getConnection();
        FdbAutoAddVo fdbAutoAddVo = new FdbAutoAddVo();
        PreparedStatement pstmt = null;
        ResultSet rsAdd = null;

        if (con == null) {
            LOG.error("getFdbAddedItems(ndc): get Connection is null");

            return fdbAutoAddVo;
        }

        try {
            String query = "select PMID, GCNSEQNO, "
                    + "ADDDATE FROM FDB_PACKAGEDDRUG WHERE PMID = ?";

            LOG.debug("Query is: " + query);
            pstmt = con.prepareStatement(query);
            pstmt.setString(1, ndc);
            rsAdd = pstmt.executeQuery();

            while (rsAdd.next()) {
                fdbAutoAddVo.setNdc(rsAdd.getString(1));
                fdbAutoAddVo.setGcnSeqno(rsAdd.getLong(2));
                fdbAutoAddVo.setAddDate(rsAdd.getString(PPSConstants.I3));
             }
        } catch (Exception e) {
            LOG.error(ESAPIValidator.validateStringInput(e.getMessage(), ESAPIValidationType.LOG_FORGING));
        } finally {
            safeClose(rsAdd);
            safeClose(pstmt);
            safeClose(con);
        }

        return fdbAutoAddVo;
    }

    @Override
    public List<FdbAutoUpdateVo> getFdbUpdatedItems(Date startDate) {

        Connection con = getConnection();
        List<FdbAutoUpdateVo> ndcList = new ArrayList<FdbAutoUpdateVo>();
        PreparedStatement pstmt = null;
        ResultSet rs = null;

        if (con == null) {
            LOG.error("getFdbUpdatedItems: get Connection is null");

            return ndcList;
        }

        try {

            String query = "select PMID, FORMATCODE, GCNSEQNO, UPDATEDATE FROM FDB_PACKAGEDDRUG WHERE to_date "
                + "(UPDATEDATE,'YYYYMMDD') between to_date("
                + dateFormat.format(startDate) + ",'YYYYMMDD')  and to_date("
                + dateFormat.format(new Date()) + ",'YYYYMMDD') ";

            LOG.debug(query);
            pstmt = con.prepareStatement(query);
            rs = pstmt.executeQuery();

            // extract data from the ResultSet
            while (rs.next()) {
                FdbAutoUpdateVo vo = new FdbAutoUpdateVo();
                vo.setNdc(rs.getString(1));
                vo.setFormatIndicator(rs.getString(PPSConstants.I2));
                vo.setGcnSequenceNumber(String.valueOf(rs.getLong(PPSConstants.I3)));
                vo.setUpdateDate(rs.getString(PPSConstants.I4));
                ndcList.add(vo);
            }
        } catch (Exception e) {
            LOG.error(ESAPIValidator.validateStringInput(e.getMessage(), ESAPIValidationType.LOG_FORGING));
        } finally {
            safeClose(rs);
            safeClose(pstmt);
            safeClose(con);
        }

        return ndcList;
    }
    
    @Override
    public String getFdbUpdateDate() {

        Connection con = getConnection();
        PreparedStatement pstmt = null;
        ResultSet rs = null;

        if (con == null) {
            LOG.error("getFdbUpdatedItems: get Connection is null");

            return "";
        }

        String version = "";
        String issueDate = "";
        
        try {

            String query = "select DBVERSION, ISSUEDATE FROM FDB_VERSION";
            
            LOG.debug("Executing query: {}", query);
            pstmt = con.prepareStatement(query);
            rs = pstmt.executeQuery();

            // extract data from the ResultSet
            while (rs.next()) {
                version = rs.getString(1);
                issueDate = rs.getString(2);
                
            }
        } catch (Exception e) {
            LOG.error(ESAPIValidator.validateStringInput(e.getMessage(), ESAPIValidationType.LOG_FORGING));
        } finally {
            safeClose(rs);
            safeClose(pstmt);
            safeClose(con);
        }

        try {
            Date date = dateFormat.parse(issueDate);
            SimpleDateFormat formatter = new SimpleDateFormat("EEEE, dd MMMM yyyy", Locale.US);
            issueDate = formatter.format(date);
          
        } catch (ParseException e) {
            LOG.debug(e.getMessage());
        }
        
        return "FDB Version " + version + " - Issue Date: " + issueDate;
    }
    
    
    /**
     * getConnection
     * 
     * @return Connection;
     */
    private Connection getConnection() {

        Connection conn = null;
      
        try {
            conn = fdbDataSource.getConnection();
        } catch (SQLException ex) {
            LOG.error("SQL Exception in getting connection: " + ex);
        }

        if (conn == null) {
            LOG.error("Connection is null");
        }

        return conn;
    }
}
