/**
 * Source file created in 2006 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.external.common.preencapsulation.outbound.capability.impl;


import java.io.IOException;
import java.math.BigInteger;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;

import gov.va.med.pharmacy.peps.common.exception.InterfaceException;
import gov.va.med.pharmacy.peps.common.utility.PropertyUtility;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.external.common.callback.InterfaceCounterDomainCapabilityCallback;
import gov.va.med.pharmacy.peps.external.common.document.ResponseDocument;
import gov.va.med.pharmacy.peps.external.common.preencapsulation.outbound.capability.QuickActionCapability;
import gov.va.med.pharmacy.peps.external.common.preencapsulation.outbound.document.DrugFileToExternalInterfaceDocument;
import gov.va.med.pharmacy.peps.external.common.preencapsulation.outbound.utility.VistaLinkConnectionUtility;
import gov.va.med.pharmacy.peps.external.common.preencapsulation.utility.quickaction.DrugFileToExternalInterfaceConverter;
import gov.va.med.pharmacy.peps.external.common.vo.outbound.quickaction.drugfiletoexternalinterface.DrugFileToExternalInterface;
import gov.va.med.pharmacy.peps.external.common.vo.status.response.Response;
import gov.va.med.pharmacy.peps.external.common.vo.status.response.ResponseStatusType;
import gov.va.med.vistalink.rpc.RpcResponse;


/**
 * Sends updates to VistaLink. This is a local only class and will be needed for PPS-L.
 */
public class QuickActionCapabilityImpl implements QuickActionCapability {
 
    private static final org.apache.logging.log4j.Logger LOG =
        org.apache.logging.log4j.LogManager.getLogger(QuickActionCapabilityImpl.class);

    private String quickActionRpcContext;
    private String quickActionRpcName;
    private VistaLinkConnectionUtility vistaLinkConnectionUtility;
    private InterfaceCounterDomainCapabilityCallback interfaceCounter;

    /**
     * The constructor sets some base properties
     */
    public QuickActionCapabilityImpl() {

        try {
            Properties properties = PropertyUtility.loadProperties(QuickActionCapabilityImpl.class);
            quickActionRpcContext = properties.getProperty("quickActionRpcContext");
            quickActionRpcName = properties.getProperty("quickActionRpc");
        } catch (IOException e) {
            LOG.error("unable to load properties file", e);
            throw new InterfaceException(e, InterfaceException.INTERFACE_ERROR, InterfaceException.PRE_ENCAPSULATION);
        }
    }

    /**
     * This method is used to set the VistaLink connection.
     * 
     * @param connection The Vista Link Connection Utility
     */
    public void setVistaLinkConnectionUtility(VistaLinkConnectionUtility connection) {
        this.vistaLinkConnectionUtility = connection;
    }

    /**
     * This method is used to tell Vista send the Drug file to the external interface
     * 
     * @param user The user who initiated the request
     */
    @Override
    public void sendDrugFileToExternalInterface(UserVo user) {

        DrugFileToExternalInterface drugFileToExternalInterface = DrugFileToExternalInterfaceConverter
            .toDrugFileToExternalInterface();

        drugFileToExternalInterface.setPepsIdNumber(BigInteger.valueOf(interfaceCounter.incrementCounter(
            InterfaceCounterDomainCapabilityCallback.INTERFACE_VISTA_SYNCHRONIZATION,
            InterfaceCounterDomainCapabilityCallback.COUNTER_MESSAGE_ID, user)));

        String xml = DrugFileToExternalInterfaceDocument.instance().marshal(drugFileToExternalInterface);
        List<String> params = new LinkedList<String>();
        params.add(xml);

        sendItem(params, user);
    }

    /**
     * Callback.
     * 
     * @param counter counter
     */
    public void setInterfaceCounterDomainCapability(InterfaceCounterDomainCapabilityCallback counter) {
        this.interfaceCounter = counter;
    }

    /**
     * This method is used to send a single quick action update.
     * 
     * @param params The list of parameters
     * @param user The user
     */
    private void sendItem(List params, UserVo user) {
        RpcResponse rpcResponse = vistaLinkConnectionUtility.sendRequest(quickActionRpcContext, quickActionRpcName, params,
            user);
 
        Response response = ResponseDocument.instance().unmarshal(rpcResponse.getResults());
 
        if (ResponseStatusType.FAILURE.equals(response.getResponse().getStatus())) {
            throw new InterfaceException(InterfaceException.INTERFACE_VISTA_FAILURE, quickActionRpcName, response
                .getResponse().getValue());
        }
        else if (ResponseStatusType.QUEUED.equals(response.getResponse().getStatus())) { // shouldn't happen
            LOG.debug("VistA Quick Action queued: " + response.getResponse().getValue());
        }
        else if (ResponseStatusType.SUCCESS.equals(response.getResponse().getStatus())) {
            LOG.debug("VistA Quick Action successful: " + response.getResponse().getValue());
        }
    }

}
