package gov.va.med.pharmacy.peps.common.utility.security;

import static org.owasp.esapi.ESAPI.validator;

import java.io.File;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;

import org.owasp.esapi.ESAPI;
import org.owasp.esapi.errors.ValidationException;

/**
 * ESAPI Validator wrapper.
 *
 * @author DNS   DNS
 *
 */
public final class ESAPIValidator {
	private static final org.apache.logging.log4j.Logger LOG = org.apache.logging.log4j.LogManager.getLogger(ESAPIValidator.class);
	
    /**
     * The maximum valid length of a URL intended to trigger the execution of a Spring MVC controller method. Used to prevent
     * denial of service attacks based on the length of the URL submitted by the browser.
     */
    public static final int MAX_MVC_EXECUTION_URL_LENGTH = 10000;

    /**
     * Private constructor.
     */
    private ESAPIValidator() {
    }

    /**
     * Validate method.
     *
     * @param input
     *            the input string to validate
     * @param type
     *            the type of validation to be performed
     * @return the scrubbed output string if able to validate/scrub - null if unable for Log Forging.
     *
     */
    public static String validateStringInput(String input, ESAPIValidationType type) {

        // Exit immediately if input is null
        if (input == null) {
            return null;
        }

        // Exit immediately if input is Empty
        if (input.length() < 1) {
            return "";
        }

        switch (type) {

        case ACCESS_CONTROL_DB:
            try {
                return validator().getValidInput("accessControlDb", input, "accessControlDb", Integer.MAX_VALUE, false, false);
            } catch (Exception e) {
                throw new RuntimeException("Invalid characters found in input.  Access Control: DB violation.");
            }

        case COMMAND_INJECTION:
            try {

                return validator()
                        .getValidInput("commandInjection", input, "commandInjection", Integer.MAX_VALUE, false, false);

            } catch (Exception e) {
                throw new RuntimeException("Invalid characters found in input.  Command Injection violation.");
            }

        case CROSS_SITE_SCRIPTING_REFLECTED:
            try {
                return validator().getValidInput("crossSiteScriptingReflected", input, "crossSiteScriptingReflected",
                    Integer.MAX_VALUE, false, false);
            } catch (Exception e) {
                throw new RuntimeException("Invalid characters found in input.  Cross Site Scripting:  Reflected violation.");
            }

        case LOG_FORGING:
            // scrub out cr/lf
            input = input.replace('\n', ' ').replace('\r', ' ');
            try {
                return validator().getValidInput("logForging", input, "logForging", Integer.MAX_VALUE, false, false);
            } catch (Exception e) {
                return null;
            }

        case PATH_MANIPULATION:
            try {
                return validator()
                        .getValidInput("pathManipulation", input, "pathManipulation", Integer.MAX_VALUE, false, false);
            } catch (Exception e) {
                LOG.error("String failed ESAPI PATH_MANIPULATION validation: " + input, e);
                throw new RuntimeException("Invalid characters found in input. Path Manipulation violation.");
            }

        case PORTABILITY_FLAW_FILE_SEPARATOR:
            try {
                input = input.replace('\\', File.separator.toCharArray()[0]).replace('/', File.separator.toCharArray()[0]);
                return validator().getValidInput("portabilityFlawFileSeparator", input, "portabilityFlawFileSeparator",
                        Integer.MAX_VALUE, false, false);
            } catch (Exception e) {
                throw new RuntimeException("Invalid characters found in input. Portability Flaw File Separator violation.");
            }
            
        case REQUEST_URI:
            try {
                return validator().getValidInput(
                    // We should probably pass in the context parameter, but just use something vague for now.
                    "some_uri", input, "requestUri", MAX_MVC_EXECUTION_URL_LENGTH, true, false);
            } catch (ValidationException e) {
                // It would probably be better just to throw the ValidationException here but have to keep the method
                // contract for now.
                throw new RuntimeException("Input was not a valid " + type, e);
            }

        case SYSTEM_INFORMATION_LEAK_EXTERNAL:
            try {
                return validator().getValidInput("systemInformationLeakExternal", input, "systemInformationLeakExternal",
                        Integer.MAX_VALUE, false, false);
            } catch (Exception e) {
                return null;
            }

        case XML_EXT_ENTITY_INJ:
            try {
                return validator().getValidInput("xmlExtEntityInj", input, "xmlExtEntityInj", Integer.MAX_VALUE, false, false);
            } catch (Exception e) {
                throw new RuntimeException("Invalid characters found in input.  Open Redirect violation.");
            }
            
        case VA_HTTPS_URL:
            try {
                return validator().getValidInput(
                    // We should probably pass in the context parameter, but just use something vague for now.
                    "some_url", input, "vaHttpsUrl", Integer.MAX_VALUE, true);
            } catch (ValidationException e) {
                // It would probably be better just to throw the ValidationException here but have to keep the method
                // contract for now.
                throw new RuntimeException("Input was not a valid " + ESAPIValidationType.VA_HTTPS_URL, e);
            }

        default:
            return null;
        }
    }

    /**
     * Validate long method.
     *
     * @param input
     *            the input string to validate
     * @return the long value if able to validate - -1 if unable.
     */
    public static long validateLongInput(String input) {
        try {
            return ESAPI.validator().getValidNumber("validateLong", input, 0, Long.MAX_VALUE, false).longValue();
        } catch (Exception e) {
            throw new RuntimeException("Invalid long value found in input.");
        }
    }

    /**
     * Validate double input method.
     *
     * @param input the input string to validate
     * @return the double value if able to validate
     */
    public static double validateDoubleInput(String input) {
        try {
            return ESAPI.validator().getValidDouble("validateDouble", input, 0, Double.MAX_VALUE, false).doubleValue();
        } catch (Exception e) {
            throw new RuntimeException("Invalid double value found in input.");
        }
    }

    /**
     * Validate file name method.
     *
     * @param input
     *            the input file name string to validate
     * @param allowedExtensions
     *            List of allowable file extensions in string format
     * @return a canonicalized and validated file name as a String.
     */
    public static String validateFileNameInput(String input, List<String> allowedExtensions) {
        try {
            return ESAPI.validator().getValidFileName("validateFileNameInput", input, allowedExtensions, false);
        } catch (Exception e) {
            throw new RuntimeException("Invalid file name value found in input.");
        }
    }



    /**
     * Validates a file name at the end of a path. Performs essentially no validation on the other parts (e.g.,
     * preceding directories) of the path.
     *
     * @param input
     *            the input file name string to validate
     * @param allowedExtensions
     *            List of allowable file extensions in string format
     * @return a canonicalized and validated file name as a String.
     */
    public static String validateFileNameInputWithPath(String input, List<String> allowedExtensions) {
        final String processedInput = validateStringInput(input, ESAPIValidationType.LOG_FORGING);
        Path path = Paths.get(processedInput);
        String fileName = path.getFileName().toString();
        if (validateFileNameInput(fileName, allowedExtensions).equals(fileName)) {
            return processedInput;
        } else {
            throw new RuntimeException("Invalid file name value found in input.");
        }

    }
}
