package gov.va.med.pharmacy.peps.common.utility.security;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;
import java.util.ArrayList;
import java.util.List;

/**
 * Extension of Java's ObjectInputStream that provides security during deserialization of objects. Instantiate an instance of
 * this class instead of ObjectInputStream, and specify the valid classes. Standard Java classes are already on a list of safe
 * classes within StdJavaListEnum and part of the logic used here to determine if a class is safe. The VoListEnum was created to
 * provide a standard list of safe classes that are ValueObject assignable. The calling method can use the VoListEnum or specify
 * a list of classes to be used for the safe list. The VoListEnumTest is used to verify that the VoListEnum and the
 * StdJavaListEnum safe lists are complete for all ValueObject assignable classes within PS_PPS_common. 
*/
public class SafeObjectInputStream extends ObjectInputStream {

    /**
     * Whitelist of classes safe for deserialization from the stream.
     */
    private List<Class<?>> safeClasses;

    /**
     * No-arg constructor, uses only the StdJavaListEnum as the safe list.
     * 
     * @throws IOException if an issue encountered opening the stream
     * @throws SecurityException if an issue encountered with a SecurityManager
     * @see java.io.ObjectInputStream#ObjectInputStream()
     */
    public SafeObjectInputStream() throws IOException, SecurityException {
        super();
        this.safeClasses = new ArrayList<Class<?>>();
    }

    /**
     * Constructor that allows specifying a whitelist of classes safe for deserialization.
     * 
     * @param in the InputStream on which to base this SafeObjectInputStream
     * @param safeClasses whitelist of classes safe to deserialize
     * @throws IOException if an issue encountered opening the stream
     */
    public SafeObjectInputStream(InputStream in, List<Class<?>> safeClasses) throws IOException {
        super(in);
        this.safeClasses = safeClasses;
    }

    /**
     * Constructor that allows specifying a single type of class safe for deserialization.
     * 
     * @param in the InputStream on which to base this SafeObjectInputStream
     * @param safeClass class safe to deserialize
     * @throws IOException if an issue encountered opening the stream.
     */
    public SafeObjectInputStream(InputStream in, Class<?> safeClass) throws IOException {
        this(in, new ArrayList<Class<?>>());
        this.safeClasses.add(safeClass);
    }

    /**
     * Resolves the class to deserialize, checking that it is in the whitelist of safe classes. The
     * VoListEnumTest.checkIfSafeClass method uses the same logic to verify if a class is on the safe list. Any changes to this
     * method should be reviewed to determine if changes need to be made within VoListEnumTest.checkIfSafeClass.
     * 
     * @see java.io.ObjectInputStream#resolveClass(java.io.ObjectStreamClass)
     */
    @Override
    protected final Class<?> resolveClass(ObjectStreamClass desc) throws IOException, ClassNotFoundException {
        Class<?> clazz = super.resolveClass(desc);

        if (clazz.isPrimitive() 
            || clazz.isArray() 
            || Enum.class.isAssignableFrom(clazz) 
            || List.class.isAssignableFrom(clazz)
            || StdJavaListEnum.getStdJavaList().contains(clazz) 
            || safeClasses.contains(clazz)) {
            return clazz;
        }
        throw new SecurityException("Security Violation while deserializing - invalid class: " + clazz);
    }

}
