package gov.va.med.pharmacy.peps.common.utility.security;

import static gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidator.validateStringInput;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertThat;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

/**
 * <p>Automated tests for {@link ESAPIValidator}.</p>
 * 
 * <p>
 * These are not strictly unit tests since they use the ESAPI Validator, but they don't require a database or even a Spring
 * context. Therefore we run these tests with the other unit tests.
 * </p>
 */
public class ESAPIValidatorTest {

    /**
     * Validate long: invalid input test.
     */
    @Test
    public void validateLong_InvalidInput_Test() {

        long output = -1;
        String input = "9x";
        try {
            output = ESAPIValidator.validateLongInput(input);
            fail("No exception thrown for invalid input character(s).");
        } catch (Throwable t) {
            // success
            System.out.println(output + "");
        }
    }

    /**
     * Validate long: valid input test.
     */
    @Test
    public void validateLong_ValidInput_Test() {

        long output = -1;
        String input = "9";
        try {
            output = ESAPIValidator.validateLongInput(input);
            // success
            System.out.println(output);
            assertEquals(9, output);
        } catch (Throwable t) {
            fail("Exception thrown for valid input character(s).");
        }
    }

    /**
     * Validate long: valid input test.
     */
    @Test
    public void validateLong_ValidInputZero_Test() {

        long output = -1;
        String input = "0";
        try {
            output = ESAPIValidator.validateLongInput(input);
            // success
            System.out.println(output);
            assertEquals(0, output);
        } catch (Throwable t) {
            fail("Exception thrown for valid input character(s).");
        }
    }

    /**
     * Validate Double: invalid input test.
     */
    @Test
    public void validateDouble_InvalidInput_Test() {

        double output = -1;
        String input = "9x";
        try {
            output = ESAPIValidator.validateDoubleInput(input);
            fail("No exception thrown for invalid input character(s).");
        } catch (Throwable t) {
            // success
            System.out.println(output + "");
        }
    }

    /**
     * Validate long: valid input test.
     */
    @Test
    public void validateDouble_ValidInput_Test() {

        double output = -1;
        Double input = 9.25;
        try {
            output = ESAPIValidator.validateDoubleInput(input.toString());
            // success
            System.out.println(output);
            assertEquals(9.25, output, 0);
        } catch (Throwable t) {
            fail("Exception thrown for valid input character(s).");
        }
    }

    /**
     * Validate long: valid input test.
     */
    @Test
    public void validateDouble_ValidInputZero_Test() {

        double output = -1;
        Double input = 0.25;
        try {
            output = ESAPIValidator.validateDoubleInput(input.toString());
            // success
            System.out.println(output);
            assertEquals(0.25, output, 0);
        } catch (Throwable t) {
            fail("Exception thrown for valid input character(s).");
        }
    }

    /**
     * Validate file name: no path invalid file extension test.
     */
    @Test
    public void validateFileName_NoPath_InvalidFileExtension_Test() {

        String output = null;
        List<String> allowedExtensions = new ArrayList<String>();

        // With bad type (file extension) Errors out/throws exception
        String input = "someFileName.txt";
        allowedExtensions.add("tmp");
        try {
            output = ESAPIValidator.validateFileNameInput(input, allowedExtensions);
            fail("No exception thrown for invalid input character(s).");
        } catch (Throwable t) {
            assertThat(t.getMessage(), is("Invalid file name value found in input."));
            System.out.println("Output = " + output);
        }

    }

    @Test
    public void validateFileName_NoPath_ValidFileExtension_Test() {

        String output;
        List<String> allowedExtensions = new ArrayList<String>();

        // Valid file name
        output = null;
        String input = "someFileName.tmp";
        allowedExtensions.add("tmp");
        try {
            output = ESAPIValidator.validateFileNameInput(input, allowedExtensions);
            // success
            System.out.println(output);
            assertEquals(input, output);
        } catch (Throwable t) {
            fail("Exception thrown for valid input character(s).");
        }
    }

    /**
     * Validate file name: with unexpected path and valid file extension.
     */
    @Test
    public void validateFileName_WithPath_ValidFileExtension_Test() {

        String output;
        List<String> allowedExtensions = new ArrayList<String>();

        // With path data (file separators) Errors out/throws exception
        output = null;
        String input = "somePath/someFileName.tmp";
        allowedExtensions.add("tmp");
        try {
            output = ESAPIValidator.validateFileNameInput(input, allowedExtensions);
            fail("No exception thrown for invalid input character(s).");
        } catch (Throwable t) {
            System.out.println("Output = " + output);
            assertThat(t.getMessage(), is("Invalid file name value found in input."));
        }
    }

    @Test
    public void validateFileNameTest() {

        String output;
        List<String> allowedExtensions = new ArrayList<String>();

        // Valid file name
        output = null;
        String input = "someFileName.tmp";
        allowedExtensions.add("tmp");
        try {
            output = ESAPIValidator.validateFileNameInput(input, allowedExtensions);
            // success
            System.out.println(output);
            assertEquals(input, output);
        } catch (Throwable t) {
            fail("Exception thrown for valid input character(s).");
        }

        // With path data (file separators) Errors out/throws exception
        output = null;
        input = "somePath/someFileName.tmp";
        allowedExtensions.add("tmp");
        try {
            output = ESAPIValidator.validateFileNameInput(input, allowedExtensions);
            fail("No exception thrown for invalid input character(s).");
        } catch (Throwable t) {
            // success
            System.out.println("Output = " + output);
        }
    }

    /**
     * Validate file name with path: Valid path and valid file extension.
     */
    @Test
    public void validateFileNameWithPath_ValidPath_ValidFileExtension_Test() {

        String output = null;
        List<String> allowedExtensions = new ArrayList<String>();

        String input = "C:/tmp/someFileName.tmp";
        allowedExtensions.add("tmp");
        try {
            output = ESAPIValidator.validateFileNameInputWithPath(input, allowedExtensions);
            assertEquals(input, output);
        } catch (Throwable t) {
            System.out.println("Output = " + output);
        }
    }

    /**
     * Validate file name with path: Valid path and invalid file extension.
     */
    @Test
    public void validateFileNameWithPath_ValidPath_InvalidFileExtension_Test() {

        String output = null;
        List<String> allowedExtensions = new ArrayList<String>();

        String input = "C:/tmp/someFileName.txt";
        allowedExtensions.add("tmp");
        try {
            output = ESAPIValidator.validateFileNameInputWithPath(input, allowedExtensions);
            System.out.println("Output = " + output);
            fail("Expected an RuntimeException to be thrown");
        } catch (Throwable t) {
            assertThat(t.getMessage(), is("Invalid file name value found in input."));
        }
    }


    /**
     * Validate ACCESS_CONTROL_DB: invalid input.
     */
    @Test
    public final void validateAccessControlDb_InvalidInput_Test() {
        String output = null;
        String input = "*xx";

        try {
            output = ESAPIValidator.validateStringInput(input, ESAPIValidationType.ACCESS_CONTROL_DB);
            fail("No exception thrown for invalid input character(s).");
        } catch (Throwable t) {
            System.out.println(output);
            assertTrue(t.getMessage().contains("Invalid characters found in input"));
        }
    }

    /**
     * Validate ACCESS_CONTROL_DB valid input test.
     */
    @Test
    public final void validateAccessControlDb_ValidInputTest() {
        String output = null;
        String input = "xx";

        try {
            output = ESAPIValidator.validateStringInput(input, ESAPIValidationType.ACCESS_CONTROL_DB);
            System.out.println(output);
            assertEquals(input, output);
        } catch (Throwable t) {
            System.out.println(output);
            fail("Exception thrown for valid input character(s).");
        }
    }

    
    @Test
    public void validateCrossSiteScriptingReflected_ValidInput_WithLessThan_Test() {
        final String input = "This is a valid input with a lot of characters include < and the word script";
        final String output = ESAPIValidator.validateStringInput(input, ESAPIValidationType.CROSS_SITE_SCRIPTING_REFLECTED);
        assertEquals(input, output);
    }

    /**
     * Test method validateString for case CrossSiteScriptingReflected.
     *
     * */
    @Test
    public void validateCrossSiteScriptingReflected_InvalidInput_Test() {
        final String input = "This is a invalid input with a lot of characters include the tag <script";
        try {
            ESAPIValidator.validateStringInput(input, ESAPIValidationType.CROSS_SITE_SCRIPTING_REFLECTED);
            fail("Should not get here: an exception should have been thrown.");
        } catch (RuntimeException t) {
            assertTrue(t.getMessage().contains("Invalid characters found in input"));
        }
    }

    /**
     * Test method validateString for case {@link ESAPIValidationType#REQUEST_URI}.
     */
    @Test
    public void validateRequestUri_ValidInput_Test() {
        // basic string test 
        String input = "This is a valid input with a lot of characters include the word script";
        input = validateStringInput(input, ESAPIValidationType.REQUEST_URI);

        // test including all valid special characters and all valid % encoding values
        // encoded values correspond to the characters %.,':#$&?+-_=[]/\ ()
        input = "special chars:.,'#$&?+-_=[0]\\/ encoding: %25%2E%2C%27%3A%23%24%26%3F%2B%2D%5F%3D%5B%5D%2F%5C%20%28%29";
        input = validateStringInput(input, ESAPIValidationType.REQUEST_URI);
    }

    /**
     * Test method validateString for case {@link ESAPIValidationType#REQUEST_URI}.
     */
    @Test
    public void validateRequestUri_InvalidInput_Test() {
        // test a string containing "<" 
        // the RegEx does not allow "<" or ">"
        String output = null;
        String input = "This is a invalid input with a lot of characters include the tag <script";
        try {
            output = ESAPIValidator.validateStringInput(input, ESAPIValidationType.REQUEST_URI);
        } catch (RuntimeException t) {
            output = null;
        } finally {
            assertNull("Validation was expected to fail with a RuntimeException", output);            
        }

        // test a string containing%3C which corresponds to <
        // the RegEx only allows specific encoded values   
        output = null;
        input = "/PRE/searchItems.go?&searchTerms%5B0%3C.value=oxygen%25";
        try {
            output = ESAPIValidator.validateStringInput(input, ESAPIValidationType.REQUEST_URI);
        } catch (RuntimeException t) {
            output = null;
        } finally {
            assertNull("Validation was expected to fail with a RuntimeException", output);            
        }

        // check a set of invalid special characters 
        String partialInput = "null";
        Integer i = 0;
        input = "!@%^*()<>{}|\"";

        for (i = 0; i < input.length(); i++) {
            partialInput = input.charAt(i) + " char";
            try {
                output = validateStringInput(partialInput, ESAPIValidationType.REQUEST_URI);
            } catch (RuntimeException e) {
                output = null;
            } finally {
                assertNull("Validation was expected to fail with a RuntimeException", output);            
            }
        }
    }

    /**
     * This test will fail if an exception is thrown, indicating that the known good URL in the test data failed the validation
     * for CrossSiteScriptingReflected.
     */
    @Test
    public void testValidUrlInput() {
        // simple search containing "aspirin"
        String url = "/PRE/searchItems.go?_subCategories=on&searchTerms%5B0%5D.searchType=CONTAINS"
            + "&searchTerms%5B0%5D.value=aspirin&searchTerms%5B0%5D.searchField.key=product.va.product.name"
            + "&strength=&entityType=PRODUCT&dosageForm=&_requestStatus=on&_itemStatus=on&isFirstRun=false"
            + "&hasEntityChanged=false&itemStatus=ACTIVE&itemStatus=INACTIVE&requestStatus=APPROVED"
            + "&requestStatus=PENDING&requestStatus=REJECTED&_categories=on&tableId=searchResultsTable";

        url = validateStringInput(url, ESAPIValidationType.REQUEST_URI);

        // search containing "aspirin" and other criteria
        url = "/PRE/searchItems.go?_subCategories=on&searchTerms%5B0%5D.searchType=CONTAINS&searchTerms%5B0%5D.value=aspirin"
            + "&searchTerms%5B0%5D.searchField.key=product.va.product.name&strength=1&entityType=PRODUCT&dosageForm="
            + "&subCategories=HERBAL&subCategories=CHEMOTHERAPY&subCategories=OTC&subCategories=VETERINARY&_requestStatus=on"
            + "&_itemStatus=on&isFirstRun=false&hasEntityChanged=false&itemStatus=ACTIVE&itemStatus=INACTIVE"
            + "&categories=MEDICATION&categories=INVESTIGATIONAL&categories=COMPOUND&categories=SUPPLY&requestStatus=APPROVED"
            + "&requestStatus=PENDING&requestStatus=REJECTED&_categories=on&tableId=searchResultsTable";

        url = validateStringInput(url, ESAPIValidationType.REQUEST_URI);

        // simple search containing "oxygen%"
        url = "/PRE/searchItems.go?isFirstRun=false&hasEntityChanged=false"
            + "&searchTerms%5B0%5D.value=oxygen%25&searchTerms%5B0%5D.searchType=BEGINS_WITH"
            + "&entityType=PRODUCT&searchTerms%5B0%5D.searchField.key=product.va.product.name&strength=&dosageForm="
            + "&_categories=on&_subCategories=on&_itemStatus=on&_requestStatus=on";

        url = validateStringInput(url, ESAPIValidationType.REQUEST_URI);
    }

    /**
     * Test method validateString for case CommandInjection.
     *
     * */
    @Test
    @Ignore
    public void validateCommandInjectionTest() {

    }

    /**
     * Test method validateString for case LogForging.
     *
     * */
    @Test
    public final void validateLogForgingTest() {
        String output = null;
        String input = null;

        input = "xx xxx\r\nxxx";
        output = ESAPIValidator.validateStringInput(input, ESAPIValidationType.LOG_FORGING);
        System.out.println(output);
        assertEquals("xx xxx  xxx", output);

        output = null;
        input = "xx xxxo xxx";
        output = ESAPIValidator.validateStringInput(input, ESAPIValidationType.LOG_FORGING);
        System.out.println(output);
        assertEquals(input, output);

        output = null;
        input = "** xx^o xxx yyyyyyyyyyyyyyyyyyyyyyyyyyy ,./432";
        output = ESAPIValidator.validateStringInput(input, ESAPIValidationType.LOG_FORGING);
        System.out.println(output);
        assertEquals(input, output);
    }

    /**
     * Test method validateString for case PathManipulation.
     *
     * */
    @Test
    @Ignore
    public void validatePathManipulationTest() {

    }

    /**
     * Test method validateString for case PortabilityFlawFileSeparator.
     *
     * */
    @Test
    @Ignore
    public void validatePortabilityFlawFileSeparatorTest() {

    }

    /**
     * Test method validateString for case SystemInformationLeakExternal.
     *
     * */
    @Test
    @Ignore
    public void validateSystemInformationLeakExternalTest() {

    }

    /**
     * Test method validateString for case XmlExtEntityInj.
     *
     * */
    @Test
    @Ignore
    public void validateXmlExtEntityInjTest() {

    }
    
    @Test(expected = RuntimeException.class)
    public void validateVaHttpsUrlTest_NoHttps() {
        ESAPIValidator.validateStringInput("http://foobar.va.gov", ESAPIValidationType.VA_HTTPS_URL);
    }
    
    @Test(expected = RuntimeException.class)
    public void validateVaHttpsUrlTest_NoVa() {
        ESAPIValidator.validateStringInput("https://www.google.com", ESAPIValidationType.VA_HTTPS_URL);
    }
    
    @Test
    public void validateVaHttpsUrlTest_Valid() {
        final List<String> urls = Arrays.asList(
            // Try lots of permutations to make sure we're as flexible as we should be.
            "https://www.va.gov",
            "https://www.va.gov/",
            "https://va.gov",
            "https://va.gov/foobar",
            "https://DNS.URL/centrallogin/centrallanding.aspx");

        for (final String url : urls) {
            assertThat(
                ESAPIValidator.validateStringInput(url, ESAPIValidationType.VA_HTTPS_URL),
                equalTo(url));
        }
    }

}
