package gov.va.med.pharmacy.peps.service.common.capability;

import java.util.Objects;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;

import gov.va.med.pharmacy.peps.common.vo.RematchVo;

/**
 * Similar to {@link gov.va.med.pharmacy.peps.common.vo.ModDifferenceVo}, but represents a rematch to save instead of a
 * difference.
 */
public class ModRematch {

    private Optional<RematchVo> optionalRematch;
    
    private String reason;
    
    private boolean shouldOverride;
    
    /**
     * Constructs an instance with all properties. Clients should use the static factory methods instead.
     * 
     * @param pOptionalRematch See {@link #getOptionalRematch()}. Must not be null.
     * @param pReason See {@link #getReason()}. May be null: will be converted to an empty string
     * @param pShouldOverride See {@link #getShouldOverride()}.
     */
    protected ModRematch(Optional<RematchVo> pOptionalRematch, String pReason, boolean pShouldOverride) {
        optionalRematch = Objects.requireNonNull(pOptionalRematch);
        reason = StringUtils.defaultString(pReason);
        shouldOverride = pShouldOverride;
    }

    /**
     * Constructs a non-override instance. ({@link #getShouldOverride()} will be false.)
     * 
     * @param rematch must not be null
     * @param reason See {@link #getReason()}. May be null: will be converted to an empty string
     * @return the new instance
     */
    public static ModRematch nonOverride(final RematchVo rematch, final String reason) {
        return new ModRematch(Optional.of(rematch), reason, false);
    }
    
    /**
     * Constructs an override instance. ({@link #getShouldOverride() will be true.)
     * @param optionalRematch must not be null
     * @param reason See {@link #getReason()}. May be null: will be converted to an empty string
     * @return the new instance
     */
    public static ModRematch override(final Optional<RematchVo> optionalRematch, final String reason) {
        return new ModRematch(optionalRematch, reason, true);
    }

    /**
     * Returns the rematch to save. An empty optional here indicates that the existing rematch for the product should be
     * deleted.
     * @return never null
     */
    public Optional<RematchVo> getOptionalRematch() {
        return optionalRematch;
    }

    /**
     * Sets the rematch to save.
     * @param optionalRematch Must not be null.
     */
    public void setOptionalRematch(Optional<RematchVo> optionalRematch) {
        this.optionalRematch = Objects.requireNonNull(optionalRematch);
    }

    /**
     * Returns the rematch reason for recording in the audit history.
     * @return never null, but possibly empty
     */
    public String getReason() {
        return reason;
    }

    /**
     * Sets the rematch reason. See {@link #getReason()}.
     * @param reason May be null: will be converted to the empty string.
     */
    public void setReason(String reason) {
        this.reason = StringUtils.defaultString(reason);
    }

    /**
     * Returns whether the given Rematch should override any rematch already present for the item.
     * 
     * <p>
     * This property must be true to delete an existing rematch. If this property is false, {@link #getOptionalRematch()} will
     * therefore always be present.
     * </p>
     * 
     * @return true if it should override; false otherwise
     */
    public boolean getShouldOverride() {
        return shouldOverride;
    }

    public void setShouldOverride(boolean pShouldOverride) {
        shouldOverride = pShouldOverride;
    }

}
