/**
 * Source file created in 2008 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.service.common.capability.impl;


import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Locale;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.Logger;

import gov.va.med.pharmacy.peps.common.exception.DomainException;
import gov.va.med.pharmacy.peps.common.exception.ItemNotFoundException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.exception.ValueObjectValidationException;
import gov.va.med.pharmacy.peps.common.utility.PPSConstants;
import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.EventCategory;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.IntendedUseVo;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryDetailsVo;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryVo;
import gov.va.med.pharmacy.peps.common.vo.ItemStatus;
import gov.va.med.pharmacy.peps.common.vo.ModDifferenceVo;
import gov.va.med.pharmacy.peps.common.vo.NdcVo;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.SynonymVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.datafield.DataField;
import gov.va.med.pharmacy.peps.common.vo.datafield.DataFields;
import gov.va.med.pharmacy.peps.common.vo.validator.ErrorKey;
import gov.va.med.pharmacy.peps.common.vo.validator.Errors;
import gov.va.med.pharmacy.peps.domain.common.capability.IntendedUseDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.NotificationDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.ProductDomainCapability;
import gov.va.med.pharmacy.peps.service.common.utility.ItemAuditHistoryUtility;


/**
 * Enforces NDC business rules.
 */
public class NdcRulesCapabilityImpl extends DefaultRulesCapabilityImpl<NdcVo> {
    private static final String NDC_DELIMITER = "-";
    
    private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(NdcRulesCapabilityImpl.class);

    private IntendedUseDomainCapability intendedUseDomainCapability;
    
    /**
     * For saving item audit histories. Most rules do not independently save audit histories, but this one does.
     */
    private ItemAuditHistoryUtility itemAuditHistoryUtility;

    /**
     * Enforce additional NDC rules beyond simple ValueObject validation.
     */
    @Override
    protected void handleEnforceRules(NdcVo ndc, UserVo user, boolean canPerformUpdate)
        throws ValidationException {

        Errors errors = new Errors();

        ndc.setNdc(padNdcWithLeadingZero(NDC_DELIMITER, ndc.getNdc()));
        padPreviousNdc(ndc);
        handleNdcPricingFields(ndc);
        
        // getProduct() should never be null, but check just in case.
        if (ndc.getProduct() == null || ndc.getProduct().isMinimallyPopulated()) {
            LOG.debug("Retrieving full parent product for enforcing rules.");
            ndc.setProduct(retrieveParentProduct(ndc));
        }

        if (ndc.getRequestItemStatus().isApproved() && ndc.getProduct().getRequestItemStatus().isPending()) {
            errors.addError(ErrorKey.PRODUCT_PENDING_NDC_APPROVED);
            throw new ValueObjectValidationException(errors);
        }
        
        try {
            if (ndc.getProduct().getItemStatus().isInactive() && ndc.getItemStatus().isActive()) {
                errors.addError(ErrorKey.PRODUCT_INACTIVE_NDC_ACTIVE);
                throw new ValueObjectValidationException(errors);
            }
        } catch (Exception e) {
            // do nothing if there is an exception.
            LOG.debug("Swallowing exception.", e);
        }
        
        if (canPerformUpdate) {
            
            ItemAuditHistoryVo audit = new ItemAuditHistoryVo();
            // This method is called for any modification, so we have to be pretty vague with the reason.
            audit.setReason("Child NDC " + ndc.getNdc() + " updated");
            audit.setAuditItemId(ndc.getProduct().getId());
            audit.setCreatedBy(user.getUsername());
            audit.setEventCategory(EventCategory.NATIONAL_MODIFICATION);
            audit.setDetailCategory(EventCategory.NATIONAL_MODIFICATION);
            audit.setAuditItemType(EntityType.PRODUCT);
            audit.setSiteName(user.getLocation());
            audit.setUsername(user.getUsername());
            
            Collection<ItemAuditHistoryDetailsVo> collection = new ArrayList<ItemAuditHistoryDetailsVo>();
            audit.setDetails(collection);
           
            // Only create a new synonym if the NDC is Active
            if (ndc.getItemStatus().equals(ItemStatus.ACTIVE)) {
                updateProductSynonym(ndc, user, audit);
            }
            updateProductGcnSeqNo(ndc, user, audit);
            
            if (CollectionUtils.isNotEmpty(audit.getDetails())) {
                LOG.debug("Parent product was updated. Saving its Item Audit History.");
                itemAuditHistoryUtility.saveItemAuditHistoryRecords(audit, user);
            }
        }
        
    }

    /**
     * Retrieves the parent product of the NDC
     *
     * @param ndc ndc
     * @return parent product
     * @throws ItemNotFoundException ItemNotFoundException
     */
    private ProductVo retrieveParentProduct(NdcVo ndc) throws ItemNotFoundException {
        ProductVo product = (ProductVo) getManagedItemCapability().retrieve(ndc.getParentId(), ndc.getParent().getEntityType());

        return product;
    }

    /**
     * Pad the NDC value with leading zeros if any of its segments are not already of the correct length.
     * <p>
     * DF144: NDC
     * <p>
     * Rule 3: Provide leading zeros in each segment of the NDC when a segment does not have the correct number of digits.
     * 
     * @param ndc {@link NdcVo}
     */
    private void padPreviousNdc(NdcVo ndc) {
        DataFields<DataField> vadf = ndc.getVaDataFields();
        DataField<String> previousNdc = vadf.getDataField(FieldKey.PREVIOUS_NDC);

        if ((previousNdc != null) && (previousNdc.getValue() != null) && (!previousNdc.getValue().isEmpty())) {

            if ((previousNdc.getValue() != null) && (previousNdc.getValue().trim().length() > 0)) {
                previousNdc.selectStringValue(padNdcWithLeadingZero(NDC_DELIMITER, previousNdc.getValue()));

            }

            
        }
    }


    /**
     * When the Unit Price or NDC Price Per Order Unit fields are empty, enter zero for the field value before saving the NDC
     * item.
     * <p>
     * For DF200: NDC Price Per Dispense Unit, enforce rules 3-5:
     * <ul>
     * <li>Rule 3: This field is calculated by dividing NDC Price per Order Unit (DF202) by NDC Dispense Unit per Order Unit
     * (DF331).</li>
     * <li>Rule 4: If NDC Price per Order Unit (DF202) is Empty or "0.0", then NDC Price per Dispense Unit must be set to
     * "0.00".</li>
     * Rule 5: If NDC Dispense Unit per Order Unit (DF331) is Empty or "0.0", then NDC Price per Dispense Unit must be set to
     * "0.00".</li>
     * </ul>
     * 
     * @param ndc NdcVo
     */
    private void handleNdcPricingFields(NdcVo ndc) {
        DataFields<DataField> dataFields = ndc.getVaDataFields();

        DataField<Double> unitPrice = dataFields.getDataField(FieldKey.UNIT_PRICE);

        if (unitPrice == null) {
            unitPrice = DataField.newInstance(FieldKey.UNIT_PRICE);
            dataFields.setDataField(unitPrice);
        }

        if (unitPrice.getValue() == null) {
            unitPrice.selectValue(Double.valueOf(0));
        }

        DataField<Double> ndcPricePerOrderUnit = dataFields.getDataField(FieldKey.NDC_PRICE_PER_ORDER_UNIT);

        if (ndcPricePerOrderUnit == null) {
            ndcPricePerOrderUnit = DataField.newInstance(FieldKey.NDC_PRICE_PER_ORDER_UNIT);
            dataFields.setDataField(ndcPricePerOrderUnit);
        }

        if (ndcPricePerOrderUnit.getValue() == null) {
            ndcPricePerOrderUnit.selectValue(Double.valueOf(0));
        }

        DataField<Double> ndcPricePerDispenseUnit = dataFields.getDataField(FieldKey.NDC_PRICE_PER_DISPENSE_UNIT);

        if (ndcPricePerDispenseUnit == null) {
            ndcPricePerDispenseUnit = DataField.newInstance(FieldKey.NDC_PRICE_PER_DISPENSE_UNIT);
            dataFields.setDataField(ndcPricePerDispenseUnit);
        }

        if (ndcPricePerOrderUnit.getValue() == 0 || ndc.getNdcDispUnitsPerOrdUnit() == null
            || ndc.getNdcDispUnitsPerOrdUnit() == 0) {
            ndcPricePerDispenseUnit.selectValue(Double.valueOf(0));
        } else {
            
            // calculate the NDC PPOU and round/format it to four decimal digits
            double calculatedValue = ndcPricePerOrderUnit.getValue() / ndc.getNdcDispUnitsPerOrdUnit();
            double roundedValue = new BigDecimal(calculatedValue)
                .setScale(PPSConstants.I4, BigDecimal.ROUND_HALF_UP).doubleValue();
            DecimalFormat format = new DecimalFormat("0.####");
            String formatted = format.format(roundedValue);
            ndcPricePerDispenseUnit.selectValue(Double.valueOf(formatted));
        }
    }

    /**
     * Updates the parent product's synonyms from the NDC.
     * 
     * @param ndc NdcVo
     * @param user UserVo for which to update
     * @throws ValidationException exception
     */
    private void updateProductSynonym(NdcVo ndc, UserVo user, ItemAuditHistoryVo audit)
        throws ValidationException {
        String newSynonym = ndc.getTradeName().toUpperCase(Locale.US);

        // Get current list of synonyms
        Collection<SynonymVo> synonyms = ndc.getProduct().getSynonyms();

        StringBuffer synonymNames = new StringBuffer();

        // Check if NDC's trade name is already in synonyms
        for (SynonymVo synonym : synonyms) {
            if (synonym.getSynonymName() != null && synonym.getSynonymName().equals(newSynonym)) {
                newSynonym = null;
                break;
            }
            synonymNames.append("Synonym Name: " + synonym.getSynonymName() + "<p>");
        }

        // If Trade name was not found in synonyms table
        if (newSynonym != null && newSynonym.length() != 0) {
            // get the ndc vadfs
            DataFields<DataField> vadfs = ndc.getDNS  aFields();

            // Set synonym's properties
            SynonymVo synonymVo = new SynonymVo();
            synonymVo.setSynonymName(newSynonym);

            synonymVo.setNdcCode(ndc.getNdc());

            // vendor
            String vendor = ndc.getVendor();

            if ((vendor != null) && (vendor.trim().length() > 0)) {
                synonymVo.setSynonymVendor(vendor);
            }

            // vendor stock number
            String vsn = ndc.getVendorStockNumber();

            if ((vsn != null) && (vsn.trim().length() > 0)) {
                synonymVo.setSynonymVsn(vsn);
            }

            // order unit
            if (ndc.getOrderUnit() != null) {
                synonymVo.setSynonymOrderUnit(ndc.getOrderUnit());
            }

            // price per order unit
            DataField<Double> ndcPPOU = vadfs.getDataField(FieldKey.NDC_PRICE_PER_ORDER_UNIT);

            if ((ndcPPOU != null) && (ndcPPOU.getValue() != null) && (ndcPPOU.getValue().doubleValue() > 0)) {
                synonymVo.setSynonymPricePerOrderUnit(ndcPPOU.getValue());
            }

            // price per dispense unit
            DataField<Double> ndcPPDU = vadfs.getDataField(FieldKey.NDC_PRICE_PER_DISPENSE_UNIT);

            if ((ndcPPDU != null) && (ndcPPDU.getValue() != null) && (ndcPPDU.getValue().doubleValue() > 0)) {
                synonymVo.setSynonymPricePerDispenseUnit(ndcPPDU.getValue());
            }

            // NDC's disp units per ord unit is an optional field, make sure that the value is not null
            if (ndc.getNdcDispUnitsPerOrdUnit() != null) {
                synonymVo.setSynonymDispenseUnitPerOrderUnit(new Double(ndc.getNdcDispUnitsPerOrdUnit().longValue()));
            }

            synonymVo.setSynonymIntendedUse(getIntendedUse());

            // Insert trade name into synonyms table
            ProductDomainCapability productDomainCapbility = 
                (ProductDomainCapability) this.getManagedItemDomainCapability(EntityType.PRODUCT);
            ndc.getProduct().getSynonyms().add(synonymVo);
            // Note: we directly update the property in the database, skipping any normal business logic for updates.
            // This works here because synonyms are not sent to VistA. In the future this update should just use the
            // normal update business logic.
            productDomainCapbility.addSingleSynonym(synonymVo, ndc.getProduct(), user);
            String synonymNamesNew = synonymNames.toString() + "Synonym Name: " + synonymVo.getSynonymName();
            
            ItemAuditHistoryDetailsVo detail = new ItemAuditHistoryDetailsVo();
            detail.setColumnName("Synonyms");
            detail.setNewValue(synonymNamesNew);
            detail.setEventCategory(EventCategory.NATIONAL_MODIFICATION);
            detail.setDetailReason("New Trade Name added to Product Synonym table");
            detail.setOldValue(synonymNames.toString());
            audit.addDetail(detail);
        }
    }

    /**
     * Updates the GCNSEQNO for the NDC's parent product as appropriate.
     *  
     * @param ndc NdcVo
     * @param user UserVo for which to update
     * @throws ValidationException exception
     */
    private void updateProductGcnSeqNo(NdcVo ndc, UserVo user, ItemAuditHistoryVo audit)
        throws ValidationException {
        ProductDomainCapability productDomainCapbility = (ProductDomainCapability) this
            .getManagedItemDomainCapability(EntityType.PRODUCT);
        // Put parent product in a variable for convenience since we use it often.
        final ProductVo parentProduct = ndc.getProduct();
        if (StringUtils.isBlank(parentProduct.getGcnSequenceNumber())) {
            if (ndc.getFdbNdcVo() != null && ndc.getFdbNdcVo().getGcnSeqno() != null) {
                parentProduct.setGcnSequenceNumber(ndc.getFdbNdcVo().getGcnSeqno().toString());

                ItemAuditHistoryDetailsVo detail = new ItemAuditHistoryDetailsVo();
                detail.setColumnName("GCNSEQNO");
                detail.setNewValue(parentProduct.getGcnSequenceNumber());
                detail.setEventCategory(EventCategory.NATIONAL_MODIFICATION);
                detail.setDetailReason("Product GCNSEQNO updated from NDC");
                detail.setOldValue("");
                audit.addDetail(detail);
                
                // Note: we directly update the property in the database, skipping any normal business logic for updates.
                // This works here because GCNSEQNO's are always sent for every product in the update file. In the future
                // this update should just use the normal update business logic.
                productDomainCapbility.updateGcnSeqNo(parentProduct, user);
            }
        }
    }

    @Override
    public Errors checkForWarnings(NdcVo item, Collection<ModDifferenceVo> modDifferences, boolean newAdd)
        throws ValidationException {

        Errors warnings = super.checkForWarnings(item, modDifferences, newAdd);

        ProductVo product = retrieveParentProduct(item);

        if (item.getRequestItemStatus().isPending() && product.getRequestItemStatus().isPending()) {
            warnings.addError(ErrorKey.PRODUCT_AND_NDC_PENDING);
        }
        
        
//REMOVED this code on 4/3 becaue the VA doesn't want this warning message.  
//Left it here in case they change their mind.
        
//        if (modDifferences != null && modDifferences.size() > 0) {
//            Iterator itr = modDifferences.iterator();
//       
//            while (itr.hasNext()) {
//              
//                ModDifferenceVo modDifferenceVo = (ModDifferenceVo) (itr.next());
//                Difference difference = modDifferenceVo.getDifference();
//            
//                if (difference.getFieldKey().equals(FieldKey.PRODUCT)) {
//                    ProductVo newValue = (ProductVo) difference.getNewValue();
//                    ProductVo oldValue = (ProductVo) difference.getOldValue();
//                    
//                    if (oldValue != null && newValue != null && oldValue.getGcnSequenceNumber() != null 
//                        && newValue.getGcnSequenceNumber() != null) {
//                        
//                        if (!(oldValue.getGcnSequenceNumber().equalsIgnoreCase(newValue.getGcnSequenceNumber()))) {
//                            Object[] args = { oldValue.getGcnSequenceNumber(), newValue.getDNS  ductName(), 
//                                newValue.getGcnSequenceNumber() };
//                            warnings.addError(ErrorKey.GCN_SEQNUM_NOMATCH, args);
//                        }
//                    }
//                }
//            }
//        }


        if (item.getParent().getGcnSequenceNumber() != null  
            && item.getFdbNdcVo() != null) {
            if (!(item.getFdbNdcVo().getGcnSeqno().equals(new Long(item.getParent().getGcnSequenceNumber())))) {
                String name = ((ProductVo) item.getParent()).getDNS  ductName();
                Object[] args = { item.getFdbNdcVo().getGcnSeqno(), name, item.getParent().getGcnSequenceNumber()};
                warnings.addError(ErrorKey.GCN_FDBSEQNUM_NOMATCH, args);
            }
        }

        return warnings;
    }

    /**
     * Get Intended user for Trade Name
     * 
     * 
     * @return IntendedUseVo
     * @throws ItemNotFoundException when intended use for trade name not found
     */
    private IntendedUseVo getIntendedUse() throws ItemNotFoundException {

        IntendedUseVo intendedUseVo = null;
        List<IntendedUseVo> intendedUseList = intendedUseDomainCapability.retrieveDomain();

        for (IntendedUseVo intendedUse : intendedUseList) {
            if (intendedUse.isTradeName()) {
                intendedUseVo = intendedUse;
                break;
            }
        }

        // Throw exception if item not found
        if (intendedUseVo == null) {
            throw new DomainException(DomainException.DATA_NOT_FOUND, new Object[] {"'Trade Name' Intended Use selection"});
        }

        return intendedUseVo;
    }

    public void setIntendedUseDomainCapability(IntendedUseDomainCapability intendedUseDomainCapability) {
        this.intendedUseDomainCapability = intendedUseDomainCapability;
    }

    /**
     * Setter for Spring.
     * @param pNotificationDomainCapability the dependency to set
     */
    public void setNotificationDomainCapability(NotificationDomainCapability pNotificationDomainCapability) {
        // Note that we don't save the NotificationDomainCapability but just create an ItemAuditHistoryUtility from it.
        itemAuditHistoryUtility = new ItemAuditHistoryUtility(pNotificationDomainCapability);
    }
}
