/*
 * 
 */
package gov.va.med.pharmacy.peps.service.common.capability.impl;

import gov.va.med.pharmacy.peps.common.email.Email;
import gov.va.med.pharmacy.peps.common.email.EmailService;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.exception.ValueObjectValidationException;
import gov.va.med.pharmacy.peps.common.exception.WrappedIOException;
import gov.va.med.pharmacy.peps.common.utility.security.UntrustedLogParam;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.NdfUpdateFileVo;
import gov.va.med.pharmacy.peps.common.vo.PaginatedList;
import gov.va.med.pharmacy.peps.common.vo.StatusVo;
import gov.va.med.pharmacy.peps.common.vo.UpdateSearchCriteria;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.validator.ErrorKey;
import gov.va.med.pharmacy.peps.common.vo.validator.Errors;
import gov.va.med.pharmacy.peps.domain.common.capability.NdfUpdateFileMgtDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.NdfUpdateFileStatusDomainCapability;
import gov.va.med.pharmacy.peps.service.common.capability.NdfUpdateCapability;
import gov.va.med.pharmacy.peps.service.common.capability.NdfUpdateProcessCapability;
import gov.va.med.pharmacy.peps.service.common.utility.NdfUpdateProcessFile;

import java.io.IOException;
import java.nio.file.Path;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Optional;

import javax.annotation.Resource;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.core.env.Environment;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * Contains the main logic to create the NDF Update File and send email notifications. This class is not completely
 * thread-safe but see the {@link NdfUpdateCapability} class documentation for what is thread-safe.
 */
@Configuration
@PropertySource("classpath:gov.va.med.pharmacy.peps.emailservice.properties")
public class NdfUpdateCapabilityImpl implements NdfUpdateCapability {
  
  private static final String THREAD_NAME = "On-demand Update File Thread";

  /**
   * The property name of the email address list to send file rejected emails to.
   */
  static final String FILE_REJECTED_TO_EMAIL_PROPERTY = "update_file_rejected_to_email";

  /**
   * The property name of the email address list to send file approved emails to.
   */
  static final String FILE_APPROVED_TO_EMAIL_PROPERTY = "update_file_approved_to_email";

  /**
   * The property name of the email address list to send file ready for testing emails to.
   */
  static final String FILE_READY_FOR_TESTING_TO_EMAIL_PROPERTY = "update_file_on_test_site_to_email";

    /** The Logger. */
    private static final Logger LOG = LogManager
            .getLogger(NdfUpdateCapabilityImpl.class);
    
    /** The NdfUpdateProcessCapability. */
    private NdfUpdateProcessCapability ndfUpdateProcessCapability;
    
    /** The NdfUpdateFileMgtDomainCapability. */
    private NdfUpdateFileMgtDomainCapability ndfUpdateFileMgtDomainCapability;
    
    /** The NdfUpdateFileStatusDomainCapability. */
    private NdfUpdateFileStatusDomainCapability ndfUpdateFileStatusDomainCapability;
    
    /** The NdfUpdateProcessFile. */
    private NdfUpdateProcessFile ndfUpdateProcessFile;

    /** The transaction manager. */
    private PlatformTransactionManager transactionManager;

    /** The email service. */
    private EmailService emailService;

    /** The environment. */
    private Environment environment;
    
  // This method is synchronized because this process should only be executed by one thread at a time. This class
  // should also be a singleton.
  @Override
  public synchronized NdfUpdateFileVo createOnDemandFileAsync(UserVo user) throws ValidationException {

    final Optional<NdfUpdateFileVo> optionalFileToGenerate = getFileRecordForGeneration(user);
    // If a file was already in process, throw the exception per the method contract.
    if (!optionalFileToGenerate.isPresent()) {
      throw new ValidationException(ValidationException.NON_FINAL_STATE_FILE_IN_PROCESS);
    }
    
    final NdfUpdateFileVo fileToGenerate = optionalFileToGenerate.get();

    // It would be better to use a pre-existing thread for performance here, but we don't have a common
    // ExecutorService available and performance is not a major concern for this operation.
    new Thread(new Runnable() {
      @Override
      public void run() {
        try {
          LOG.info("Executing on-demand file generation.");
          // generateAndTransmitFile() is also synchronized so this new thread will not execute it until the
          // kicking-off method returns.
          generateAndTransmitFile(fileToGenerate, user);
        } catch (final Exception e) {
          handleExceptionDuringCreateAndSendToTestProcess(user, fileToGenerate, e);
          cleanupFromExceptionDuringCreateAndSendToTestProcess(fileToGenerate);
          
          // Don't re-throw an exception because we're on a separate thread. We have already updated the file metadata
          // to report the error.
        }
      }
    }, THREAD_NAME).start();

    return fileToGenerate;
  }

    @Override
    // This method is synchronized because this process should only be executed by one thread at a time. This class
    // should also be a singleton.
    public synchronized void createScheduledFile(UserVo user) throws IOException {

      // Get the file record in a separate transaction because we want to commit it before we start the transaction for
      // generateAndTransmitFile().
      final TransactionTemplate txTemplate = new TransactionTemplate(transactionManager);
      txTemplate.setPropagationBehavior(TransactionTemplate.PROPAGATION_REQUIRES_NEW);
      final Optional<NdfUpdateFileVo> optionalNewFile = txTemplate.execute(
          new TransactionCallback<Optional<NdfUpdateFileVo>>() {
            @Override
            public Optional<NdfUpdateFileVo> doInTransaction(TransactionStatus status) {
              return getFileRecordForGeneration(user);
            }
          });

      if (optionalNewFile.isPresent()) {
        final NdfUpdateFileVo newFileVo = optionalNewFile.get(); // Never returns null.

        try {
          // This method executes in its own transaction.
          generateAndTransmitFile(newFileVo, user);
        } catch (Exception e) {
          handleExceptionDuringCreateAndSendToTestProcess(user, newFileVo, e);
          cleanupFromExceptionDuringCreateAndSendToTestProcess(newFileVo);

          // Still throw the exception per the method contract.
          throw e;
        }

      }
    }

    /**
     * Returns the {@link NdfUpdateFileVo} to use for generating a new update file. Creates one or resets an existing
     * one as appropriate.
     * 
     * @return an "empty" optional if no appropriate record could be returned and the file should not be generated now.
     * Otherwise, a "present" optional containing the record to use. Never null.
     */
    private synchronized Optional<NdfUpdateFileVo> getFileRecordForGeneration(UserVo user) {
      final NdfUpdateFileVo latestFileVo = ndfUpdateFileMgtDomainCapability.retrieveCurrentStatus();
      if (latestFileVo == null) {
        // very first time creating a file
        return Optional.of(createInitiatedFile(user));

      } else if (StatusVo.NON_FINAL_STATE_STATUSES.contains(latestFileVo.getStatus().getStatusId())) {
        LOG.info("Cannot create new file, as the previous file is not in a final state.");
        return Optional.empty();

      } else if (StatusVo.PPSN_ERROR == latestFileVo.getStatus().getStatusId()) {
        // reuse the NdfUpdateFileVo
        latestFileVo.setComments(null);
        latestFileVo.setDirectoryPath(null);
        latestFileVo.setFileName(null);
        latestFileVo.getStatus().setStatusId(StatusVo.INITIATED);
        // Go ahead and update the record in the database to prevent anyone from trying to kick off another process.
        updateNdfUpdateFile(latestFileVo, user);
        return Optional.of(latestFileVo);

      } else {
        return Optional.of(createInitiatedFile(user));
      }
    }

    /**
     * Generates an update file, transmits it to the test folder, and updates its metadata in the database, all in its
     * own transaction.
     * 
     * @param newFileVo the associated {@link NdfUpdateFileVo}. Must be in {@link StatusVo#INITIATED} status.
     * @param user the user generating the file
     * @throws IOException if there is an error generating or transferring the update file.
     */
    // This method is mainly protected just to allow automated testing, but theoretically a subclass could use it.
    protected synchronized void generateAndTransmitFile(NdfUpdateFileVo newFileVo, UserVo user) throws IOException {
      
      // Precondition check.
      if (StatusVo.INITIATED != newFileVo.getStatus().getStatusId()) {
        throw new IllegalArgumentException("The given update file is not ready to generate.");
      }

      // Use TransactionTemplate because @Transactional does not work when calling methods on 'this'.
      final TransactionTemplate txTemplate = new TransactionTemplate(transactionManager);
      txTemplate.setPropagationBehavior(TransactionTemplate.PROPAGATION_REQUIRES_NEW);
      try {
        txTemplate.execute(new TransactionCallbackWithoutResult() {
          @Override
          protected void doInTransactionWithoutResult(TransactionStatus status) {
            try {
              createAndAttachPhysicalFile(user, newFileVo);
        
              ndfUpdateProcessCapability.addDataToFile(newFileVo, ndfUpdateProcessFile);
          
              transmitUpdateFileToTest(newFileVo, user);
            } catch (IOException ex) {
              throw new WrappedIOException(ex);
            }

            updateNdfUpdateFile(newFileVo, user);
      
            LOG.info("Generated and transmitted update file {}.", newFileVo.getFileName());
          }
        });
      } catch (final WrappedIOException ex) {
        LOG.debug("Caught WrappedIOException. Translating back to a normal IOException.", ex);
        throw new IOException("Failed to generate update file.", ex.getWrappedException());
      }
    }

    /**
     * Create the local physical file and attach it to specified NdfUpdateFileVo.
     * @param user User
     * @param newFileVo NdfUpdateFileVo to associate to the new physical file
     * @throws IOException Thrown if errors encountered during physical file creation
     */ 
    private void createAndAttachPhysicalFile(UserVo user, NdfUpdateFileVo newFileVo) throws IOException {

        Path filePath = ndfUpdateProcessFile.createFile(newFileVo.getFileId());
   
        newFileVo.setFileName(filePath.getFileName().toString());
        newFileVo.setDirectoryPath(ndfUpdateProcessFile.getPath().toString());
        newFileVo.setStatus(new StatusVo(StatusVo.CREATED));
        updateNdfUpdateFile(newFileVo, user);
    }

    /**
     * Handles an exception during the file creation process.
     * @param user User who invoked the file creation
     * @param newFileVo the NdfUpdateFileVo being processed. Must not be null.
     * @param e the exception encountered
     */
    private void handleExceptionDuringCreateAndSendToTestProcess(UserVo user, NdfUpdateFileVo newFileVo, Exception e) {
        try{
            LOG.error(
                "Exception in NdfUpdateCapabilityImpl.process() - " + new UntrustedLogParam(e.getMessage()), e);
            newFileVo.setStatus(new StatusVo(StatusVo.PPSN_ERROR));
            newFileVo.setComments("PPS-N/NDF Update Test File Creation Failed. Log a CA ticket");
            updateNdfUpdateFileInSeparateTxn(newFileVo, user);
        }
        catch (Exception e1){
            LOG.error("Secondary exception encountered during cleanup after a primary exception.", e1);
        }
    }
    
    /**
     * Creates an NdfUpdateFileVo with initiated status. 
     * @param user User initiating the file
     * @return NdfUpdateFileVo with initiated status
     */
    private NdfUpdateFileVo createInitiatedFile(UserVo user) {
        NdfUpdateFileVo initiatedFile = new NdfUpdateFileVo();
        initiatedFile.setStatus(new StatusVo(StatusVo.INITIATED));
        initiatedFile.setFileName("");
        NdfUpdateFileVo updateFile = insertNdfUpdateFile(initiatedFile, user);
        return updateFile;
    }

    @Override
    public synchronized void approve(NdfUpdateFileVo toUpdate, UserVo user) throws ValidationException {
        try {
            transmitUpdateFileToProd(toUpdate, user);
        } catch (Exception e) {
            LOG.error("Error while transmitting file to production.", e);
            throw new ValidationException(ValidationException.FILE_APPROVAL_FAILED);
        }
    }
    
    @Override
    public synchronized void reject(NdfUpdateFileVo toUpdate, UserVo user)
            throws ValidationException {
        
      if (StringUtils.isBlank(toUpdate.getComments())) {
        Errors errors = new Errors();
        errors.addError(FieldKey.UPDATE_FILE, ErrorKey.FILE_REJECTION_COMMENTS_REQUIRED);
        throw new ValueObjectValidationException(errors);
      }

      try {
        transmitUpdateFileToReject(toUpdate, user);
        ndfUpdateProcessCapability.processRejectedFile(toUpdate);                
      } catch (Exception e) {
        LOG.error("Error while transmitting file to reject.", e);
        throw new ValidationException(ValidationException.FILE_REJECTION_FAILED);
      }
    }       
    
    /**
     * Transmit update file to production - Moves the file to the production folder, sends an email notification, and stores 
     * the NdfUpdateFileVo with an updated status.  
     *
     * @param updateFile the update file
     * @param user the user
     * @throws IOException if the file could not be moved
     */
    private void transmitUpdateFileToProd(NdfUpdateFileVo updateFile, UserVo user) throws IOException {
      LOG.info("Preparing to transmit " + updateFile.getFileName() + " file to Prod.");

      ndfUpdateProcessFile.transmitFileToProd(updateFile.getFileName());

      Calendar gc = GregorianCalendar.getInstance();
      updateFile.setProdTransmissionDtm(gc.getTime());
      updateFile.setStatus(new StatusVo(StatusVo.TRANSMITTED_TO_PRODUCTION));

      try {
        sendEmailNotification(updateFile.getStatus(), updateFile.getFileName(),
            ndfUpdateProcessFile.getProductionFileSize(updateFile.getFileName()));
      } catch (final Exception ex) {
        // The file was sent, but the email failed. We don't want to fail the whole process for this.
        updateFile.setComments(
            "PPS-N/NDF Update Production File Transmission Successful, but subsequent errors encountered sending "
                + "email. Log a CA ticket.");
        LOG.error("File was sent, but subsequent error occurred sending email.", ex);
      }

      updateNdfUpdateFile(updateFile, user);
    }
    
    /**
     * Transmit update file to test - Moves the file to the test folder and sends an email notification. If the transmit
     * is successful, this method will update the status of the passed {@link NdfUpdateFileVo} but will not persist the
     * update to the database.
     *
     * @param updateFile the update file
     * @param user the user
     * @throws IOException if file transmission failed
     */    
    protected void transmitUpdateFileToTest(NdfUpdateFileVo updateFile, UserVo user) throws IOException {
        LOG.info("Preparing to transmit " + updateFile.getFileName() + " file to Test.");

        try {
            ndfUpdateProcessFile.transmitFileToTest(updateFile.getFileName());
            Calendar gc = GregorianCalendar.getInstance();
            updateFile.setTestTransmissionDtm(gc.getTime());
            updateFile.setStatus(new StatusVo(StatusVo.READY_FOR_TESTING));
            try {
                sendEmailNotification(updateFile.getStatus(), updateFile.getFileName(), 
                        ndfUpdateProcessFile.getPendingFileSize(updateFile.getFileName()));
            }
            catch (Exception e) {
                //the file was sent, but the email or status updates failed.
                LOG.error("File was sent, but subsequent error occurred sending email.", e);
                updateFile.setComments(
                        "PPS-N/NDF Update Test File Transmission Successful, but subsequent errors encountered sending email. "
                                + "Log a CA ticket.");                
            }
        } 
        catch (final IOException e) {
          // Don't update the file status or anything, just re-wrap in a new IOException with an error message at the
          // appropriate level of detail. We'll rely on the calling method to update the file status.
          throw new IOException("Failed to transmit file to test folder.", e);
        } 
    }    
    
    /**
     * Transmit update file to reject - Renames and moves the file to the reject folder, sends an email notification, 
     * and stores the NdfUpdateFileVo with an updated status.  
     *
     * @param updateFile the update file
     * @param user the user
     * @throws IOException if the file could not be moved
     */
    private void transmitUpdateFileToReject(NdfUpdateFileVo updateFile, UserVo user) throws IOException {
        LOG.info("Preparing to rename and transmit " + updateFile.getFileName() + " file to the Rejected folder.");
        
        String rejectedFileName = ndfUpdateProcessFile.getRejectedFileName(updateFile.getFileName());
        ndfUpdateProcessFile.transmitFileToReject(updateFile.getFileName(), rejectedFileName);
        
        //If the caller specified VISTA_ERROR, store VISTA_ERROR, otherwise, set the status to REJECTED_BY_QA 
        if (updateFile.getStatus().getStatusId() != StatusVo.VISTA_ERROR)
        {
            updateFile.setStatus(new StatusVo(StatusVo.REJECTED_BY_QA));
        }
        
        // we rename the file name in the database also
        updateFile.setFileName(rejectedFileName);
        
        try {
          sendEmailNotification(updateFile.getStatus(), updateFile.getFileName(), 
                  ndfUpdateProcessFile.getRejectedFileSize(updateFile.getFileName()));
        } catch (final Exception ex) {
          // The file was sent, but the email failed. We don't want to fail the whole process for this.
          updateFile.setComments(
              "PPS-N/NDF Update Reject File Rename/Move Successful, but subsequent errors encountered sending email. "
                  + "Log a CA ticket.");
          LOG.error("File was sent, but subsequent error occurred sending email.", ex);
        }

        updateNdfUpdateFile(updateFile, user);      
    }       
    
    /**
     * Update the given NdfUpdateFileVo in the database.
     * 
     * @param updatedFile
     *            ManagedItemVo to update
     * @param user
     *            UserVo updating item
     */
    protected void updateNdfUpdateFile(NdfUpdateFileVo updatedFile, UserVo user) {
        LOG.debug("Updating NDFUpdateFile Info.");
        Calendar gc = GregorianCalendar.getInstance();

        updatedFile.setModifiedDate(gc.getTime());
        updatedFile.setStatusModifiedDtm(gc.getTime());
        updatedFile.setModifiedBy(user.getUsername());

        ndfUpdateFileMgtDomainCapability.update(updatedFile, user);
    }

    /**
     * Update the given NdfUpdateFileVo in the database, in a separate transaction.  Useful for updating status and comments 
     * during error scenarios.
     * 
     * @param updatedFile
     *            ManagedItemVo to update
     * @param user
     *            UserVo updating item
     */   
    protected void updateNdfUpdateFileInSeparateTxn(final NdfUpdateFileVo updatedFile, final UserVo user)
    {
      // Use TransactionTemplate because @Transactional does not work when calling methods on 'this'.
      final TransactionTemplate txTemplate = new TransactionTemplate(transactionManager);
      txTemplate.setPropagationBehavior(TransactionTemplate.PROPAGATION_REQUIRES_NEW);
      txTemplate.execute(new TransactionCallbackWithoutResult() {

        @Override
        protected void doInTransactionWithoutResult(final TransactionStatus txStatus) {
          updateNdfUpdateFile(updatedFile, user);       
        }
      });
    }

    /**
     * Insert the given NdfUpdateFileVo in the database.
     *
     * @param insertFile the insert file
     * @param user            UserVo updating item
     * @return the ndf update file vo
     */
    private NdfUpdateFileVo insertNdfUpdateFile(NdfUpdateFileVo insertFile, UserVo user) {
        NdfUpdateFileVo insertedFile = null;
        Calendar gc = GregorianCalendar.getInstance();

        insertFile.setModifiedBy(user.getUsername());
        insertFile.setModifiedDate(gc.getTime());
        insertFile.setStatusModifiedDtm(gc.getTime());

        insertedFile = ndfUpdateFileMgtDomainCapability.insert(insertFile,
                user);
        return insertedFile;
    }

    /**
     * Retrieve all NdfUpdateFileVo.
     * @return List<FileUpdateVo> updateFiles
     */
    @Override
    public List<NdfUpdateFileVo> retrieveFileUpdates() {
        return ndfUpdateFileMgtDomainCapability.retrieveAll();
    }

    /**
     * Retrieve paged NdfUpdateFileVo.
     *
     * @param searchCriteria
     *            the search criteria
     * @return List<FileUpdateVo> updateFiles
     */
    @Override
    public PaginatedList<NdfUpdateFileVo> retrievePagedFileUpdates(
            UpdateSearchCriteria searchCriteria) {
        return ndfUpdateFileMgtDomainCapability.retrievePaged(searchCriteria);
    }

    /**
     * Retrieve all the status values (EPL_STATUS).
     * @return List<StatusVo> all update file statuses
     */
    @Override
    public List<StatusVo> retrieveAllFileStatuses() {
        return ndfUpdateFileStatusDomainCapability.retrieveAll();
    }

    /**
     * Retrieve NdfUpdateFileVo ready for approval/rejection.
     * @return List<NdfUpdateFileVo> pendingUpdateFiles
     */
    @Override
    public List<NdfUpdateFileVo> retrievePendingUpdate() {
        return ndfUpdateFileMgtDomainCapability.retrievePending();
    }

    @Override
    public NdfUpdateFileVo retrieveById(Long id) {
        return ndfUpdateFileMgtDomainCapability.retrieveById(id);
    }

    /**
     * Retrieve a list of FileUpdateVo with initiated status.  Realistically, there should only ever be one.
     *
     * @return List<FileUpdateVo> initiatedUpdateFiles
     */
    @Override
    public List<NdfUpdateFileVo> retrieveInitiatedUpdate() {
        return ndfUpdateFileMgtDomainCapability.retrieveInitiated();
    }

    /**
     * gets the TransactionManager for NdfUpdateCapabilityImpl.
     * 
     * @return the transactionManager
     */
    public PlatformTransactionManager getTransactionManager() {
        return transactionManager;
    }

    /**
     * sets the TransactionManager for NdfUpdateCapabilityImpl.
     * 
     * @param transactionManager
     *            the transactionManager to set
     */
    @Resource
    public void setTransactionManager(
            PlatformTransactionManager transactionManager) {
        this.transactionManager = transactionManager;
    }

    /**
     * setNdfUpdateProcessCapability in NdfUpdateProcessCapabilityImpl.
     *
     * @param ndfUpdateProcessCapability the new NdfUpdateProcessCapability
     */
    public void setNdfUpdateProcessCapability(NdfUpdateProcessCapability ndfUpdateProcessCapability) {
        this.ndfUpdateProcessCapability = ndfUpdateProcessCapability;
    }

    /**
     * Gets the NdfUpdateFileMgtDomainCapability.
     *
     * @return the NdfUpdateFileMgtDomainCapability
     */
    public NdfUpdateFileMgtDomainCapability getNdfUpdateFileMgtDomainCapability() {
        return ndfUpdateFileMgtDomainCapability;
    }

    /**
     * Sets the NdfUpdateFileMgtDomainCapability.
     *
     * @param ndfUpdateFileMgtDomainCapability the new NdfUpdateFileMgtDomainCapability
     */
    public void setNdfUpdateFileMgtDomainCapability(
            NdfUpdateFileMgtDomainCapability ndfUpdateFileMgtDomainCapability) {
        this.ndfUpdateFileMgtDomainCapability = ndfUpdateFileMgtDomainCapability;
    }

    /**
     * Get the reference to the NdfUpdateProcessFile, which represents a physical file and its actions.
     *
     * @return the NdfUpdateProcessFile
     */
    public NdfUpdateProcessFile getNdfUpdateProcessFile() {
        return ndfUpdateProcessFile;
    }

    /**
     * Set the reference to the NdfUpdateProcessFile, which represents a physical file and its actions.
     *
     * @param ndfUpdateProcessFile the new NdfUpdateProcessFile
     */
    public void setNdfUpdateProcessFile(NdfUpdateProcessFile ndfUpdateProcessFile) {
        this.ndfUpdateProcessFile = ndfUpdateProcessFile;
    }

    /**
     * Gets the email service.
     *
     * @return the emailService
     */
    public EmailService getEmailService() {
        return emailService;
    }

    /**
     * Sets the email service.
     *
     * @param emailService the emailService to set
     */
    @Resource
    public void setEmailService(EmailService emailService) {
        this.emailService = emailService;
    }

    /**
     * Sends text email notification.
     *
     * @param status the status
     * @param filename the filename
     * @param fileSize the file size
     */
    private void sendEmailNotification(StatusVo status, String filename, String fileSize)
    {
        String toEmailIds = null;  
        String emailSubject = null;
        String emailMessage = null;
        
                
        if(status.isReadyForTesting()){                      
            // File ready for SQA            
            emailSubject="PPS-N/NDF File ["+ filename +"] CREATED FOR QA";
            emailMessage="\n\nThe PPS-N/NDF File " + filename + " (Size " + fileSize  + " bytes) "
                    + "has been CREATED and TRANSMITTED to the test sFTP.  It is now ready to "
                    + "be DOWNLOADED and INSTALLED.";
            toEmailIds = environment.getRequiredProperty(FILE_READY_FOR_TESTING_TO_EMAIL_PROPERTY);
         }  // File has been Approved/Released      
        else if(status.isTransmittedToProduction()){
            emailSubject="PPS-N/NDF File " + filename + " RELEASED NATIONALLY";
            emailMessage="\n\nThe PPS-N/NDF Update File " + filename + " (Size " + fileSize  + " bytes) "
                    + "has been Nationally Released to the Product sFTP and is available to be "
                    + "DOWNLOADED and INSTALLED at the local sites.";
            toEmailIds = environment.getRequiredProperty(FILE_APPROVED_TO_EMAIL_PROPERTY);
        }  // File has been Rejected
        else if (status.isRejectedByQA()){
            emailSubject="PPS-N/NDF File " + filename + " REJECTED";
            emailMessage="\n\nThe PPS-N/NDF Update Test File " + filename + " (Size " + fileSize  + " bytes) "
                    + "has been REJECTED by the PPS-N Manager.";
            toEmailIds = environment.getRequiredProperty(FILE_REJECTED_TO_EMAIL_PROPERTY);
        } else if ( status.isVistaError()) {
            emailSubject="PPS-N/NDF File " + filename + " AUTO REJECTED";
            emailMessage="\n\nThe PPS-N/NDF File " + filename + " (Size " + fileSize + " bytes) has been "
                    + "AUTO REJECTED by QA.";
           toEmailIds = environment.getRequiredProperty(FILE_REJECTED_TO_EMAIL_PROPERTY);
        }

        Email emailBean = new Email();

        /*
         * Need to set to in another way by converting to an array as
         * setting addresses like PII             PII              Gives
         * error by javax mail Address Parser.
         */
        String[] toEmailId = new String[] { "" };

        // if toEmailIdStr string is a comma separated string then split it.
        if (null != toEmailIds && toEmailIds.length() > 0) {
            toEmailId = toEmailIds.split(",");
        }

        emailBean.setTo(toEmailId);
        emailBean.setSubject(emailSubject);
        emailBean.setEmailMessage(emailMessage);

        emailService.sendEmailMutlipleTimes(emailBean);
    }  
    
    /**
     * Cleanup from a PPSN error during file creation or sending to test.
     *
     * @param ndfUpdateFileVo the NdfUpdateFileVo with {@code StatusVo.PPSN_ERROR} status. Must not be null.
     * @return true, if successful
     */   
    private boolean cleanupFromExceptionDuringCreateAndSendToTestProcess(NdfUpdateFileVo ndfUpdateFileVo) {
        try{            
            //cleanup the physical file on the local drive, if it exists.
            NdfUpdateProcessFile.deleteFile(ndfUpdateFileVo);
        }
        catch (Exception e1){
            LOG.error("Secondary exception encountered during cleanup after a primary exception.", e1);
        }        
        return true;
    }  
    
    /**
     * Gets the ndf update file status domain capability.
     *
     * @return the ndf update file status domain capability
     */
    public NdfUpdateFileStatusDomainCapability getNdfUpdateFileStatusDomainCapability() {
        return ndfUpdateFileStatusDomainCapability;
    }

    /**
     * Sets the ndf update file status domain capability.
     *
     * @param ndfUpdateFileStatusDomainCapability the new ndf update file status domain capability
     */
    public void setNdfUpdateFileStatusDomainCapability(
            NdfUpdateFileStatusDomainCapability ndfUpdateFileStatusDomainCapability) {
        this.ndfUpdateFileStatusDomainCapability = ndfUpdateFileStatusDomainCapability;
    }


    /**
     * Sets the {@link Environment} instance to use.
     * @param environment instance to use
     */
    @Resource
    public void setEnvironment(Environment environment) {
      this.environment = environment;
    }
}
