package gov.va.med.pharmacy.peps.service.common.capability.impl;

import gov.va.med.pharmacy.peps.common.utility.VistaFileNumber;
import gov.va.med.pharmacy.peps.common.vista.VistaProductFile;
import gov.va.med.pharmacy.peps.domain.common.capability.ProductDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.dao.EplNdfOutgoingDifferencesDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplRematchDao;
import gov.va.med.pharmacy.peps.domain.common.model.EplRematchDo;
import gov.va.med.pharmacy.peps.service.common.utility.NdfUpdateProcessFile;
import java.io.IOException;
import java.util.Date;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;

import static gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidationType.LOG_FORGING;
import static gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidator.validateStringInput;

/**
 * Writes miscellaneous sections (e.g., rematches and inactivated products) of the update file.
 */
public class NdfUpdateProcessMiscData {
    private static final org.apache.logging.log4j.Logger LOG = org.apache.logging.log4j.LogManager
        .getLogger(NdfUpdateProcessCapabilityImpl.class);
    
    private final ProductDomainCapability productDomainCapability;
    private final EplNdfOutgoingDifferencesDao eplNdfOutgoingDifferencesDao; 
    private final EplRematchDao eplRematchDao;
    
    final String newLine = System.getProperty("line.separator");
    
    /**
     * Constructs an instance with the required dependencies.
     * @param pProductDomainCapability
     * @param pEplNdfOutgoingDifferencesDao
     * @param pEplRematchSuggestionDao
     */
    @Autowired
    public NdfUpdateProcessMiscData(
        final ProductDomainCapability pProductDomainCapability,
        final EplNdfOutgoingDifferencesDao pEplNdfOutgoingDifferencesDao,
        final EplRematchDao pEplRematchSuggestionDao) {
      productDomainCapability = pProductDomainCapability;
      eplNdfOutgoingDifferencesDao = pEplNdfOutgoingDifferencesDao;
      eplRematchDao = pEplRematchSuggestionDao;
    }
    
    /**
     * Finds the automatic rematches that need to be put in the update file and puts them there. (REM section)
     * 
     * @param startDTM the start date of the update file process
     * @param ndfUpdateFile the update file to put the automatic rematches in
     * @throws IOException if there is an error when putting rows in the update file
     */
    public void rematchProducts(Date startDTM, NdfUpdateProcessFile ndfUpdateFile) throws IOException{
        
        final List<EplRematchDo> rematchesForInactivation =
            eplRematchDao.findOutgoingRematches(startDTM);
        for (final EplRematchDo rematch : rematchesForInactivation) {
          // This method is only sending automatic rematches.
          if (rematch.getToAutoProduct() != null) {
            
            final long oldIen = rematch.getFromProduct().getNdfProductIen();
            
            try {
              ndfUpdateFile.putNextRow("REM^" + oldIen);
              ndfUpdateFile.putNextRow(rematch.getToAutoProduct().getNdfProductIen().toString());
            } catch (IOException e) {
              LOG.error("error processing row for inactive products with ien of " + oldIen);
              throw e;
            }
          }
        }
    }

    //5000.68 data inactive product list
    public void inactiveProducts(Date startDTM, NdfUpdateProcessFile ndfUpdateFile) throws IOException{
        List<Object[]> inactiveProducts = productDomainCapability.retrieveInactive();
        List<Object[]> inactivatedProducts = 
            eplNdfOutgoingDifferencesDao.findInactivatedProductsForLocal(VistaFileNumber.FIFTY_68.getFileNumber(), startDTM);
        for (Object[] inactiveProduct : inactiveProducts) {
                
            try {
                ndfUpdateFile.putNextRow("PRODUCT^" + inactiveProduct[0]);
                ndfUpdateFile.putNextRow(" ");
            } catch (IOException e) {
              LOG.error("error processing row for inactive products with ien of " + validateStringInput(inactiveProduct[0].toString(), LOG_FORGING));
              throw e;
            }
        }
        
        for (Object[] inactivatedProduct : inactivatedProducts) {
            
            try {
                ndfUpdateFile.putNextRow("PRODUCT^" + inactivatedProduct[0]);
                ndfUpdateFile.putNextRow(" ");
            } catch (IOException e) {
              LOG.error("error processing row for unmatched products with ien of " + validateStringInput(inactivatedProduct[0].toString(), LOG_FORGING));
              throw e;
            }
        }
    }
    
    //5000.3 data inactive generic name list
    public void inactiveGenericName(Date startDTM, NdfUpdateProcessFile ndfUpdateFile) throws IOException {
        List<String> inactiveGenericNames = 
            eplNdfOutgoingDifferencesDao.findInactiveGenericName(VistaFileNumber.FIFTY_6.getFileNumber(),"1", startDTM);
        for (String ien : inactiveGenericNames) {            
            try {                
                ndfUpdateFile.putNextRow("GENERIC^" + ien);
                ndfUpdateFile.putNextRow(" ");
            } catch (IOException e) {
              LOG.error("error processing row for inactive generic name with ien of " + validateStringInput(ien, LOG_FORGING));
              throw e;
            }
    
        }
    }
    
    //5000.4 Strength Change data list
    public void productStrengthChanges(Date startDTM, NdfUpdateProcessFile ndfUpdateFile) throws IOException{
        List<Object[]> productStrengthChanges = 
            eplNdfOutgoingDifferencesDao.findByFileAndField(VistaProductFile.FILE_NUMBER,
                VistaProductFile.STRENGTH_FIELD_NUMBER, startDTM);
        for (Object[] productStrengthIen : productStrengthChanges){
             try {
                ndfUpdateFile.putNextRow("POE^" + productStrengthIen[0]);
                ndfUpdateFile.putNextRow(" ");
            } catch (IOException e) {
              LOG.error("error processing row for strength changes for product ien " + validateStringInput(productStrengthIen[0].toString(), LOG_FORGING));
              throw e;
            }
        }
    }
    
    //5000.5 National Formulary data list
    public void productFormularyChanges(Date startDTM, NdfUpdateProcessFile ndfUpdateFile) throws IOException{
        List<Object[]> productFormularyChanges = eplNdfOutgoingDifferencesDao.findByFileAndField(VistaProductFile.FILE_NUMBER,
            VistaProductFile.NATFORMULARYIND_FIELD_NUMBER, startDTM);
        for (Object[] productFormularyIen : productFormularyChanges){
             try {
                ndfUpdateFile.putNextRow("NFI^" + productFormularyIen[0]);
                ndfUpdateFile.putNextRow(" ");
            } catch (IOException e) {
              LOG.error("error processing row for formulary changes for product ien " + validateStringInput(productFormularyIen[0].toString(), LOG_FORGING));
              throw e;
            }
        }
    }
    
    //5000.6 Drug Class change
    public void productDrugClassChanges(Date startDTM, NdfUpdateProcessFile ndfUpdateFile) throws IOException{
        List<Object[]> productDrugClassChanges = eplNdfOutgoingDifferencesDao.findDrugClassChanges(VistaProductFile.FILE_NUMBER,
            VistaProductFile.PRIMARYDRUGCLASS_FIELD_NUMBER, startDTM);
        for (Object[] productDrugClassData : productDrugClassChanges){
            try {
                 ndfUpdateFile.putNextRow("CLASS^" + productDrugClassData[0] + "^0");
                 ndfUpdateFile.putNextRow(productDrugClassData[0] + "^" + productDrugClassData[1] + "^" + productDrugClassData[2] ); 
           } catch (IOException e) {
                LOG.error("error processing row for drug class changes for product ien " + validateStringInput(productDrugClassData[0].toString(), LOG_FORGING));
                throw e;
            }
        }
    }
    
    
    //5000.7 Unmark for CMOP
    public void productUnmarkForCmopChanges(Date startDTM, NdfUpdateProcessFile ndfUpdateFile) throws IOException{
        List<Object[]> productUnmarkForCmopChanges = eplNdfOutgoingDifferencesDao
            .findByFileAndField(VistaProductFile.FILE_NUMBER, VistaProductFile.CMOPINDICATOR_FIELD_NUMBER, startDTM);
        for (Object[] productCmopIen : productUnmarkForCmopChanges){
            if(productCmopIen[1].equals("0")){ 
                try {
                    ndfUpdateFile.putNextRow("CMOP^" + productCmopIen[0]);
                    ndfUpdateFile.putNextRow(" ");
                } catch (IOException e) {
                  LOG.error("error processing row for cmop indicator changes for product ien " + validateStringInput(productCmopIen[0].toString(), LOG_FORGING));
                  throw e;
                }
            }
        }
    }

    public EplRematchDao getEplRematchSuggestionDao() {
      return eplRematchDao;
    }       
}
