package gov.va.med.pharmacy.peps.service.common.capability.impl;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;

import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.apache.commons.collections4.ListUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.logging.log4j.LogManager;

import gov.va.med.pharmacy.peps.common.exception.DuplicateRematchException;
import gov.va.med.pharmacy.peps.common.utility.EnvironmentUtility;
import gov.va.med.pharmacy.peps.common.utility.security.UntrustedLogParam;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryDetailsVo;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryVo;
import gov.va.med.pharmacy.peps.common.vo.ManagedItemVo;
import gov.va.med.pharmacy.peps.common.vo.MatchReason;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.RematchStatus;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.domain.common.capability.RematchMgtDomainCapability;
import gov.va.med.pharmacy.peps.service.common.capability.ModRematch;
import gov.va.med.pharmacy.peps.service.common.capability.RematchCapability;
import gov.va.med.pharmacy.peps.service.common.capability.RematchPreperationBean;

/**
 * Main implementation of {@link RematchCapability}.
 */
public class RematchCapabilityImpl implements RematchCapability{

    private static final Logger LOG = LogManager.getLogger(RematchCapabilityImpl.class);

    private final RematchMgtDomainCapability rematchMgtDomainCapability;
    private final EnvironmentUtility environmentUtility;

    /**
     * Constructs an instance with the required dependencies.
     * 
     * @param pRematchMgtDomainCapability the {@link RematchMgtDomainCapability} to use. Must not be null.
     * @param pEnvironmentUtility the {@link EnvironmentUtility} to use. Must not be null.
     */
    @Autowired
    public RematchCapabilityImpl(
        final RematchMgtDomainCapability pRematchMgtDomainCapability, final EnvironmentUtility pEnvironmentUtility) {
        rematchMgtDomainCapability = Objects.requireNonNull(pRematchMgtDomainCapability);
        environmentUtility = Objects.requireNonNull(pEnvironmentUtility);
    }
    
    @Override
    public RematchPreperationBean prepareForRematch(ProductVo fromProduct, Set<MatchReason> matchReasons) {
        final long fromProductId = Long.parseLong(fromProduct.getId());

        // Get existing rematch, if any.
        Optional<RematchVo> rematchForProduct = rematchMgtDomainCapability.getRematchForProduct(fromProductId);

        // Get initial potential matches.
        final List<ProductVo> basePotentialMatches = rematchMgtDomainCapability.findRematchProducts(fromProductId);
        final boolean shouldDefaultToSameProduct = shouldDefaultToSameProduct(matchReasons);
        final List<ProductVo> potentialMatches = shouldDefaultToSameProduct
                ? ListUtils.union(Arrays.asList(fromProduct), basePotentialMatches)
                : basePotentialMatches;

        return new RematchPreperationBean(rematchForProduct.isPresent(), potentialMatches, shouldDefaultToSameProduct);
    }
    
    private boolean shouldDefaultToSameProduct(final Set<MatchReason> matchReasons) {
        // Should return true if and only if all match reasons return true.
        for (final MatchReason reason : matchReasons) {
            if (!reason.getDefaultToSameProduct()) {
                return false;
            }
        }
        
        return true;
    }
    
    @Override
    public void updateUponProposedInactivation(final ManagedItemVo item, UserVo user) {
        if (item instanceof ProductVo) {
            final long itemId = Long.parseLong(item.getId());
            final Optional<RematchVo> existingOptionalRematch = rematchMgtDomainCapability.getRematchForProduct(itemId);

            if (existingOptionalRematch.isPresent() && existingOptionalRematch.get().getStatus().equals(RematchStatus.FUTURE)) {
                final RematchVo existingRematch = existingOptionalRematch.get();
                final RematchStatus newStatus = RematchStatus.PENDING;
                LOG.debug("Updating match status to {} to associate it with the generated request.", newStatus);
                existingRematch.setStatus(newStatus);
                try {
                    rematchMgtDomainCapability.saveOrUpdate(existingRematch, user);
                } catch (final DuplicateRematchException e) {
                    // We are only updating the match reason and status here: this should not trigger a
                    // DuplicateRematchException.
                    throw new RuntimeException("Unexpected exception!", e);
                }
            }
        }
    }

    @Override
    public Optional<RematchVo> saveOrUpdateUncommittedRematch(
        final Collection<Difference> itemDiffs,
        final RematchVo rematch,
        final UserVo user)
            throws DuplicateRematchException {

        return saveOrUpdateRematch(itemDiffs, false, rematch, user);
    }

    @Override
    public Optional<RematchVo> updateRematchesOnItemCommit(
        final ManagedItemVo item, final Collection<Difference> committedDiffs, Optional<ModRematch> optionalModRematch,
        final UserVo user) throws DuplicateRematchException {

        LOG.debug("Updating rematches for changes {}.", new UntrustedLogParam(committedDiffs));

        // Multiple operations below check for the existing rematch in the database. Go ahead and pull it now.
        final long itemId = Long.parseLong(item.getId());
        final Optional<RematchVo> existingRematch = rematchMgtDomainCapability.getRematchForProduct(itemId);
        
        if (existingRematch.isPresent()) {
            deleteReversedRematch(existingRematch.get(), committedDiffs);
        }
        
        if (optionalModRematch.isPresent()) {
            return commitModRematchChanges(item, committedDiffs, existingRematch, optionalModRematch.get(), user);
        } else {
            return Optional.empty();
        }
    }

    /**
     * Commits the given {@link ModRematch} changes.
     * 
     * @param item the item being modified
     * @param committedDiffs the differences being committed along with the rematch changes, if any
     * @param existingRematch the existing rematch in the DB for the item, if any
     * @param modRematch the changes to commit
     * @param user the user performing the commit
     * @return the saved rematch, if any
     */
    private Optional<RematchVo> commitModRematchChanges(
        final ManagedItemVo item, final Collection<Difference> committedDiffs, final Optional<RematchVo> existingRematch,
        final ModRematch modRematch, final UserVo user) throws DuplicateRematchException {

        final Optional<RematchVo> optionalRematch = modRematch.getOptionalRematch();
        
        final Optional<RematchVo> deletedRematch = deletePreviousRematchWhenAppropriate(existingRematch, modRematch);
        
        // If the caller requested overriding any current rematch, auto-approve the given rematch. This avoids the
        // rematch being declared obsolete later due to no differences.
        approveOverrideRematch(modRematch);
        
        final ItemAuditHistoryVo newAuditHistory = ObjectUtils.defaultIfNull(
            item.getNewAuditHistory(), item.createModificationAuditHistory(user, environmentUtility));
        
        // Actually save the rematch if one given.
        final Optional<RematchVo> savedRematch = optionalRematch.isPresent()
            ? saveOrUpdateRematch(committedDiffs, true, optionalRematch.get(), user)
            : Optional.empty();

        // If there is no audit history entry at all (no diffs), then add one. There is a User Story (162889) to do
        // this for all rematches. This if block may need to be moved elsewhere if we add audit history for other rematches.
        if (committedDiffs.isEmpty() && (deletedRematch.isPresent() || savedRematch.isPresent())) {
            LOG.debug("Adding manual rematch audit history.");
            newAuditHistory.addDetail(new ItemAuditHistoryDetailsVo(
                newAuditHistory.getEventCategory(),
                modRematch.getReason(),
                FIELD_NAME,
                deletedRematch.isPresent() ? deletedRematch.get().getToProductsString() : Difference.NO_VALUE,
                savedRematch.isPresent() ? savedRematch.get().getToProductsString() : Difference.NO_VALUE));
        }
        
        // If we created the audit history, set it on the item.
        if (item.getNewAuditHistory() == null && !newAuditHistory.getDetails().isEmpty()) {
            item.setNewAuditHistory(newAuditHistory);
        }
        
        return savedRematch;
    }

    private void approveOverrideRematch(final ModRematch modRematch) {
        if (modRematch.getShouldOverride() && modRematch.getOptionalRematch().isPresent()) {
            final RematchStatus newStatus = RematchStatus.APPROVED;
            modRematch.getOptionalRematch().get().setStatus(newStatus);
            LOG.debug("Override rematch set to " + newStatus);
        }
    }
    
    /**
     * Updates the rematch status per the current set of differences and calls
     * {@link RematchMgtDomainCapability#saveOrUpdate(RematchVo, UserVo)}.
     * @param diffsCommitted whether the given differences have been committed to the item
     */
    private Optional<RematchVo> saveOrUpdateRematch(
        final Collection<Difference> itemDiffs, final boolean diffsCommitted, final RematchVo rematch, final UserVo user)
            throws DuplicateRematchException {

        rematch.updateStatusPerChanges(itemDiffs, diffsCommitted);
        
        if (rematch.getStatus().equals(RematchStatus.OBSOLETE)) {
            if (rematch.getEplRematchId() != null) {
                // The rematch has previously been saved to the database: delete it.
                rematchMgtDomainCapability.delete(rematch);
            }
            return Optional.empty();
        } else {
            return Optional.of(rematchMgtDomainCapability.saveOrUpdate(rematch, user));
        }
    }

    /**
     * Deletes the pre-existing rematch if it should be deleted.
     * @return the deleted rematch, if any
     */
    private Optional<RematchVo> deletePreviousRematchWhenAppropriate(
        Optional<RematchVo> previousRematch, final ModRematch modRematch) {
        if (previousRematch.isPresent() && modRematch.getShouldOverride()) {
            LOG.debug("Deleting the previous rematch for the given product.");
            delete(previousRematch.get());
            return previousRematch;
        }
        
        return Optional.empty();
    }

    /**
     * Deletes an unsent rematch for the given item that the given diffs are reversing. E.g., if there is an unsent rematch for
     * product inactivation and the given diffs are re-activating the product, then the rematch will be deleted.
     * 
     * @param item the affected item. May or may not have the changes applied.
     * @param committedDiffs the {@link Difference}s being applied to to the item
     */
    private void deleteReversedRematch(final RematchVo existingRematch, final Collection<Difference> committedDiffs) {
        if (existingRematch.isObsoletePerChanges(committedDiffs)) {
            rematchMgtDomainCapability.delete(existingRematch);
            // Log this at INFO level because there's no audit history for this operation. The log will provide us some
            // history in case of trouble.
            LOG.info(
                "Deleted pre-existing rematch for item {} since the committed changes reverse it.",
                new UntrustedLogParam(existingRematch.getFromProduct().getId()));
        }
    }

    @Override
    public void delete(RematchVo rematch) {
        rematchMgtDomainCapability.delete(rematch);    
    }

}
