package gov.va.med.pharmacy.peps.service.common.capability.impl;

import static gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidationType.LOG_FORGING;
import static gov.va.med.pharmacy.peps.common.utility.security.ESAPIValidator.validateStringInput;

import gov.va.med.pharmacy.peps.common.email.EmailService;
import gov.va.med.pharmacy.peps.common.utility.ConfigFileUtility;
import gov.va.med.pharmacy.peps.common.vo.NationalSetting;
import gov.va.med.pharmacy.peps.common.vo.NationalSettingVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.domain.common.capability.NationalSettingDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.model.EplProductDo;
import gov.va.med.pharmacy.peps.domain.common.utility.RxNormCapability;
import gov.va.med.pharmacy.peps.service.common.capability.RxNormUpdateCapability;
import gov.va.med.pharmacy.peps.service.common.scheduler.ProcessStatus;

import java.util.Date;
import java.util.List;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

/**
 * This class is the main implementation that manages the RxNorm update process.
 * 
 * @author VHAISHEdgecK
 *
 */
public class RxNormUpdateCapabilityImpl implements RxNormUpdateCapability {

    private static final Logger LOG = LogManager.getLogger(RxNormUpdateCapabilityImpl.class);
    private NationalSettingDomainCapability nationalSettingDomainCapability;
    private RxNormCapability rxNormCapability;
    private static final int TIMEOUT = 200;
    private static final int LONG_TIMEOUT = 1200;

    @Autowired
    private PlatformTransactionManager transactionManager;

    @Autowired
    private EmailService emailService;

    /*
     * (non-Javadoc)
     * 
     * @see
     * gov.va.med.pharmacy.peps.service.common.capability.RxNormUpdateCapability#process(gov.va.med.pharmacy.peps.common.vo.
     * UserVo)
     */
    @Override
    public void process(UserVo user) {

        try {
            updateJobStatusToRunning(user);

            List<EplProductDo> products = fetchRxNormUpdateProducts();

            processProducts(user, products);

            updateJobStatusToCompleted(user);

        } catch (Exception e) {
            e.printStackTrace();
            LOG.error("Exception in RxNormUpdateCapabilityImpl.process()  " + validateStringInput(e.getMessage(), LOG_FORGING),
                e);
            if (e.getCause() != null) {
                LOG.error("The Underlying cause is " + e.getCause().getMessage(), e);
            }
            updateJobStatusToError(user);
        }
    }

    /**
     * Calls the RxNorm domain capability to fetch and return a list of EPL Product objects that are enabled for rxNorm updates
     * 
     * @return the list of EPL Product objects to process
     */
    private List<EplProductDo> fetchRxNormUpdateProducts() {
        DefaultTransactionDefinition def;
        TransactionStatus status;
        def = new DefaultTransactionDefinition(TransactionDefinition.PROPAGATION_REQUIRES_NEW);
        def.setTimeout(LONG_TIMEOUT);
        status = transactionManager.getTransaction(def);

        List<EplProductDo> products = rxNormCapability.fetchRxNormUpdateProducts();
        transactionManager.commit(status);

        return products;
    }

    /**
     * Updates the RxNorm job status to Error
     * 
     * @param user
     * @param e
     */
    private void updateJobStatusToError(UserVo user) {
        DefaultTransactionDefinition def;
        TransactionStatus status;
        def = new DefaultTransactionDefinition(TransactionDefinition.PROPAGATION_REQUIRES_NEW);
        def.setTimeout(TIMEOUT);
        status = transactionManager.getTransaction(def);

        updateStatus(ProcessStatus.ERROR, user);

        transactionManager.commit(status);
    }

    /**
     * This method iterates thru a list of products, creates a new transaction for each product, and passes processing down into
     * the populate and persist method of the RxNorm domain capability object.
     * 
     * @param user
     * @param products the list of products to send to rxnorm
     */
    private void processProducts(UserVo user, List<EplProductDo> products) {
        // We create and assigned a brand new ConfigFileUtility instance every time we
        // run the RXNorm update process.
        ConfigFileUtility configFile = new ConfigFileUtility();
        Date runDate = new Date();
        for (EplProductDo product : products) {
            processEachProduct(user, product, runDate, configFile);
        }
    }

    /**
     * Processes each product
     * 
     * @param user
     * @param productDo
     * @param runDate
     */
    private void processEachProduct(UserVo user, EplProductDo productDo, Date runDate, ConfigFileUtility configFile) {
        final TransactionTemplate txTemplate = new TransactionTemplate(transactionManager);
        txTemplate.setPropagationBehavior(TransactionTemplate.PROPAGATION_REQUIRED);
        txTemplate.setTimeout(TIMEOUT);
        txTemplate.execute(new TransactionCallbackWithoutResult() {
            @Override
            protected void doInTransactionWithoutResult(TransactionStatus status) {
                rxNormCapability.populateAndPersist(user, productDo, runDate, configFile);
            }
        });
    }

    /**
     * Updates the RxNorm job status to Completed
     * 
     * @param user
     */
    private void updateJobStatusToCompleted(UserVo user) {
        DefaultTransactionDefinition def;
        TransactionStatus status;
        def = new DefaultTransactionDefinition(TransactionDefinition.PROPAGATION_REQUIRES_NEW);
        def.setTimeout(TIMEOUT);
        status = transactionManager.getTransaction(def);

        updateStatus(ProcessStatus.COMPLETED, user);
        updateRuntime(user);

        transactionManager.commit(status);
    }

    /**
     * Updates the RxNorm job status to Running
     * 
     * @param user
     */
    private void updateJobStatusToRunning(UserVo user) {
        DefaultTransactionDefinition def;
        TransactionStatus status;
        def = new DefaultTransactionDefinition(TransactionDefinition.PROPAGATION_REQUIRES_NEW);
        def.setTimeout(TIMEOUT);
        status = transactionManager.getTransaction(def);

        LOG.info("---------------processing RXNORM Update-------------------");

        updateStatus(ProcessStatus.RUNNING, user);

        transactionManager.commit(status);
    }

    /**
     * The updateRuntime method will update the RxNorm Update run status.
     * 
     * @param processStatus ProcessStatus
     */
    private void updateStatus(ProcessStatus processStatus, UserVo user) {

        // This code can be replaced with ProcessStatusUpdater.
        NationalSettingVo returnedSetting = nationalSettingDomainCapability
            .retrieve(NationalSetting.RXNORM_UPDATE_RUN_STATE.toString());

        returnedSetting.setStringValue(processStatus.toString());
        nationalSettingDomainCapability.update(returnedSetting, user);
    }

    /**
     * The updateRuntime method will update the RxNorm Update runtime date.
     */
    private void updateRuntime(UserVo user) {

        // This code can be replaced with ProcessStatusUpdater.
        NationalSettingVo returnedSetting = nationalSettingDomainCapability
            .retrieve(NationalSetting.RXNORM_UPDATE_LAST_RUN.toString());

        returnedSetting.setDateValue(new Date());
        nationalSettingDomainCapability.update(returnedSetting, user);
    }

    /**
     * gets the TransactionManager for RxNormUpdateCapabilityImpl.
     * 
     * @return the transactionManager
     */
    public PlatformTransactionManager getTransactionManager() {
        return transactionManager;
    }

    /**
     * sets the TransactionManager for RxNormUpdateCapabilityImpl.
     * 
     * @param transactionManager the transactionManager to set
     */
    public void setTransactionManager(PlatformTransactionManager transactionManager) {
        this.transactionManager = transactionManager;
    }

    /**
     * setNationalSettingDomainCapability in ProposedInactivateCapabilityImpl.
     * 
     * @param nationalSettingDomainCapability the nationalSettingDomainCapability to set
     */
    public void setNationalSettingDomainCapability(NationalSettingDomainCapability nationalSettingDomainCapability) {
        this.nationalSettingDomainCapability = nationalSettingDomainCapability;
    }

    /**
     * Gets the rx norm capability.
     * 
     * @return the rx norm capability
     */
    public RxNormCapability getRxNormCapability() {
        return rxNormCapability;
    }

    /**
     * Sets the rx norm capability.
     * 
     * @param rxNormCapability the new rx norm capability
     */
    public void setRxNormCapability(RxNormCapability rxNormCapability) {
        this.rxNormCapability = rxNormCapability;
    }

    /**
     * @return the emailService
     */
    public EmailService getEmailService() {
        return emailService;
    }

    /**
     * @param emailService the emailService to set
     */
    public void setEmailService(EmailService emailService) {
        this.emailService = emailService;
    }


}
