package gov.va.med.pharmacy.peps.service.common.migration;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import gov.va.med.pharmacy.peps.common.vo.MigrationRow;

/**
 * This class writes the result file for a migration. The result file contains every row from the import file,
 * plus an error flag column to indicate whether a validation error occurred for that row, as well as an error message column
 * describing the error.
 *
 * @param <R> the generic type of MigrationRow processed
 */
public abstract class AbstractMigrationResultFileWriter<R extends MigrationRow> {
    /**
     * The class's logger.
     */
    private static final Logger LOG = LogManager.getLogger(AbstractMigrationResultFileWriter.class);

    /**
     * Write the given rows to the given resultFile, per business rules.
     *
     * @param rows to write
     * @param resultFile which should contain rows in error
     * @param sheetName the name to apply to the sheet
     */
    public void write(List<R> rows, File resultFile, String sheetName) {
        try (XSSFWorkbook workbook = new XSSFWorkbook()) {
            XSSFSheet sheet = workbook.createSheet(sheetName);
            sheet = addHeaderRow(sheet);
            sheet = addDetailRows(sheet, rows);

            try (FileOutputStream out = new FileOutputStream(resultFile)) {
                workbook.write(out);
            } catch (IOException e) {
                LOG.error("Failed to write Migration results file.", e);
            }
        } catch (IOException e) {
            LOG.error("Failed to close Migration results file.", e);
        }
    }

    private XSSFSheet addDetailRows(XSSFSheet sheet, List<R> rows) {
        int outputRowNumber = 0;

        for (R row : rows) {
            sheet = addDetailRow(sheet, row, ++outputRowNumber);
        }

        return sheet;
    }

    /**
     * Adds the detail row.
     *
     * @param sheet the sheet
     * @param importRow the import row
     * @param rowNumber the row number
     * @return the XSSF sheet
     */
    protected abstract XSSFSheet addDetailRow(XSSFSheet sheet, R importRow, int rowNumber);
    
    /**
     * Adds the header row.
     *
     * @param sheet the sheet
     * @return the XSSF sheet
     */
    protected abstract XSSFSheet addHeaderRow(XSSFSheet sheet);
}
