package gov.va.med.pharmacy.peps.service.common.migration;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.POIXMLException;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import gov.va.med.pharmacy.peps.common.vo.HazardousWasteImportRow;
import gov.va.med.pharmacy.peps.service.common.session.impl.MigrationImportFormatException;

/**
 * Reads {@link HazardousWasteImportRow}s from Excel spreadsheets.
 */
public class HazardousWasteImportReader extends AbstractMigrationImportReader<HazardousWasteImportRow>{
    /**
     * The index of the workbook sheet containing the hazardous waste data.
     */
    private static final int SHEET_INDEX = 0;

    /**
     * Converts all rows of the first sheet of the given Excel workbook.
     * 
     * @param is the stream providing the Excel workbook. Must be closed by the caller.
     * @return a List containing one {@link HazardousWasteImportRow} for each data row in the spreadsheet, or an empty list if
     *         there are no data rows.
     * @throws IOException if the given InputStream cannot be opened.
     * @throws POIXMLException if the given InputStream is not a spreadsheet file (.xlsx only).
     * @throws MigrationImportFormatException if the EPL ID column is not formatted as text in the Excel spreadsheet, if
     *             the first row is not a header row, or if the columns are in the wrong order.
     */
    @Override
    public List<HazardousWasteImportRow> getRows(InputStream is)
        throws IOException, POIXMLException, MigrationImportFormatException {
        List<HazardousWasteImportRow> rows = new ArrayList<HazardousWasteImportRow>();
        MigrationImportFormatException importFormatException = null;
        XSSFWorkbook workbook = null;

        try {
            workbook = new XSSFWorkbook(is);
            XSSFSheet sheet = workbook.getSheetAt(SHEET_INDEX);
            Iterator<Row> iterator = sheet.iterator();
            iterator = discardHeaderRow(iterator, HazardousWasteMigrationConstants.VUID_COLUMN, 
                HazardousWasteMigrationConstants.VUID_COLUMN_HEADER);

            while (iterator.hasNext()) {
                Row xlsxRow = iterator.next();
                String vuid = getCellValue(xlsxRow.getCell(HazardousWasteMigrationConstants.VUID_COLUMN));
                String eplIdString = getCellValue(xlsxRow.getCell(HazardousWasteMigrationConstants.EPL_ID_COLUMN));
                Long eplId = (StringUtils.isEmpty(eplIdString)) ? null : Long.valueOf(eplIdString);
                String cmopId = getCellValue(xlsxRow.getCell(HazardousWasteMigrationConstants.CMOP_ID_COLUMN));
                String ndfProductIenString = getCellValue(
                    xlsxRow.getCell(HazardousWasteMigrationConstants.NDF_PRODUCT_IEN_COLUMN));
                Long ndfProductIen = (StringUtils.isEmpty(ndfProductIenString)) ? null : Long.valueOf(ndfProductIenString);
                String vaProductName = getCellValue(xlsxRow.getCell(HazardousWasteMigrationConstants.VA_PRODUCT_NAME_COLUMN));
                String genericName = getCellValue(xlsxRow.getCell(HazardousWasteMigrationConstants.GENERIC_NAME_COLUMN));
                String scheduleName = getCellValue(xlsxRow.getCell(HazardousWasteMigrationConstants.SCHEDULE_NAME_COLUMN));
                String dosageFormName = getCellValue(xlsxRow.getCell(HazardousWasteMigrationConstants.DOSAGE_FORM_NAME_COLUMN));
                String haz2Handle = getCellValue(xlsxRow.getCell(HazardousWasteMigrationConstants.HAZARDOUS_TO_HANDLE_COLUMN));
                String haz2Dispose = getCellValue(
                    xlsxRow.getCell(HazardousWasteMigrationConstants.HAZARDOUS_TO_DISPOSE_COLUMN));
                String primaryEpa = getCellValue(xlsxRow.getCell(HazardousWasteMigrationConstants.PRIMARY_EPA_COLUMN));
                String wasteSortCode = getCellValue(xlsxRow.getCell(HazardousWasteMigrationConstants.WASTE_SORT_CODE_COLUMN));
                String dotShippingName = getCellValue(
                    xlsxRow.getCell(HazardousWasteMigrationConstants.DOT_SHIPPING_NAME_COLUMN));

                HazardousWasteImportRow resultRow = new HazardousWasteImportRow(vuid, eplId, cmopId, ndfProductIen,
                    vaProductName, genericName, scheduleName, dosageFormName, haz2Dispose, haz2Handle, primaryEpa,
                    wasteSortCode, dotShippingName);
                rows.add(resultRow);
            }
        } catch (IllegalArgumentException e) {
            importFormatException = new MigrationImportFormatException(e.getMessage());
        } catch (IllegalStateException e) {
            importFormatException = new MigrationImportFormatException(
                "Please ensure the EPL ID column is formatted as text in Excel.");
        } finally {
            if (workbook != null) {
                workbook.close();
            }
        }

        if (importFormatException != null) {
            throw importFormatException;
        }

        return rows;
    }

}
