package gov.va.med.pharmacy.peps.service.common.session.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import gov.va.med.pharmacy.peps.common.utility.PpsConfiguration;
import gov.va.med.pharmacy.peps.common.vo.MigrationRow;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.service.common.migration.AbstractMigrationResultFileWriter;
import gov.va.med.pharmacy.peps.service.common.migration.MigrationStatus;

/**
 * Main implementation of {@link MigrationService}.
 *
 * @param <R> the type of MigrationRow to be processed
 * @param <V> the type of AbstractMigrationImportRowValidator to be used for validation
 * @param <M> the type of AbstractMigrator to be used for migration
 * @param <W> the type of AbstractMigrationResultFileWriter to be used for writing the results file
 */
public abstract class AbstractMigrationServiceImpl<R extends MigrationRow, 
        V extends AbstractMigrationImportRowValidator<R>, 
        M extends AbstractMigrator<R>, 
        W extends AbstractMigrationResultFileWriter<R>> 
        implements MigrationService<R>{
  
    /** The config. */
    private PpsConfiguration config;

    /** The validator. */
    private V validator;

    /** The migrator. */
    private M migrator;
    
    /** The status. */
    private MigrationStatus status;

    /** The writer. */
    private W writer;

    /** The user. */
    private UserVo user;
    
    /**
     * The constructor.
     *
     * @param config containing application configuration settings
     * @param validator for checking migration business rules
     * @param migrator for processing rows
     * @param writer for creating the result file
     */
    @Autowired
    public AbstractMigrationServiceImpl(
            PpsConfiguration config,
            V validator,
            M migrator,
            W writer) {
        this.config = config;
        this.validator = validator;
        this.migrator = migrator;
        this.writer = writer;
        this.status = new MigrationStatus();
        this.user = null;
    }
    
    /* (non-Javadoc)
     * @see gov.va.med.pharmacy.peps.service.common.session.impl.MigrationService#process(java.util.List)
     */
    @Override
    @Transactional
    public List<R> process(List<R> rows) {
        int rowCounter = 0;
        
        for(R row : rows) {
            row = validator.validate(row);
            
            if(row.isValid()) {
                row = migrator.migrate(row, user);
            }
            
            synchronized(this) {
                status.setProgressCount(++rowCounter);
            }
        }
        
        return rows;
    }

    /* (non-Javadoc)
     * @see gov.va.med.pharmacy.peps.service.common.session.impl.MigrationService#initialize(int, 
     * gov.va.med.pharmacy.peps.common.vo.UserVo)
     */
    @Override
    public synchronized void initialize(int rowCount, UserVo migrationUser) 
            throws MigrationInProgressException {
        if(MigrationStatus.INPROGRESS.equals(status.getStatus())) {
            throw new MigrationInProgressException();
        }
        
        status.setProgressCount(0);
        status.setTotalCount(rowCount);
        status.setStatus(MigrationStatus.INPROGRESS);
        this.user = migrationUser;
    }
    
    /* (non-Javadoc)
     * @see gov.va.med.pharmacy.peps.service.common.session.impl.MigrationService#getStatus()
     */
    @Override
    public synchronized MigrationStatus getStatus() {
        return status;
    }
    
    /* (non-Javadoc)
     * @see gov.va.med.pharmacy.peps.service.common.session.impl.MigrationService#writeResultFile(java.util.List, 
     * String)
     */
    @Override
    public void writeResultFile(List<R> rows, String sheetName) {
        writer.write(rows, getResultFile(), sheetName);
        
        synchronized(this) {
            status.setStatus(MigrationStatus.DONE);
        }
    }

    /**
     * Gets the config.
     *
     * @return the config
     */
    protected final PpsConfiguration getConfig() {
        return config;
    }

    /**
     * Sets the config.
     *
     * @param config the config to set
     */
    protected final void setConfig(PpsConfiguration config) {
        this.config = config;
    }

    /**
     * Gets the validator.
     *
     * @return the validator
     */
    protected final V getValidator() {
        return validator;
    }

    /**
     * Sets the validator.
     *
     * @param validator the validator to set
     */
    protected final void setValidator(V validator) {
        this.validator = validator;
    }

    /**
     * Gets the migrator.
     *
     * @return the migrator
     */
    protected final M getMigrator() {
        return migrator;
    }

    /**
     * Sets the migrator.
     *
     * @param migrator the migrator to set
     */
    protected final void setMigrator(M migrator) {
        this.migrator = migrator;
    }

    /**
     * Gets the writer.
     *
     * @return the writer
     */
    protected final W getWriter() {
        return writer;
    }

    /**
     * Sets the writer.
     *
     * @param writer the writer to set
     */
    protected final void setWriter(W writer) {
        this.writer = writer;
    }

    /**
     * Gets the user.
     *
     * @return the user
     */
    protected final UserVo getUser() {
        return user;
    }

    /**
     * Sets the user.
     *
     * @param user the user to set
     */
    protected final void setUser(UserVo user) {
        this.user = user;
    }

    /**
     * Sets the status.
     *
     * @param status the status to set
     */
    protected final void setStatus(MigrationStatus status) {
        this.status = status;
    }
}
