/**
 * Source file created in 2008 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.service.common.session.impl;


import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionException;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

import gov.va.med.pharmacy.peps.common.exception.CsvFileException;
import gov.va.med.pharmacy.peps.common.vo.NationalSetting;
import gov.va.med.pharmacy.peps.common.vo.ReportProductVo;
import gov.va.med.pharmacy.peps.common.vo.ReportType;
import gov.va.med.pharmacy.peps.service.common.session.AbstractReportExportProcess;
import gov.va.med.pharmacy.peps.service.common.session.ReportIngredientProcess;
import gov.va.med.pharmacy.peps.service.common.utility.ReportListIngredientCsvFile;


/** Generates the Ingredient list report in a background process. */
@Service
public class ReportIngredientProcessImpl 
    extends AbstractReportExportProcess implements ReportIngredientProcess {

    private static final Logger LOG = org.apache.logging.log4j.LogManager.getLogger(ReportIngredientProcessImpl.class);
    private static final String ERROR = "Cound not open Ingredient file for export: ";

    /** 
     * Generates Ingredient list Report.
     * @throws InterruptedException 
     */
    @Override
    public void doExport() throws InterruptedException {
        LOG.debug("Exporting Ingredient List Report");

        DefaultTransactionDefinition def = null;
        TransactionStatus status = null;
        int i = 0;

        try {
            //Set start time
            setDateValue(NationalSetting.REPORT_INGREDIENT_START, getDateTime());

            ReportListIngredientCsvFile reportListIngredientCsvFile = new ReportListIngredientCsvFile();

            try {
                reportListIngredientCsvFile.createFile();
            } catch (CsvFileException e) {
                /*
                 * TODO: There is no indication to the user that an exception has occurred.
                 */
                LOG.error("Could not create List Ingredients file for export: " + e);
            }

            def = new DefaultTransactionDefinition(TransactionDefinition.PROPAGATION_REQUIRES_NEW);
            def.setTimeout(TIMEOUT);
            status = getTransactionManager().getTransaction(def);
            List<Long> list = getReportDomainCapability().getIds(ReportType.LIST_INGREDIENTS);
            getExportState().setRecordTotal(list.size());
            getTransactionManager().commit(status);

            // Notify observer
            notifyObservers();

            List<Long> actionList = new ArrayList<Long>(BATCH_SIZE);

            for (Long eplId : list) {
                actionList.add(eplId);
                i++;

                if (actionList.size() >= BATCH_SIZE) {
                    processBatch(i, reportListIngredientCsvFile, list, actionList);
                }
            }

            //Write any records where the remaining total is less than the
            //batch size.
            if (actionList.size() > 0) {
                processBatch(i, reportListIngredientCsvFile, list, actionList);
            }

            reportListIngredientCsvFile.closeExport();

            //Set end time
            setDateValue(NationalSetting.REPORT_INGREDIENT_COMPLETE, getDateTime());

        } catch (TransactionException e) {
            LOG.error("FAILED TO retrieve List. Error Message is " + e.getMessage());
            this.stopProcess();
        }
    }

    private void processBatch(int recordCounter, ReportListIngredientCsvFile reportListIngredientCsvFile, List<Long> list,
            List<Long> actionList) {
        DefaultTransactionDefinition def;
        TransactionStatus status;
        LOG.info("Ingredient Processing: " + recordCounter + " of " + list.size());
        def = new DefaultTransactionDefinition(TransactionDefinition.PROPAGATION_REQUIRES_NEW);
        def.setTimeout(TIMEOUT);
        status = getTransactionManager().getTransaction(def);
        List<ReportProductVo> rtList = 
                getReportDomainCapability().getProductIngredientData(actionList);
        getTransactionManager().commit(status);

        // print the ingredient list
        try {
            reportListIngredientCsvFile.printListIngredient(rtList);
        } catch (CsvFileException e) {
            /*
             * TODO: There is no indication to the user that an exception has occurred.
             * This exception would occur if there was an error writing to the file
             * on disk. The report might be incomplete or missing records, and the user
             * wouldn't know.
             */
            LOG.error(ERROR + e);
        }

        // the action list needs to be cleared for the next run through the loop
        actionList.clear();

        // Update record count
        getExportState().setRecordCounter(recordCounter);
        notifyObservers();
    }
}
