package gov.va.med.pharmacy.peps.service.common.capability.impl;

import static org.junit.Assert.assertEquals;
import static org.quartz.JobBuilder.newJob;
import static org.quartz.SimpleScheduleBuilder.simpleSchedule;
import static org.quartz.TriggerBuilder.newTrigger;

import org.junit.Test;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.quartz.Trigger.TriggerState;
import org.quartz.impl.StdScheduler;
import org.quartz.impl.StdSchedulerFactory;

import gov.va.med.pharmacy.peps.service.common.scheduler.FdbJobNames;
import gov.va.med.pharmacy.peps.service.common.scheduler.jobs.FdbAddJob;
import gov.va.med.pharmacy.peps.service.common.scheduler.jobs.FdbUpdateJob;
import gov.va.med.pharmacy.peps.service.common.scheduler.jobs.InactivationCheckJob;

/**
 * Unit tests for {@link FdbSchedulerProcessCapabiltyImpl}.
 */
public class FdbSchedulerProcessCapabiltyImplTest {

  @Test
  public void testGetJobKey() {
    /* Setup */
    final FdbSchedulerProcessCapabiltyImpl capability = new FdbSchedulerProcessCapabiltyImpl();
    final JobKey expectedRxNorm = new JobKey(
        FdbJobNames.RXNORM_UPDATE_JOB.getJobName(), FdbSchedulerProcessCapabiltyImpl.FDBGROUP);
    final JobKey expectedFdbAdd = new JobKey(
        FdbJobNames.FDB_ADD_JOB.getJobName(), FdbSchedulerProcessCapabiltyImpl.FDBGROUP);
    
    /* Behavior & Verification */
    assertEquals(expectedRxNorm, capability.getJobKey(FdbJobNames.RXNORM_UPDATE_JOB));
    assertEquals(expectedFdbAdd, capability.getJobKey(FdbJobNames.FDB_ADD_JOB));
  }

  @Test
  public void testGetQuartzTriggerState() throws SchedulerException {
    
    /* Setup */
    final FdbSchedulerProcessCapabiltyImpl capability = new FdbSchedulerProcessCapabiltyImpl();
    final Scheduler scheduler = StdSchedulerFactory.getDefaultScheduler();
    // Note: do NOT start the scheduler. We don't want any jobs to run.

    // Create a job with one normal trigger.
    final JobKey fdbAddJobKey = capability.getJobKey(FdbJobNames.FDB_ADD_JOB);
    final JobDetail fdbAddJob = newJob(FdbAddJob.class)
        .withIdentity(fdbAddJobKey)
        .build();
    final Trigger normalTrigger = newTrigger()
        .startNow()
        .withSchedule(simpleSchedule().withIntervalInHours(1))
        .build();
    scheduler.scheduleJob(fdbAddJob, normalTrigger);
    
    // Create a job with no triggers.
    final JobKey fdbUpdateJobKey = capability.getJobKey(FdbJobNames.FDB_UPDATE_JOB);
    final JobDetail fdbUpdateJob = newJob(FdbUpdateJob.class)
        .withIdentity(fdbUpdateJobKey)
        .storeDurably()
        .build();
    scheduler.addJob(fdbUpdateJob, false);
    
    // Create a job with a paused trigger.
    final JobKey inactivationJobKey = capability.getJobKey(FdbJobNames.INACTIVATION_CHECK_JOB);
    final JobDetail inactivationJob = newJob(InactivationCheckJob.class)
        .withIdentity(inactivationJobKey)
        .build();
    final Trigger pausedTrigger = newTrigger()
        .startNow()
        .withSchedule(simpleSchedule().withIntervalInHours(2))
        .build();
    scheduler.scheduleJob(inactivationJob, pausedTrigger);
    scheduler.pauseJob(inactivationJobKey);

    // For some reason, the CapabilityImpl needs an StdScheduler instead of a normal Scheduler. Just
    // cast it since it is one already.
    capability.setScheduler((StdScheduler)scheduler);
    
    /* Behavior & Verification */
    assertEquals(TriggerState.NORMAL, capability.getQuartzTriggerState(fdbAddJobKey));
    assertEquals(TriggerState.PAUSED, capability.getQuartzTriggerState(fdbUpdateJobKey));
    assertEquals(TriggerState.PAUSED, capability.getQuartzTriggerState(inactivationJobKey));
  }

}
