package gov.va.med.pharmacy.peps.service.common.capability.impl;

import static gov.va.med.pharmacy.peps.service.national.updatefile.UpdateFileReportFormatter.formatPairWithSquareBraces;
import static gov.va.med.pharmacy.peps.service.national.updatefile.UpdateFileReportFormatter.doubleIndent;
import static gov.va.med.pharmacy.peps.service.national.updatefile.UpdateFileReportFormatter.indentLevel2;
import static org.easymock.EasyMock.*;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.time.LocalDate;
import java.util.Arrays;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Optional;

import org.easymock.EasyMockSupport;
import org.junit.Before;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.utility.VistaFileNumber;
import gov.va.med.pharmacy.peps.common.vista.VistaFile;
import gov.va.med.pharmacy.peps.common.vista.VistaProductFile;
import gov.va.med.pharmacy.peps.common.vo.MatchReason;
import gov.va.med.pharmacy.peps.domain.common.dao.EplNdfOutgoingDifferencesDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplProductDao;
import gov.va.med.pharmacy.peps.domain.common.dao.EplRematchDao;
import gov.va.med.pharmacy.peps.domain.common.model.dto.ProductChangeDto;
import gov.va.med.pharmacy.peps.service.common.update.CopayTierChange;
import gov.va.med.pharmacy.peps.service.national.updatefile.ReportMessageCode;
import gov.va.med.pharmacy.peps.service.national.updatefile.UpdateFileReportGenerator;

/**
 * Unit tests for {@link MessageDataCapabilityImpl}.
 */
public class MessageDataCapabilityImplTest extends EasyMockSupport {

    /**
     * Mock stored as a member variable for convenient creation in {@link #setup()}.
     */
    private EplNdfOutgoingDifferencesDao mockEplNdfOutgoingDifferencesDao;

    /**
     * Mock stored as a member variable for convenient creation in {@link #setup()}.
     */
    private EplRematchDao mockEplRematchDao;

    /**
     * Mock stored as a member variable for convenient creation in {@link #setup()}.
     */
    private EplProductDao mockEplProductDao;

    /**
     * Mock stored as a member variable for convenient creation in {@link #setup()}.
     */
    private UpdateFileReportGenerator mockUpdateFileReportGenerator;
    
    /**
     * Instantiates all the mock objects for convenience.
     */
    @Before
    public void setup() {
        mockEplNdfOutgoingDifferencesDao = mock(EplNdfOutgoingDifferencesDao.class);
        mockEplRematchDao = mock(EplRematchDao.class);
        mockEplProductDao = mock(EplProductDao.class);
        mockUpdateFileReportGenerator = mock(UpdateFileReportGenerator.class);
    }

    /**
     * Builds a {@link MessageDataCapabilityImpl} with the member variable mock objects.
     */
    private MessageDataCapabilityImpl buildWithMocks() {
        return new MessageDataCapabilityImpl(
            mockEplNdfOutgoingDifferencesDao, mockEplRematchDao, mockEplProductDao, mockUpdateFileReportGenerator);
    }
    
    /**
     * Tests {@link MessageDataCapabilityImpl#buildDataUpdateForNdfReport(Date)} with the minimal possible data.
     */
    @Test
    public void testBuildDataUpdateForNdfReport_Minimal() {
        /* Setup */
        
        final Date testDate = new Date();
        final String natFormularyHeaderText = "Nat Formulary Header";
        final String formularyDesignatorHeaderText = "Formulary Designator Header";
        
        final List<String> expectedReport = Arrays.asList(
            // Nat Formulary report - always included even if no changes
            natFormularyHeaderText,
            "  " + MessageDataCapabilityImpl.FORMULARY,
            doubleIndent("NONE"),
            "",
            "  " + MessageDataCapabilityImpl.NONFORMULARY,
            doubleIndent("NONE"),
            "",
            // Formulary designator change report - always included even if no changes
            formularyDesignatorHeaderText,
            doubleIndent("NONE"),
            "",
            "");
        
        final MessageDataCapabilityImpl objUnderTest = buildWithMocks();
        
        /* Expected method calls */
        // Common calls
        expect(mockEplRematchDao.findOutgoingRematches(testDate)).andReturn(Collections.emptyList()).anyTimes();
        // Report-specific calls
        expect(mockEplNdfOutgoingDifferencesDao.findAddedProducts(VistaFileNumber.FIFTY_68.getFileNumber(), ".01", testDate))
            .andReturn(Collections.emptyList());
        expect(mockEplNdfOutgoingDifferencesDao.findInactivatedCMOPProducts(testDate))
            .andReturn(Collections.emptyList());
        expect(mockEplProductDao.findFutureInactive()).andReturn(Collections.emptyList());
        expect(mockEplRematchDao.findUnsentRematches()).andReturn(Collections.emptyList());
        expect(mockEplNdfOutgoingDifferencesDao.findInactivatedProducts(testDate))
            .andReturn(Collections.emptyList());
        expect(
            mockEplNdfOutgoingDifferencesDao.findReactivatedProducts(VistaFileNumber.FIFTY_68.getFileNumber(), "21", testDate))
            .andReturn(Collections.emptyList());
        expect(mockEplNdfOutgoingDifferencesDao.findActiveNatFormularyProducts(
                VistaFileNumber.FIFTY_68.getFileNumber(), "17", testDate))
            .andReturn(Collections.emptyList());
        expect(
            mockEplNdfOutgoingDifferencesDao.findInactiveNatFormularyProducts(
                VistaFileNumber.FIFTY_68.getFileNumber(), "17", testDate))
            .andReturn(Collections.emptyList());
        expect(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.NATIONALFORMULARY))
            .andReturn(Arrays.asList(natFormularyHeaderText));
        expect(mockEplNdfOutgoingDifferencesDao.findFormularyDesignatorProducts(testDate))
            .andReturn(Collections.emptyList());
        expect(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.FORMULARYDESIGNATOR))
            .andReturn(Arrays.asList(formularyDesignatorHeaderText));
        expect(mockEplNdfOutgoingDifferencesDao.findCopayTierChanges(testDate)).andReturn(Collections.emptyList());
        expect(mockEplNdfOutgoingDifferencesDao.genNameAllChange(testDate))
            .andReturn(Collections.emptyList()).atLeastOnce();  // The other rematch report may call this one, too.
        expect(mockEplNdfOutgoingDifferencesDao.messageGenericChange(VistaProductFile.VAPRINTNAME_FIELD_NUMBER, testDate))
            .andReturn(Collections.emptyList()).atLeastOnce();  // The other rematch report may call this one, too.
        expect(mockEplNdfOutgoingDifferencesDao.findCMOPIdChanges(testDate)).andReturn(Collections.emptyList());
        // Strength change report
        expect(mockEplNdfOutgoingDifferencesDao.messageGenericChange(VistaProductFile.STRENGTH_FIELD_NUMBER, testDate))
            .andReturn(Collections.emptyList()).atLeastOnce();  // The other rematch report may call this one, too.
        // Drug units change report
        expect(mockEplNdfOutgoingDifferencesDao.drugUnitsChange(testDate))
            .andReturn(Collections.emptyList()).atLeastOnce();  // The other rematch report may call this one, too.
        // Dispense units change report
        expect(mockEplNdfOutgoingDifferencesDao.dispenseUnitsChange(testDate))
            .andReturn(Collections.emptyList()).atLeastOnce();  // The other rematch report may call this one, too.
        // Schedule change report
        expect(mockEplNdfOutgoingDifferencesDao.scheduleAllChange(VistaFileNumber.FIFTY_68.getFileNumber(), "19", testDate))
            .andReturn(Collections.emptyList());
        // Possible dosages report.
        expect(mockEplNdfOutgoingDifferencesDao.findYesPossibleDosageProducts(
                VistaFileNumber.FIFTY_68.getFileNumber(), "40", testDate))
            .andReturn(Collections.emptyList());
        expect(mockEplNdfOutgoingDifferencesDao.findNoPossibleDosageProducts(
                VistaFileNumber.FIFTY_68.getFileNumber(), testDate))
            .andReturn(Collections.emptyList());
        // Override Dose Check Report
        expect(mockEplNdfOutgoingDifferencesDao.findOverrideDFChkExcl(VistaFileNumber.FIFTY_68.getFileNumber(), "31", testDate))
            .andReturn(Collections.emptyList());
        // Dosage form change report
        expect(mockEplNdfOutgoingDifferencesDao.dosageFormChange(testDate))
            .andReturn(Collections.emptyList()).atLeastOnce();  // The other rematch report may call this one, too.
        // Other change rematches report
        // - No new expected calls for this one.
        // Class change report
        expect(mockEplNdfOutgoingDifferencesDao.classChange(VistaFileNumber.FIFTY_68.getFileNumber(), "15", testDate))
            .andReturn(Collections.emptyList());
        // New class report
        expect(mockEplNdfOutgoingDifferencesDao.findNewClassProducts(
                VistaFileNumber.FIFTY_605.getFileNumber(), ".01", testDate))
            .andReturn(Collections.emptyList());

        /* Behavior & Verification */
        replayAll();
        
        final List<String> actualReport = objUnderTest.buildDataUpdateForNdfReport(testDate);
        assertThat(actualReport, equalTo(expectedReport));
        
        verifyAll();
    }
    
    @Test
    public void testStrengthChangeReport() throws Exception {
        /* Setup */

        final Date testStartDate = new Date();
        final String headerText = "Strength change header text";
        
        final ProductChangeDto strengthChange = new ProductChangeDto(4L, "Product A", "CMOP A", "3", "2");
        
        final List<String> expectedReport = Arrays.asList(
            headerText,
            formatPairWithSquareBraces(strengthChange.getVaProductName(), strengthChange.getCmopId()),
            doubleIndent("  Old Value: " + strengthChange.getOldValue()),
            doubleIndent("  New Value: " + strengthChange.getNewValue()),
            "");

        final MessageDataCapabilityImpl objUnderTest = buildWithMocks();
        
        /* Expected method calls */
        expect(mockEplNdfOutgoingDifferencesDao.messageGenericChange(VistaProductFile.STRENGTH_FIELD_NUMBER, testStartDate))
            .andReturn(Arrays.asList(strengthChange));
        expect(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.STRENGTH))
            .andReturn(Arrays.asList(headerText));

        /* Behavior & Verification */
        replayAll();
        
        // Note: we should ideally test buildDataUpdateForNdfReport() but this is better than nothing.
        final List<String> actualReport = objUnderTest.strengthChangeReport(testStartDate);
        assertThat(actualReport, equalTo(expectedReport));

        verifyAll();
    }

    /**
     * Tests {@link MessageDataCapabilityImpl#copayTierChangesReport(Date)}. We should really test this as part of
     * {@link MessageDataCapabilityImpl#buildDataUpdateForNdfReport(Date)}, that we haven't figured out a good way of
     * testing that giant function yet.
     */
    @Test
    public void testCopayTierChangesReport() throws Exception
    {
      /* Setup */

      final Date testStartDate = new Date();
      final String headerText = "The COPAY TIER etc. etc. ...";
      
      // Product 1
      final String product1Ien = "10001";
      final String product1Name = "PRODUCT 1";
      final String product1CmopId = "CMOPID1";
      // With an end date.
      final CopayTierChange product1Tier1 = new CopayTierChange(
          product1Ien, product1Name, product1CmopId, "23",
          Optional.of(LocalDate.of(2016, 11, 2)), Optional.of(LocalDate.of(2016, 11, 12)));
      // Without an end date.
      final CopayTierChange product1Tier2 = new CopayTierChange(
          product1Ien, product1Name, product1CmopId, "11", Optional.of(LocalDate.of(2016, 11, 13)), Optional.empty());

      // Product 2
      final String product2Ien = "20002";
      final String product2Name = "PRODUCT 2";
      final String product2CmopId = "CMOPID2";
      final CopayTierChange product2Tier1 = new CopayTierChange(
          product2Ien, product2Name, product2CmopId, VistaFile.ALL_DELETED,
          Optional.empty(), Optional.empty());
      
      // Product 3
      final String product3Ien = "30003";
      final String product3Name = product2Name; // intentional
      final String product3CmopId = "";
      final CopayTierChange product3Tier1 = new CopayTierChange(
          product3Ien, product3Name, product3CmopId, product1Tier1.getCopayTier(),
          Optional.of(LocalDate.of(2016, 3, 4)), Optional.empty());

      final List<String> expectedReport = Arrays.asList(
          "", // Starts with a blank line.
          headerText,
          "",
          formatPairWithSquareBraces(product1Name, product1CmopId),
          indentLevel2("COPAY TIER LEVEL: 23  Start Date: 11/02/2016   End Date: 11/12/2016"),
          indentLevel2("COPAY TIER LEVEL: 11  Start Date: 11/13/2016   End Date:           "),
          "",
          formatPairWithSquareBraces(product2Name, product2CmopId),
          indentLevel2(MessageDataCapabilityImpl.COPAY_TIERS_REMOVED_MESSAGE),
          "",
          formatPairWithSquareBraces(product3Name, product3CmopId),
          indentLevel2("COPAY TIER LEVEL: 23  Start Date: 03/04/2016   End Date:           "));

      final MessageDataCapabilityImpl objUnderTest = buildWithMocks();
          
      final List<CopayTierChange> testChanges = Arrays.asList(product1Tier1, product1Tier2, product2Tier1, product3Tier1);

      expect(mockEplNdfOutgoingDifferencesDao.findCopayTierChanges(testStartDate))
        .andReturn(testChanges);
      expect(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.COPAYTIER))
          .andReturn(Arrays.asList("", headerText, ""));
      
      /* Behavior */
      
      replayAll();
      
      // Note: we should ideally test buildDataUpdateForNdfReport()
      final List<String> strings = objUnderTest.copayTierChangesReport(testStartDate);
      
      assertThat(strings, equalTo(expectedReport));
      
      verifyAll();
    }
    
    /**
     * Tests {@link MessageDataCapabilityImpl#buildUpdatedInteractionsReport(Date)} with the minimal possible data.
     */
    @Test
    public void testBuildUpdatedInteractionsReport_Minimal() {
        /* Setup */
        
        final Date testDate = new Date();
        final String ddiHeaderText = "DDI Header";
        
        final List<String> expectedReport = Arrays.asList(
            // DDI Change Report - always included even if no changes
            ddiHeaderText,
            "  " + MessageDataCapabilityImpl.ADDINT,
            doubleIndent("NONE"),
            "",
            "  " + MessageDataCapabilityImpl.EDITINT,
            doubleIndent("NONE"),
            "",
            "  " + MessageDataCapabilityImpl.INACTIVATEINT,
            doubleIndent("NONE"),
            "");
        
        final MessageDataCapabilityImpl objUnderTest = buildWithMocks();
        
        /* Expected method calls */
        // FDA Med Guide Report
        expect(mockEplNdfOutgoingDifferencesDao.findAddedMedGuideProducts(
                VistaFileNumber.FIFTY_68.getFileNumber(), "100", testDate))
            .andReturn(Collections.emptyList());
        expect(mockEplNdfOutgoingDifferencesDao.findEditedMedGuideProducts(
                VistaFileNumber.FIFTY_68.getFileNumber(), "100", testDate))
            .andReturn(Collections.emptyList());
        expect(mockEplNdfOutgoingDifferencesDao.findDeletedMedGuideProducts(
                VistaFileNumber.FIFTY_68.getFileNumber(), "100", testDate))
            .andReturn(Collections.emptyList());
        // DDI Change Report
        expect(mockEplNdfOutgoingDifferencesDao.findAddedDDIs(VistaFileNumber.FIFTY_SIX.getFileNumber(), ".01", testDate))
            .andReturn(Collections.emptyList());
        expect(mockEplNdfOutgoingDifferencesDao.findReactivatedDDIs(VistaFileNumber.FIFTY_SIX.getFileNumber(), "7", testDate))
            .andReturn(Collections.emptyList());
        expect(mockEplNdfOutgoingDifferencesDao.findEditedDDIs(VistaFileNumber.FIFTY_SIX.getFileNumber(), "3", testDate))
            .andReturn(Collections.emptyList());
        expect(mockEplNdfOutgoingDifferencesDao.findInactivatedDDIs(VistaFileNumber.FIFTY_SIX.getFileNumber(), "7", testDate))
            .andReturn(Collections.emptyList());
        expect(mockUpdateFileReportGenerator.formatHeaderTextWithPadding(ReportMessageCode.DDI))
            .andReturn(Arrays.asList(ddiHeaderText));
        // Excluded DDI Report
        expect(mockEplNdfOutgoingDifferencesDao.findExcludedDDIProducts(
                VistaFileNumber.FIFTY_68.getFileNumber(), "23", testDate))
            .andReturn(Collections.emptyList());
        // Previously Excluded DDI Report
        expect(mockEplNdfOutgoingDifferencesDao.findPrevExcludedDDIProducts(
                VistaFileNumber.FIFTY_68.getFileNumber(), "23", testDate))
            .andReturn(Collections.emptyList());

        /* Behavior & Verification */
        replayAll();
        
        final List<String> actualReport = objUnderTest.buildUpdatedInteractionsReport(testDate);
        assertThat(actualReport, equalTo(expectedReport));
        
        verifyAll();
    }

    @Test
    public void testGenerateUpdateFilePreviewReport_Failure() {
      
        /* Setup */
        final Date startDate = new Date();

        // Expected calls
        expect(mockEplRematchDao.findOutgoingRematches(startDate))
            .andThrow(new RuntimeException("throwing an exception to see what happens"));
        expect(mockEplNdfOutgoingDifferencesDao.findAddedMedGuideProducts(
            VistaFileNumber.FIFTY_68.getFileNumber(), "100", startDate))
            .andThrow(new RuntimeException("throwing another exception to see what happens"));

        final MessageDataCapabilityImpl objUnderTest = buildWithMocks();

        /* Behavior & Verification */
        replayAll();
        
        final List<String> actualResult = objUnderTest.generateUpdateFilePreviewReport(startDate);

        assertEquals(
            Arrays.asList(MessageDataCapabilityImpl.REPORT_FAILURE_MESSAGE, MessageDataCapabilityImpl.REPORT_FAILURE_MESSAGE),
            actualResult);
        
        verifyAll();
    }
}
