package gov.va.med.pharmacy.peps.service.common.capability.impl;

import static org.easymock.EasyMock.*;

import java.util.Arrays;
import java.util.Collections;
import java.util.Date;

import org.easymock.EasyMockSupport;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.utility.test.generator.ProductGenerator;
import gov.va.med.pharmacy.peps.common.vo.EntityType;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.domain.common.capability.DatabaseDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.ProductDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.capability.RequestDomainCapability;
import gov.va.med.pharmacy.peps.domain.common.utility.test.NoOpTransactionManager;
import gov.va.med.pharmacy.peps.service.common.scheduler.ProcessStatus;
import gov.va.med.pharmacy.peps.service.common.scheduler.ProcessStatusUpdater;
import gov.va.med.pharmacy.peps.service.national.capability.ManagedItemCapability;

/**
 * Unit tests for {@link ProposedInactivateCapabilityImpl}.
 */
public class ProposedInactivateCapabilityImplTest extends EasyMockSupport {

  /**
   * Tests executing the job when there are no inactivations.
   */
  @Test
  public void testSynchronizedProposedInactivationDate_NoInactivations() {
    /* Setup */
    final ProcessStatusUpdater mockProcessStatusUpdater = strictMock(ProcessStatusUpdater.class);
    
    // We don't care about the order of operations for this one.
    final DatabaseDomainCapability mockDatabaseDomainCapability = mock(DatabaseDomainCapability.class);
    
    final RequestDomainCapability mockRequestDomainCapability = strictMock(RequestDomainCapability.class);

    final ProposedInactivateCapabilityImpl objUnderTest = new ProposedInactivateCapabilityImpl();
    objUnderTest.setTransactionManager(new NoOpTransactionManager());
    objUnderTest.setProcessStatusUpdater(mockProcessStatusUpdater);
    objUnderTest.setDatabaseDomainCapability(mockDatabaseDomainCapability);
    objUnderTest.setRequestDomainCapability(mockRequestDomainCapability);

    // Expected dependency method calls.
    mockProcessStatusUpdater.updateStatus(
        ProposedInactivateCapabilityImpl.RUN_STATE_SETTING, ProcessStatus.RUNNING, objUnderTest.getUser());
    expect(mockDatabaseDomainCapability.getIdsPropInact(EntityType.NDC)).andReturn(Collections.emptyList());
    expect(mockDatabaseDomainCapability.getIdsPropInact(EntityType.PRODUCT)).andReturn(Collections.emptyList());
    expect(mockDatabaseDomainCapability.getIdsPropInact(EntityType.ORDERABLE_ITEM)).andReturn(Collections.emptyList());
    mockRequestDomainCapability.deleteCompletedRequest();
    expect(mockProcessStatusUpdater.updateStatusAndRuntime(
        ProposedInactivateCapabilityImpl.RUN_STATE_SETTING, ProcessStatus.COMPLETED,
        ProposedInactivateCapabilityImpl.LAST_RUN_SETTING, objUnderTest.getUser())).andReturn(new Date());
    
    replayAll();
    
    /* Behavior & Verification */
    
    objUnderTest.synchronizedProposedInactivationDate();
    
    verifyAll();
  }

  /**
   * Just tests trying to inactivate a product that has a validation error. This test could easily be expanded to
   * check/invalidate multiple products.
   */
  @Test
  public void testSynchronizedProposedInactivationDate_InvalidProduct() throws ValidationException {
    /* Setup */
    final ProductVo product = new ProductGenerator().getFirst();
    // Set an inactivation date that just hit.
    product.getVaDataFields().getDataField(FieldKey.PROPOSED_INACTIVATION_DATE).selectValue(new Date());

    final ProcessStatusUpdater mockProcessStatusUpdater = strictMock(ProcessStatusUpdater.class);
    
    // We don't care about the order of operations for this one.
    final DatabaseDomainCapability mockDatabaseDomainCapability = mock(DatabaseDomainCapability.class);
    
    final RequestDomainCapability mockRequestDomainCapability = strictMock(RequestDomainCapability.class);
    
    final ProductDomainCapability mockProductDomainCapability = strictMock(ProductDomainCapability.class);
    
    final ManagedItemCapability mockManagedItemCapability = strictMock(ManagedItemCapability.class);

    final ProposedInactivateCapabilityImpl objUnderTest = new ProposedInactivateCapabilityImpl();
    objUnderTest.setTransactionManager(new NoOpTransactionManager());
    objUnderTest.setProcessStatusUpdater(mockProcessStatusUpdater);
    objUnderTest.setDatabaseDomainCapability(mockDatabaseDomainCapability);
    objUnderTest.setRequestDomainCapability(mockRequestDomainCapability);
    objUnderTest.setProductDomainCapability(mockProductDomainCapability);
    objUnderTest.setManagedItemCapability(mockManagedItemCapability);

    // Expected dependency method calls.
    mockProcessStatusUpdater.updateStatus(
        ProposedInactivateCapabilityImpl.RUN_STATE_SETTING, ProcessStatus.RUNNING, objUnderTest.getUser());
    expect(mockDatabaseDomainCapability.getIdsPropInact(EntityType.NDC)).andReturn(Collections.emptyList());

    // Product processing.
    expect(mockDatabaseDomainCapability.getIdsPropInact(EntityType.PRODUCT))
      .andReturn(Arrays.asList(Long.valueOf(product.getId())));
    expect(mockRequestDomainCapability.retrieve(product.getId(), EntityType.PRODUCT))
      .andReturn(Collections.emptyList());
    expect(mockProductDomainCapability.retrieve(product.getId())).andReturn(product);
    // not checking the itemModDifferences due to the difficulty involved
    expect(mockManagedItemCapability.commitModifications(anyObject(), eq(objUnderTest.getUser())))
      // This ValidationException doesn't make sense, but it will do.
      .andThrow(new ValidationException(ValidationException.INACTIVATE_COPAYTIER));

    // We should keep processing: will not roll back the transaction.
    expect(mockDatabaseDomainCapability.getIdsPropInact(EntityType.ORDERABLE_ITEM)).andReturn(Collections.emptyList());
    mockRequestDomainCapability.deleteCompletedRequest();
    expect(mockProcessStatusUpdater.updateStatusAndRuntime(
        ProposedInactivateCapabilityImpl.RUN_STATE_SETTING, ProcessStatus.COMPLETED,
        ProposedInactivateCapabilityImpl.LAST_RUN_SETTING, objUnderTest.getUser())).andReturn(new Date());
    
    replayAll();
    
    /* Behavior & Verification */
    
    objUnderTest.synchronizedProposedInactivationDate();
    
    verifyAll();
  }

}
