package gov.va.med.pharmacy.peps.service.common.capability.impl;

import static org.easymock.EasyMock.expect;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;

import org.apache.commons.collections4.ListUtils;
import org.easymock.EasyMockSupport;
import org.junit.Before;
import org.junit.Test;

import gov.va.med.pharmacy.peps.common.exception.DuplicateRematchException;
import gov.va.med.pharmacy.peps.common.utility.impl.test.EnvironmentUtilityStub;
import gov.va.med.pharmacy.peps.common.utility.test.generator.ProductGenerator;
import gov.va.med.pharmacy.peps.common.utility.test.generator.UserGenerator;
import gov.va.med.pharmacy.peps.common.vo.FieldKey;
import gov.va.med.pharmacy.peps.common.vo.IngredientVo;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryDetailsVo;
import gov.va.med.pharmacy.peps.common.vo.ItemAuditHistoryVo;
import gov.va.med.pharmacy.peps.common.vo.ItemStatus;
import gov.va.med.pharmacy.peps.common.vo.MatchReason;
import gov.va.med.pharmacy.peps.common.vo.OrderableItemVo;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.RematchStatus;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.common.vo.diff.Difference;
import gov.va.med.pharmacy.peps.common.vo.diff.test.DifferenceGenerator;
import gov.va.med.pharmacy.peps.domain.common.capability.RematchMgtDomainCapability;
import gov.va.med.pharmacy.peps.service.common.capability.ModRematch;
import gov.va.med.pharmacy.peps.service.common.capability.RematchCapability;
import gov.va.med.pharmacy.peps.service.common.capability.RematchPreperationBean;

/**
 * Unit tests for {@link RematchCapabilityImpl}.
 */
public class RematchCapabilityImplTest extends EasyMockSupport {
  
  /**
   * Just for convenience since many tests use this.
   */
  private final ProductGenerator productGenerator = new ProductGenerator();
  
  /**
   * Just for convenience since many tests use this.
   */
  private final UserGenerator userGenerator = new UserGenerator();

  private final EnvironmentUtilityStub environmentUtility = new EnvironmentUtilityStub();
  
  /**
   * Mock stored as a member variable for convenient creation in {@link #setup()}.
   */
  private RematchMgtDomainCapability mockRematchDomainCapability;
  
  @Before
  public void setup() {
    mockRematchDomainCapability = strictMock(RematchMgtDomainCapability.class);
  }
  
  /**
   * Tests calling {@link RematchCapabilityImpl#updateUponProposedInactivation(
   * gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, UserVo)} for a non-product.
   */
  @Test
  public void testUpdateUponProposedInactivation_NonProduct() {
    /* Setup */

    final UserVo updatingUser = userGenerator.getNationalManagerOne();

    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

    // No expected method calls
    
    replayAll();
    
    /* Behavior & verification */
    
    objUnderTest.updateUponProposedInactivation(new OrderableItemVo(), updatingUser);
    objUnderTest.updateUponProposedInactivation(new IngredientVo(), updatingUser);
    
    verifyAll();
  }

  @Test
  public void testUpdateUponProposedInactivation_NoExisting() {
    /* Setup */

    final UserVo updatingUser = userGenerator.getNationalManagerTwo();
    final ProductVo testProduct = productGenerator.getFirst();

    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

    // Expected method calls
    expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testProduct.getId())))
      .andReturn(Optional.empty());
    
    replayAll();
    
    /* Behavior & verification */
    
    objUnderTest.updateUponProposedInactivation(testProduct, updatingUser);
    
    verifyAll();
  }

  @Test
  public void testUpdateUponProposedInactivation_ExistingFutureInact() throws DuplicateRematchException {
    /* Setup */

    final UserVo updatingUser = userGenerator.getNationalManagerOne();
    final RematchVo existingRematch = createExistingRematch(MatchReason.FUTURE_INACTIVATION, RematchStatus.FUTURE);
    final ProductVo testProduct = existingRematch.getFromProduct();

    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

    // Expected method calls
    expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testProduct.getId())))
      .andReturn(Optional.of(existingRematch));
    expect(mockRematchDomainCapability.saveOrUpdate(existingRematch, updatingUser)).andReturn(existingRematch);
    
    replayAll();
    
    /* Behavior & verification */
    
    objUnderTest.updateUponProposedInactivation(testProduct, updatingUser);
    
    verifyAll();

    assertThat("should have changed match status", existingRematch.getStatus(), equalTo(RematchStatus.PENDING));
  }

  @Test
  public void testUpdateUponProposedInactivation_ExistingOther() {
    /* Setup */

    final MatchReason initialMatchReason = MatchReason.MANUAL_REMATCH;
    final RematchStatus initialStatus = RematchStatus.APPROVED;
    final RematchVo existingRematch = createExistingRematch(initialMatchReason, initialStatus);
    final ProductVo testProduct = existingRematch.getFromProduct();

    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

    // Expected method calls
    expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testProduct.getId())))
      .andReturn(Optional.of(existingRematch));
    
    replayAll();
    
    /* Behavior & verification */
    
    objUnderTest.updateUponProposedInactivation(testProduct, userGenerator.getNationalManagerOne());
    
    verifyAll();
    
    assertThat("should not have changed rematch status", existingRematch.getStatus(), equalTo(initialStatus));
  }

    /**
     * Tests that
     * {@link RematchCapabilityImpl#saveOrUpdateUncommittedRematch(Collection, RematchVo, UserVo)} deletes
     * the rematch if it is obsolete.
     */
  @Test
  public void testSaveOrUpdateUncommittedRematch_DeleteObsolete() throws DuplicateRematchException {
    
    /* Setup */
    
    final List<ProductVo> testProducts = productGenerator.getList();
    final ProductVo fromProduct = testProducts.get(0);
    final ProductVo toProduct = testProducts.get(1);
    final UserVo testUser = userGenerator.getNationalManagerTwo();

    final Collection<Difference> differences = Arrays.asList(
        // Red herring difference.
        new Difference(FieldKey.HERBAL, Boolean.FALSE, Boolean.TRUE));
    final RematchVo testRematch = RematchVo.automaticRematch(fromProduct, toProduct, RematchStatus.PENDING);
    testRematch.setEplRematchId(null);  // Unsaved.
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    // No expected method calls, should just return an empty optional.
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.saveOrUpdateUncommittedRematch(differences, testRematch, testUser),
        equalTo(Optional.empty()));
    
    verifyAll();
  }

  /**
   * Tests that {@link
   * RematchCapabilityImpl#updateRematchesOnItemCommit(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
   * Optional, UserVo)} does not delete an existing rematch if it shouldn't.
   */
  @Test
  public void testUpdateRematchesOnItemCommit_PreserveExisting() throws DuplicateRematchException {

    /* Setup */
    
    final UserVo testUser = userGenerator.getNationalManagerTwo();
    final RematchVo existingRematch = createExistingRematch(MatchReason.FUTURE_INACTIVATION, RematchStatus.FUTURE);
    final ProductVo testProduct = existingRematch.getFromProduct();
    
    final Collection<Difference> differences = Arrays.asList(
        // The item has a rematch and we are inactivating it. This shouldn't actually happen in production, but it's a good
        // edge case for this test.
        new Difference(FieldKey.ITEM_STATUS, ItemStatus.ACTIVE, ItemStatus.INACTIVE));
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    // Expected method calls
    expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testProduct.getId())))
        .andReturn(Optional.of(existingRematch)).anyTimes();
    // Should _not_ call delete.
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.updateRematchesOnItemCommit(testProduct, differences, Optional.empty(), testUser),
        equalTo(Optional.empty()));
    
    verifyAll();
  }

  
  /**
   * Tests that {@link
   * RematchCapabilityImpl#updateRematchesOnItemCommit(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
   * Optional, UserVo)} deletes the existing rematch if we are reversing the trigger.
   */
  @Test
  public void testUpdateRematchesOnItemCommit_DeleteReversed() throws DuplicateRematchException {

    /* Setup */
    
    // Create the existing rematch.
    final RematchVo existingRematch = createExistingRematch(MatchReason.FUTURE_INACTIVATION, RematchStatus.FUTURE);
    final UserVo testUser = userGenerator.getNationalManagerTwo();
    final ProductVo testProduct = existingRematch.getFromProduct();

    // Hardcoding some RematchVo business logic here: clearing a Proposed Inactivation Date obsoletes rematch.
    final Collection<Difference> differences = Arrays.asList(
        DifferenceGenerator.makeDifferenceForDataField(FieldKey.PROPOSED_INACTIVATION_DATE, new Date(), null));
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    // Expected method calls
    expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testProduct.getId())))
      .andReturn(Optional.of(existingRematch));
    mockRematchDomainCapability.delete(existingRematch);
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.updateRematchesOnItemCommit(testProduct, differences, Optional.empty(), testUser),
        equalTo(Optional.empty()));
    
    verifyAll();
  }
  
  /**
   * Tests that {@link
   * RematchCapabilityImpl#updateRematchesOnItemCommit(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
   * Optional, UserVo)} works properly if we are reversing the trigger and there is no existing rematch.
   */
  @Test
  public void testUpdateRematchesOnItemCommit_DeleteReversedNonExisting() throws DuplicateRematchException {

    /* Setup */
    
    final ProductVo testProduct = productGenerator.getFirst();
    final UserVo testUser = userGenerator.getNationalManagerOne();

    final Collection<Difference> differences = Arrays.asList(
        DifferenceGenerator.makeDifferenceForDataField(FieldKey.PROPOSED_INACTIVATION_DATE, new Date(), null));
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

    // Expected method calls
    expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testProduct.getId())))
      .andReturn(Optional.empty());
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.updateRematchesOnItemCommit(testProduct, differences, Optional.empty(), testUser),
        equalTo(Optional.empty()));
    
    verifyAll();
  }

  /**
   * Tests that {@link
   * RematchCapabilityImpl#updateRematchesOnItemCommit(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
   * Optional, UserVo)} deletes the rematch if it is obsolete.
   */
  @Test
  public void testUpdateRematchesOnItemCommit_DeleteObsolete() throws DuplicateRematchException {
    
    /* Setup */
    
    final List<ProductVo> testProducts = productGenerator.getList();
    final ProductVo fromProduct = testProducts.get(0);
    final ProductVo toProduct = testProducts.get(1);
    final UserVo testUser = userGenerator.getNationalManagerTwo();

    final Collection<Difference> differences = Arrays.asList(
        // Red herring difference.
        new Difference(FieldKey.HERBAL, Boolean.FALSE, Boolean.TRUE));
    final RematchVo testRematch = RematchVo.automaticRematch(fromProduct, toProduct, RematchStatus.PENDING);
    testRematch.setEplRematchId("1999");
    final ModRematch modRematch = ModRematch.nonOverride(testRematch,
        "ignored reason");  // The reason is currently ignored for non-manual rematches.
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    // Expected method calls
    noExistingRematch(fromProduct);
    mockRematchDomainCapability.delete(testRematch);
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.updateRematchesOnItemCommit(fromProduct, differences, Optional.of(modRematch), testUser),
        equalTo(Optional.empty()));

    assertThat("should not add audit history", fromProduct.getNewAuditHistory(), nullValue());
    
    verifyAll();
  }
  
  /**
   * Tests that {@link
   * RematchCapabilityImpl#updateRematchesOnItemCommit(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
   * Optional, UserVo)} does not delete the rematch if it is not obsolete.
   */
  @Test
  public void testUpdateRematchesOnItemCommit_KeepCurrent() throws DuplicateRematchException {
    
    /* Setup */
    
    final List<ProductVo> testProducts = productGenerator.getList();
    final ProductVo fromProduct = testProducts.get(0);
    final ProductVo toProduct = testProducts.get(1);
    final UserVo testUser = userGenerator.getNationalManagerTwo();
    final Collection<Difference> differences = Arrays.asList(
        new Difference(FieldKey.ITEM_STATUS, ItemStatus.ACTIVE, ItemStatus.INACTIVE));
    final RematchVo testRematch = RematchVo.automaticRematch(fromProduct, toProduct, RematchStatus.PENDING);
    final ModRematch modRematch = ModRematch.nonOverride(testRematch,
        "ignored reason");  // The reason is currently ignored for non-manual rematches.
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    // Expected method calls.
    noExistingRematch(fromProduct);
    expect(mockRematchDomainCapability.saveOrUpdate(testRematch, testUser)).andReturn(testRematch);
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.updateRematchesOnItemCommit(fromProduct, differences, Optional.of(modRematch), testUser),
        equalTo(Optional.of(testRematch)));
    
    assertThat("should not add audit history", fromProduct.getNewAuditHistory(), nullValue());
    assertThat("should approve rematch", testRematch.getStatus(), equalTo(RematchStatus.APPROVED));
    
    verifyAll();
  }
  
  /**
   * <p>
   * Tests that {@link RematchCapabilityImpl#updateUponCommittingItemChanges(RequestVo,
   * gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, java.util.Collection)} updates the rematch if the trigger reason
   * changed.
   * </p>
   * 
   * <p>
   * I'd really like to test that the method calls {@link RematchVo#updateStatusPerChanges(Collection, boolean)}, but I
   * want to use a real {@link RematchVo} instead of a mock. Mockito provides "spy" functionality on a real object but
   * EasyMock apparently does not. So we just test this one case of the status changing instead.
   * </p>
   */
  @Test
  public void testUpdateRematchesOnItemCommit_UpdateCurrent() throws DuplicateRematchException {
    
    /* Setup */
    
    final List<ProductVo> testProducts = productGenerator.getList();
    final ProductVo fromProduct = testProducts.get(0);
    final ProductVo toProduct = testProducts.get(1);
    final UserVo testUser = userGenerator.getNationalManagerOne();
    final Collection<Difference> differences = Arrays.asList(
        DifferenceGenerator.makeDifferenceForDataField(FieldKey.PROPOSED_INACTIVATION_DATE, null, new Date()),
        new Difference(FieldKey.ITEM_STATUS, ItemStatus.ACTIVE, ItemStatus.INACTIVE));
    final RematchVo testRematch = RematchVo.automaticRematch(fromProduct, toProduct, RematchStatus.FUTURE);
    final RematchVo expectedRematch = RematchVo.automaticRematch(fromProduct, toProduct, RematchStatus.APPROVED);
    final ModRematch modRematch = ModRematch.nonOverride(testRematch,
        "ignored reason");  // The reason is currently ignored for non-manual rematches.
    
    final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);
    
    // Expected method calls.
    noExistingRematch(fromProduct);
    expect(mockRematchDomainCapability.saveOrUpdate(expectedRematch, testUser)).andReturn(expectedRematch);
    
    replayAll();
    
    /* Behavior & verification */
    
    assertThat(
        objUnderTest.updateRematchesOnItemCommit(fromProduct, differences, Optional.of(modRematch), testUser),
        equalTo(Optional.of(expectedRematch)));
    
    verifyAll();
  }
  
    /**
     * Tests that {@link
     * RematchCapabilityImpl#updateRematchesOnItemCommit(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
     * Optional, UserVo)} does not delete a manual rematch.
     */
    @Test
    public void testUpdateRematchesOnItemCommit_KeepManual() throws DuplicateRematchException {

        final String providedReason = "manual rematch reason";
        final List<ProductVo> testProducts = productGenerator.getList();
        final ProductVo fromProduct = testProducts.get(0);
        final ProductVo toProduct = testProducts.get(1);
        final UserVo testUser = userGenerator.getNationalManagerTwo();
        final Collection<Difference> differences = Collections.emptyList(); // No differences.
        final RematchVo testRematch = RematchVo.automaticRematch(fromProduct, toProduct, RematchStatus.PENDING);
        final ModRematch modRematch = ModRematch.override(Optional.of(testRematch), providedReason);
        
        final ItemAuditHistoryVo expectedHistory = expectedManualRematchHistory(
            testUser, fromProduct, providedReason, Difference.NO_VALUE, testRematch.getToProductsString());

        final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

        // Expected method calls.
        expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(testRematch.getFromProduct().getId())))
            .andReturn(Optional.empty());
        expect(mockRematchDomainCapability.saveOrUpdate(testRematch, testUser)).andReturn(testRematch);

        replayAll();

        /* Behavior & verification */

        assertThat(
            objUnderTest.updateRematchesOnItemCommit(fromProduct, differences, Optional.of(modRematch), testUser),
            equalTo(Optional.of(testRematch)));

        assertThat(
            "should add correct audit history",
            fromProduct.getNewAuditHistory(),
            equalTo(expectedHistory));

        assertThat("should approve rematch", testRematch.getStatus(), equalTo(RematchStatus.APPROVED));

        verifyAll();
    }
    
    /**
    * Tests that {@link
    * RematchCapabilityImpl#updateRematchesOnItemCommit(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo, Collection,
    * Optional, UserVo)} overwrites a previous rematch for it's fromProduct if it exists.
    */
    @Test
    public void testUpdateRematchesOnItemCommit_OverwritePrevious() throws DuplicateRematchException {
        
        final String providedReason = "overwrite rematch reason";
        final List<ProductVo> testProducts = productGenerator.getList();
        final ProductVo fromProduct = testProducts.get(0);
        final ProductVo toProductPrevious = testProducts.get(1);
        final ProductVo toProductManual = testProducts.get(2);
        final UserVo testUser = userGenerator.getNationalManagerTwo();
        final Collection<Difference> differences = Collections.emptyList(); // No differences.
        final RematchVo previousRematch = RematchVo.automaticRematch(fromProduct, toProductPrevious, RematchStatus.APPROVED);
        final RematchVo manualRematch = RematchVo.automaticRematch(fromProduct, toProductManual, RematchStatus.PENDING);
        final ModRematch modRematch = ModRematch.override(Optional.of(manualRematch), providedReason);
        
        final ItemAuditHistoryVo expectedHistory = expectedManualRematchHistory(
            testUser, fromProduct, providedReason, previousRematch.getToProductsString(), manualRematch.getToProductsString());

        final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

        // Expected method calls.
        expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(manualRematch.getFromProduct().getId())))
            .andReturn(Optional.of(previousRematch));
        mockRematchDomainCapability.delete(previousRematch);
        expect(mockRematchDomainCapability.saveOrUpdate(manualRematch, testUser)).andReturn(manualRematch);

        replayAll();

        /* Behavior & verification */

        assertThat(
            objUnderTest.updateRematchesOnItemCommit(fromProduct, differences, Optional.of(modRematch), testUser),
            equalTo(Optional.of(manualRematch)));
        
        assertThat(
            "should add correct audit history",
            fromProduct.getNewAuditHistory(),
            equalTo(expectedHistory));

        assertThat("should approve rematch", manualRematch.getStatus(), equalTo(RematchStatus.APPROVED));

        verifyAll();
        
    }

    /**
     * Tests that {@link RematchCapabilityImpl#updateRematchesOnItemCommit(gov.va.med.pharmacy.peps.common.vo.ManagedItemVo,
     * Collection, Optional, UserVo)} deletes an existing rematch if requested.
     */
    @Test
    public void testUpdateRematchesOnItemCommit_DeleteExisting() throws DuplicateRematchException {

        /* Setup */

        final String providedReason = "delete reason";
        final List<ProductVo> testProducts = productGenerator.getList();
        final ProductVo fromProduct = testProducts.get(0);
        final ProductVo toProductPrevious = testProducts.get(1);
        final RematchVo existingRematch = RematchVo.automaticRematch(fromProduct, toProductPrevious, RematchStatus.APPROVED);
        final UserVo testUser = userGenerator.getNationalManagerTwo();
        final ModRematch modRematch = ModRematch.override(Optional.empty(), providedReason);

        final ItemAuditHistoryVo expectedHistory = expectedManualRematchHistory(
            testUser, fromProduct, providedReason, existingRematch.getToProductsString(), Difference.NO_VALUE);

        final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

        // Expected method calls.
        expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(fromProduct.getId())))
            .andReturn(Optional.of(existingRematch));
        mockRematchDomainCapability.delete(existingRematch);

        replayAll();

        /* Behavior & verification */

        assertThat(
            objUnderTest.updateRematchesOnItemCommit(fromProduct, Collections.emptyList(), Optional.of(modRematch), testUser),
            equalTo(Optional.empty()));
        
        assertThat(
            "should add correct audit history",
            fromProduct.getNewAuditHistory(),
            equalTo(expectedHistory));
    }
    
    @Test
    public void testPrepareForRematch_NoDefault() {
        /* Setup */
        final List<ProductVo> testProducts = productGenerator.getList();
        final ProductVo fromProduct = testProducts.get(0);
        final long fromProductId = Long.parseLong(fromProduct.getId());
        final RematchVo existingRematch = RematchVo.automaticRematch(fromProduct, testProducts.get(1), RematchStatus.APPROVED);
        final List<ProductVo> basePotentialMatches = Arrays.asList(testProducts.get(1));

        final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

        // Expected method calls.
        expect(mockRematchDomainCapability.getRematchForProduct(fromProductId))
            .andReturn(Optional.of(existingRematch));
        expect(mockRematchDomainCapability.findRematchProducts(fromProductId))
            .andReturn(basePotentialMatches);
        
        replayAll();

        /* Behavior & verification */
        
        final HashSet<MatchReason> reasons = new HashSet<>(Arrays.asList(MatchReason.INACTIVATION, MatchReason.VA_PRINT_NAME));
        final RematchPreperationBean bean = objUnderTest.prepareForRematch(fromProduct, reasons);
        
        assertTrue("should indicate existing rematch", bean.getRematchExistsForProduct());
        assertThat(bean.getPotentialRematchProducts(), equalTo(basePotentialMatches));
    }
    
    @Test
    public void testPrepareForRematch_DefaultSelected() {
        /* Setup */
        final List<ProductVo> testProducts = productGenerator.getList();
        final ProductVo fromProduct = testProducts.get(0);
        final long fromProductId = Long.parseLong(fromProduct.getId());
        final List<ProductVo> basePotentialMatches = Arrays.asList(testProducts.get(1));

        // Expected method calls.
        final RematchCapabilityImpl objUnderTest = new RematchCapabilityImpl(mockRematchDomainCapability, environmentUtility);

        // Expected method calls.
        expect(mockRematchDomainCapability.getRematchForProduct(fromProductId))
            .andReturn(Optional.empty());
        expect(mockRematchDomainCapability.findRematchProducts(fromProductId))
            .andReturn(basePotentialMatches);
        
        replayAll();

        /* Behavior & verification */
        final RematchPreperationBean bean = objUnderTest.prepareForRematch(
            fromProduct, Collections.singleton(MatchReason.VA_PRINT_NAME));

        assertFalse("should indicate no existing rematch", bean.getRematchExistsForProduct());
        final List<ProductVo> expectedPotentialMatches = ListUtils.union(Arrays.asList(fromProduct), basePotentialMatches);
        assertThat(bean.getPotentialRematchProducts(), equalTo(expectedPotentialMatches));
    }

    /**
     * Returns the expected {@link ItemAuditHistoryVo} for a manual rematch.
     * @return never null
     */
    private ItemAuditHistoryVo expectedManualRematchHistory(
        final UserVo testUser, final ProductVo product, final String reason, final String oldValue, final String newValue) {
        final ItemAuditHistoryVo expectedAuditHistory =
            product.createModificationAuditHistory(testUser, environmentUtility);
        expectedAuditHistory.addDetail(new ItemAuditHistoryDetailsVo(
            expectedAuditHistory.getEventCategory(),
            reason,
            RematchCapability.FIELD_NAME,
            oldValue,
            newValue));
        return expectedAuditHistory;
    }
    
  private RematchVo createExistingRematch(final MatchReason matchReason, final RematchStatus status) {
    
    // Create the existing rematch.
    final List<ProductVo> testProducts = productGenerator.getList();
    final ProductVo fromProduct = testProducts.get(0);
    final ProductVo toProduct = testProducts.get(1);
    final RematchVo existingRematch = RematchVo.automaticRematch(fromProduct, toProduct, status);
    existingRematch.setEplRematchId("1244");
    
    return existingRematch;
  }

    /**
     * Configures {@link #mockRematchDomainCapability} to act like there is not existing rematch for the given product.
     */
    private void noExistingRematch(final ProductVo fromProduct) {
        expect(mockRematchDomainCapability.getRematchForProduct(Long.parseLong(fromProduct.getId())))
            .andReturn(Optional.empty()).anyTimes();
    }

}
