/**
 * Source file created in 2011 by Southwest Research Institute
 */


package gov.va.med.pharmacy.peps.presentation.common.auth;

import java.io.IOException;
import java.util.Collections;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;

import gov.va.med.pharmacy.peps.common.exception.AuthenticationException;
import gov.va.med.pharmacy.peps.common.exception.ValidationException;
import gov.va.med.pharmacy.peps.common.utility.security.UntrustedLogParam;
import gov.va.med.pharmacy.peps.common.vo.Role;
import gov.va.med.pharmacy.peps.common.vo.UserVo;
import gov.va.med.pharmacy.peps.presentation.common.context.UserContext;
import gov.va.med.pharmacy.peps.presentation.common.controller.UserController;
import gov.va.med.pharmacy.peps.presentation.common.spring.interceptor.PepsSpringInterceptor;
import gov.va.med.pharmacy.peps.service.common.session.UserService;


/**
 * Translates a SSOi authentication to a PPS authentication.
 */
public class LoginSpringInterceptor extends PepsSpringInterceptor {

    private static final Logger LOG = LogManager.getLogger(LoginSpringInterceptor.class);

    private final UserService userService;
    
    /**
     * Constructs an instance with the required dependencies.
     * @param pUserService The {@link UserService} to use.
     */
    @Autowired
    public LoginSpringInterceptor(final UserService pUserService) {
        userService = pUserService;
    }

    /**
     * SSOi actually authenticates the user, this interceptor just maps the SSOi authentication into our {@link UserVo}
     * and populates the roles.
     * 
     * @param request current HttpServletRequest
     * @param response current HttpServletResponse
     * @param handler The handler
     * @return true if the user is logged in, else false
     * @throws ValidationException if the data coming from SSOi is incomplete or invalid.
     * @throws IOException if an input or output exception occurs.
     * @see org.springframework.web.servlet.handler.HandlerInterceptorAdapter#preHandle(HttpServletRequest,
     *          HttpServletResponse, Object)
     */
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
        throws IOException, ValidationException {

        final String userVauid = request.getHeader(SsoiHeader.VAUID.getHeaderName());
        LOG.debug("User VAUID is: {}", new UntrustedLogParam(userVauid));
        
        HttpSession session = request.getSession();
        
        LOG.debug(request.getRequestURL());
        
        // Need to add the user to the session.
        if (session.getAttribute(PpsAuthentication.USER_ATTRIBUTE) == null
            && request.getRequestURI().endsWith(UserController.INITIAL_LOGIN_URL)) {
            LOG.debug("New user login detected: setting user context.");
            
            logSsoiHeaders(request);
            
            final String firstnameHeader = request.getHeader(SsoiHeader.FIRSTNAME.getHeaderName());
            final String lastnameHeader = request.getHeader(SsoiHeader.LASTNAME.getHeaderName());
            if (firstnameHeader != null && lastnameHeader != null) {
                
                if (!(firstnameHeader.isEmpty()) && !(lastnameHeader.isEmpty())) {
                    LOG.debug("firstname: " + firstnameHeader + ", lastname: " + lastnameHeader 
                                            + ", vauid: " + userVauid);
                    
                    UserVo user = toUser(userVauid, firstnameHeader, lastnameHeader, request);
                    
                    UserContext userContext = getUserContext(session);
                    userContext.setUser(user);
                    
                    //Set the user attribute on the session so the controller can use it
                    setUserContext(request, session, userContext);
                    session.setAttribute(PpsAuthentication.USER_ATTRIBUTE, user);
                }
                else {
                    LOG.error("firstname header and/or lastname header from IAM SSOi are empty");
                    throw new AuthenticationException(AuthenticationException.SSOI_TRAITS_MISSING);
                }
            }
            else {
                LOG.error("firstname header and/or lastname header from IAM SSOi are null");
                throw new AuthenticationException(AuthenticationException.SSOI_TRAITS_MISSING);
            }
        }
        
        if (!request.isRequestedSessionIdValid()) {
            LOG.debug("Session invalid: will redirect the user to the log out page.");
            final String logOutUrl = PpsAuthentication.getLogoutTargetUrl(request);
            if (StringUtils.isNotEmpty(logOutUrl)) {
                LOG.debug("Redirecting to {}.", new UntrustedLogParam(logOutUrl));
                response.sendRedirect(logOutUrl);
                return false;
            }
        }

        if (session.getAttribute(PpsAuthentication.USER_ATTRIBUTE) == null) {
            LOG.debug("No user: redirecting to index page");
            response.sendRedirect(request.getContextPath() + "/index.go");
            return false;
        }
        
        // everything is fine, return true
        return true;
    }

    private void logSsoiHeaders(HttpServletRequest request) {
        if (LOG.isDebugEnabled()) {
          LOG.debug("All HTTP headers: {}", Collections.list(request.getHeaderNames()));
          for (final SsoiHeader header : SsoiHeader.values()) {
            final String headerName = header.getHeaderName();
            LOG.debug("SSOi Header {} is: {}", headerName, new UntrustedLogParam(request.getHeader(headerName)));
          }
        }
    }


    /**
     * @throws ValidationException if the user didn't exist in the database and the newly created {@link UserVo} is not valid
     */
    private UserVo toUser(String vauidStr, String firstName, String lastName, HttpServletRequest request)
        throws ValidationException {

        Long vauid = (vauidStr != null && !vauidStr.isEmpty()) ? Long.parseLong(vauidStr) : 0L;
        LOG.debug("User VAUID is " + vauid);

        UserVo user = userService.retrieve(vauid);

        if (user == null) {
            LOG.debug("User does not exist");
            user = new UserVo();
            user.setDuz(0L);
            user.setVauid(vauid);
            user.setFirstName(firstName);
            user.setLastName(lastName);
            // We used to get a full name attribute from VistA, but we don't with SSOi. Just concatenate first and last
            // name instead.
            final String userName = firstName + " " + lastName;
            user.setUsername(userName);
            user.setLocation("200");
            user.setStationNumber("200");
            addRole(user, Role.PSS_PPSN_VIEWER);
            
            user = userService.create(user);
        } else {
            LOG.debug("User Exists :: VAUID = " + vauid);
            
            addRole(user, Role.PSS_PPSN_VIEWER);

            if (user.getRolePnmApvdFlag()) {
                addRole(user, Role.PSS_PPSN_MANAGER);
            }

            if (user.getRolePnsApvdFlag()) {
                addRole(user, Role.PSS_PPSN_SUPERVISOR);
            }

            if (user.getRolePsrApvdFlag()) {
                addRole(user, Role.PSS_PPSN_SECOND_APPROVER);
            }

            if (user.getRoleMigApvdFlag()) {
                addRole(user, Role.PSS_PPSN_MIGRATOR);
            }
            
            // User update not required as the keys are not coming from VistA
            // user = userService.update(user);
        }

        return user;
    }

    /**
     * Adds the role to the user and logs it.
     */
    private void addRole(final UserVo user, final Role role) {
        LOG.debug("Adding role {} to user.", role);
        user.addRole(role);
    }

}
