package gov.va.med.pharmacy.peps.presentation.common.controller.requests;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.replay;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.HashSet;
import java.util.List;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;

import org.junit.Test;

import gov.va.med.pharmacy.peps.common.exception.BusinessRuleException;
import gov.va.med.pharmacy.peps.common.vo.ProductVo;
import gov.va.med.pharmacy.peps.common.vo.RematchStatus;
import gov.va.med.pharmacy.peps.common.vo.RematchVo;
import gov.va.med.pharmacy.peps.common.vo.RequestItemStatus;
import gov.va.med.pharmacy.peps.common.vo.RequestVo;


/**
 * ManageRequestsController unit tests.
 *
 */
public class ManageRequestsControllerTest {

    /**
     * test prepare matches to commit.
     * @throws BusinessRuleException exception
     */
    @Test
    public void testPrepareMatchesToCommit() throws BusinessRuleException {
        HashSet<ProductVo> productcs = new HashSet<ProductVo>();
        ProductVo product = new ProductVo();
        product.setId("123");
        product.setVaProductName("p1");
        product.setCmopId("id1");
        productcs.add(product);

        product = new ProductVo();
        product.setId("153");
        product.setVaProductName("p2");
        product.setCmopId("id3");
        productcs.add(product);
        RematchVo rematch = RematchVo.suggestedRematch(
            product, productcs, RematchStatus.FUTURE);
        RequestVo requestVo = new RequestVo();
        requestVo.setTransientRematch(rematch);
        // Step 1 - create the mock object
        HttpServletRequest req = createMock(HttpServletRequest.class);

        // Step 2 - record the expected behavior
        // set my expectations for call to request on each product
        expect(req.getParameter("match_approval_123")).andReturn("APPROVED").times(2); // called twice
        expect(req.getParameter("match_approval_153")).andReturn("REJECTED").times(2); // called twice;
        // switch the mock to replay state
        replay(req);

        // Step 3 - call the method, do assertions
        new ManageRequestsController().prepareMatchesToCommit(requestVo, req);
        assertEquals(1, requestVo.getTransientRematch().getMatchProducts().size());
    }

    /**
     * test prepar matches to commit all rejected.
     * @throws BusinessRuleException ex
     */
    @Test
    public void testPrepareMatchesToCommitAllRejected() throws BusinessRuleException { 
        HashSet<ProductVo> productcs = new HashSet<ProductVo>();
       
        ProductVo product = new ProductVo();
        product.setId("123");
        product.setVaProductName("p1");
        product.setCmopId("id1");
        productcs.add(product);

        product = new ProductVo();
        product.setId("153");
        product.setVaProductName("p2");
        product.setCmopId("id3");
        productcs.add(product);
        RematchVo rematch = RematchVo.suggestedRematch(
            product, productcs, RematchStatus.FUTURE);
        RequestVo requestVo = new RequestVo();
        requestVo.setTransientRematch(rematch);
        // Step 1 - create the mock object
        HttpServletRequest req = createMock(HttpServletRequest.class);

        // Step 2 - record the expected behavior
        // set my expectations for call to request on each product
        expect(req.getParameter("match_approval_123")).andReturn("REJECTED").times(2); // called twice
        expect(req.getParameter("match_approval_153")).andReturn("REJECTED").times(2); // called twice;
        // switch the mock to replay state
        replay(req);

        // Step 3 - call the method, do assertions
        new ManageRequestsController().prepareMatchesToCommit(requestVo, req);
        assertNull(requestVo.getTransientRematch());
    }
        
    /**
     * test prepare match items as default.
     * @throws Exception ex
     */
    @Test
    public void testPrepareMatchItemsDefault() throws Exception {
        
        HashSet<ProductVo> productcs = new HashSet<ProductVo>();
        ProductVo product = new ProductVo();
        product.setId("123");
        product.setVaProductName("p1");
        product.setCmopId("id1");
        productcs.add(product);

        product = new ProductVo();
        product.setId("153");
        product.setVaProductName("p2");
        product.setCmopId("id3");
        productcs.add(product);
        RematchVo rematch = RematchVo.suggestedRematch(
            product, productcs, RematchStatus.FUTURE);
        RequestVo requestVo = new RequestVo();
        requestVo.setTransientRematch(rematch);
        // Step 1 - create the mock object
        HttpServletRequest req = createMock(HttpServletRequest.class);

        // Step 2 - record the expected behavior
        // set my expectations for call to request on each product
        expect(req.getParameter("match_approval_123")).andReturn(null).times(2); // called twice
        expect(req.getParameter("match_approval_153")).andReturn(null).times(2); // called twice;
        // switch the mock to replay state
        replay(req);

        // Step 3 - call the method, do assertions
        List<RematchApprovalItem> items = new ManageRequestsController().prepareMatchItems(requestVo, req);
        assertTrue(items.stream().allMatch(it-> it.getApproval().equals(RequestItemStatus.APPROVED)));
    }
    
    /**
     * test prepare match items.
     * @throws Exception ex
     */
    @Test
    public void testPrepareMatchItemsSet() throws Exception {
        
        TreeSet<ProductVo> productcs = new TreeSet<ProductVo>();
        ProductVo product = new ProductVo();
        product.setId("123");
        product.setVaProductName("p1");
        product.setCmopId("id1");
        productcs.add(product);
        
        product = new ProductVo();
        product.setId("153");
        product.setVaProductName("p2");
        product.setCmopId("id3");
        productcs.add(product);
        RematchVo rematch = RematchVo.suggestedRematch(
            product, productcs, RematchStatus.FUTURE);
        RequestVo requestVo = new RequestVo();
        requestVo.setTransientRematch(rematch);
        // Step 1 - create the mock object
        HttpServletRequest req = createMock(HttpServletRequest.class);
        
        // Step 2 - record the expected behavior
        // set my expectations for call to request on each product
        expect(req.getParameter("match_approval_123")).andReturn("REJECTED").times(2); // called twice
        expect(req.getParameter("match_approval_153")).andReturn("APPROVED").times(2); // called twice;
        // switch the mock to replay state
        replay(req);
        
        // Step 3 - call the method, do assertions
        List<RematchApprovalItem> items = new ManageRequestsController().prepareMatchItems(requestVo, req);
        assertEquals(RequestItemStatus.REJECTED, items.get(0).getApproval());
        assertEquals(RequestItemStatus.APPROVED, items.get(1).getApproval());
    }
    
    /**
     * test prepareMatchItems with auto rematch.
     * @throws BusinessRuleException ex
     */
    @Test
    public void testPrepareMatchItemsWithAuto() throws BusinessRuleException {
        HashSet<ProductVo> productcs = new HashSet<ProductVo>();
        ProductVo automatic = new ProductVo();
        automatic.setId("123");
        automatic.setVaProductName("p1");
        automatic.setCmopId("id1");
        productcs.add(automatic);

        RequestVo requestVo = new RequestVo();
        requestVo.setTransientRematch(RematchVo.automaticRematch(
            new ProductVo(), automatic, RematchStatus.FUTURE));
        assertNull(requestVo.getTransientRematch().getToSuggestedProducts());
        assertNotNull(requestVo.getTransientRematch().getToAutomaticProduct());
        
        HttpServletRequest req = createMock(HttpServletRequest.class);

        expect(req.getParameter("match_approval_123")).andReturn("APPROVED").times(2); // called twice
        replay(req);

        List<RematchApprovalItem> items = new ManageRequestsController().prepareMatchItems(requestVo, req);
        assertEquals(1, items.size());
    }
    
    /**
     * test prepare matches to commit auto rematch with rejected.
     * @throws BusinessRuleException ex
     */
    @Test
    public void testPrepareMatchesToCommitAutoRejected() throws BusinessRuleException {
        HashSet<ProductVo> productcs = new HashSet<ProductVo>();
        ProductVo automatic = new ProductVo();
        automatic.setId("123");
        automatic.setVaProductName("p1");
        automatic.setCmopId("id1");
        productcs.add(automatic);

        RequestVo requestVo = new RequestVo();
        requestVo.setTransientRematch(RematchVo.automaticRematch(
            new ProductVo(), automatic, RematchStatus.FUTURE));
        assertNull(requestVo.getTransientRematch().getToSuggestedProducts());
        assertNotNull(requestVo.getTransientRematch().getToAutomaticProduct());
        
        HttpServletRequest req = createMock(HttpServletRequest.class);
        
        expect(req.getParameter("match_approval_123")).andReturn("REJECTED").times(2); // called twice
        replay(req);
        
        new ManageRequestsController().prepareMatchesToCommit(requestVo, req);
        assertNull(requestVo.getTransientRematch());
    }
   
 }
